# EMPROC -- Compute correlation and emission line velocities for
#	    a range of reduced file numbers
#	    December 11, 1991
#	    By Doug Mink, Center for Astrophysics TDC

procedure emproc (rfnrange)

string rfnrange=""		{prompt="RFN range or file"}
bool emis_vel=no		{prompt="Compute emission line velocities (y or n)"}
bool corr_vel=no		{prompt="Compute cross correlation velocities (y or n)"}
bool save_arch=no		{prompt="Save result in TDC archive (y or n)"}
bool plot=yes			{prompt="Plot results on display (y or n)"}
bool hard_copy=no		{prompt="Make printer hard copies (y or n)"}
bool cursor=no			{prompt="Wait after plot with cursor (y or n)"}
bool delfile=yes		{prompt="Delete IRAF copy of archive file (y or n)"}
bool verbose=no			{prompt="Print what's happening (y or n)"}
bool debug=no			{prompt="Debugging mode (y or n)"}

begin

bool firstrfn, oldfile, vb, delf, savarc, arctest, corvel,emvel,plres,hcres
bool runtwice,curwait
int irfn, nrfn
file rfnlist, rfnfile, rfntemp, iraffile, irafhead, arcdir, arcrec, rfn_file
string rfn,tchar, xcspectra, inst, arg, arg1, logfiles

	vb = verbose
	delf = delfile
	savarc = save_arch
	arctest = debug
	corvel = corr_vel
	emvel = emis_vel
	plres = plot
	hcres = hard_copy
	curwait = cursor
	
	rfntemp = mktemp ("arcproc")
	rfn_file = mktemp ("arcrfn")
	rfn = rfnrange
	inst = rarc.instrument
	arcdir = rarc.arc_dir

#  Make list of files from rfn range or file of rfn's

	rfnfile = rfn
	rfsub (rfn,1,1)
	tchar = rfsub.outstring
	firstrfn = yes
	if (tchar == '@') {
	    rflist (rfn,0)
	    nrfn = rflist.nlist
	    if (vb)
		print (nrfn," rfns = ",rflist.rfile)
 
	    for (irfn=1; irfn <= nrfn; irfn=irfn+1) {
		rflist (rfnfile,irfn)
		rfn = rflist.rfn
		rfnobj (rfn, list_file=rfntemp,
			instrument=inst,arc_dir=arcdir,verbose=no)
		if (rfnfile != "") {
		    if (firstrfn) {
			set clobber = yes
			type (rfntemp, >rfn_file)
			set clobber = no
			firstrfn = no
			}
		    else
			type (rfntemp, >>rfn_file)
		    }
		if (vb)
			type (rfntemp)
		delete (rfntemp)
		}
	    rfnfile = "@" // rfn_file
	    }
	else {
	    rfnobj (rfn, list_file=rfn_file,
		    instrument=inst,arc_dir=arcdir,verbose=no)
	    nrfn = rfnobj.nfiles
	    rfnfile = "@" // rfn_file
	    if (vb)
		type (rfn_file)
	    }

#  Set up parameters in archive writing, graphic display, and hard copies

	if (corvel) {
	    redshift
	    if (savarc)
		xcsao.archive=yes
	    else if (emvel)
		xcsao.archive=yes
	    else
		xcsao.archive=no
	    if (plres) {
		xcsao.hard_copy=yes
		xcsao.device="stdgraph"
		}
	    else
		xcsao.hard_copy=no
	    runtwice=no
	    if (hcres) {
		if (xcsao.hard_copy) {
		    runtwice=yes
		    }
		else {
		    xcsao.hard_copy=yes
		    xcsao.device="stdplot"
		    }
		}
	    }

	if (emvel) {
	    emspec.archive=savarc
	    emspec.plot=plres
	    emspec.hardcopy=hcres
	    emspec.cursor=curwait
	    }

	set clobber = yes

	if (vb)
	    print (nrfn," rfns = ",rflist.rfile)
 
#  Loop through reduced files in list

	for (irfn=1; irfn <= nrfn; irfn=irfn+1) {

	    rflist (rfnfile,irfn)
	    iraffile = rflist.rfile
	    rfn = rflist.rfn
	    arg = "-inst " // inst // " " // rfn
	    arg1 = "-inst " // inst // " " // rflist.rfile
	    irafhead = rflist.rfile // ".imh"
	    oldfile = access (irafhead)
	    if (vb)
		print (irfn, iraffile," ", rfn)

#	Find cross-correlation velocity

	    if (corvel) {
		if (!oldfile) {
		    rarc (rfn)
		    demach (iraffile)
		    }
		xcsao (iraffile)
		if (runtwice) {
		    xcsao.device="stdplot"
		    logfiles=xcsao.logfiles
		    xcsao.logfiles=""
		    xcsao (iraffile)
		    xcsao.device="stdgraph"
		    xcsao.logfiles=logfiles
		    }

#	Save cross-correlation results to archive file

		if (savarc) {
		    cspec (arg)
		    }

#	Delete IRAF file

		if (delf)
		    imdelete (iraffile)
		if (hcres)
		    gflush
		}

#	Find emission line velocity

	    if (emvel)
		emspec (rfn)
	    }
	delete (rfn_file)
end
# Nov 18 1991	run demach on all created IRAF files
# Dec 11 1991	Rename EMSAO to EMSPEC to avoid conflicts with new EMSAO
