# File saotdc/archive/Scripts/newproc.cl
# November 18, 1991
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# NEWPROC -- Compute correlation and emission line velocities for
#	     a range of reduced file numbers using RVSAO package

procedure newproc (rfnrange)

string rfnrange=""		{prompt="RFN range or file"}
bool emis_vel=no		{prompt="Compute emission line velocities (y or n)"}
bool corr_vel=no		{prompt="Compute cross correlation velocities (y or n)"}
bool save_arch=no		{prompt="Save result in TDC archive (y or n)"}
bool plot=yes			{prompt="Plot results on display (y or n)"}
bool hard_copy=no		{prompt="Make printer hard copies (y or n)"}
bool cursor=no			{prompt="Wait after plot with cursor (y or n)"}
bool delfile=yes		{prompt="Delete IRAF copy of archive file (y or n)"}
bool verbose=no			{prompt="Print what's happening (y or n)"}
bool debug=no			{prompt="Debugging mode (y or n)"}

begin

bool firstrfn, oldfile, vb, delf, savarc, arctest, corvel,emvel,plres,hcres
bool curwait
int irfn, nrfn
file rfnlist, rfnfile, rfntemp, iraffile, irafhead, arcdir, arcrec, rfn_file
string rfn,tchar, xcspectra, inst, arg, arg1, logfiles

	vb = verbose
	delf = delfile
	savarc = save_arch
	arctest = debug
	corvel = corr_vel
	emvel = emis_vel
	plres = plot
	hcres = hard_copy
	curwait = cursor
	
	rfntemp = mktemp ("arcproc")
	rfn_file = mktemp ("arcrfn")
	rfn = rfnrange
	inst = rarc.instrument
	arcdir = rarc.arc_dir

#  Make list of files from rfn range or file of rfn's

	rfnfile = rfn
	rfsub (rfn,1,1)
	tchar = rfsub.outstring
	firstrfn = yes
	if (tchar == '@') {
	    rflist (rfn,0)
	    nrfn = rflist.nlist
	    if (vb)
		print (nrfn," rfns = ",rflist.rfile)
 
	    for (irfn=1; irfn <= nrfn; irfn=irfn+1) {
		rflist (rfnfile,irfn)
		rfn = rflist.rfn
		rfnobj (rfn, list_file=rfntemp,
			instrument=inst,arc_dir=arcdir,verbose=no)
		if (rfnfile != "") {
		    if (firstrfn) {
			set clobber = yes
			type (rfntemp, >rfn_file)
			set clobber = no
			firstrfn = no
			}
		    else
			type (rfntemp, >>rfn_file)
		    }
		if (vb)
			type (rfntemp)
		delete (rfntemp)
		}
	    rfnfile = "@" // rfn_file
	    }
	else {
	    rfnobj (rfn, list_file=rfn_file,
		    instrument=inst,arc_dir=arcdir,verbose=no)
	    nrfn = rfnobj.nfiles
	    rfnfile = "@" // rfn_file
	    if (vb)
		type (rfn_file)
	    }
	rvsao

#  Set up parameters in archive writing, graphic display, and hard copies

	if (corvel) {
	    rarc.dispcor = yes
	    rarc.log = yes
	    xcsao.archive=savarc
	    if (emvel)
		xcsao.save_vel=yes
	    xcsao.displot = plres
	    xcsao.hardcopy = hcres
	    xcsao.curmode = curwait
	    }

	if (emvel) {
	    emsao.archive = savarc
	    emsao.displot = plres
	    emsao.hardcopy = hcres
	    emsao.curmode = curwait
	    rarc.dispcor = yes
	    rarc.log = no
	    }

	set clobber = yes

	if (vb)
	    print (nrfn," rfns = ",rflist.rfile)
 
#  Loop through reduced files in list

	for (irfn=1; irfn <= nrfn; irfn=irfn+1) {

	    rflist (rfnfile,irfn)
	    iraffile = rflist.rfile
	    rfn = rflist.rfn
	    arg = "-inst " // inst // " " // rfn
	    arg1 = "-inst " // inst // " " // rflist.rfile
	    irafhead = rflist.rfile // ".imh"
	    oldfile = access (irafhead)
	    if (vb)
		print (irfn, iraffile," ", rfn)

#	Find cross-correlation velocity

	    if (corvel) {
		if (!oldfile) {
		    rarc (rfn)
		    demach (iraffile)
		    oldfile = yes
		    }
		xcsao (iraffile)
		}

#	Find emission line velocity

	    if (emvel) {
		if (!oldfile) {
		    rarc (rfn)
		    demach (iraffile)
		    }
		emsao (iraffile)
		}

#	Save cross-correlation results to archive file

	    if (savarc) {
		cspec (arg)
		archinstall (arg1)
		}
	    if (delf && xcsao.archive) {
		arcrec = iraffile // ".corr"
		if (access (arcrec))
		    delete (arcrec)
		arcrec = iraffile // ".ansum"
		if (access (arcrec))
		    delete (arcrec)
		}
	    if (delf && emsao.archive) {
		arcrec = iraffile // ".emlines"
		if (access (arcrec))
		    delete (arcrec)
		arcrec = iraffile // ".ansum"
		if (access (arcrec))
		    delete (arcrec)
		}

#	Delete IRAF file

	    if (delf)
		imdelete (iraffile)
	    if (hcres)
		gflush
	    }
	delete (rfn_file)
end
# Nov 18 1991	Always call demach when rarc creates a file
