# File saotdc/archive/Scripts/afitd.cl
# April 19, 2004
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# AFITD -- Convert archived spectrum files to FITS files for a range of
#          reduced file numbers (use task rarcd to read from archive)
#	   Use whatever instrument archive has been set by archive,setinst

procedure afitd (rfnrange)

string	rfnrange=""	{prompt="RFN range or file"}
int	offstart=1	{prompt="First RFN with dispersion offset of 1 pixel"}
bool	wdir = yes	{prompt="Write to century subdirectories (yes or no)"}
string	outroot=""	{prompt="Output root directory"}
bool	verbose=no	{prompt="Print what's happening (yes or no)"}
bool	debug=no	{prompt="Debugging mode (yes or no)"}

begin

bool firstrfn, vb, wd
bool curwait
int irfn, jrfn, nrfn, npix
file rfnfile, iraffile, fitsfile, irafpix, arcdir, arcrec, rfn_temp
file rootdir, outr
string rfn,tchar, xcspectra, inst, logfiles, outdir

	vb = verbose
	wd = wdir
	set imtype = "fits"
	outr = outroot

	rfn = rfnrange
	inst = rarcd.instrument
	arcdir = rarcd.arc_dir
	rootdir = rarcd.root_dir

#  Set up parameters in archive writing, graphic display, and hard copies
	rarcd.dispcor = yes
	rarcd.log = no
	set clobber = yes
 
	rfnobj.root_dir = rootdir
	rfnobj.instrument = inst
	rfnobj.arc_dir = arcdir
	rfnobj.verbose = no

#  Process data files from rfn range or file of rfn's
	rfnfile = rfn
	rfsub (rfn,1,1)
	tchar = substr (rfnfile,1,1)
	if (tchar != '@') {
	    rfn_temp = mktemp ("arcrfn")
	    rfnobj (rfn, list_file=rfn_temp,listdir=no,
		    instrument=inst,arc_dir=arcdir,verbose=no)
	    rfnfile = "@" // rfn_temp
	    }

	rflist (rfnfile,0)
	nrfn = rflist.nlist
	if (vb)
	    print (nrfn," rfns from ",substr(rfnfile,2,20))

#  Loop through list of reduced files
	for (irfn=1; irfn <= nrfn; irfn=irfn+1) {
	    rflist (rfnfile,irfn)
	    rfn = rflist.rfn
	    rfnobj (rfn)
 	    iraffile = rfnobj.arcfile
	    if (vb)
		print (irfn,rfn," ",iraffile)

	# If reduced file is found for this RFN, process it
	    if (iraffile != "") {
		if (wd) {
		    outdir = outr // "/rfdir/r" // substr (iraffile,1,3)
		    if (!access (outdir)) {
			mkdir (outdir)
			}
		    rarcd.out_dir = outdir
		    }
		else {
		    rarcd.out_dir=""
		    }
		jrfn = int (rfn)
		if (jrfn < offstart)
		    rarcd (rfn, dispoff=0)
		else
		    rarcd (rfn, dispoff=1)
		fitsfile = outdir // "/" // rarcd.iraffile
		if (!access (fitsfile)) {
		    print ("AFITD: No FITS file ",fitsfile)
		    next
		    }
		imgetp (fitsfile,"i_naxis1")
		npix = int (imgetp.value)
		if (npix < 1) {
		    imdelete (fitsfile)
		    next
		    }
		}
	    flpr
	    }
	if (tchar != '@')
	    delete (rfn_temp)
end
# Jun 14 1996	New task

# Mar 18 1998	Do not add file extension to FITS file; wfits does it now

# Apr  4 2002	Fix code

# Apr 23 2003	Fix output directory code

# Apr 19 2004	Put "rfdir/r" in path, not just "/"
