# File saotdc/archive/Scripts/cfits.cl
# April 21, 2003
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# CFITS -- Convert archived spectrum files to FITS files for a range of
#          reduced file numbers, placing FITS files in appropriate directories
#	   Use whatever instrument archive has been set by archive,setinst
#	   and archive reading parameters in rarcd

procedure cfits (rfnrange)

string	rfnrange=""	{prompt="RFN range or file"}
int	offstart=1	{prompt="First RFN with dispersion offset of 1 pixel"}
bool	wdir = yes	{prompt="Write to century subdirectories (yes or no)"}
bool	verbose=no	{prompt="Print what's happening (yes or no)"}
bool	debug=no	{prompt="Debugging mode (yes or no)"}

begin

bool firstrfn, vb, wd
bool curwait
int irfn, jrfn, nrfn, npix
file rfnfile, iraffile, fitsfile, irafpix, arcdir, arcrec, rfn_temp
file rootdir
string rfn,tchar, xcspectra, inst, logfiles, outdir

	vb = verbose
	wd = wdir
	set imtype = "fits"

	rfn = rfnrange
	inst = rarcd.instrument
	arcdir = rarcd.arc_dir
	rootdir = rarcd.root_dir

#  Set up parameters in archive writing, graphic display, and hard copies
	rarcd.dispcor = yes
	rarcd.log = yes
	set clobber = yes

#  Process data files from rfn range or file of rfn's
	rfnfile = rfn
	rfsub (rfn,1,1)
	tchar = substr (rfnfile,1,1)
	if (tchar != '@') {
	    rfn_temp = mktemp ("arcrfn")
	    rfnobj (rfn, list_file=rfn_temp,
		    instrument=inst,arc_dir=arcdir,verbose=no)
	    rfnfile = "@" // rfn_temp
	    }

	rflist (rfnfile,0)
	nrfn = rflist.nlist
	if (vb)
	    print (nrfn," rfns from ",substr(rfnfile,2,20))
 
	rfnobj.root_dir = rootdir
	rfnobj.instrument = inst
	rfnobj.arc_dir = arcdir
	rfnobj.verbose = no

#  Loop through list of reduced files
	for (irfn=1; irfn <= nrfn; irfn=irfn+1) {
	    rflist (rfnfile,irfn)
	    dpar rflist
	    rfn = rflist.rfn
	    rfnobj (rfn)
 	    iraffile = rfnobj.arcfile
	    if (vb)
		print (irfn,rfn," ",iraffile)

	# If reduced file is found for this RFN, process it
	    if (iraffile != "") {
		if (!rarcd.keep_name)
		    iraffile = rfn
		fitsfile = iraffile // ".fits"
		if (!access (fitsfile)) {
		    jrfn = int (rfn)
		    if (jrfn < offstart)
			rarcd (rfn, dispoff=0)
		    else
			rarcd (rfn, dispoff=1)
		    if (!access (fitsfile)) {
			print ("CFITS: No FITS file ",fitsfile)
			next
			}
		    }
		imgetp (fitsfile,"i_naxis1")
		npix = int (imgetp.value)
		if (npix < 1) {
		    imdelete (fitsfile)
		    next
		    }
		if (wd) {
		    if (rfn < 10) {
			newname = "0000" // rfn // ".fits"
			}
		    else if (rfn < 100) {
			newname = "000" // rfn // ".fits"
			}
		    else if (rfn < 1000) {
			newname = "00" // rfn // ".fits"
			}
		    else if (rfn < 10000) {
			newname = "0" // rfn // ".fits"
			}
		    else {
			newname = rfn // ".fits"
			}
		    outdir = substr (newname,1,3)
		    if (!access (outdir)) {
			mkdir (outdir)
			}
		    rename (fitsfile, newname)
		    move (newname, outdir)
		    }
		}
	    flpr
	    }
	if (tchar != '@')
	    delete (rfn_temp)
end

# Apr 21 2003	New task, based on afits
