# File archive/Scripts/coremf.cl
# May 28, 1997
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# COREMF -- Compute emission line and correlation velocities for
#          a list of FAST observations numbers using the RVSAO package

procedure coremf (rfnfile)

string	rfnfile=""	{prompt="RFN or file"}
string	rootdir="/ret/ret7/fast/"	{prompt="Root directory"}
bool	emis_vel=no	{prompt="Compute emission line velocities (y or n)"}
bool	corr_vel=no	{prompt="Compute cross correlation velocities (y or n)"}
bool	plot=yes	{prompt="Plot results on display (y or n)"}
bool	hard_copy=no	{prompt="Make printer hard copies (y or n)"}
bool	cursor=no	{prompt="Wait after plot with cursor (y or n)"}
bool	delfile=yes	{prompt="Delete IRAF copy of archive file (y or n)"}
bool	verbose=no	{prompt="Print what's happening (y or n)"}
bool	debug=no	{prompt="Debugging mode (y or n)"}

begin

bool	vb, delf, arctest, corvel,emvel,plres,hcres
bool	curwait
int	nobs, npix, i
file	iraffile, irafhead, irafpix, rfn_temp
file	localfile, localpix, localhead, newpix,newpath
string	line1, line2
file	listfile, obsdir, irafpath
string	tchar, obsdate, arg, arg1, rfile

	rfile = rfnfile
	vb = verbose
	delf = delfile
	arctest = debug
	corvel = corr_vel
	emvel = emis_vel
	plres = plot
	hcres = hard_copy
	curwait = cursor
	rfast1.verbose = vb
	rfast1.rootdir = rootdir
	rfast1.destdir = ""

#  Set up parameters in archive writing, graphic display, and hard copies

	if (emvel) {
	    if (corvel)
		emsao.save_vel=yes
	    emsao.displot = plres
	    emsao.hardcopy = hcres
	    emsao.curmode = curwait
	    }

	if (corvel) {
	    xcsao.displot = plres
	    xcsao.hardcopy = hcres
	    xcsao.curmode = curwait
	    }

	set clobber = yes

#  Process data files from RFN or file of RFN's
	tchar = substr (rfile,1,1)
	if (tchar != '@') {
	    rfn_temp = mktemp ("arcrfn")
	    print (rfile,>rfn_temp)
	    listfile = rfn_temp
	    }
	else
	    listfile = substr(rfile,2,64)

	if (vb)
	    print ("RFNs from ",listfile)
 
	obnobj.root_dir = rootdir
	obnobj.verbose = no

#  Loop through list of reduced files
	list = listfile
	i = 0
	while (fscan (list, line) != EOF) {
	    i = i + 1
	    rfast1 (line)
	    localfile = rfast1.destpath

	# If the reduced file is found, process a local copy of it
	    if (localfile != "") {

	# Find emission line velocity first if always doing it
		if (emvel)
		    emsao (localfile)

	# Find cross-correlation velocity
		if (corvel)
		    xcsao (localfile)

	# Delete IRAF file
		if (delf)
		    imdelete (localfile)
		if (hcres)
		    gflush
		}
	    }

	if (tchar != '@')
	    delete (rfn_temp)
end
# May 28 1997	New task same as EMCORF except that EMSAO is run before XCSAO
