# File archive/Scripts/emcorf.cl
# May 9, 1995
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# EMCORF -- Compute correlation and emission line velocities for
#          a list of FAST observations numbers using the RVSAO package

procedure emcorf (rfnfile)

string	rfnfile=""	{prompt="RFN or file"}
string	rootdir=""	{prompt="Root directory"}
bool	corr_vel=no	{prompt="Compute cross correlation velocities (y or n)"}
bool	emis_vel=no	{prompt="Compute emission line velocities (y or n)"}
bool	emis_xcor=no	{prompt="Check em. line vel. if emis_temp best (y or n)"}
string	emis_temp="emtemp"  {prompt="Name of emission line template"}
string	best_key="BESTTEMP" {prompt="Keyword for best template"}
bool	save_arch=no	{prompt="Save result in TDC archive (y or n)"}
bool	plot=yes	{prompt="Plot results on display (y or n)"}
bool	hard_copy=no	{prompt="Make printer hard copies (y or n)"}
bool	cursor=no	{prompt="Wait after plot with cursor (y or n)"}
bool	delfile=yes	{prompt="Delete IRAF copy of archive file (y or n)"}
bool	verbose=no	{prompt="Print what's happening (y or n)"}
bool	debug=no	{prompt="Debugging mode (y or n)"}

begin

bool vb, delf, savarc, arctest, corvel,emvel,plres,hcres
bool curwait, emxc
int nobs, npix
file	iraffile, irafhead, irafpix, rfn_temp
file	localfile, localpix, localhead, newpix,newpath
string	line1, line2
file	listfile, obsdir, irafpath
string	tchar, obsdate, arg, arg1, rfile, emtemp, besttemp

	rfile = rfnfile
	vb = verbose
	rfast.verbose = vb
	delf = delfile
	savarc = save_arch
	arctest = debug
	corvel = corr_vel
	emvel = emis_vel
	plres = plot
	hcres = hard_copy
	curwait = cursor
	emxc = emis_xcor
	besttemp = best_key
	emtemp = emis_temp
	rfast.rootdir = rootdir
	if (!corvel && emxc) {
	    print ("EMCORF: Cannot check templates if XCSAO not run")
	    stop
	    }

#  Set up parameters in archive writing, graphic display, and hard copies

	if (corvel) {
	    rarce.dispcor = yes
	    rarce.log = yes
	    if (emvel || emxc)
		xcsao.save_vel=yes
	    xcsao.displot = plres
	    xcsao.hardcopy = hcres
	    xcsao.curmode = curwait
	    }

	if (emvel || emxc) {
	    rarce.dispcor = yes
	    rarce.log = no
	    emsao.displot = plres
	    emsao.hardcopy = hcres
	    emsao.curmode = curwait
	    }

	set clobber = yes

#  Process data files from RFN or file of RFN's

	tchar = substr (rfile,1,1)
	if (tchar != '@') {
	    rfn_temp = mktemp ("arcrfn")
	    print (rfile,>rfn_temp)
	    listfile = rfn_temp
	    }
	else
	    listfile = substr(rfile,2,64)

	if (vb)
	    print ("RFNs from ",listfile)
 
	obnobj.root_dir = rootdir
	obnobj.verbose = no

#  Loop through list of reduced files
	list = listfile
	while (fscan (list,line) != EOF) {
	    rfpath (line)
	    nobs = rfpath.nobs
	    if (nobs == 0)
		next
	    obsdate = rfpath.obsdate
	    rfast (obsdate, nobs)
	    localfile = rfast.obsfile

	# If the reduced file is found, process a local copy of it
	    if (localfile != "") {

	# Find cross-correlation velocity
		if (corvel)
		    xcsao (localfile)

	# Find emission line velocity
		if (emvel)
		    emsao (localfile)
		else if (emxc) {
		    imgetp (localfile,besttemp)
		    if (imgetp.value == emtemp)
			emsao (localfile)
		    }
 
	# Delete IRAF file
		if (delf)
		    imdelete (localfile)
		if (hcres)
		    gflush
		}

	    }

	if (tchar != '@')
	    delete (rfn_temp)
end
# Feb 24 1995	New task
# Mar 31 1995	Drop unused parameters
# May  9 1995	Make local spectrum using RFAST
