# File archive/Scripts/emcorf6.cl
# June 22, 1999
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# EMCORF6 -- Compute correlation and emission line velocities for
#            a list of FAST observations numbers using the RVSAO package
#	     and print results 6 graphs (3 or 6 spectra) to a page

procedure emcorf6 (rfnfile)

string	rfnfile=""	{prompt="RFN or file"}
string	rootdir="/ret/ret7/fast/"	{prompt="Root directory"}
bool	corr_vel=no	{prompt="Compute cross correlation velocities (y or n)"}
bool	emis_vel=no	{prompt="Compute emission line velocities (y or n)"}
bool	corr_print=no	{prompt="Print XCSAO output (yes or no)"}
bool	emis_print=no	{prompt="Print EMSAO output (yes or no)"}
int	npage=6		{prompt="Number per page: 6(1-sided) or 12(2-sided)"}
bool	plot=yes	{prompt="Plot results on display (y or n)"}
bool	cursor=no	{prompt="Wait after plot with cursor (y or n)"}
bool	delfile=yes	{prompt="Delete IRAF copy of archive file (y or n)"}
bool	verbose=no	{prompt="Print what's happening (y or n)"}
bool	debug=no	{prompt="Debugging mode (y or n)"}

begin

bool	vb, delf, corvel, emvel, xcp, emp
bool	pldisp, curwait, emhard, xchard
file	iraffile, irafhead, irafpix, rfn_temp
file	localfile, localpix, localhead, newpix,newpath
file	listfile
int	np
string	tchar, obsdate, arg, arg1, rfile, emtemp, besttemp, s1, s2

	rfile = rfnfile
	vb = verbose
	delf = delfile
	corvel = corr_vel
	emvel = emis_vel
	curwait = cursor
	pldisp = plot
	rfast1.verbose = vb
	rfast1.rootdir = rootdir
	rfast1.destdir = ""

	set stdplot = "epsf"
	emsao.hardcopy = yes
	xcsao.hardcopy = yes
	xchard = yes
	emhard = yes
	xcp = corr_print
	emp = emis_print

#  Clear old sgi files from disk
	delete("sgi*",>&"dev$null")

#  Set up parameters in archive writing, graphic display, and hard copies

	if (corvel) {
	    xcsao.save_vel = yes
	    xcsao.displot = pldisp
	    xcsao.curmode = curwait
	    }

	if (emvel) {
	    emsao.displot = pldisp
	    emsao.curmode = curwait
	    }

	set clobber = yes

#  Process data files from RFN or file of RFN's
	tchar = substr (rfile,1,1)
	if (tchar != '@') {
	    rfn_temp = mktemp ("arcrfn")
	    print (rfile,>rfn_temp)
	    listfile = rfn_temp
	    }
	else
	    listfile = substr(rfile,2,64)

	if (vb)
	    print ("RFNs from ",listfile)
 
	obnobj.root_dir = rootdir
	obnobj.verbose = no
	nprint.ipage = 0
	nprint.npage = 0
	nprint.nper = npage

#  Loop through list of reduced files
	list = listfile
	while (fscan (list, line) != EOF) {
	    rfast1 (line)
	    localfile = rfast1.destpath

	# If the reduced file is found, process a local copy of it
	    if (localfile != "") {

	# Find cross-correlation velocity
		if (corvel) {
		    xcsao (localfile)
		    if (xchard) {
			gflush
			if (xcp)
			    nprint ("emcorf6")
			else
			    delete("sgi*")
			}
		    }

	# Find emission line velocity after successful emtemp correlation
		if (emvel) {
		    emsao (localfile)
		    if (emhard) {
			gflush
			if (emp)
			    nprint ("emcorf6")
			else
			    delete("sgi*")
			}
		    }
 
	# Delete IRAF file
		if (delf)
		    imdelete (localfile)

		}
	    }

    # Print out remaining graphs, if any
	if (nprint.ipage > 0)
	    nprint ("emcorf6",clprint=yes)

	if (tchar != '@')
	    delete (rfn_temp)
end
# Jun 21 1999	New task based on emcorf.cl to print 6 hardcopies per page
# Jun 22 1999	Move tex file creation to separate task
