# File archive/Scripts/emrangf.cl
# December 6, 2002
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# EMRANGF -- Compute correlation and emission line velocities for
#            a range of FAST reduced files using the RVSAO package

procedure emrangf (date1, first_obn, date2, last_obn)

string	date1="1994.0101"		{prompt="First date (yyyy.mmdd)"}
int	first_obn=1	{prompt="First observation number on first day"}
string	date2="1994.0101"		{prompt="Last date (yyyy.mmdd)"}
int	last_obn=300	{prompt="Last observation number on last day"}
int	obn1=1		{prompt="First observation number per day"}
int	obn2=300	{prompt="Last observation number per day"}
string	firstfits="2000.0510"	{prompt="First date in FITS format"}
string	rootdir="/ret/ret7/fast"	{prompt="Root directory"}
string	emis_temp="emtemp"	{prompt="Name of emission line template"}
string	best_key="BESTTEMP"	{prompt="Keyword for best template"}
bool	corr_vel=no	{prompt="Compute cross correlation velocities (y or n)"}
bool	emis_vel=no	{prompt="Always compute emission line velocities (y or n)"}
bool	emis_xcor=no	{prompt="Check em. line vel. only if emis_temp best (y or n)"}
bool	emis_first=no	{prompt="Check em. line vel. before correlation (y or n)"}
bool	plot=yes	{prompt="Plot results on display (y or n)"}
bool	hard_copy=no	{prompt="Make printer hard copies (y or n)"}
bool	cursor=no	{prompt="Wait after plot with cursor (y or n)"}
bool	delfile=yes	{prompt="Delete IRAF copy of archive file (y or n)"}
bool	verbose=no	{prompt="Print what's happening (y or n)"}

begin

bool	vb, delf, corvel,emvel,plres,hcres
bool	curwait, emxc, mcomp, hcomp, emf
string	obsdate
int	npix, nobs, nobs1, nobs2, jd, jd1, jd2, jdf, firstobs, lastobs
file	iraffile, irafhead
file	obsdir
string	inst, emt, besttemp

	vb = verbose
	delf = delfile
	corvel = corr_vel
	emvel = emis_vel
	plres = plot
	hcres = hard_copy
	curwait = cursor
	besttemp = best_key
	emxc = emis_xcor
	emf = emis_first
	emt = emis_temp
	if (!corvel && emxc) {
	    print ("EMRANGF: Cannot check templates if XCSAO not run")
	    stop
	    }

#  Set up parameters in archive writing, graphic display, and hard copies

	if (corvel) {
	    xcsao.archive=no
	    if (emvel)
		xcsao.save_vel=yes
	    xcsao.displot = plres
	    xcsao.hardcopy = hcres
	    xcsao.curmode = curwait
	    }

	if (emvel || emxc) {
	    emsao.archive = no
	    emsao.displot = plres
	    emsao.hardcopy = hcres
	    emsao.curmode = curwait
	    }

	set clobber = yes

	rfast1.rootdir = rootdir
#	rfast1.verbose = vb

# First date to process
	vtoj.frac_day = 0.5
	vtoj (date1)
	jd1 = vtoj.julian_day
	firstobs = first_obn

# Last date to process
	vtoj (date2)
	jd2 = vtoj.julian_day
	lastobs = last_obn

# Date of first archive file in FITS format
	vtoj (firstfits)
	jdf = vtoj.julian_day

	if (verbose)
	    print ("Julian Dates ", jd1, " to ", jd2)

#  Loop through range of dates
	for (jd = jd1; jd<=jd2; jd=jd+1) {
	    jtov (jd)
	    obsdate = jtov.date
	    obsdir = rootdir // "/" // obsdate
	    if (access (obsdir)) {

	#  Loop through range of observation numbers
		if (jd == jd1) {
		    nobs1 = firstobs
		    }
		else {
		    nobs1 = obn1
		    }
		if (jd == jd2) {
		    nobs2 = lastobs
		    }
		else {
		    nobs2 = obn2
		    }
		for (nobs = nobs1; nobs <= nobs2; nobs = nobs + 1) {
		    rfast1.arcfile = ""
		    rfast1.destpath = ""
		    rfast1 (obsdate, nobs)
		    iraffile = rfast1.destpath

		# If reduced file is found, process it in the working directory
		    if (iraffile != "") {
			smatch (iraffile,"COMP")
			mcomp = smatch.found
			smatch (iraffile,"henear")
			hcomp = smatch.found
			if (!mcomp && !hcomp) {
			    if (vb)
				print (nobs," ",iraffile)
			    if (jd < jdf) {
				irafhead = iraffile // ".imh"
				}
			    else {
				irafhead = iraffile // ".fits"
				}
			    if (access (irafhead)) {
				if (jd < jdf) {
				    demach (iraffile)
				    }
				imgetp (iraffile,"i_naxis1")
				npix = int (imgetp.value)
				}
			    if (vb)
				print (npix," ",npix)
			    if (npix > 1) {

			# Find emission line velocity first, if requested
				if (emvel && emf)
				    emsao (iraffile)

			# Find cross-correlation velocity
				if (corvel)
				    xcsao (iraffile)

			# Find emission line velocity last, if requested
				if (emvel && !emf) {
				    if (emxc ) {
					imgetp (iraffile,besttemp)
					if (vb)
					    print ("EMRANGF: best template is ",imgetp.value)
					if (imgetp.value == emt)
					    emsao (iraffile)
					}
				    else
					emsao (iraffile)
				    }
				}
			    else if (vb)
				print (iraffile," has ",npix," pixels")
 
			# Delete IRAF file
			    if (delf)
				imdelete (iraffile)
			    if (hcres)
				gflush
			    }

		    # Print error message if file could not be copied
			else if (vb)
			    print ("Cannot acces ",irafhead)
			}

		# Print error message if file is not found
#		    else if (vb)
#			print ("IRAF file for observation ",nobs," not found")
		    }
		}

	# Print error message if directory is not found
	    else if (vb)
		print ("Directory for ",obsdate," not found")
	    }
end

# Jan 24 1995	Task written
# Feb 16 1995	Make best template keyword a parameter
# Feb 16 1995	Use new task SMATCH to check for calibration spectra

# Oct 17 1996	Change root directory

# Mar 14 1997	Rewrite using RFAST1
# May 28 1997	Add option to find emission line velocity first

# Jun 15 2000	Add date to switch from .imh to .fits files in the archive

# Dec  6 2002	Fix coding problems which seem not to have killed the program
