# File archive/Scripts/eqrange.cl
# December 6, 2002
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# EQRANGE -- Compute equivalent widths for a range of reduced file
#	     numbers using the EQWIDTH task from the RVSAO package

procedure eqrange (rfn1,rfn2)

int	rfn1=1		{prompt="First RFN"}
int	rfn2=1		{prompt="Last RFN"}
int	offstart=1	{prompt="First RFN with dispersion offset of 1 pixel"}
bool	delfile=yes	{prompt="Delete IRAF copy of archive file (y or n)"}
bool	verbose=no	{prompt="Print what's happening (y or n)"}
bool	debug=no	{prompt="Debugging mode (y or n)"}

begin

bool vb, delf, arctest
bool curwait, emxc
int rfn, nrfn, firstrfn, lastrfn, npix
file rfnlist, rfnfile, iraffile, irafhead, arcdir, arcrec, irafpix
file rootdir
string inst, arg, arg1, rfn_temp

	vb = verbose
	delf = delfile
	arctest = debug
	firstrfn = rfn1
	lastrfn = rfn2
	inst = rarce.instrument
	arcdir = rarce.arc_dir
	rootdir = rarce.root_dir

	set clobber = yes

	rfnobj.root_dir = rootdir
	rfnobj.instrument = inst
	rfnobj.arc_dir = arcdir
	rfnobj.verbose = no

#  Loop through range of reduced file numbers
	for (rfn=firstrfn; rfn <= lastrfn; rfn=rfn+1) {
	    rfn_temp = rfn
	    rfnobj (rfn_temp)
	    iraffile = rfnobj.arcfile

	# If reduced file is found, read it into IRAF file and process it
	    if (iraffile != "") {
		if (!rarce.keep_name)
		    iraffile = rfn_temp
		if (vb)
		    print (rfn,rfn_temp," ",iraffile)
		arg = "-inst " // inst // " " // rfn_temp
		irafhead = iraffile // ".imh"
		if (!access (irafhead)) {
		    if (rfn < offstart)
			rarce (rfn_temp, dispoff=0)
		    else
			rarce (rfn_temp, dispoff=1)
		    if (!access (irafhead)) {
			print ("EMRANGE: No IRAF header file ",irafhead)
			next
			}
		    demach (iraffile)
		    }
		irafpix = iraffile // ".pix"
		if (!access (irafpix)) {
		    print ("EMRANGE: No IRAF pixel file ",irafpix)
		    if (access (irafhead))
			imdel (irafhead)
		    next
		    }
		imgetp (irafhead,"i_naxis1")
		npix = int (imgetp.value)
		if (npix < 1) {
		    imdel (irafhead)
		    next
		    }

	# Find equivalent widths
		eqwidth (iraffile)

#	Delete IRAF file
		if (delf)
		    imdelete (iraffile)
		}

	# Print error message if file is not found
	    else if (vb)
		print ("IRAF file for RFN ",rfn," not found")
	    }
end
# Dec  6 2002	New task based on archive.emrange
