# File archive/Scripts/eqrange.cl
# January 31, 2003
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# EQRANGF -- Compute equivalent widths for a range of FAST reduced
#            files using the EQWIDTH task from the RVSAO package

procedure eqrangf (date1, first_obn, date2, last_obn)

string	date1="1994.0101"		{prompt="First date (yyyy.mmdd)"}
int	first_obn=1	{prompt="First observation number on first day"}
string	date2="1994.0101"		{prompt="Last date (yyyy.mmdd)"}
int	last_obn=300	{prompt="Last observation number on last day"}
int	obn1=1		{prompt="First observation number per day"}
int	obn2=300	{prompt="Last observation number per day"}
string	rootdir="/ret/ret7/fast"	{prompt="Root directory"}
bool    delfile=yes     {prompt="Delete IRAF copy of archive file (y or n)"}
bool	verbose=no	{prompt="Print what's happening (y or n)"}

begin

bool	vb, delf
bool	mcomp, hcomp, emf
string	obsdate
int	npix, nobs, nobs1, nobs2, jd, jd1, jd2, firstobs, lastobs
file	iraffile, irafhead
file	obsdir
string	inst, emt, besttemp

	vb = verbose
	delf = delfile

#  Set up parameters in archive writing, graphic display, and hard copies
	set clobber = yes
	rfast1.rootdir = rootdir
#	rfast1.verbose = vb

# First date to process
	vtoj.frac_day = 0.5
	vtoj (date1)
	jd1 = vtoj.julian_day
	firstobs = first_obn

# Last date to process
	vtoj (date2)
	jd2 = vtoj.julian_day
	lastobs = last_obn

	if (verbose)
	    print ("Julian Dates ", jd1, " to ", jd2)

#  Loop through range of dates
	for (jd = jd1; jd<=jd2; jd=jd+1) {
	    jtov (jd)
	    obsdate = jtov.date
	    obsdir = rootdir // "/" // obsdate
	    if (access (obsdir)) {

	#  Loop through range of observation numbers
		if (jd == jd1) {
		    nobs1 = firstobs
		    }
		else {
		    nobs1 = obn1
		    }
		if (jd == jd2) {
		    nobs2 = lastobs
		    }
		else {
		    nobs2 = obn2
		    }
		for (nobs = nobs1; nobs <= nobs2; nobs = nobs + 1) {
		    rfast1.arcfile = ""
		    rfast1.destpath = ""
		    rfast1 (obsdate, nobs)
		    iraffile = rfast1.destpath

		# If reduced file is found, process it in the working directory
		    if (iraffile != "") {
			smatch (iraffile,"COMP")
			mcomp = smatch.found
			smatch (iraffile,"henear")
			hcomp = smatch.found
			if (!mcomp && !hcomp) {
			    if (vb)
				print (nobs," ",iraffile)
			    irafhead = iraffile // ".fits"
			    if (access (irafhead)) {
				imgetp (iraffile,"i_naxis1")
				npix = int (imgetp.value)
				}
			    if (vb)
				print (npix," ",npix)

			# Find equivalent widths
			    if (npix > 1) {
				eqwidth (iraffile)
				}
			    else if (vb)
				print (iraffile," has ",npix," pixels")

			# Delete IRAF file
			    if (delf) {
				imdelete (iraffile)
				}
			    }

		# Print error message if file could not be copied
			else if (vb)
			    print ("Cannot access ",irafhead)
			}

		# Print error message if file is not found
#		    else if (vb)
#			print ("IRAF file for observation ",nobs," not found")
		    }
		}

	# Print error message if directory is not found
	    else if (vb)
		print ("Directory for ",obsdate," not found")
	    }
end

# Dec  6 2002	New task based on archive.emrangf
# Jan 31 2003	Drop IMH to FITS switch; archive is all FITS
