#  File archive/Scripts/imhfits.cl
#  May 7, 1993
#  By Doug Mink

#  Write object+sky files to one file per object spectrum

procedure imhfits (imagelist)

file	imagelist=""	{prompt="Image file name or @list file"}
string	extension=""	{prompt="Image file extension"}
int	rfnlast=0	{prompt="Last reduced file number assigned (RETURNED)"}
int	rfninc=1	{prompt="RFN increment between objects"}
bool	write_fits=no	{prompt="Write FITS file after combining (y or n)"}
file	outdir="fits/"	{prompt="Directory for FITS files"}
bool	eso_flag=no	{prompt="ESO header data (y or n)"}
bool	del_iraf=yes	{prompt="Delete IRAF object+sky files (y or n)"}
string	observatory	{prompt="Observatory name"}
string	longitude	{prompt="Observatory longitude"}
string	latitude	{prompt="Observatory latitude"}
int	altitude	{prompt="Observatory altitude in meters"}
string	epoch="1950.0"	{prompt="Coordinate epoch"}
bool	verbose=no	{prompt="Note progress with lots of text (yes or no)"}

begin

bool	wrfits, diraf, eso, vb
file	imf,imfa,outfile,idf,aps,imsect,outsect,outpath,imagefile, iml,outd
file	listfile,imsky,imobj
int	ispec, iobj, idno, rfn, i, npix, idd, idm, irh, irm, ifib, ioff
int	drfn, ispec1, ifile, nfiles, naxis
string	name,si,id, srfn, obj, ra, dec, jrh, ext, imlist, tstring
real	lw0,dlw,vel, rsec, dsec, sec1, sec2, seconds
char	tchar,echar

	eso = eso_flag
	wrfits = write_fits
	diraf = del_iraf
	drfn = rfninc
	ext = extension
	iml = imagelist
	outd = outdir
	vb = verbose

#  If first character of imagelist is @, get number of images in list
	tchar = substr (iml,1,1)
	if (tchar == "@") {
	    listfile = substr (iml,2,100)
	    rlist (listfile,0)
	    nfiles = rlist.nlist
	    list = listfile
	    if (vb)
		print ("IMHFITS: Reading list of files from ",listfile)
	    }

#  Otherwise, read only one file
	else {
	    imagefile = iml
	    nfiles = 1
	    }

	for (ifile = 1; ifile <= nfiles; ifile = ifile + 1) {

	# Read next file from list if indirect
	    if (tchar == "@") {
		if (fscan (list,imagefile) == EOF)
		    break
		if (vb)
		    print ("IMHFITS: Next file name is ",imagefile)
		}

	# Add extension to image file name
	    imf = imagefile // ext
	    rfn = rfnlast
	    print ("IMHFITS:  Image file is ",imf)

	# Get size of image
	    imgetp (imf,"i_naxis1")
	    npix = int (imgetp.value)


	# Set up reduced file number part of file name
	    rfn = rfn + drfn
	    srfn = rfn
	    if (rfn < 10)
		srfn = "000" // srfn
	    else if (rfn < 100)
		srfn = "00" // srfn
	    else if (rfn < 1000)
		srfn = "0" // srfn
	    else
		srfn = srfn

	    imgetp (imf,"i_title")
	    outfile = srfn // "." // imgetp.value
	    namefilt (outfile)
	    outfile = namefilt.newname

	# Set up object and sky image sections for stacking
            imgetp (imf,"i_naxis")
            naxis = int (imgetp.value)
            print ("Input file is ",imf," x ",naxis)
            if (naxis == 3) {
                imobj = imf // "[*,1,1]"
                imsky = imf // "[*,1,3]"
                }
            else {
                imobj = imf

	# Sky image name
		imgetp (imf,"IMAGETYP")
		if (substr (imgetp.value,1,3) != "OBJ")
		    imsky = imagefile // ext
		else
		    imsky = imagefile // "sky" // ext
                }
            imlist = imobj // "," // imsky

	# Stack object and sky files into output file
	    imstack (imlist,outfile)

	# Remove IRAF stuff from header
	    impdel (outfile,"IRAFTYPE")
	    impdel (outfile,"IRAF-B/P")
	    impdel (outfile,"IRAF-MAX")
	    impdel (outfile,"IRAF-MIN")

	# If ESO data, recompute positions and times
	    if (eso) {
		hedit.add = yes
		hedit.show = no
		hedit.verify = no
		hedit.delete = no
		sechms.fraction = no
		degdms.fraction = no

	    # Start time
		imgetp (outfile,"TM-START")
		tstring = imgetp.value
		sechms (tstring)
		imputp (outfile,"UTOPEN",sechms.value)
		sec1 = real (imgetp.value)

	    # End time
		imgetp (outfile,"TM-END")
		tstring = imgetp.value
		sechms (tstring)
		imputp (outfile,"UT",sechms.value)
		sec2 = real (imgetp.value)

	    # Exposure duration
		seconds = sec2 - sec1
		hedit (outfile,"EXPTIME",seconds)

	    # Sidereal start time
		imgetp (outfile,"TIME-SID")
		tstring = imgetp.value
		sechms (tstring)
		imputp (outfile,"ST",sechms.value)

	    # Right Ascension
		imgetp (outfile,"POSTN-RA")
		tstring = imgetp.value
		degdms.hours = yes
		degdms (tstring)
		imputp (outfile,"RA",degdms.value)

	    # Declination
		imgetp (outfile,"POSTN-DEC")
		tstring = imgetp.value
		degdms.hours = no
		degdms (tstring)
		imputp (outfile,"DEC",degdms.value)

	    # Epoch
		imgetp (outfile,"EPOCH")
		echar = substr (imgetp.value,1,1)
		if (echar == "0")
		    hedit (outfile,"EPOCH",epoch)
		}

#	Add observatory position to header
	    hedit.add = yes
	    hedit.show = no
	    hedit.verify = no
	    hedit.delete = no
	    imputp (outfile,"SITENAME",observatory)
	    imputp (outfile,"SITELAT",latitude)
	    imputp (outfile,"SITELONG",longitude)
	    hedit (outfile,"SITEELEV",altitude)
	    imgetp (outfile,"UT")
	    imputp (outfile,"UTOPEN",imgetp.value)

	# Write FITS file
	    if (wrfits) {
		outpath = outd // outfile
		wfits (outfile,outpath)
		}

	# Delete object+sky IRAF file
	    if (diraf)
		imdel (outfile,verify=no)

	# Update last RFN parameter
	    rfnlast = rfn

	    }

end

# Apr 17 1992	Add eso_flag and ESO header translation
# Sep 18 1992	Add EPOCH

# May  7 1993	Automatically deal with new single-file aperture extraction
