#  File archive/Scripts/msfits.cl
#  January 11, 1994
#  By Doug Mink

#  Split multispec file into one file per spectrum

procedure msfits (spectfile,apref,calfile1,calfile2,flatfile)

file	spectfile=""	{prompt="Multi-object spectrum image file"}
file	apref=""	{prompt="Aperture reference image"}
file	calfile1=""	{prompt="First calibration lamp spectrum image file"}
file	calfile2=""	{prompt="Second calibration lamp spectrum image file"}
file	flatfile=""	{prompt="Incandescent flat spectrum image file"}

file	biasfile=""	{prompt="Bias image file"}
file	imagedir=""	{prompt="Directory containing image files"}
bool	doapsum=""	{prompt="Make multispec files (yes or no)"}
file	idfile=""	{prompt="Fiber assignment table"}
int	apoffset=2	{prompt="Offset between apertures and fibers"}
int	rfnlast=0	{prompt="Last reduced file number assigned (RETURNED)"}
int	rfninc=4	{prompt="RFN increment between objects"}
bool	write_fits=no	{prompt="Write FITS file for each fiber (y or n)"}
file	outdir="fits/"	{prompt="Directory for FITS files"}
bool	del_iraf=yes	{prompt="Delete IRAF fiber files (y or n)"}
string	observatory="MMT"	{prompt="Observatory name"}
string	longitude="+110:53:03.9"	{prompt="Observatory longitude"}
string	latitude="+37:20:29.9"	{prompt="Observatory latitude"}
real	altitude=2606.	{prompt="Observatory altitude in meters"}

begin
	bool doap,wfts
	file dir,fbias,bfile,tempfile,infile,cfile1,cfile2,ffile,sfile,afile
	file rfile, odir
	int irfn

	doap = doapsum
	dir = imagedir
	sfile = spectfile
	rfile = apref
	cfile1 = calfile1
	cfile2 = calfile2
	if (rfile == "") rfile = cfile2
	odir = outdir
	wfts = write_fits
	tempfile = mktemp ("argfits")
	hedit.add = yes
	hedit.verify = no
	hedit.update = yes

#  Set observatory position
	msobj.observatory = observatory
	msobj.latitude = latitude
	msobj.longitude = longitude
	msobj.altitude = altitude
	msplits.observatory = observatory
	msplits.latitude = latitude
	msplits.longitude = longitude
	msplits.altitude = altitude

#  Copy over bias file
	fbias = biasfile
	if (fbias != "" ) {
	    afile = fbias // ".imh"
	    if (!access (afile)) {
		bfile = dir // fbias
		imcopy (bfile,fbias)
		}
	    }

#  Split first calibration lamp image file
	msobj.object="HeAr"
	msobj.apoffset = apoffset
	msobj.idfile= idfile
	msobj.rfnlast = rfnlast
	if (ffile != "") {
	    rfninc = 4
	    }
	else {
	    rfninc = 3
	    }
	msobj.rfninc = rfninc
	msobj.write_fits = wfts
	if (wfts) {
	    if (!access (odir)) {
		mkdir (odir)
		if (!access (odir)) {
		    print ("MSFITS: Cannot write to directory ",odir)
		    return
		    }
		}
	    }
	msobj.outdir = odir
	msobj.del_iraf = del_iraf
	infile = dir // cfile1
	afile = cfile1 // ".imh"
	if (!access (afile)) {
	    imcopy (infile,cfile1)
	    hedit (cfile1,"DISPAXIS",2)
	    if (fbias != "") {
		imarith (cfile1,"+",fbias,tempfile)
		imdelete (cfile1)
		imrename (tempfile,cfile1)
		}
	    }
	if (doap) {
	    afile = cfile1 // ".ms.imh"
	    if (access (afile)) imdelete (afile)
	    apsum.references = rfile
	    apsum (cfile1)
	    }
	msobj (cfile1)

#  Split second calibration lamp image file
	msobj.object="HeAr"
	msobj.rfnlast = rfnlast + 2
	infile = dir // cfile2
	afile = cfile2 // ".imh"
	if (!access (afile)) {
	    imcopy (infile,cfile2)
	    hedit (cfile2,"DISPAXIS",2)
	    if (fbias != "") {
		imarith (cfile2,"+",fbias,tempfile)
		imdelete (cfile2)
		imrename (tempfile,cfile2)
		}
	    }
	if (doap) {
	    afile = cfile2 // ".ms.imh"
	    if (access (afile)) imdelete (afile)
	    apsum.references = rfile
	    apsum (cfile2)
	    }
	msobj (cfile2)

#  Split dome flat field image file, if present
	ffile = flatfile
	if (ffile != "") {
	    msobj.object="domeflat"
	    msobj.rfnlast = rfnlast + 3
	    infile = dir // ffile
	    afile = ffile // ".imh"
	    if (!access (afile)) {
		imcopy (infile,ffile)
		hedit (ffile,"DISPAXIS",2)
		if (fbias != "") {
		    imarith (ffile,"+",fbias,tempfile)
		    imdelete (ffile)
		    imrename (tempfile,ffile)
		    }
		}
	    if (doap) {
		afile = ffile // ".ms.imh"
		if (access (afile)) imdelete (afile)
		apsum.references = rfile
		apsum (ffile)
		}
	    msobj (ffile)
	    }

#  Split image file of object spectra
	msplits.apoffset = apoffset
	msplits.idfile= idfile
	msplits.rfnlast = rfnlast + 1
	msplits.rfninc = rfninc
	msplits.write_fits = write_fits
	msplits.outdir = odir
	msplits.del_iraf = del_iraf
	infile = dir // sfile
	afile = sfile // ".imh"
	if (!access (afile)) {
	    imcopy (infile,sfile)
	    hedit (sfile,"DISPAXIS",2)
	    if (fbias != "") {
		imarith (sfile,"+",fbias,tempfile)
		imdelete (sfile)
		imrename (tempfile,sfile)
		}
	    }
	if (doap) {
	    afile = sfile // ".ms.imh"
	    if (access (afile)) imdelete (afile)
	    apsum.references = rfile
	    apsum (sfile)
	    }
	msplits (sfile)

end
