#  File archive/Scripts/msplit.cl
#  April 17, 1992
#  By Doug Mink

#  Split multispec file into one file per spectrum

procedure msplit (imagefile)

file	imagefile=""	{prompt="Multichannel spectrum image file"}
file	idfile=""	{prompt="Fiber assignment table"}
int	apoffset=2	{prompt="Offset between apertures and fibers"}
int	rfnlast=0	{prompt="Last reduced file number assigned (RETURNED)"}
int	rfninc=1	{prompt="RFN increment between objects"}
bool	write_fits=no	{prompt="Write FITS file for each fiber (y or n)"}
file	outdir="fits/"	{prompt="Directory for FITS files"}
bool	del_iraf=yes	{prompt="Delete IRAF fiber files (y or n)"}
string	observatory	{prompt="Observatory name"}
string	longitude	{prompt="Observatory Longitude"}
string	latitude	{prompt="Observatory Latitude"}
real	altitude	{prompt="Observatory altitude in meters"}

begin

bool	wrfits, diraf
file	imf,msf0,msf,outfile,idf,aps,imsect,outsect,outpath,afile
int	nspec,ispec, iobj, idno, rfn, i, npix, idd, idm, irh, irm, ifib, ioff
int	drfn, ispec1
string	name,si,id, srfn, obj, ra, dec, jrh, appar, apinfo, apr
real	lw0,dlw,vel, rsec, dsec, epoch

	ioff = apoffset
	wrfits = write_fits
	diraf = del_iraf
	drfn = rfninc

	idf = idfile
	imf = imagefile
	msf0 = imf // ".ms"
	msf = imf // ".ms1"
	afile = msf // ".imh"
	if (access (afile)) imdelete (afile)
	rfn = rfnlast
	imcopy (msf0,msf)

	imgetp (msf,"i_naxis1")
	npix = int (imgetp.value)
	imgetp (msf,"i_naxis2")
	nspec = int (imgetp.value)

	impdel (msf,"IRAFTYPE")
	impdel (msf,"IRAF-B/P")
	impdel (msf,"IRAF-MAX")
	impdel (msf,"IRAF-MIN")
	impdel (msf,"RA")
	impdel (msf,"DEC")
	impdel (msf,"EPOCH")
	for (i = 1; i <= 46; i = i + 1) {
	    si = i
	    appar = "APNUM" // si
	    impdel (msf,appar)
	    }

#  Add observatory position to header
	hedit.add = yes
	hedit.show = no
	hedit.verify = no
	hedit.delete = no
	imputp (msf,"SITENAME",observatory)
	imputp (msf,"SITELAT",latitude)
	imputp (msf,"SITELONG",longitude)
	hedit (msf,"SITEELEV",altitude)
	imgetp (msf,"UT")
	imputp (msf,"UTOPEN",imgetp.value)

# Extract object information from line of ID file
	list = idf
	if (idf != "") {
	    while (fscan (list,ifib,iobj,jrh,irm,rsec,idd,idm,dsec,epoch,idno) != EOF) {
		ispec = ifib - ioff
		print ("Spectrum ",ispec," Fiber ",ifib," Object ",iobj)

	    # If object, write position to header
		if (iobj == 1) {
		    irh = int (jrh)
		    if (irh > 9)
			ra = irh // ":"
		    else
			ra = " " // irh // ":"
		    if (irm > 9)
			ra = ra // irm // ":"
		    else
			ra = ra // "0" // irm // ":"
		    if (rsec >= 10.0)
			ra = ra // rsec
		    else
			ra = ra // "0" // rsec
		    if (idd > 9)
			dec = idd // ":"
		    else
			dec = " " // idd // ":"
		    if (idm > 9)
			dec = dec // idm // ":"
		    else
			dec = dec // "0" // idm // ":"
		    if (dsec >= 10.0)
			dec = dec // dsec
		    else
			dec = dec // "0" // dsec
		    print (idno," ",epoch," RA= ",ra," Dec= ",dec

	    # Set up reduced file number part of file name
		    id = idno
		    rfn = rfn + drfn
		    srfn = rfn
		    if (rfn < 10)
			srfn = "000" // srfn
		    else if (rfn < 100)
			srfn = "00" // srfn
		    else if (rfn < 1000)
			srfn = "0" // srfn
		    else
			srfn = srfn

		    obj = imf // "_" // id
		    outfile = srfn // "." // obj

	    # Image section for object spectrum
		    ispec1 = ispec + 1
		    imsect = msf // "[1:" // npix // "," // ispec // 
			     ":" // ispec1 // "]"
		    outsect = outfile
		    }

	    # Image section for sky spectrum
		else {
		    imsect = msf // "[1:" // npix // "," // ispec // "]"
		    outsect = outfile // "[1:" // npix // ",2]"
		    }

	    # Copy one line of input file to output file
		imcopy (imsect,outsect)

	    # Add APNUM parameters for this fiber from header
		si = ispec
		appar = "APNUM" // si
		imgetp (msf0,appar)
		apinfo = imgetp.value
	 	if (iobj == 1) {
		    imputp (outfile,"APOBJ",apinfo)

	    # Add title to header
		    imputp (outfile,"i_title", obj)
		    imputp (outfile,"RA",ra)
		    imputp (outfile,"DEC",dec)
		    hedit (outfile,"EPOCH",epoch)
		    hedit (outfile,"FIBOBJ",ifib)
		    }
		else {
		    hedit (outfile,"FIBSKY",ifib)
		    imputp (outfile,"APSKY",apinfo)
		    }

	    # Write FITS file
		if (iobj == 0 && wrfits) {
		    outpath = outdir // outfile
		    wfits (outfile,outpath)

	    # Delete IRAF file
		    if (diraf)
			imdel (outfile)

	    # Update last RFN parameter
		    rfnlast = rfn
		    }
		}
	    }
	imdel (msf)

end
