# File archive/Scripts/nprint.cl
# June 22, 1999
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# NPRINT -- Set up and print IRAF graphs sent to device EPSF N per page.
#            a list of FAST observations numbers using the RVSAO package
#	     and print results 6 graphs (3 or 6 spectra) to a page

procedure nprint (filename)

string	filename="output"	{prompt="TeX file in which to set up ouput page"}

int	ipage=0		{prompt="Position of this graph on page"}
int	npage=0		{prompt="Page number of last page"}
string	pprinter=""	{prompt="Printer for Postscript output"}
int	nper=6		{prompt="Graphs per page 6=1-sided, 12=2-sided"}
bool	clprint=no	{prompt="Print current buffer (yes or no)"}

begin

int	ip
string	s1, texfile, dvifile, logfile, outfile, hpcom

	if (clprint)
	    ipage = 0
	else
	    ipage = ipage + 1
	ip = ipage
	print("ipage = ", ip)
	texfile = filename // ".tex"
	dvifile = filename // ".dvi"
	logfile = filename // ".log"
	outfile = filename // ".ps"

	# First page of group
	if (ip == 1) {

	    #  Set up page heading
	    npage = npage + 1
	    delete(texfile,>&"dev$null")
	    print("\pageno=  ",npage,>>texfile)
	    print("\headline{\hfil \\tt FAST Spectra \hfil}",>>texfile)
	    print("\input psfig",>>texfile)

	    #  Set up first graph on page
	    files ("sgi*ps") | scan (s1)
	    rename(s1,"x"//s1)
	    print("\vbox to 2.truein {",>>texfile)
	    print("\hfil",>>texfile)
	    print("\special{psfile=x",s1,>>texfile)
	    print("  hoffset=-260 voffset=-350 hscale=50 vscale=60}",>>texfile)
	    }

	#  Set up second graph on page
	else if (ip == 2) {
	    files ("sgi*ps") | scan (s1)
	    rename(s1,"x"//s1)
	    print("\special{psfile=x",s1,>>texfile)
	    print("  hoffset=-20 voffset=-350 hscale=50 vscale=60}",>>texfile)
	    }

	#  Set up third graph on page
	else if (ip == 3) {
	    files ("sgi*ps") | scan (s1)
	    rename(s1,"x"//s1)
	    print("\hskip -6pt",>>texfile)
	    print("\special{psfile=x",s1,>>texfile)
	    print("  hoffset=-260 voffset=-570 hscale=50 vscale=60}",>>texfile)
	    }

	#  Set up fourth graph on page
	else if (ip == 4) {
	    files ("sgi*ps") | scan (s1)
	    rename(s1,"x"//s1)
	    print("\special{psfile=x",s1,>>texfile)
	    print("  hoffset=-20 voffset=-570 hscale=50 vscale=60}",>>texfile)
	    }

	#  Set up fifth graph on page
	else if (ip == 5) {
	    files ("sgi*ps") | scan (s1)
	    rename(s1,"x"//s1)
	    print("\hskip -6pt",>>texfile)
	    print("\special{psfile=x",s1,>>texfile)
	    print("  hoffset=-260 voffset=-790 hscale=50 vscale=60}",>>texfile)
	    }

	# Last page of group of six
	else if (ip == 6) {

	    #  Set up sixth graph on page
	    files ("sgi*ps") | scan (s1)
	    rename(s1,"x"//s1)
	    print("\special{psfile=x",s1,>>texfile)
	    print("  hoffset=-20 voffset=-790 hscale=50 vscale=60}",>>texfile)

	    #  Print out six pages
	    if (nper == 6) {
		print("\hfil",>>texfile)
		print("\vfil",>>texfile)
		print("}",>>texfile)
		print("\vfill",>>texfile)
		print("\eject",>>texfile)
		print(" ",>>texfile)
		print("\bye",>>texfile)
		tex (texfile)
		dvips ("-o",outfile,dvifile)
		hpcom = "-d"//pprinter// " "// outfile
		hprint (hpcom)
		delete("*.dvi")
		delete("xsgi*.eps")
		delete(logfile)
		delete(texfile,>&"dev$null")
		ipage = 0
		}
	    }

	# Seventh page of group
	else if (ip == 7) {

	    #  Set up page heading
	    npage = npage + 1
	    print("\hfil",>>texfile)
	    print("\vfil",>>texfile)
	    print("}",>>texfile)
	    print("\vfill",>>texfile)
	    print("\eject",>>texfile)
	    print("\pageno=  ",npage,>>texfile)
	    print("\headline{\hfil \\tt FAST Spectra \hfil}",>>texfile)
	    print("\input psfig",>>texfile)

	    #  Set up first graph on page
	    files ("sgi*ps") | scan (s1)
	    rename(s1,"x"//s1)
	    print("\vbox to 2.truein {",>>texfile)
	    print("\hfil",>>texfile)
	    print("\special{psfile=x",s1,>>texfile)
	    print("  hoffset=-260 voffset=-350 hscale=50 vscale=60}",>>texfile)
	    }

	#  Set up second graph on page 2
	else if (ip == 8) {
	    files ("sgi*ps") | scan (s1)
	    rename(s1,"x"//s1)
	    print("\special{psfile=x",s1,>>texfile)
	    print("  hoffset=-20 voffset=-350 hscale=50 vscale=60}",>>texfile)
	    }

	#  Set up third graph on page 2
	else if (ip == 9) {
	    files ("sgi*ps") | scan (s1)
	    rename(s1,"x"//s1)
	    print("\hskip -6pt",>>texfile)
	    print("\special{psfile=x",s1,>>texfile)
	    print("  hoffset=-260 voffset=-570 hscale=50 vscale=60}",>>texfile)
	    }

	#  Set up fourth graph on page 2
	else if (ip == 10) {
	    files ("sgi*ps") | scan (s1)
	    rename(s1,"x"//s1)
	    print("\special{psfile=x",s1,>>texfile)
	    print("  hoffset=-20 voffset=-570 hscale=50 vscale=60}",>>texfile)
	    }

	#  Set up fifth graph on page 2
	else if (ip == 11) {
	    files ("sgi*ps") | scan (s1)
	    rename(s1,"x"//s1)
	    print("\hskip -6pt",>>texfile)
	    print("\special{psfile=x",s1,>>texfile)
	    print("  hoffset=-260 voffset=-790 hscale=50 vscale=60}",>>texfile)
	    }

	# Last page of group of six 2
	else if (ip == 12) {

	    #  Set up sixth graph on page 2
	    files ("sgi*ps") | scan (s1)
	    rename(s1,"x"//s1)
	    print("\special{psfile=x",s1,>>texfile)
	    print("  hoffset=-20 voffset=-790 hscale=50 vscale=60}",>>texfile)

	    #  Print out six pages
	    if (nper == 12) {
		print("\hfil",>>texfile)
		print("\vfil",>>texfile)
		print("}",>>texfile)
		print("\vfill",>>texfile)
		print("\eject",>>texfile)
		print(" ",>>texfile)
		print("\bye",>>texfile)
		tex (texfile)
		dvips ("-o",outfile,dvifile)
		hpcom = "-duplex -d"//pprinter// " "// outfile
		hprint (hpcom)
		delete("*.dvi")
		delete("xsgi*.eps")
		delete(logfile)
		delete(texfile,>&"dev$null")
		ipage = 0
		}
	    }

    # Print out remaining graphs
	if (clprint) {
	    print("\hfil",>>texfile)
	    print("\vfil",>>texfile)
	    print("}",>>texfile)
	    print("\vfill",>>texfile)
	    print("\eject",>>texfile)
	    print(" ",>>texfile)
	    print("\bye",>>texfile)
	    tex (texfile)
	    dvips ("-o",outfile,dvifile)
	    if (ipage >6) {
		hpcom = "-duplex -d"//pprinter// " "// outfile
		hprint (hpcom)
		}
	    else {
		hpcom = "-d"//pprinter// " "// outfile
		hprint (hpcom)
		}
	    delete("*.dvi")
	    delete("xsgi*.eps")
	    delete(logfile)
	    delete(texfile,>&"dev$null")
	    ipage = 0
	    }

	flpr
	return
end
# Jun 22 1999	New task
