# File saotdc/archive/Scripts/qplota.cl
# August 24, 1999
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# QPLOTA -- Plot spectrum and tabulate results from previous use of RVSAO

procedure qplota (rfnrange)

string	rfnrange=""	{prompt="RFN range or file"}
string	qtask="xcsao"	{prompt="program to run (xcsao or emsao)"}
string	velplot="combination"	{prompt="Velocity to plot",
				 enum="combination|emission|correlation"}
int	nsmooth=5	{prompt="Number of times to smooth spectrum"}
int	offstart=1	{prompt="First RFN with dispersion offset of 1 pixel"}
bool	delfile=yes	{prompt="Delete IRAF copy of archive file (y or n)"}
bool	verbose=no	{prompt="Print what's happening (y or n)"}
bool	debug=no	{prompt="Debugging mode (y or n)"}

begin

bool vb, delf
int irfn, jrfn, nrfn, npix, ns
file rfnfile, iraffile, irafhead, arcdir, rfn_temp, irafpix
file rootdir
string rfn,tchar, inst, arg
string imtype, qt, vplot

	imtype = envget ("imtype")
	vb = verbose
	delf = delfile
	qt = qtask
	vplot = velplot
	ns = nsmooth

	rfn = rfnrange
	inst = rarce.instrument
	arcdir = rarce.arc_dir
	rootdir = rarce.root_dir
	rarce.dispcor = yes

	set clobber = yes

#  Process data files from rfn range or file of rfn's
	rfnfile = rfn
	rfsub (rfn,1,1)
	tchar = substr (rfnfile,1,1)
	if (tchar != '@') {
	    rfn_temp = mktemp ("arcrfn")
	    rfnobj (rfn, list_file=rfn_temp,
		    instrument=inst,arc_dir=arcdir,verbose=no)
	    rfnfile = "@" // rfn_temp
	    }

	rflist (rfnfile,0)
	nrfn = rflist.nlist
	if (vb)
	    print (nrfn," rfns from ",substr(rfnfile,2,20))
 
	rfnobj.root_dir = rootdir
	rfnobj.instrument = inst
	rfnobj.arc_dir = arcdir
	rfnobj.verbose = no

#  Loop through list of reduced files
	for (irfn=1; irfn <= nrfn; irfn=irfn+1) {
	    rflist (rfnfile,irfn)
	    rfn = rflist.rfn
	    rfnobj (rfn)
 	    iraffile = rfnobj.arcfile
	    if (vb)
		print (irfn,rfn," ",iraffile)

	# If reduced file is found for this RFN, process it
	    if (iraffile != "") {
		if (!rarce.keep_name)
		    iraffile = rfn
		arg = "-inst " // inst // " " // rfn
		irafhead = iraffile // "." // imtype
		print (irafhead)
		if (!access (irafhead)) {
		    jrfn = int (rfn)
		    if (jrfn < offstart)
			rarce (rfn, dispoff=0)
		    else
			rarce (rfn, dispoff=1)
		    if (!access (irafhead)) {
			print ("EMCOR: No IRAF header file ",irafhead)
			next
			}
		    if (imtype == "imh")
			demach (iraffile)
		    }
		if (imtype == "imh") {
		    irafpix = iraffile // ".pix"
		    if (!access (irafpix)) {
			print ("EMCOR: No IRAF pixel file ",irafpix)
			if (access (irafhead))
			    imdel (irafhead)
			next
			}
		    }
		imgetp (irafhead,"i_naxis1")
		npix = int (imgetp.value)
		if (npix < 1) {
		    imdel (irafhead)
		    next
		    }

	    # Qplot this file
		if (qtask == "emsao") {
		    emsao (irafhead,vel_init="combination",linefit=no,
			   curmode=yes,save_vel=yes,vel_plot=vplot,nsmooth=ns)
		    }
		else {
		    xcsao (irafhead,correlate=no,curmode=yes,dispmode=2,
			   save_vel=yes,vel_plot=vplot,nsmooth=ns)
		    }
		}
 
#	Delete IRAF file
	    if (delf)
		imdelete (iraffile)
	    flpr
	    }
	if (tchar != '@')
	    delete (rfn_temp)
end

# Aug 24 1999	New task combining features of archive$emcor.cl and rvsao$qplot.cl
