# File archive/Scripts/rebinf.cl
# December 3, 1998
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# REBINF -- Rebin spectra into 1-D files from a list of FAST observation numbers

procedure rebinf (rfnfile)

string	rfnfile=""	{prompt="RFN or file"}
string	rootdir="/ret/ret7/fast/"	{prompt="Root directory"}
real	wl1 = 3500.0	{prompt="Starting wavelength in Angstroms"}
real	wl2 = 7000.0	{prompt="Ending wavelength in Angstroms"}
int	npix=2000	{prompt="Number of pixels in output spectra"}
int	band = 1	{prompt="Band of multispec image to copy"}
bool	newxcor=no	{prompt="Run XCSAO to get new correlation velocity"}
bool	newemis=no	{prompt="Run EMSAO to get new emission line velocity"}
bool	delfile=yes	{prompt="Delete IRAF copy of archive file (y or n)"}
bool	fitsout=no	{prompt="Write FITS output instead of IRAF"}
bool	verbose=no	{prompt="Print what's happening (y or n)"}
bool	debug=no	{prompt="Debugging mode (y or n)"}

begin

bool	vb, delf, newxc, newem, fitsw
int	i
file	iraffile, irafhead, irafpix, rfn_temp, fitsfile
file	localfile, localpix, localhead, newpix,newpath
string	line1, line2
file	listfile, obsdir, irafpath, outfile
string	tchar, arg, arg1, rfile

	rfile = rfnfile
	vb = verbose
	delf = delfile
	newxc = newxcor
	newem = newemis
	fitsw = fitsout
#	rfast1.verbose = vb
	rfast1.rootdir = rootdir
	rfast1.destdir = ""

	set clobber = yes

#  Process data files from RFN or file of RFN's
	tchar = substr (rfile,1,1)
	if (tchar != '@') {
	    rfn_temp = mktemp ("arcrfn")
	    print (rfile,>rfn_temp)
	    listfile = rfn_temp
	    }
	else
	    listfile = substr(rfile,2,64)

	if (vb)
	    print ("RFNs from ",listfile)
 
	obnobj.root_dir = rootdir
	obnobj.verbose = no

	sumspec.specband = band
	sumspec.st_lambda = wl1
	sumspec.end_lambda = wl2
	sumspec.npts = npix
#	sumspec.velcomp = INDEF
#	sumspec.zcomp = 0.0

#  Loop through list of reduced files
	list = listfile
	i = 0
	while (fscan (list, line) != EOF) {
	    i = i + 1
	    rfast1 (line)
	    localfile = rfast1.destpath

	# If the reduced file is found, process a local copy of it
	    if (localfile != "") {

	    # Get new correlation velocity if desired
		if (newxc) {
		    xcsao (localfile)
		    }

	    # Get new emission line velocity if desired
		if (newem) {
		    emsao (localfile)
		    }

	    # Output file name is full FAST observation sequence number
		outfile = substr (localfile,1,14)
		if (band == 3)
		    outfile = outfile // "sky"
		if (band == 4)
		    outfile = outfile // "raw"
		if (vb)
		    print ("REBINF ",i,": ",localfile," -> ", outfile)

	# Rebin this file into output file
		sumspec (localfile,outfile)

	# Delete IRAF file
		if (delf)
		    imdelete (localfile)

	# Write FITS file, if requested
		if (fitsw) {
		    iraffile = outfile // ".imh"
		    if (access (iraffile)) {
			fitsfile = outfile // ".fits"
			wfits (iraffile, fitsfile)
			imdelete (iraffile)
			}
		    }
		}
	    else
		print ("REBINF ",i,": ",substr(line,1,30)," not found",>>"missing")
	    }

	if (tchar != '@')
	    delete (rfn_temp)
end
# May 13 1997	New task
# May 19 1997	Use sumspec instead of scombine for better rebinning
# Aug 26 1997	Get velocities from new XCSAO and/or EMSAO, as an option

# Dec  3 1998	Set sumspec.band instead of putting band in local filename
# Dec  3 1998	Add option to write FITS output file
