#  File archive/Scripts/redext.cl
#  September 25, 1992
#  By Doug Mink

#  Split MMT Red Channel files into one 2-D file per spectrum

procedure redext (spectfile,idfile,calfile1,flatfile)

file	spectfile=""	{prompt="Multi-object spectrum image file"}
file	idfile=""	{prompt="Fiber assignment table"}
file	calfile1=""	{prompt="First calibration lamp spectrum image file"}
file	calfile2=""	{prompt="Second calibration lamp spectrum image file"}
file	flatfile=""	{prompt="Incandescent flat spectrum image file"}

file	imagedir=""	{prompt="Directory containing image files"}
int	x1a=16	{prompt="Left column limit for aperture 1"}
int	x2a=75	{prompt="Right column limit for aperture 1"}
int	x1b=111	{prompt="Left column limit for aperture 2"}
int	x2b=171	{prompt="Right column limit for aperture 2"}
int	x1c=207	{prompt="Left column limit for aperture 3"}
int	x2c=267	{prompt="Right column limit for aperture 3"}
int	x1d=303	{prompt="Left column limit for aperture 4"}
int	x2d=362	{prompt="Right column limit for aperture 4"}
int	x1e=403	{prompt="Left column limit for aperture 5"}
int	x2e=459	{prompt="Right column limit for aperture 5"}
int	x1f=506	{prompt="Left column limit for aperture 6"}
int	x2f=555	{prompt="Right column limit for aperture 6"}
int	x1g=592	{prompt="Left column limit for aperture 7"}
int	x2g=652	{prompt="Right column limit for aperture 7"}
int	x1h=748	{prompt="Left column limit for aperture 8"}
int	x2h=748	{prompt="Right column limit for aperture 8"}
string	observatory="MMT"	{prompt="Observatory name"}
string	longitude="+110:53:04.3"	{prompt="Observatory longitude"}
string	latitude="31:41:19.6"	{prompt="Observatory latitude"}
real	altitude=2608.	{prompt="Observatory altitude in meters"}

begin
	file dir,infile,cfile1,cfile2,ffile,sfile,afile,insect
	file outfile,idf
	int irh,jrh,irm,idd,idm,x1,x2,ispec
	real rsec,dsec,ut,exp,epoch
	char objname,ra,dec,applim,sect

	dir = imagedir
	sfile = spectfile
	cfile1 = calfile1
	cfile2 = calfile2
	ffile = flatfile
	idf = idfile

#  Add observatory position to header
        print ("REDEXT: Ready to add observatory")
        hedit.add = yes
        hedit.show = yes
        hedit.verify = no
        hedit.update = yes
        hedit.delete = no

# Proceed based on object information from line of ID file
	list = idf
	if (idf != "") {
	    while (fscan (list,ispec,jrh,irm,rsec,idd,idm,dsec,epoch,objname) != EOF) {
		print ("Spectrum ",ispec," Object ",objname)

	    # Write position to object header
		irh = int (jrh)
		if (irh > 9)
		    ra = irh // ":"
		else
		    ra = " " // irh // ":"
		if (irm > 9)
		    ra = ra // irm // ":"
		else
		    ra = ra // "0" // irm // ":"
		if (rsec >= 10.0)
		    ra = ra // rsec
		else
		    ra = ra // "0" // rsec
		if (idd > 9)
		    dec = idd // ":"
		else
		    dec = " " // idd // ":"
		if (idm > 9)
		    dec = dec // idm // ":"
		else
		    dec = dec // "0" // idm // ":"
		if (dsec >= 10.0)
		    dec = dec // dsec
		else
		    dec = dec // "0" // dsec
		print (ispec," ",epoch," RA= ",ra," Dec= ",dec

	    # extract appropriate columns for this spectrum
		if (ispec == 1)
		    sect = "[" // x1a // ":" // x2a // ",*]"
		else if (ispec == 2)
		    sect = "[" // x1b // ":" // x2b // ",*]"
		else if (ispec == 3)
		    sect = "[" // x1c // ":" // x2c // ",*]"
		else if (ispec == 4)
		    sect = "[" // x1d // ":" // x2d // ",*]"
		else if (ispec == 5)
		    sect = "[" // x1e // ":" // x2e // ",*]"
		else if (ispec == 6)
		    sect = "[" // x1f // ":" // x2f // ",*]"
		else if (ispec == 7)
		    sect = "[" // x1g // ":" // x2g // ",*]"
		else if (ispec == 8)
		    sect = "[" // x1h // ":" // x2h // ",*]"

	    # Make copy of object spectrum for this aperture
		afile = dir // sfile // ".imh"
		if (access (afile)) {
		    infile = dir // sfile // sect
		    outfile = sfile // "." // ispec
		    afile = outfile // ".imh"
		    if (access (afile))
			imdelete (afile)
		    imcopy (infile,outfile)

	    # Add title and position to header
		    imputp (outfile,"i_title", obj)
		    hedit (outfile,"DISPAXIS",2)
		    imputp (outfile,"RA",ra)
		    imputp (outfile,"DEC",dec)
		    hedit (outfile,"EPOCH",epoch)
		    hedit (outfile,"APERTURE",ispec)

	    # Add UT open and close times as expected by reduction software

		    imgetp (outfile,"UT")
		    imputp (outfile,"UTOPEN",imgetp.value)
		    harith.param1="UT"
		    harith.operation = "+"
		    harith.format = "%011.2h"
		    harith.verbose = no
		    imgetp (outfile,"EXPTIME")
		    exp = real (imgetp.value) / 3600.d0
		    harith (outfile,"UT",exp)
		    imputp (outfile,"UT",harith.outval)

	    # Add observatory parameters as expected by reduction software

		    imputp (outfile,"SITENAME",observatory)
		    imputp (outfile,"SITELONG",longitude)
		    imputp (outfile,"SITELAT",latitude)
		    hedit (outfile,"SITEELEV",altitude)
		    }
		else
		    print ("REDEXT:  Cannot find file = ",afile)

	    # Extract first calibration lamp image file
		afile = dir // cfile1 // ".imh"
		if (access (afile)) {
		    infile = dir // cfile1 // sect
		    outfile = cfile1 // "." // ispec
		    afile = outfile // ".imh"
		    if (access (afile))
			imdelete (afile)
		    imcopy (infile,outfile)
		    hedit (outfile,"DISPAXIS",2)
		    hedit (outfile,"APERTURE",ispec)

	    # Add UT open and close times as expected by reduction software

		    imgetp (outfile,"UT")
		    imputp (outfile,"UTOPEN",imgetp.value)
		    harith.param1="UT"
		    harith.operation = "+"
		    harith.format = "%011.2h"
		    harith.verbose = no
		    imgetp (outfile,"EXPTIME")
		    exp = real (imgetp.value) / 3600.d0
		    harith (outfile,"UT",exp)
		    imputp (outfile,"UT",harith.outval)

	    # Add observatory parameters as expected by reduction software

		    imputp (outfile,"SITENAME",observatory)
		    imputp (outfile,"SITELONG",longitude)
		    imputp (outfile,"SITELAT",latitude)
		    hedit (outfile,"SITEELEV",altitude)
		    }
		else
		    print ("REDEXT:  Cannot find cfile1 = ",afile)

	    # Extract second calibration lamp image file, if present
		if (cfile2 != "") {
		    afile = dir // cfile2 // ".imh"
		    if (access (afile)) {
			infile = dir // cfile2 // sect
			outfile = cfile2 // "." // ispec
			afile = outfile // ".imh"
			if (access (afile))
			    imdelete (afile)
			imcopy (infile,outfile)
			hedit (outfile,"DISPAXIS",2)
			hedit (outfile,"APERTURE",ispec)

	    # Add UT open and close times as expected by reduction software

			imgetp (outfile,"UT")
			imputp (outfile,"UTOPEN",imgetp.value)
			harith.param1="UT"
			harith.operation = "+"
			harith.format = "%011.2h"
			harith.verbose = no
			imgetp (outfile,"EXPTIME")
			exp = real (imgetp.value) / 3600.d0
			harith (outfile,"UT",exp)
			imputp (outfile,"UT",harith.outval)

	    # Add observatory parameters as expected by reduction software

			imputp (outfile,"SITENAME",observatory)
			imputp (outfile,"SITELONG",longitude)
			imputp (outfile,"SITELAT",latitude)
			hedit (outfile,"SITEELEV",altitude)
			}
		    else
			print ("REDEXT:  Cannot find cfile2 = ",afile)
		    }

	    # Extract dome flat field image file
		afile = dir // ffile // ".imh"
		if (access (afile)) {
		    infile = dir // ffile // sect
		    outfile = ffile // "." // ispec
		    afile = outfile // ".imh"
		    if (access (afile))
			imdelete (afile)
		    imcopy (infile,outfile)
		    hedit (outfile,"DISPAXIS",2)
		    hedit (outfile,"APERTURE",ispec)

	    # Add UT open and close times as expected by reduction software

		    imgetp (outfile,"UT")
		    imputp (outfile,"UTOPEN",imgetp.value)
		    harith.param1="UT"
		    harith.operation = "+"
		    harith.format = "%011.2h"
		    harith.verbose = no
		    imgetp (outfile,"EXPTIME")
		    exp = real (imgetp.value) / 3600.d0
		    harith (outfile,"UT",exp)
		    imputp (outfile,"UT",harith.outval)

	    # Add observatory parameters as expected by reduction software

		    imputp (outfile,"SITENAME",observatory)
		    imputp (outfile,"SITELONG",longitude)
		    imputp (outfile,"SITELAT",latitude)
		    hedit (outfile,"SITEELEV",altitude)
		    }
		else
		    print ("REDEXT:  Cannot find ffile = ",afile)
		}
	    }

end
# Jun 30 1992	New task
# Sep 11 1992	Fix cal and flat bug; add file existence checking
# Sep 25 1992	Add UTOPEN and SITE* parameters to header
