# File archive/Scripts/reskyf.cl
# June 19, 1997
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# RESKYF -- Rebin spectra into 1-D files from a list of FAST observation numbers

procedure rebinf (rfnfile)

string	rfnfile=""	{prompt="RFN or file"}
string	rootdir="/ret/ret7/fast/"	{prompt="Root directory"}
real	wl1 = 3500.0	{prompt="Starting wavelength in Angstroms"}
real	wl2 = 7000.0	{prompt="Ending wavelength in Angstroms"}
int	npix=2000	{prompt="Number of pixels in output spectra"}
int	nsky = 1	{prompt="Number of sky spectra to include"}
bool	delfile=yes	{prompt="Delete IRAF copy of archive file (y or n)"}
bool	verbose=no	{prompt="Print what's happening (y or n)"}
bool	debug=no	{prompt="Debugging mode (y or n)"}

begin

bool	vb, delf
int	i, j
file	iraffile, irafhead, irafpix, rfn_temp
file	localfile, localband, localpix, localhead, newpix,newpath
string	line1, line2, localtemp
file	listfile, obsdir, irafpath, outfile, skyband, specband
string	tchar, arg, arg1, rfile, inspec

	rfile = rfnfile
	vb = verbose
	delf = delfile
	rfast1.verbose = yes
	rfast1.rootdir = rootdir
	rfast1.destdir = ""

	set clobber = yes

#  Process data files from RFN or file of RFN's
	tchar = substr (rfile,1,1)
	if (tchar != '@') {
	    rfn_temp = mktemp ("fastrfn")
	    print (rfile,>rfn_temp)
	    listfile = rfn_temp
	    }
	else
	    listfile = substr(rfile,2,64)

	if (vb)
	    print ("RFNs from ",listfile)
 
	obnobj.root_dir = rootdir
	obnobj.verbose = no

	sumspec.st_lambda = wl1
	sumspec.end_lambda = wl2
	sumspec.npts = npix
	sumspec.velcomp = INDEF
	sumspec.zcomp = INDEF

#  Loop through list of reduced files
	list = listfile
	i = 0
	while (fscan (list, line) != EOF) {
	    i = i + 1
	    rfast1 (line)
	    localfile = rfast1.destpath

	# If the reduced file is found, process a local copy of it
	    if (localfile != "") {

	    # Output file name is full FAST observation sequence number
		outfile = substr (localfile,1,14)
		if (nsky > 1)
		    outfile = outfile // "faint"
		else
		    outfile = outfile // "spec"
		if (nsky > 1) {
		    localtemp = substr (localfile,11,14)
		    imrename (localfile,localtemp)
		    }
		else
		    localtemp = localfile
		skyband = localtemp // "[3]"
		specband = localtemp // "[1]"
		inspec = specband
		for (j = 0; j < nsky; j = j + 1) {
		    inspec = inspec // "," // skyband
		    }
		print inspec
		if (vb)
		    print ("RESKYF ",i,": ",specband, " + ",nsky," * ",skyband," -> ", outfile)

	    # Rebin this file into output file
		sumspec (inspec, outfile)

	    # Delete IRAF file
		if (delf)
		    imdelete (localtemp)
		}
	    else
		print ("RESKYF ",i,": ",substr(line,1,30)," not found",>>"missing")
	    }

	if (tchar != '@')
	    delete (rfn_temp)
end
# Jun 18 1997	New task
