# File archive/Scripts/rfast1.cl
# April 30, 2003
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# RFAST1 -- Make a copy of one reduced file from the FAST archive

procedure rfast1 (obdate,obname)

string	obdate="1994.0101"	{prompt="Date (yyyy.mmdd)"}
string	obname="1"		{prompt="Observation sequence number on date"}
string	rootdir="/ret/ret7/fast"	{prompt="Root directory"}
file	destdir=""		{prompt="Destination directory pathname (no /)"}
file	arcfile=""		{prompt="File name in archive (returned)"}
file	destpath=""		{prompt="Destination file name (returned)"}
string	firstfits="1994.0101"	{prompt="First date in FITS format"}
bool	datedir=no	{prompt="Destination subdirectories by date (y or n)"}
bool	verbose=no		{prompt="Print what's happening (y or n)"}

begin

bool	vb, dd
string	obsdate,date,date1,date2
int	npix, nobs, jd, jdf, obnum, iob1, iob2, iob3, isp
string	obsfile, obsdir, outfile, realpix, outd, tstring
string	obspath, obshead, outhead, outpix, outdir, outpath

	vb = verbose
	dd = datedir
	arcfile = ""

	set clobber = yes

#  Separate out number from date if one string
	date = obdate
	iob1 = stridx ('.',date)
	date1 = substr (date,iob1+5,64)
	date = substr (date,iob1-4,iob1+4)
	iob2 = stridx ('.',date1)
	iob3 = 0
	obsfile = ""
	if (iob2 == 0)
	    iob2 = stridx ('/',date1)
	if (iob2 == 0)
	    iob2 = stridx (' ',date1)
	if (iob2 > 0) {
	    date2 = substr (date1,iob2+1,64)
	    date = substr (date,1,iob1+iob2+4)
	    iob3 =stridx ('.',date2)
	    if (iob3 > 0) {
		obnum = int (substr (date2,1,iob3-1))
		isp = stridx (' ', date2)
		if (isp > 0)
		    obsfile = substr (date2,1,isp-1)
		else
		    obsfile = substr (date2,1,64)
		}
	    else {
		isp = stridx (' ', date2)
		if (isp > 0)
		    obnum = int (substr (date2,1,isp-1))
		else
		    obnum = int (substr (date2,1,64))
		}
	    }
	else {
	    iob3 = stridx ('.', obname)
	    if (iob3 == 0) {
		isp = stridx (' ', obname)
		if (isp > 0)
		    obnum = int (substr (obname, 1, isp-1))
		else
		    obnum = int (substr (obname, 1, 64))
		}
	    else {
		obnum = int (substr (obname, 1, iob3-1))
		isp = stridx (' ', obname)
		if (isp > 0)
		    obsfile = substr (obname,1,isp-1)
		else
		    obsfile = substr (obname,1,64)
		}
	    }

# Date of first archive file in FITS format
        vtoj (firstfits)
        jdf = vtoj.julian_day

#  Remove trailing .imh or .fits
	smatch (obsfile, ".imh")
	if (smatch.found) {
	    tstring = substr (obsfile, 1, smatch.start-1)
	    if (vb)
		print (obsfile," -> ", tstring)
	    obsfile = tstring
	    }
	smatch (obsfile, ".fit")
	if (smatch.found) {
	    tstring = substr (obsfile, 1, smatch.start-1)
	    if (vb)
		print (obsfile," -> ", tstring)
	    obsfile = tstring
	    }

#  Add .ms if not present in filename
	smatch (obsfile, ".ms")
	if (smatch.found == no)
	    obsfile = obsfile // ".ms"

# Set up destination directory
	outd = destdir
	destpath = ""
	arcfile = ""

# Convert filename date to directory date
	vtoj.frac_day = 0.5
	vtoj (date)
	jd = vtoj.julian_day
	jtov (jd)
	obsdate = jtov.date
	obnobj.date = obsdate
	obnobj.root_dir = rootdir
#	obnobj.verbose = vb
	nobs = obnum
	obnobj (nobs)
	obsdir = obnobj.arcdir
	if (iob3 == 0)
	    obsfile = obnobj.arcfile

# If reduced file is found, copy it to the destination directory
	if (obsfile != "") {
	    if (dd) {
		outfile = obsfile
		if (outd == "")
		    outdir = obsdate // "/"
		else
		    outdir = outd // "/" // obsdate // "/"
		if (!access (outdir))
		    mkdir (outdir)
		}
	    else {
		outfile = obsdate
		outfile = outfile // "." // obsfile
		if (outd == "")
		    outdir = outd
		else
		    outdir = outd // "/"
		}
	    outpath = outdir // outfile
	    if (jd < jdf) {
		outhead = outpath // ".imh"
		}
	    else {
		outhead = outpath // ".fits"
		}
	    obspath = obsdir // obsfile
	    if (jd < jdf) {
		obshead = obspath // ".imh"
		}
	    else {
		obshead = obspath // ".fits"
		}
	    if (!access (outhead)) {
		if (vb)
		    print (obshead," -> ",outhead)
		imcopy (obshead,outhead)
		if (!access (outhead)) {
		    if (vb)
		    print ("RFAST1: IRAF header file ",outhead," not copied.")
		    return
		    }
		if (jd < jdf) {
		    demach (outpath)
		    }
		}
	    else if (vb)
		print (outpath," already exists")

	    if (jd < jdf) {
		outpix = outdir // outfile // ".pix"
		realpix = outpath // ".pix"
		if (!access (realpix)) {
		    realpix = outpath // ".pi"
		    if (!access (realpix)) {
			realpix = outpath // ".p"
			if (!access (realpix)) {
			    realpix = outpath // "."
			    if (!access (realpix)) {
				realpix = outpath
				if (!access (realpix)) {
				    print ("RFAST1: No pixel file ",realpix)
				    if (access (outhead))
					imdel (outhead)
				    return
				    }
				}
			    }
			}
		    if (vb)
			print ("RFAST1: ",realpix," -> ",outpix)
		    rename (realpix,outpix)
		    }
		if (vb)
		    print ("RFAST1: ",realpix," -> ",outpix)
		}
	    imgetp (outpath,"i_naxis1")
	    npix = int (imgetp.value)
	    if (npix < 1) {
		imdel (outhead)
		if (vb)
		    print ("RFAST1:  No pixels in file ",outpath)
		return
		}
	    arcfile = obsfile
	    destpath = outpath
	    }
	else if (vb) {
	    if (obsdir == "")
		print ("RFAST1:  Directory for observation date ",obsdate," not found")
	    else
		print ("RFAST1:  File for observation ",nobs," not found in ",obsdir)
	    }
end
# Nov 21 1995	New task
# Dec 22 1995	Add option of writing to a date-specific directory

# May 13 1997	Change outfile to outpath in pixel name checking

# Jul 13 1998	Keep file name from argument if more than number present
# Jul 17 1998	imcopy header file names
# Jul 21 1998	Set destpath to NULL if no file is found

# Jun 15 2000	Add date to switch from .imh to .fits files in the archive
# Jul 18 2000	Set first FITS date to 1994.0101 because archive is now FITS
