#*** File saotdc/archive/src/t_rfnobj.x
#*** October 9, 1990
#*** By Doug Mink, Center for Astrophysics

# RFNOBJ -- Get reduced file name(s) from reduced file number ranges

include <error.h>
include <fset.h>

define	MAX_RANGES	100
define	NTYPES	7
define	SZ_LIST	1000

procedure t_rfnobj()

char	rfn_list[SZ_LIST]		# list of rfn ranges
char	instrument[SZ_FNAME]		# instrument (zmach or mmt)
char	rootdir[SZ_FNAME]		# archive root directory
char	arcdir[SZ_FNAME]		# archive directory
char	arcfile[SZ_FNAME]		# archive file
int	rfn				# reduced file number
char	next_rfname[SZ_FNAME], next_rfn[SZ_FNAME]
int	nfiles, fd, open(), i
int	get_next_number(), decode_ranges(), btoi()
int	rfn_range[MAX_RANGES*2+1]
char	listfile[SZ_FNAME]		# file with list of archive file names
int	verbose				# true to echo filename
int	listdir				# true to include directory pathnames
int	arcout				# YES to use arc_dir
bool	clgetb()
pointer	fntopnb(), list_ptr
int	fntlenb(),fntgfnb(),junk, strmatch(), nlist, nc

begin
	# Set up the standard output to flush on a newline
	call fseti (STDOUT, F_FLUSHNL, YES)

	# Get reduced file numbers from file or input list
	call clgstr ("rfn_list", rfn_list, SZ_LINE)
	if (rfn_list[1] == '@') {
	    list_ptr = fntopnb (rfn_list,NO)
	    nlist = fntlenb (list_ptr)
	    rfn_list[1] = EOS
	    for (i=1; i <= nlist; i=i+1) {
		call strcat (",",rfn_list,SZ_LIST)
		junk = fntgfnb (list_ptr,next_rfname,SZ_LINE)
		nc = strmatch (next_rfname,".")
		if (nc > 2) {
		    next_rfn[1] = EOS
		    call strcat (next_rfname,next_rfn,nc-2)
		    call strcat (next_rfn,rfn_list,SZ_LIST)
		    }
		else
		    call strcat (next_rfname,rfn_list,SZ_LIST)
		}
	    call fntclsb (list_ptr)
	    }
	else {
	    nc = strmatch (rfn_list,".")
	    if (nc > 1) {
		for (i=nc; i<=SZ_LIST; i=i+1) {
		    rfn_list[i] = EOS
		    }
		}
	    }

	# Zero and decode reduced file number ranges
	do i = 1, MAX_RANGES*2+1 {
	    rfn_range[i] = 0
	    }
	if (decode_ranges (rfn_list, rfn_range, MAX_RANGES, nfiles) == ERR)
	    call error (1, "RFNOBJ: Illegal reduced file number list")

	call clgstr ("instrument", instrument, SZ_FNAME)
	call clgstr ("arc_dir", rootdir, SZ_FNAME)
	if (rootdir[1] == EOS) {
	    arcout = NO
	    call clgstr ("arc_dir", rootdir, SZ_FNAME)
	    }
	else
	    arcout = YES
	call clgstr ("list_file", listfile, SZ_FNAME)
	fd = 0
	if (listfile[1] != NULL)
	    fd = open (listfile, WRITE_ONLY, TEXT_FILE)
	verbose = btoi (clgetb ("verbose"))
	listdir = btoi (clgetb ("listdir"))

	# Read successive archive files, convert and write into a
	# succession of output IRAF files.
	rfn = 0
	nfiles = 0
	while (get_next_number (rfn_range, rfn) != EOF) {

	    # Get input pathname and file name
	    call arc_name (rootdir,instrument,rfn,arcout,arcdir,arcfile)
	    if (verbose==YES) {
		if (arcfile[1] == EOS) {
		    call printf ("RFNOBJ:  %s rfn %d not in TDC archive\n")
			call pargstr (instrument)
			call pargi (rfn)
		    }
		else if (listdir==YES) {
		    call printf ("%s%s\n")
			call pargstr (arcdir)
			call pargstr (arcfile)
		    }
		else {
		    call printf ("%s\n")
			call pargstr (arcfile)
		    }
		}
	    if (fd > 0 && arcfile[1] != EOS) {
		if (listdir==YES) {
		    call fprintf (fd,"%s%s\n")
			call pargstr (arcdir)
			call pargstr (arcfile)
		    }
		else {
		    call fprintf (fd,"%s\n")
			call pargstr (arcfile)
		    }
		nfiles = nfiles + 1
		}
	    }

	call clpstr ("arcdir",arcdir)
	call clpstr ("arcfile",arcfile)
	call clputi ("nfiles",nfiles)
	if (fd > 0)
	    call close (fd)

end
