#*** File saotdc/archive/Src/t_rtemp.x
#*** July 13, 1992
#*** By Doug Mink, Center for Astrophysics

# RTEMP -- Read SAO archive format template data.
#          Further documentation given in rtemp.hlp

include <error.h>
include <fset.h>

define	MAX_RANGES	100
define	NTYPES	7

procedure t_rtemp()

char	file_list[SZ_LINE]		# list of template file names
char	tempdir[SZ_FNAME]		# template directory
char	tempfile[SZ_FNAME]		# template file
char	in_fname[SZ_FNAME]		# input file name
char	out_fname[SZ_FNAME]		# output file name

int	stat
bool	clgetb()
char	clgetc()
int	arc_get_image_type(), clgeti(), btoi()
int	arc_read()
int	imtopen(),imtgetim(), inlist

include	"rarc.com"

begin
	# Set up the standard output to flush on a newline
	call fseti (STDOUT, F_FLUSHNL, YES)

	# Get RTEMP parameters.
	call clgstr ("files", file_list, SZ_FNAME)
	inlist = imtopen (file_list)
	call clgstr ("directory", tempdir, SZ_FNAME)

	call clgstr ("instrument", instrument, SZ_FNAME)
	long_header = btoi (clgetb ("long_header"))
	short_header = btoi (clgetb ("short_header"))
	debug = btoi (clgetb ("debug"))
	dispaxis = clgeti ("dispaxis")
	call clgstr ("disptype", disptype, SZ_LINE)
	call clgstr ("dispunit", dispunit, SZ_LINE)

	make_image = btoi (clgetb ("make_image"))
	if (make_image == YES)
	    data_type = arc_get_image_type (clgetc ("datatype"))

	# Read successive template files, convert and write into a
	# succession of output IRAF files in the current directory.

	while (imtgetim (inlist, tempfile,SZ_FNAME) != EOF) {

	    # Set input path name
	    call strcpy (tempdir,in_fname,SZ_FNAME)
	    call strcat (tempfile,in_fname,SZ_FNAME)

	    # Set output file name
	    call strcpy (tempfile,out_fname,SZ_FNAME)
	    call printf ("TDC template file %s -> %s\n")
		call pargstr (in_fname)
	    	call pargstr (out_fname)

	    # Convert SAO archive file to the output IRAF file.
	    # If EOT is reached then exit.
	    # If an error is detected then print a warning and continue with
	    # the next file.

	    iferr (stat = arc_read (in_fname, out_fname))
		call erract (EA_FATAL)
	    if (stat == EOF)
		break
	}

end
# Jan 10 1989	Add parameter for archive directory

# Mar  7 1990	Return 0 for data type set from file

# Jul 13 1992	Move arc_get_image_type to arc_utilities.x
