# File archive/Src/arc_data.x
# July 11, 2005
# By Doug Mink, Center for Astrophysics

#  Decode archive file data

include <imhdr.h>
include <imio.h>
include <mach.h>
include <ctype.h>
include "arc.h"

procedure arc_data (im, arc_fd, headbuf, arcspec, arctype, arcrec, ier)

pointer	im		# Image structure pointer
int	arc_fd		# File descriptor for archive data file
char	headbuf[ARB]	# Archive record header string
pointer arcspec		# Archive spectrum data
int	arctype		# Archive data type
pointer	arcrec		# Archive record data (NULL on first call)
int	ier		# Error returned (0=good, 1=bad)

int	arclen		# Length of record
char	info[1500]	# unpacked ASCII header
char	tstring[50]	# temporary ASCII string
char	dstring[50]	# temporary ASCII string
char	tempname[20]	# template file name (inpacked)
char	tname[20]	# template file name (packed)
char	pname[20]	# image header parameter name
int	nspec		# number of spectrum to read (usually 1 or 2)
int	nboff		# offset in data record to appropriate spectrum
char	mline[SZ_LINE]	# one line character buffer
int	clgeti()
bool	clgetb()
pointer	ptnam, ptemp	# Pointers to name and data for xcor template
short	ntmp,nchp	# Number of xcor templates and lines chopped
int	nfnd,nfit	# Number of emission lines found and fit
bool	keeptemp

double	gmst,lst,d2pi,dnj, exposure, jdn, hjdn0, gjdn0, teps
int	nbytes,npread,nskip,nw,ic
int	id, npix, nb, nbits, strncmp(), irfn, itemp, ichop, i, nchar
int	read(), npts, itmp
#int	nc,i0,stridx(),nextcom
#char	nl
#pointer comments	# temporary storage for unpacked COMMENTS record
#char	comment[80]	# ASCII comment from header
double	c0		# Speed of light
real	tcz,tczerr,tr,bcv
double	rad2deg, dlat, dlong

include "summary.com"
include "wpconv.com"
include "comp_lines.com"
include "head.com"
include "rarc.com"
include "corr.com"

common/darc/ oldtype
int	oldtype

begin
	st = 0.
	ier = 0
	call arc_head (headbuf, nbytes)
        nw = nbytes / 2
	arclen = nbytes
	rad2deg = 1.74532925199433d-2

#  Read data directly into appropriate array
	if (strncmp (headbuf,"SPECTRUM",8) == 0) {
	    call ctoi (headbuf,tok0[4],nbits)

#	Get data type
	    if (headbuf[tok0[5]] == 'F') {
		if (nbits == ARC_REAL)
		    arctype = TY_REAL
		else if (nbits == ARC_DOUBLE)
		    arctype = TY_DOUBLE
		}
	    else if (headbuf[tok0[5]] == 'I') {
		switch (nbits) {
		case ARC_LONG:
	    	    arctype = TY_LONG
		case ARC_SHORT:
	    	    arctype = TY_SHORT
		case ARC_BYTE:
	    	    arctype = TY_CHAR
		}
		}
	    else {
		if (debug == YES) {
		    call printf ("*** Illegal data type %s\n")
			call pargstr (headbuf[tok0[5]])
		    }
		}
	    if (data_type == 0)
		PIXTYPE(im) = arctype
	    else
		PIXTYPE(im) = data_type

#	Get size of data array
	    call ctoi (headbuf,tok0[7],naxis)
	    call ctoi (headbuf,tok0[8],naxis1)
	    if (naxis > 1)
		call ctoi (headbuf,tok0[9],naxis2)
	    else
		naxis2 = 1
	    if (naxis > 2)
		call ctoi (headbuf,tok0[10],naxis3)
	    else
		naxis3 = 1
	    nspec = clgeti ("specno")
	    npread = naxis1 * naxis2 * naxis3
	    if (nspec == 0)
		npix = naxis1 * naxis2 * naxis3
	    else
		npix = naxis1
	    if (nspec == naxis2)
		npread = naxis1
	    nb = npix * nbits / 8
	    nw = npread * nbits / 16
	    if (debug == YES) {
		call printf ("ARC_DATA: Reading %d words of data = %d x %d x (%d/%d) x %d bits\n")
		    call pargi (npread)
		    call pargi (naxis1)
		    call pargi (naxis2)
		    call pargi (nspec)
		    call pargi (naxis3)
		    call pargi (nbits)
		}
	    NAXIS(im) = naxis
	    if (NAXIS(im) > IM_MAXDIM)
		call error (5, "ARC_DATA: TDC archive NAXIS too large")
	    NAXISN(im,1) = naxis1
	    NAXISN(im,2) = naxis2
	    NAXISN(im,3) = naxis3
	    NBPIX(im) = 0
	    if (nspec > 0)
		NAXIS(im) = 1
	    if (nspec > 1)
		nskip = nspec - 1
	    else
		nskip = 0

	# Allocate space for spectrum or spectra
	    if (arcspec != NULL)
		call mfree (arcspec,oldtype)
	    call malloc (arcspec,npread,arctype)
	    oldtype = arctype

	# Read spectrum data
	    switch (arctype) {
		case TY_SHORT:
		    if (nspec == 2) {
			nb = read (arc_fd, Mems[arcspec], nw)
			nb = read (arc_fd, Mems[arcspec], nw)
			}
		    else
			nb = read (arc_fd, Mems[arcspec], nw)
		    call bufswap (Mems[arcspec],2,nb)
		case TY_LONG:
		    if (nspec == 2) {
			nb = read (arc_fd, Meml[arcspec], nw)
			nb = read (arc_fd, Meml[arcspec], nw)
			}
		    else
			nb = read (arc_fd, Meml[arcspec], nw)
		    call bufswap (Mems[arcspec],4,nb)
		case TY_REAL:
		    if (nspec == 2) {
			nb = read (arc_fd, Memr[arcspec], nw)
			nb = read (arc_fd, Memr[arcspec], nw)
			}
		    else
			nb = read (arc_fd, Memr[arcspec], nw)
		    call bufswap (Mems[arcspec],4,nb)
		case TY_DOUBLE:
		    if (nspec == 2) {
			nb = read (arc_fd, Memd[arcspec], nw)
			nb = read (arc_fd, Memd[arcspec], nw)
			}
		    else
			nb = read (arc_fd, Memd[arcspec], nw)
		    call bufswap (Mems[arcspec],8,nb)
		    call moveb (arcrec,nboff,arcspec,0,nb)
		default:
		    call error (10, "ARC_DATA: Unsupported IRAF image type")
		}
	    if (debug == YES) {
		call printf ("ARC_DATA:  %d bytes of data read\n")
		    call pargi (nb)
		}
	if (debug == YES) {
	    call printf ("ARC_DATA:  7th order fit from %f to %f Angstroms\n")
	    call pargr (w7blue)
	    call pargr (w7red)
	    }
	    return
	    }

#  Allocate buffer for this record
#	if (arcrec != NULL)
#	    call mfree (arcrec,TY_SHORT)
#	call malloc (arcrec,nw,TY_SHORT)
	call realloc (arcrec,nw,TY_SHORT)
	nb = read (arc_fd, Mems[arcrec], nw)

#  Read ASCII header information
	call imastr (im,"INSTRUMENT",instrument)
	if (strncmp (headbuf,"HEADER",6) == 0) {
	    
	    nchar = arclen
	    if (nchar > 1500) nchar = 1500
	    call chrupk (Mems[arcrec],0,info,0,nchar)
	    call arc_geti (info,"RFN   ",irfn)
	    call imaddi (im,"RFN",irfn)
	    call arc_gstr (info,"OBJECT",tstring)
	    call strcpy (tstring,OBJECT(im),SZ_LINE)
	    call arc_gstr (info,"RA    ",tstring)
	    call imastr (im,"RA",tstring)
	    call arc_gstr (info,"HA    ",tstring)
	    call imastr (im,"HA",tstring)
	    call arc_gstr (info,"DEC   ",tstring)
	    call imastr (im,"DEC",tstring)
	    call arc_getr (info,"EPOCH ",epoch)
	    call imaddr (im,"EPOCH",epoch)
	    call arc_gstr (info,"UT    ",tstring)
	    call imastr (im,"UT",tstring)
	    call arc_gstr (info,"ST    ",tstring)
	    call imastr (im,"ST",tstring)
	    call arc_gstr (info,"DATE-OBS",tstring)
	    call imastr (im,"DATE-OBS",tstring)
	    tstring[1] = EOS
	    call arc_gstr (info,"OBSERVER",tstring)
	    if (tstring[1] != EOS)
		call imastr (im,"OBSERVER",tstring)
	    tstring[1] = EOS
	    call arc_gstr (info,"PROGRAM",tstring)
	    if (tstring[1] != EOS)
		call imastr (im,"PROGRAM",tstring)
	    tstring[1] = EOS
	    call arc_gstr (info,"PI    ",tstring)
	    if (tstring[1] != EOS)
		call imastr (im,"PI",tstring)
	    tstring[1] = EOS
	    call arc_gstr (info,"FILTER",tstring)
	    if (tstring[1] != EOS)
		call imastr (im,"FILTER",tstring)
	    call arc_getd (info,"EXPTIME",exposure)
	    call imaddd (im,"EXPOSURE",exposure)
	    hjdn0 = 0
	    call arc_getd (info,"HJDN  ",hjdn0)
	    if (hjdn0 > 0) call imaddd (im,"HJDN",hjdn0)
	    gjdn0 = 0
	    call arc_getd (info,"GJDN  ",gjdn0)
	    if (gjdn0 > 0) call imaddd (im,"GJDN",gjdn0)
	    jdn = 0
	    call arc_getd (info,"JDN   ",jdn)
	    if (jdn > 0) call imaddd (im,"JDN",jdn)
	    dispoff = -1
	    call arc_geti (info,"PIXORG",dispoff)
	    }

#	Read comments

#	else if (strncmp (headbuf,"COMMENTS",8) == 0) {
#	    if (comments != NULL)
#		call mfree (comments,TY_SHORT)
#	    call malloc (comments, arclen, TY_SHORT)
#	    call chrupk (Mems[arcrec],0,Mems[comments],0,arclen)
#	    i0 = 0
#	    nl = '\n'
#	    nextcom = stridx (nl,Mems[comments+i0]) + 1
#	    while (nextcom > 0) {
#		nc = nextcom - i0 - 1
#		call strcpy (Mems[comments+i0],comment,nc)
#		call printf ("ARC_DATA: %d:%d %s\n")
#		    call pargi (i0)
#		    call pargi (nextcom)
#		    call pargstr (comment)
#		if (strncmp (comment,"END",3) == 0) {
#		    call imputh (im,"COMMENT",comment)
#		    i0 = nextcom
#		    nextcom = stridx ('\n',Mems[comments+i0]) + 1
#		    }
#		else
#		    nextcom = 0
#		}
#	    call mfree (comments,TY_SHORT)
#	    }

#	Read reduction summary
	else if (strncmp (headbuf,"REDUCE_SUMMARY",14) == 0) {
	    call moveb (Mems[arcrec],0,rfn,0,arclen)
	    call bufswap (rfn, 4, 16)
	    call bufswap (hjdn, 8, 8)
	    call bufswap (secint, 4, 8)
	    call bufswap (telnum, 2, 8)
	    call bufswap (oblong, 4, 56)
	    call imaddr (im,"HCV",velh)

#	Fill in Sidereal Time if it's not present
	    lst = st
	    if (lst <= 0 || lst > 24.d0) {
		dnj = 0.d0
#		iferr (dnj = hjdn)
#		    dnj = 0.d0
#		call printf ("ARC_DATA:  dnj = hjdn = %f \n")
#		    call pargd (dnj)
		if (dnj <= 0 || dnj > 2.488d6) dnj = hjdn0
		if (dnj <= 0 || dnj > 2.488d6) dnj = gjdn0
		if (dnj <= 0 || dnj > 2.488d6) dnj = jdn
		if (dnj <= 0 || dnj > 2.488d6) {
		    ier = 1
		    return
		    }
		call ut2gmst (dnj,gmst)
		d2pi = 6.2831853071795865d0
		lst = (oblong + gmst) * 24.d0 / d2pi
		}
	    }

#	Read reduction summary (extended version)
	else if (strncmp (headbuf,"REDUCE_SUMMARY2",15) == 0) {
	    call moveb (Mems[arcrec],0,rfn,0,arclen)
	    call bufswap (rfn, 4, 16)
	    call bufswap (hjdn, 8, 8)
	    call bufswap (secint, 4, 8)
	    call bufswap (telnum, 2, 8)
	    call bufswap (oblong, 4, 56)
	    call bufswap (gjdn, 8, 8)
	    call bufswap (velb, 4, 8)
	    call imaddr (im,"HCV",velh)
	    call imaddr (im,"BCV",velb)
	    bcv = velb
	    if (bcv == 0) bcv = velh

#	Fill in Sidereal Time if it's not present
	    lst = st
	    if (lst <= 0) {
		dnj = hjdn
		if (dnj < 0) dnj = hjdn0
		if (dnj < 0) dnj = gjdn0
		if (dnj < 0) dnj = jdn
		call ut2gmst (dnj,gmst)
		d2pi = 6.283185307179586476925287d0
		lst = (oblong + gmst) * 24.d0 / d2pi
		}
	    }

#	Read analysis summary
	else if (strncmp (headbuf,"ANALYSIS_SUMMARY",16) == 0) {
	    call moveb (Mems[arcrec],0,qcstat,0,arclen)
	    call bufswap (qcstat, 4, 40)
	    call imaddi (im,"QCSTAT",qcstat)
	    call imaddr (im,"VELOCITY",cz)
	    call imaddr (im,"CZERR",czerr)
	    call imaddr (im,"CZCONF",czconf)
	    call imaddr (im,"CZXC",czxc)
	    call imaddr (im,"CZXCERR",czxcerr)
	    call imaddr (im,"CZXCR",czxcr)
	    call imaddr (im,"CZEM",czem)
	    call imaddr (im,"CZEMERR",czemerr)
	    if (instrument[1] == 'z')
		call strcpy ("Mt. Hopkins 60-inch",tstring,50)
	    else if (instrument[1] == 'm')
		call strcpy ("MMT",tstring,50)
	    else if (instrument[1] == 'o')
		call strcpy ("Oak Ridge 61-inch",tstring,50)
	    else
		call strcpy (instrument,tstring,50)
	    call imastr (im,"SITENAME",tstring)
	    dlong = double (oblong) / rad2deg
	    call sprintf (dstring,50,"%h")
		call pargd (dlong)
	    call imastr (im,"SITELONG",dstring)
	    dlat = double (oblat) / rad2deg
	    call sprintf (dstring,50,"%h")
		call pargd (dlat)
	    call imastr (im,"SITELAT",dstring)
	    call imaddr (im,"SITEELEV",obsalt)
	    if (debug==YES) {
		call printf ("ARC_DATA:  qc= %d cz= %f czerr= %f\n")
		    call pargi (qcstat)
		    call pargr (cz)
		    call pargr (czerr)
		call printf ("ARC_DATA:  czxc= %f czxcerr= %f czxcr= %f\n")
		    call pargr (czxc)
		    call pargr (czxcerr)
		    call pargr (czxcr)
		call printf ("ARC_DATA:  czem= %f czemerr= %f\n")
		    call pargr (czem)
		    call pargr (czemerr)
		call printf ("ARC_DATA:  %s: %h %h %.1fm\n")
		    call pargstr (tstring)
		    call pargd (dlong)
		    call pargd (dlat)
		    call pargr (obsalt)
		}
	    }

#	Read correlation record
	else if (strncmp (headbuf,"CORRELATION",11) == 0) {
	    nb = 12
	    call moveb (Mems[arcrec],0,templw1,0,nb)
	    call bufswap (templw1, 4, 12)
	    call bufswap (Mems[arcrec+6],2,4)
	    ntmp = Mems[arcrec+6]
	    ntemp = ntmp
	    nchp = Mems[arcrec+7]
	    nchop = nchp
	    if (debug == YES) {
		call printf ("ARC_DATA: %d templates, %f - %f A %d lines chopped\n")
		    call pargi (ntemp)
		    call pargr (templw1)
		    call pargr (templw2)
		    call pargs (nchop)
		}
	    keeptemp = FALSE
	    keeptemp = clgetb ("save_temp")
	    if (keeptemp) {
		call imadds (im,"NTEMP",ntmp)
		if (nchop > 0) {
		    ic = 8
		    do ichop = 1, nchop {
			cchop = Mems[arcrec+ic]
			rchop = Mems[arcrec+ic+1]
			ic = ic + 2
			}
		}
		if (ntemp > 0) {
		    ptnam = arcrec + 8 + (nchop * 2)
		    ptemp = ptnam + 8
		    itmp = 0
		    do itemp = 1, ntemp {
			nb = 16
			call moveb (Mems[ptnam],0,tname,0,nb)
			call chrupk (tname,0, tempname,0, 16)
			do i = 1, 16 {
			    if (tempname[i] < 48) tempname[i] = EOS
			    if (tempname[i] > 63 && tempname[i] < 96)
				tempname[i] = tempname[i] + 32
			    }
			id = id + nb
			nb = 28
			call moveb (Mems[ptemp],0,tcenter,0,nb)
			call bufswap (tcenter, 4, 28)
			if (tpw <= 0.0 || trmsa <= 0.0) {
			    ntmp = ntmp - 1
			    call imputs (im,"NTEMP",ntmp)
			    }
			else {
			itmp = itmp + 1
			call sprintf (pname,8,"XCTEMP%d")
			    call pargi (itmp)
			c0 = 299792.5d0
			tcz = c0 * (dexp (dble (tcenter) / dble (tpw)) - 1.d0)
			tcz = tcz + bcv + tshft
			tr = theight / (2. * trmsa)
			npts = 1024
			teps = npts / (8.d0 * 70.d0) / (1.d0 + tr)
			tczerr = c0 * (10.d0 ** (teps / tpw) - 1.d0)
			call sprintf (mline,70,
			    "%s %.3f %.3f %7g %.3f %.3f 0.000 0.000 0 0")
			    call pargstr (tempname)
			    call pargr (tcz)
			    call pargr (tczerr)
			    call pargr (tr)
			    call pargr (theight)
			    call pargr (twidth)
			call imastr (im,pname,mline)
			if (tshft != 0.) {
			    call sprintf (pname,7,"TSHIFT%d")
				call pargi (itmp)
			    call imaddr (im,pname,tshft)
			    }
#			call sprintf (pname,8,"TXCARMS%d")
#			    call pargi (itmp)
#			call imaddr (im,pname,trmsa)

			if (debug == YES) {
			    call printf ("ARC_DATA: Template %d %s tshift: %f  center: %f\n")
				call pargi (itmp)
				call pargstr (tempname)
				call pargr (tshft)
				call pargr (tcenter)
			    call printf ("ARC_DATA:     height: %f width: %f  arms: %f\n")
				call pargr (theight)
				call pargr (twidth)
				call pargr (trmsa)
			    call printf ("ARC_DATA:     cz: %f czerr: %f  r: %f\n")
				call pargr (tcz)
				call pargr (tczerr)
				call pargr (tr)
			    }
			}
			if (itemp < ntemp) {
			    id = id + nb
			    ptnam = ptnam + 22
			    ptemp = ptemp + 22
			    }
			}
		    }
		}
	    }

#	Read emission line record
	else if (strncmp (headbuf,"EMISSION_LINES",14) == 0) {
	    nfnd = Mems[arcrec]
	    call bufswap (nfnd, 2, 2)
	    call imaddi (im,"CZEMNL",nfnd)
	    nfit = Mems[arcrec+1]
	    call bufswap (nfnd, 2, 2)
	    call imaddi (im,"CZEMNLF",nfit)
	    if (debug == YES) {
		call printf ("ARC_DATA: %d emission lines found, %d fit\n")
		    call pargi (nfnd)
		    call pargi (nfit)
		}
	    }

#	Read wavelength polynomials
	else if (strncmp (headbuf,"COARSE_WAVER",12) == 0) {
	    nb = 44
	    call moveb (Mems[arcrec],0,w5blue,0,nb)
	    call bufswap (w5blue, 4, 28)
	    call bufswap (w5mid, 8, 8)
	    call bufswap (w5scale, 8, 8)
	    id = nb
	    nb = w5dim * 8
	    call moveb (Mems[arcrec],id,w5coeff,0,nb)
	    call bufswap (w5coeff, 8, nb)
	    id = id + nb
	    nb = 24
	    call moveb (Mems[arcrec],id,p5dim,0,nb)
	    call bufswap (p5dim, 4, 8)
	    call bufswap (p5mid, 8, 8)
	    call bufswap (p5scale, 8, 8)
	    if (debug == YES) {
		call printf ("ARC_DATA:  Order %d fit from %.4f to %.4f Angstroms / %.4f\n")
		    call pargi (w5dim)
		    call pargr (w5blue)
		    call pargr (w5red)
		    call pargd (p5scale)
		call printf ("ARC_DATA:  Order %d fit, pixel %.5f = %.5f Angstroms\n")
		    call pargi (p5dim)
		    call pargd (w5mid)
		    call pargd (p5mid)
		}
	    id = id + nb
	    nb = p5dim * 8
	    call moveb (Mems[arcrec],id,p5coeff,0,nb)
	    call bufswap (p5coeff, 8, nb)
	    if (debug == YES) {
		call printf ("%f  %f\n%f %f\n%f %f\n")
		    call pargd (p5coeff[1])
		    call pargd (p5coeff[2])
		    call pargd (p5coeff[3])
		    call pargd (p5coeff[4])
		    call pargd (p5coeff[5])
		    call pargd (p5coeff[6])
		}
	    }

	else if (strncmp (headbuf,"FINE_WAVER",10) == 0) {
	    nb = 44
	    call moveb (Mems[arcrec],0,w7blue,0,nb)
	    call bufswap (w7blue, 4, 28)
	    call bufswap (w7mid, 8, 8)
	    call bufswap (w7scale, 8, 8)
	    id = nb
	    nb = w7dim * 8
	    call moveb (Mems[arcrec],id,w7coeff,0,nb)
	    call bufswap (w7coeff, 8, nb)
	    id = id + nb
	    nb = 24
	    call moveb (Mems[arcrec],id,p7dim,0,nb)
	    call bufswap (p7dim, 4, 8)
	    call bufswap (p7mid, 8, 8)
	    call bufswap (p7scale, 8, 8)
	    id = id + nb
	    nb = p7dim * 8
	    call moveb (Mems[arcrec],id,p7coeff,0,nb)
	    if (debug == YES) {
		call printf ("ARC_DATA:  Order %d fit from %.4f to %.4f Angstroms / %.4f\n")
		    call pargi (w7dim)
		    call pargr (w7blue)
		    call pargr (w7red)
		    call pargd (p7scale)
		call printf ("ARC_DATA:  Order %d fit, pixel %.5f = %.5f Angstroms\n")
		    call pargi (p7dim)
		    call pargd (w7mid)
		    call pargd (p7mid)
		}
	    call bufswap (p7coeff, 8, nb)
	    if (debug == YES) {
		call printf ("%f  %f\n%f %f\n%f %f\n%f %f\n")
		    call pargd (p7coeff[1])
		    call pargd (p7coeff[2])
		    call pargd (p7coeff[3])
		    call pargd (p7coeff[4])
		    call pargd (p7coeff[5])
		    call pargd (p7coeff[6])
		    call pargd (p7coeff[7])
		    call pargd (p7coeff[8])
		}
	    }

#	else if (strncmp (headbuf,"COMP_LINES",10) == 0) {
#	    nb = 20
#	    call moveb (Mems[arcrec],0,nlines,0,nb)
#	    call bufswap (nlines, 4, 20)
#	    id = nb
#	    if (nlines > 0) {
#		for (il = 1; il <= nlines; il = il + 1) {
#		    nb = 32
#		    call moveb (Mems[arcrec],id,compcenter,0,nb)
#		    call bufswap (compcenter, 4, 32)
#	    	    id = id + nb
#	    	    center[il] = compcenter
#	    	    height[il] = compheight
#	    	    width[il] = compwidth
#	    	    cont[il] = compcont
#	    	    table[il] = comptable
#	    	    respix[il] = comprpix
#	    	    resang[il] = comprang
#	    	    cflag[il] = compflag
#		    if (debug == YES) {
#			call printf ("%d %f %f %f \n")
#			    call pargi (il)
#			    call pargr (center[il])
#			    call pargr (table[il])
#			    call pargr (width[il])
#	    		}
#		    }
#		}
#	    }

#	Return to read next data segment
	return

end


#  Conversion from universal time to sidereal time
#  By P.T.Wallace, Starlink
#  Modified by Doug Mink, Center for Astrophysics

procedure ut2gmst (ut1,gmst)

#  The IAU 1982 expression (see page S15 of 1984 Astronomical
#  Almanac) is used, but rearranged to reduce rounding errors.

double ut1	# Universal time (strictly ut1) as Julian Date
double gmst	# Greenwich mean sidereal time (radians)

double d2pi,s2r,tu,angle

begin
	d2pi = 6.283185307179586476925287d0
	s2r = 0.7272205216643039849d-4		# Seconds to radians

#  Julian centuries from fundamental epoch J2000 to this ut

	tu = (ut1 - 2451545.d0) / 36525.d0

#  gmst at this ut

	angle = (dmod(ut1,1.d0) * d2pi) + (24110.54841d0 + (8640184.812866d0 +
     		(0.093104d0 - 6.2d - 6*tu) * tu) * tu) * s2r

	gmst = dmod (angle,d2pi)
	if (gmst < 0.d0) gmst = gmst + d2pi
end

# Nov  9 1990	Allocate new array for spectrum data if it is not Int*2
# May 22 1991	Add template velocities to header
# Aug 27 1991	Add template height, width, and ARMS to header
# Oct 29 1991	Write number of emission lines fit to header as CZEMFIT
# Dec  3 1991	Save template information only if requested

# Jan 29 1993	Debug dispersion correction problem under IRAF 2.10

# Jun  3 1994	Quit if bad Julian Date
# Jun 22 1994	Read PIXORG switch from HEADER to set common variable
# Jun 22 1994	Add number of lines found to image header

# Feb 24 1995	Skip templates with problems
# Jul  7 1995	Write observatory position to IRAF header
# Jul  7 1995	Write template information to IRAF header in one-line format

# Apr  2 1997	Add observatory info to debugging

# Jul 11 2005	Add byte swapping so this runs under Linux
