#*** File iodata/archive/arc_get.x
#*** June 22, 1994
#*** by Doug Mink, Center for Astrophyics

#  Subroutines to extract values from an archive file parameter string

#	arc_geti (hstring,vname,ival) returns integer ival
#	arc_gets (hstring,vname,sval) returns integer*2 sval
#	arc_getl (hstring,vname,lval) returns integer*4 lval
#	arc_getr (hstring,vname,rval) returns real*4 rval
#	arc_getd (hstring,vname,dval) returns real*8 dval
#	arc_gstr (hstring,vname,cval) returns character string cval

include <ctype.h>
include <chars.h>

#--- arc_geti  extract integer value for variable from fits header string

procedure arc_geti (hstring,vname,ival)

char hstring[ARB]	# Header string
char vname[ARB]		# Parameter name
int ival		# parameter value

char value[30]
int ip,arc_gstr()

begin
	ip = 1
	if (arc_gstr (hstring,vname,value) != 0)
	    call ctoi (value,ip,ival)
	return
end


#--- arc_gets  extract short integer value for variable from fits header string

procedure arc_gets (hstring,vname,sval)

char hstring[ARB]	# Header string
char vname[ARB]		# Parameter name
short sval		# parameter value

char value[30]
int ip,arc_gstr(),ival

begin
	ip = 1
	if (arc_gstr (hstring,vname,value) != 0) {
	    call ctoi (value,ip,ival)
	    sval = ival
	    }
	return
end


#--- arc_getl  extract long integer value for variable from fits header string

procedure arc_getl (hstring,vname,lval)

char hstring[ARB]	# Header string
char vname[ARB]		# Parameter name
long lval		# parameter value

char value[30]
int ip,arc_gstr()

begin
	ip = 1
	if (arc_gstr (hstring,vname,value) != 0)
	    call ctol (value,ip,lval)
	return
end


#--- arc_getr  extract real value for variable from fits header string

procedure arc_getr (hstring,vname,rval)

char hstring[ARB]	# Header string
char vname[ARB]		# Parameter name
real rval		# parameter value

char value[30]
int ip,arc_gstr()

begin
	ip = 1
	if (arc_gstr (hstring,vname,value) != 0)
	    call ctor (value,ip,rval)
	return
end


#--- arc_getd  extract real*8 value for variable from fits header string

procedure arc_getd (hstring,vname,dval)

char hstring[ARB]	# Header string
char vname[ARB]		# Parameter name
double dval		# parameter value

char value[30]
int ip,arc_gstr()

begin
	ip = 1
	if (arc_gstr (hstring,vname,value) != 0)
	    call ctod (value,ip,dval)
	return
end


#--- arc_gstr  extract character value for variable from fits header string

int procedure arc_gstr (hstring,vname,cval)

char hstring[ARB]	# Header string
char vname[ARB]		# Parameter name
char cval[ARB]		# Parameter value (returned)
#			character string containing the value for variable
#			vname.  trailing and leading blanks are removed.

int ival,ival1,nchars
int strmatch()

begin

#  Search for parameter name in header string

	ival = strmatch (hstring,vname) + 1
	if (ival <= 1) return (0)

#  If parameter is found, find first significant character in value

	while (IS_WHITE (hstring[ival]) || hstring[ival] == '=' ||
	       hstring[ival] == SQUOTE)
	    ival = ival + 1

#  Entry lasts until quote or control character

	ival1 = ival
	while (IS_PRINT(hstring[ival1]) && hstring[ival1]!=SQUOTE)
	    ival1 = ival1 + 1
	ival1 = ival1 - 1

#  extract entry for this variable from the header

	nchars = ival1 - ival + 1
	call strcpy (hstring[ival],cval,nchars)
	cval[nchars+1] = EOS

	return (1)
end
# Jun 22 1994	Fix bug when handling unset header parameters
