# File archive/Src/arc_read0.x
# July 16, 1992
# By Doug Mink

# ARC_READ0 -- Convert an SAO TDC archive file. An EOT is signalled by returning EOF.

include <error.h>
include <imhdr.h>
include <imset.h>
include <fio.h>
include	"arc.h"

int procedure arc_read0 (arcfile, iraffile)

char	arcfile[ARB]		# SAO TDC archive file name
char	iraffile[ARB]		# IRAF file name

int	arc_fd, i, arctype
int	immap(), open(), read()
char	headbuf[48]
bool rebin, clgetb()
pointer	im			# Pointer to output image structure
pointer	arcspec			# pointer to spectrum data
data arcspec/NULL/
pointer	arcrec			# pointer to spectrum data

errchk	immap, open, mfree
errchk	imdelete, close, imunmap

define ARC_HEAD_WORDS 24

include	"rarc.com"

begin

	# Create IRAF image header.  If only a header listing is desired
	# then map the scratch image onto DEV$NULL (faster than a real file).

	if (make_image == NO)
	    call strcpy ("dev$null", iraffile, SZ_FNAME)
	im = immap (iraffile, NEW_IMAGE, 0)

	if (long_header == YES || short_header == YES) {
	    if (long_header == YES)
		call printf ("\n")
	    }

	# Read archive data

	iferr {arc_fd = open (arcfile, READ_ONLY, BINARY_FILE)} then {
	    call printf ("ARC_READ:  Cannot read %s\n")
		call pargstr (arcfile)
	    return
	    }

	# Read archive file, one record at a time

	arcrec = NULL
	while (read (arc_fd, headbuf, ARC_HEAD_WORDS) != EOF) {
	    call arc_data (im, arc_fd, headbuf, arcspec, arctype, arcrec)
	    }
	call close (arc_fd)

	call imputh (im, "HISTORY", "Created by RARC0, Nova dispersion")

	# move spectrum from buffer to image
	rebin = clgetb ("dispcor")
	if (rebin)
	    call arc_dispcor0 (im, arcspec, arctype)
	else
	    call arc_write_iraf (im, arcspec, arctype)

	if (arcrec != NULL)
	    call mfree (arcrec, TY_SHORT)
	if (arcspec != NULL)
	    call mfree (arcspec,arctype)

	# Print short header if desired
	if (short_header == YES && long_header == NO) {
	    call printf ("%-20.20s  ")
		call pargstr (OBJECT(im))
	    do i = 1, NAXIS(im) {
		if (i == 1) {
		    call printf ("Size = %d")
		        call pargl (NAXISN(im, i))
		    }
		else {
		    call printf (" x %d")
		        call pargl (NAXISN(im, i))
		    }
		}
	    call printf ("\n")
	    }

	# Close files and clean up.
	call imunmap (im)

	return (EOF)
end
