# File archive/Src/arc_reade.x
# March 21, 2005
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# ARC_READE -- Convert an SAO TDC archive file.
#              An EOT is signalled by returning EOF.

include <error.h>
include <imhdr.h>
include <imset.h>
include <fio.h>
include	"arc.h"

int procedure arc_reade (arcfile, iraffile)

char	arcfile[ARB]		# SAO TDC archive file name
char	iraffile[ARB]		# IRAF file name

int	arc_fd, arctype, ier
#int	i
int	immap(), open(), read(), clgeti()
char	headbuf[48]
char	hstring[SZ_LINE]
char	dstring[SZ_LINE]
char	tstring[16]
long	ltime, clktime()
char	outfile[SZ_PATHNAME]

bool	rebin, clgetb(), sdisp
pointer	im			# Pointer to output image structure
pointer	arcspec			# pointer to spectrum data
data	arcspec/NULL/
pointer	arcrec			# pointer to spectrum data

errchk	immap, open, mfree
errchk	imdelete, close, imunmap

define ARC_HEAD_WORDS 24

include	"rarc.com"
include "wpconv.com"

begin

#  Create IRAF image header.  If only a header listing is desired,
#  map the scratch image onto DEV$NULL (faster than a real file).

	if (make_image == NO)
	    call strcpy ("dev$null", iraffile, SZ_FNAME)
	im = immap (iraffile, NEW_IMAGE, 0)

	if (long_header == YES || short_header == YES) {
	    if (long_header == YES)
		call printf ("\n")
	    }

# Read archive data

	iferr {arc_fd = open (arcfile, READ_ONLY, BINARY_FILE)} then {
	    call printf ("ARC_READE:  Cannot read %s\n")
		call pargstr (arcfile)
	    return
	    }

#  Read archive file, one record at a time
	arcrec = NULL
	ier = 0
	while (read (arc_fd, headbuf, ARC_HEAD_WORDS) != EOF) {
	    call arc_data (im, arc_fd, headbuf, arcspec, arctype, arcrec, ier)
	    if (ier > 0 && debug == YES) {
		call printf ("ARC_READE:  Error reading record %s of %s\n")
		    call pargstr (headbuf)
		    call pargstr (iraffile)
#		break
		}
	    }
	call close (arc_fd)

	if (debug == YES) {
	    call printf ("ARC_READE: archive file %s has been read\n")
		call pargstr (arcfile)
	    }

#  Return if error reading file
#	if (ier > 0) {
#	    call imunmap (im)
#	    return (EOF)
#	    }

#  Move spectrum from buffer to image
	rebin = clgetb ("dispcor")
	if (dispoff < 0) {
	    dispoff = clgeti ("dispoff")
	    if (debug == YES) {
		call printf ("ARC_READE: dispersion offset parameter is %d\n")
		    call pargi (dispoff)
		}
	    }
	else if (debug == YES) {
	    call printf ("ARC_READE: dispersion offset from file is %d\n")
		call pargi (dispoff)
	    }
	if (rebin)
	    call arc_dispcore (im, arcspec, arctype)
	else
	    call arc_write_iraf (im, arcspec, arctype)

	if (debug == YES) {
	    call printf ("ARC_READE: Output file has been written\n")
	    }

#  Free input pointers
	if (arcrec != NULL)
	    call mfree (arcrec, TY_SHORT)
	if (arcspec != NULL)
	    call mfree (arcspec,arctype)

#  Put wavelength to pixel conversion polynomial into header, if requested
	sdisp = FALSE
	sdisp = clgetb ("save_disp")
	if (sdisp)
	    call arc_save (im)

#  Put date and method of conversion into HISTORY
	ltime = 0
	ltime = clktime (ltime)
	call cnvtime (ltime, dstring, SZ_LINE)
	call sprintf (hstring,SZ_LINE,"Created by IRAF task saotdc.archive.rarce %s")
	    call pargstr (dstring)
	call imputh (im, "HISTORY", hstring)
	if (debug == YES) {
	    call printf ("%s\n")
		call pargstr (hstring)
	    }

#  Print short header if desired (file name as of 2000-10-18)
	if (short_header == YES && long_header == NO) {
	    call strcpy (iraffile, outfile, SZ_PATHNAME)
	    call strcat (".", outfile, SZ_PATHNAME)
	    call envgets ("imtype", tstring, 16)
	    call strcat (tstring, outfile, SZ_PATHNAME)
	    call printf ("%s\n")
		call pargstr (outfile)
#	    call printf ("%-20.20s  ")
#		call pargstr (OBJECT(im))
#	    call printf ("  ")
#	    do i = 1, NAXIS(im) {
#		if (i == 1) {
#		    call printf ("Size = %d")
#		        call pargl (NAXISN(im, i))
#		    }
#		else {
#		    call printf (" x %d")
#		        call pargl (NAXISN(im, i))
#		    }
#		}
#	    call printf ("\n")
	    }

#  Close files and clean up.
	call imunmap (im)

	return (EOF)
end


# ARC_SAVE -- Convert an SAO TDC archive file.

procedure arc_save (im)

pointer	im			# Pointer to output image structure

char	kstring[10]
int	i

include "wpconv.com"

begin
	call imaddl (im, "WPNCOEFF", pdim)
	call imaddd (im, "WPSCALE", pscale)
	call imaddi (im, "WPOFFSET", dispoff)
	call imaddd (im, "WPMID", pmid)
	do i = 1, pdim {
	    call sprintf (kstring,8,"WPCOEFF%d")
		call pargi (i)
	    call imaddd (im, kstring, pcoeff[i])
	    }
end
# Jul 16 1992	Add HISTORY giving creation program
# Aug 11 1992	Add dispersion coefficients
# Oct  2 1992	Call arc_dispcore which can read old and new coefficients
# Oct 22 1992	Write dispersion pixel offset into header

# Jan 29 1993	Put dispersion coefficient header writing into subroutine

# Jun  3 1994	Add error return
# Jun 22 1994	Get pixel offset from parameter file only if not set in header

# Oct  2 2000	Don't crash if error reading record
# Oct 18 2000	Print error messages only in debug mode; short header is filename

# Mar 21 2005	Use debug is YES/NO, not boolean, variable
