# File archive/Src/arc_wiraf.x
# March 18, 1991
# By Doug Mink, Center for Astrophysics

# ARC_WRITE_IRAF --  Read SAO archive image pixels to IRAF image file

include	<imhdr.h>
include <mach.h>
include <fset.h>
include	"arc.h"

procedure arc_write_iraf (im, arcbuf, arctype)

pointer	im		# IRAF image descriptor
pointer arcbuf		# SAO TDC archive data buffer
int arctype		# SAO TDC archive data type

int	i, npix
long	v[IM_MAXDIM], nlines, il
pointer	outline, arcline
real	linemax, linemin

long	clktime()

errchk	malloc, mfree
errchk	arc_change_pix, arc_put_image_line, arc_pix_lim

include	"rarc.com"

begin
	if (NAXIS(im) == 0) {
	    call printf ("Warning: No pixel file created\n")
	    return
	}

	npix = NAXISN(im, 1)
	nlines = 1
	do i = 2, NAXIS(im)
	    nlines = nlines * NAXISN(im, i)

	IRAFMAX(im) = -MAX_REAL
	IRAFMIN(im) = MAX_REAL

	call amovkl (long(1), v, IM_MAXDIM)

	if (debug == YES) {
	    if (nlines > 1)
		call printf ("ARC_WRITE_IRAF:  %d lines of %d pixels, type %d\n")
	    else
		call printf ("ARC_WRITE_IRAF:  %d line of %d pixels, type %d\n")
	    call pargi (nlines)
	    call pargi (npix)
	    call pargi (PIXTYPE(im))
	    }

	arcline = arcbuf
	do il = 1, nlines {

	    # Set output image line buffer
	    call arc_put_image_line (im, outline, v, PIXTYPE(im))

	    # Change pixels to appropriate type
	    call arc_change_pix (arcline, outline, npix, arctype, PIXTYPE(im))
	    call printf ("ARC_WIRAF:  Spectrum converted from %d to %d\n")
		call pargi (arctype)
		call pargi (PIXTYPE(im))

	    # Calculate image maximum and minimum
	    call arc_pix_lim (outline, npix, PIXTYPE(im), linemin, linemax)
	    IRAFMAX(im) = max (IRAFMAX(im), linemax)
	    IRAFMIN(im) = min (IRAFMIN(im), linemin)


	    if (il < nlines) arcline = arcline + npix
	    }

	if (debug == YES) {
	    call printf ("ARC_WRITE_IRAF:  minimum is %f, maximum is %f\n")
	    call pargr (IRAFMIN(im))
	    call pargr (IRAFMAX(im))
	    }
	LIMTIME(im) = clktime (long(0))
	return
end


# ARC_PUT_IMAGE_LINE -- Output an image line to an IRAF file.

procedure arc_put_image_line (im, buf, v, data_type)

pointer	im			# IRAF image descriptor
pointer	buf			# Pointer to output image line
long	v[ARB]			# imio pointer
int	data_type		# output pixel type

int	impnll(), impnlr(), impnld(), impnlx(), impnls()
errchk	impnll, impnlr, impnld, impnlx, impnls

begin
	switch (data_type) {
	case TY_SHORT, TY_USHORT:
	    if (impnls (im, buf, v) == EOF)
		call error (3, "ARC_PUT_IMAGE_LINE: Error writing IRAF file")
	case TY_INT, TY_LONG:
	    if (impnll (im, buf, v) == EOF)
		call error (3, "ARC_PUT_IMAGE_LINE: Error writing IRAF file")
	case TY_REAL:
	    if (impnlr (im, buf, v) == EOF)
		call error (3, "ARC_PUT_IMAGE_LINE: Error writing IRAF file")
	case TY_DOUBLE:
	    if (impnld (im, buf, v) == EOF)
		call error (3, "ARC_PUT_IMAGE_LINE: Error writing IRAF file")
	case TY_COMPLEX:
	    if (impnlx (im, buf, v) == EOF)
		call error (3, "ARC_PUT_IMAGE_LINE: Error writing IRAF file")
	default:
	    call error (10, "ARC_PUT_IMAGE_LINE: Unsupported IRAF image type")
	}
	return
end


# ARC_CHANGE_PIX -- Change a line of numbers to the IRAF image type

procedure arc_change_pix (inbuf, outbuf, npix, in_type, out_type)

pointer inbuf			# array of archive data
pointer	outbuf			# pointer to IRAF image line
int	npix			# number of pixels
int	in_type			# input pixel type
int	out_type		# output pixel type

begin
	switch (out_type) {
	    case TY_SHORT, TY_USHORT:
		switch (in_type) {
	    	    case TY_SHORT, TY_USHORT:
			Call achtss (Mems[inbuf], Mems[outbuf], npix)
	    	    case TY_INT, TY_LONG:
			Call achtls (Memi[inbuf], Mems[outbuf], npix)
		    case TY_REAL:
			Call achtrs (Memr[inbuf], Mems[outbuf], npix)
		    case TY_DOUBLE:
			Call achtds (Memd[inbuf], Mems[outbuf], npix)
		    default:
			call error (10, "ARC_CHANGE_LINE: Illegal archive type")
		    }
	    case TY_INT, TY_LONG:
		switch (in_type) {
	    	    case TY_SHORT, TY_USHORT:
			Call achtsl (Mems[inbuf], Meml[outbuf], npix)
	    	    case TY_INT, TY_LONG:
			Call achtll (Memi[inbuf], Meml[outbuf], npix)
		    case TY_REAL:
			Call achtrl (Memr[inbuf], Meml[outbuf], npix)
		    case TY_DOUBLE:
			Call achtdl (Memd[inbuf], Meml[outbuf], npix)
		    default:
			call error (10, "ARC_CHANGE_LINE: Illegal archive type")
		    }
	    case TY_REAL:
		switch (in_type) {
	    	    case TY_SHORT, TY_USHORT:
			Call achtsr (Mems[inbuf], Memr[outbuf], npix)
	    	    case TY_INT, TY_LONG:
			Call achtlr (Memi[inbuf], Memr[outbuf], npix)
		    case TY_REAL:
			Call achtrr (Memr[inbuf], Memr[outbuf], npix)
		    case TY_DOUBLE:
			Call achtdr (Memd[inbuf], Memr[outbuf], npix)
		    default:
			call error (10, "ARC_CHANGE_LINE: Illegal archive type")
		    }
	    case TY_DOUBLE:
		switch (in_type) {
	    	    case TY_SHORT, TY_USHORT:
			Call achtsd (Mems[inbuf], Meml[outbuf], npix)
	    	    case TY_INT, TY_LONG:
			Call achtld (Memi[inbuf], Meml[outbuf], npix)
		    case TY_REAL:
			Call achtrd (Memr[inbuf], Meml[outbuf], npix)
		    case TY_DOUBLE:
			Call achtdd (Memd[inbuf], Meml[outbuf], npix)
		    default:
			call error (10, "ARC_CHANGE_LINE: Illegal archive type")
		    }
	    case TY_COMPLEX:
		switch (in_type) {
	    	    case TY_SHORT, TY_USHORT:
			Call achtsx (Mems[inbuf], Memx[outbuf], npix)
	    	    case TY_INT, TY_LONG:
			Call achtlx (Memi[inbuf], Memx[outbuf], npix)
		    case TY_REAL:
			Call achtrx (Memr[inbuf], Memx[outbuf], npix)
		    case TY_DOUBLE:
			Call achtdx (Memd[inbuf], Memx[outbuf], npix)
		    default:
			call error (10, "ARC_CHANGE_LINE: Illegal archive type")
		    }
	    default:
		call error (10, "ARC_CHANGE_LINE: Illegal IRAF image type")
	    }
	return
end


# ARC_PIX_LIMITS -- Determine the maximum and minimum values in a line

procedure arc_pix_lim (buf, npix, pixtype, linemin, linemax)

pointer	buf			# pointer to IRAF image line
int	npix			# number of pixels
int	pixtype			# output data type
real	linemax, linemin	# min and max pixel values

short	smax, smin
long	lmax, lmin
real	rmax, rmin
double	dmax, dmin
complex	xmax, xmin

begin
	switch (pixtype) {
	case TY_SHORT, TY_USHORT:
	    call alims (Mems[buf], npix, smin, smax)
	    linemax = smax
	    linemin = smin
	case TY_INT, TY_LONG:
	    call aliml (Meml[buf], npix, lmin, lmax)
	    linemax = lmax
	    linemin = lmin
	case TY_REAL:
	    call alimr (Memr[buf], npix, rmin, rmax)
	    linemax = rmax
	    linemin = rmin
	case TY_DOUBLE:
	    call alimd (Memd[buf], npix, dmin, dmax)
	    linemax = dmax
	    linemin = dmin
	case TY_COMPLEX:
	    call alimx (Memx[buf], npix, xmin, xmax)
	    linemax = xmax
	    linemin = xmin
	default:
	    call error (30, "ARC_PIX_LIMITS: Unknown IRAF type")
	}
	return
end
