#*** file procdata/Src/t_arcdate.x
#*** March 14, 2000
#*** By Doug Mink

# T_VTOJ:  Convert from date as yyyy.mmdd and time as hh.mmsss to julian date
# T_JTOV:  Convert julian date to date as yyyy.mmdd and time as hh.mmssss
# T_FTOV:  Convert FITS date dd/mm/yy to date as yyyy.mmdd

include <ctype.h>

procedure t_vtoj

double date	# date as yyyy.mmdd
		#    yyyy = calendar year (e.g. 1973)
		#    mm = calendar month (e.g. 04 = april)
		#    dd = calendar day (e.g. 15)
double	time	# Time as hh.mmssxxxx
		#   *if time<0, it is time as -(fraction of a day)
		#    hh = hour of day (0 .le. hh .le. 23)
		#    nn = minutes (0 .le. nn .le. 59)
		#    ss = seconds (0 .le. ss .le. 59)
		#  xxxx = tenths of milliseconds (0 .le. xxxx .le. 9999)
double dj	# Julian date (returned)

double tsec, dfrac
char	ctime[16], cdate[16]
int	ip, jd
bool	verbose, clgetb()

begin
	verbose = clgetb ("verbose")
	call clgstr ("date",cdate,16)
	ip = 1
	call ctod (cdate,ip,date)
	call clgstr ("time",ctime,16)
	ip = 1
	call ctod (ctime,ip,time)
	if (date < 0.0301d0) return

	call vtos (date,time,tsec)
	dj = 2433282.5d0 + (tsec / 8.64d4)
	jd = int (dj)
	dfrac = dj - double (jd)
	call clputi ("julian_day",jd)
	call clputd ("frac_day",dfrac)
	if (verbose) {
	    call printf ("%9.4f %7.4f -> %13.4f = %d + %6.4f\n")
		call pargd (date)
		call pargd (time)
		call pargd (dj)
		call pargi (jd)
		call pargd (dfrac)
	    }
	return
end


# T_JTOV:  Convert julian date to date as yyyy.mmdd and time as hh.mmssss

procedure t_jtov

double dj	# Julian date
double date	# Date as yyyy.mmdd (returned)
		#  yyyy = calendar year (e.g. 1973)
		#    mm = calendar month (e.g. 04 = april)
		#    dd = calendar day (e.g. 15)
double time	# Time as hh.mmssxxxx (returned)
		#   *if time<0, it is time as -(fraction of a day)
		#    hh = hour of day (0 .le. hh .le. 23)
		#    nn = minutes (0 .le. nn .le. 59)
		#    ss = seconds (0 .le. ss .le. 59)
		#  xxxx = tenths of milliseconds (0 .le. xxxx .le. 9999)
double	dfrac, tsec, clgetd()
char	ctime[16], cdate[16]
int	jd, clgeti()
bool	verbose, clgetb()

begin
	verbose = clgetb ("verbose")
	jd = clgeti ("julian_day")
	dfrac = clgetd ("frac_day")
	dj = double (jd) + dfrac
	tsec = (dj - 2433282.5d0) * 8.64d4
	call stov (tsec,date,time)
	call sprintf (cdate,16,"%9.4f")
	    call pargd (date)
	call clpstr ("date",cdate,16)
	call sprintf (ctime,16,"%10.7f")
	    call pargd (time)
	call clpstr ("time",ctime,16)
	if (verbose) {
	    call printf ("%13.4f -> %9.4f %7.4f\n")
		call pargd (dj)
		call pargd (date)
		call pargd (time)
	    }

	return
end


# T_FTOV:  Convert FITS date to date as yyyy.mmdd

procedure t_ftov

char	fdate[16]	# Date as mm/dd/yy
char	vdate[16]	# Date as yyyy.mmdd (returned)

double	date		# Date as yyyy.mmdd
			#  yyyy = calendar year (e.g. 1973)
			#    mm = calendar month (e.g. 04 = april)
			#    dd = calendar day (e.g. 15)
double	toffset
double	time, tsec, toff, clgetd()
int	iday,ip,imon,iyr
bool	verbose, clgetb()

begin
	verbose = clgetb ("verbose")
	call clgstr ("fdate",fdate,16)

#  Set day of month
	iday = 0
	ip = 1
	while (IS_DIGIT (fdate[ip])) {
	    iday = (10 * iday) + TO_INTEG (fdate[ip])
	    ip = ip + 1
	    }

# Set month
	imon = 0
	ip = ip + 1
	while (IS_DIGIT (fdate[ip])) {
	    imon = (10 * imon) + TO_INTEG (fdate[ip])
	    ip = ip + 1
	    }

# Set year
	iyr = 0
	ip = ip + 1
	while (IS_DIGIT (fdate[ip])) {
	    iyr = (10 * iyr) + TO_INTEG (fdate[ip])
	    ip = ip + 1
	    }
	if (iyr < 100) {
	    if (iyr > 20)
		iyr = iyr + 1900
	    else
		iyr = iyr + 2000
	    }

#  Compute vigesimal date
	date = double (iyr) + 1.d-2 * double (imon) + 1.d-4 * double (iday)

#  Convert to seconds since 1950
	time = 0.d0
	call vtos (date,time,tsec)

#  Add time offset
	toffset = clgetd ("timeoff")
	tsec = tsec + (toffset * 3600.d0)
	tsec = tsec + toff

#  Convert back to vigesimal date
	call stov (tsec,date,time)
	call sprintf (vdate,16,"%9.4f")
	    call pargd (date)
	call clpstr ("vdate",vdate,16)

	if (verbose) {
	    call printf ("%s + %.2f -> %s\n")
		call pargstr (fdate)
		call pargd (toffset)
		call pargstr (vdate)
	    }

	return
end


# JTOS:  Convert julian date to seconds since 1950.0

	procedure jtos (dj,tsec)

double	dj	# Julian date
double	tsec	# Seconds since 1950.0 (returned)

begin
	tsec = (dj - 2433282.5d0) * 8.64d4
	return
end


# VTOS:  Convert from date as yyyy.mmdd and time as hh.mmsss to seconds since 1950.0

procedure vtos (date,time,tsec)

double	date	# Date as yyyy.mmdd
		#    yyyy = calendar year (e.g. 1973)
		#    mm = calendar month (e.g. 04 = april)
		#    dd = calendar day (e.g. 15)
double	time	# Time as hh.mmssxxxx
		#   *if time<0, it is time as -(fraction of a day)
		#    hh = hour of day (0 .le. hh .le. 23)
		#    nn = minutes (0 .le. nn .le. 59)
		#    ss = seconds (0 .le. ss .le. 59)
		#  xxxx = tenths of milliseconds (0 <= xxxx <= 9999)
double	tsec	# Seconds past 1950.0 (returned)

double	dh,dm,dd
int iy,im,id

begin

	if (date < 0.0301d0) return

#  calculate the number of full years, months, and days already
#  elapsed since 0h, march 1, -1 (up to most recent midnight).

#  convert time of day to elapsed seconds

	if (time < 0)
	    tsec = time * -86400.d0
	else {
	    dh = dint (time + 1.0d-10)
	    dm = dint (((time-dh) * 1.0d2) + 1.0d-10)
	    tsec = (time * 1.0d4) - (dh * 1.0d4) - (dm * 1.0d2)
	    tsec = dint (tsec * 1.d5 + 1.d-4) / 1.d5
	    tsec = tsec + (dm * 60.0d0) + (dh * 3600.0d0)
	    }

#  calculate the number of full months elapsed since
#  the current or most recent march

	iy = idint (date + 1.0d-10)
	im = idint (((date - double (iy)) * 1.0d4) + 1.0d-8)
	id = mod (im,100)
	im = (im / 100) + 9
	if (im < 12) iy = iy - 1
	im = mod (im, 12)
	id = id - 1

#  Starting with March as month 0 and ending with the following
#  February as month 11, the calculation of the number of days
#  per month reduces to a simple formula.  The following statement
#  determines the number of whole days elapsed since 3/1/-1 and then
#  subtracts the 712163 days between then and 1/1/1950.  It converts
#  the result to seconds and adds the accumulated seconds above.

	id = id + ((im+1+im/6+im/11)/2 * 31) + ((im-im/6-im/11)/2 * 30) +
	     (iy / 4) - (iy / 100) + (iy / 400)
	dd = double (id) + (365.d0 * double (iy)) - 712163.d0
	tsec = tsec + (dd * 86400.d0)

	return
end


# STOV:  Convert seconds since 1950.0 to date as yyyy.mmdd and time as hh.mmssss

procedure stov (tsec,date,time)

double tsec	# Seconds past 1950.0
double date	# Date as yyyy.mmdd
		#    yyyy = calendar year (e.g. 1973)
		#    mm = calendar month (e.g. 04 = april)
		#    dd = calendar day (e.g. 15)
double time	# Time as hh.mmssxxxx
		#   *if time<0, it is time as -(fraction of a day)
		#    hh = hour of day (0 .le. hh .le. 23)
		#    nn = minutes (0 .le. nn .le. 59)
		#    ss = seconds (0 .le. ss .le. 59)
		#  xxxx = tenths of milliseconds (0 .le. xxxx .le. 9999)

int	iyr,imon,iday,ihr,imn
double	sec

begin
	call ts2i (tsec,iyr,imon,iday,ihr,imn,sec)

#  convert date to yyyy.mmdd
	date = double (iyr) + 1.d-2 * double (imon) + 1.d-4 * double (iday)

#  convert time to hh.mmssssss
	time = double (ihr) + 1.d-2 * double (imn) + 1.d-4 * sec

	return
end


# TS2I:  Convert seconds since 1950.0 to year month day hours minutes seconds .1ms

procedure ts2i (tsec,iyr,imon,iday,ihr,imn,sec)

double	tsec	# seconds since 1/1/1950 0:00
int	iyr	# year
int	imon	# month
int	iday	# day
int	ihr	# hours
int	imn	# minutes
double	sec	# seconds

double	t,days
int	isec,ihms,nc,nc4,nly,ny,m,im

begin

#  Calculate time of day (hours, minutes, seconds, .1 msec)
	t = dint ((tsec + 61530883200.d0) * 1.d4 + .5d0)
	ihr = idint (dmod (t/36.d6,24.d0))
	imn = idint (dmod (t/60.d4,60.d0))
	isec = idint (dmod (tsec+1.d-6,60.d0))
	ihms = idint (dmod (tsec+1.d-6,1.d0) * 1.d4)
	sec = double (isec) + 1.d-4 * double (ihms)

#  Calculate number of days since 0 hr 0/0/0000
	days = dint ((t / 864.d6) + 1.0d-6)

#  Number of leap centuries (400 years)
	nc4 = idint ((days / 146097.0d0) + 1.0d-5)

#  Number of centuries since last /400
	days = days - (146097.d0 * double (nc4))
	nc = idint ((days / 36524.d0) + 1.0d-4)
	if (nc > 3) nc = 3

#  Calculate number of leap years since last century
	days = days - (36524.d0 * nc)
	nly = idint ((days / 1461.d0) + 1.0d-10)

#  Calculate number of years since last leap year
	days = days - (1461.d0 * double (nly))
	ny = idint ((days / 365.d0) + 1.d-8)
	if (ny > 3) ny = 3

#  Calculate day of month
	days = days - (365.d0 * double (ny))
	if (days < 0) {
	    m = 0;
	    iday = 29
	    }
	else {
	    iday = idint (days + 1.0d-8) + 1
	    do m = 1, 12 {
		im = mod ((m + ((m - 1) / 5)), 2)
		if (iday-1 < im+30) break
		iday = iday - im - 30
		}
	    }

#  Month
	imon  =  mod (m+1,12) + 1

#  Year
	iyr = nc4*400 + nc*100 + nly*4 + ny + m/11

	return
end

# Dec 19 1995	Add new task for FITS dates

# Mar 14 2000	Fix bug dealing with 2000-02-29
