#*** File archive/Src/t_obnobj.x
#*** November 20, 1995
#*** By Doug Mink, Center for Astrophysics

# OBNOBJ -- Get reduced file name(s) from observation numbers

include <error.h>
include <fset.h>

define	MAX_RANGES	100
define	NTYPES	7
define	SZ_LIST	1000

procedure t_obnobj()

char	obn_list[SZ_LIST]		# list of observation number ranges
char	obsdate[SZ_FNAME]			# observation date (yyyy.mmdd)
char	rootdir[SZ_PATHNAME]		# archive root directory
char	arcdir[SZ_PATHNAME]		# archive directory
char	arcfile[SZ_FNAME]		# archive file
int	obn				# reduced file number
char	next_obn[SZ_FNAME]
int	nfiles, fd, open(), i, ip, itok
int	get_next_number(), decode_ranges(), btoi()
int	obn_range[MAX_RANGES*2+1]
char	listfile[SZ_FNAME]		# file with list of archive file names
char	nextline[SZ_LINE]		# one line from input file
bool	verbose				# true to echo filename
bool	debug				# true to print directoriy as well
int	listdir				# true to include directory pathnames
int	arcout				# YES to use arc_dir
bool	clgetb()
int	nc, idot, getline(), stridx(), strldx()
char	jdot

include "head.com"

begin

	jdot = '.'
	call clgstr ("date", obsdate, SZ_FNAME)
	call clgstr ("root_dir", rootdir, SZ_PATHNAME)
	if (rootdir[1] == EOS) {
	    arcout = YES
	    call clgstr ("arc_dir", rootdir, SZ_PATHNAME)
	    }
	else
	    arcout = NO
	call clgstr ("list_file", listfile, SZ_FNAME)
	fd = 0
	if (listfile[1] != NULL)
	    fd = open (listfile, WRITE_ONLY, TEXT_FILE)
	verbose = clgetb ("verbose")
	debug = FALSE
	listdir = btoi (clgetb ("listdir"))

	# Set up the standard output to flush on a newline
	call fseti (STDOUT, F_FLUSHNL, YES)

	# Get reduced file numbers from file or input list
	call clgstr ("obn_list", obn_list, SZ_LINE)

	# Handle file of reduced file numbers
	if (obn_list[1] == '@') {
	    fd = open (obn_list[2],READ_WRITE,TEXT_FILE)

	    if (debug) {
		call printf (" opening file %s\n")
		    call pargstr (obn_list[2])
		}
	    while (getline (fd,nextline) > 0) {
		call tokenize (nextline,SZ_LINE)
		itok = 1
		nc = tok1[itok] - tok0[itok] + itok
		if (ntok > 1) {
		    call strcpy (nextline[tok0[itok]],obsdate,nc)
		    itok = itok + 1
		    nc = tok1[itok] - tok0[itok] + 1
		    }
		call strcpy (nextline[tok0[itok]],next_obn,nc)
		idot = stridx (jdot,next_obn)
		if (nc > 2) {
		    next_obn[idot] = EOS
		    }
		ip = 1
		call ctoi (next_obn,ip,obn)
		if (obn == 0)
		    call ctoi (next_obn,ip,obn)
		if (obn == 0)
		    call ctoi (next_obn,ip,obn)

		# Get input pathname and file name
		call obs_name (rootdir, obsdate, obn, arcdir, arcfile, debug)
		if (verbose) {
		    if (arcfile[1] == EOS) {
			call printf ("OBNOBJ:  %s obs %d not in TDC archive\n")
			    call pargstr (obsdate)
			    call pargi (obn)
			}
		    else if (listdir==YES) {
			call printf ("OBNOBJ:  %s%s\n")
			    call pargstr (arcdir)
			    call pargstr (arcfile)
			}
		    else {
			call printf ("OBNOBJ:  %s\n")
			    call pargstr (arcfile)
			}
		    }
		if (fd > 0 && arcfile[1] != EOS) {
		    if (listdir==YES) {
			call fprintf (fd,"%s%s\n")
			    call pargstr (arcdir)
			    call pargstr (arcfile)
			}
		    else {
			call fprintf (fd,"%s\n")
			    call pargstr (arcfile)
			}
		    nfiles = nfiles + 1
		    }
		}

	    call close (fd)
	    }
	else {
	    nc = stridx (jdot, obn_list)
	    if (nc > 1) {
		do i = nc, SZ_LIST {
		    obn_list[i] = EOS
		    }
		}

	    # Zero and decode reduced file number ranges
	    do i = 1, MAX_RANGES*2+1 {
		obn_range[i] = 0
		}
	    if (decode_ranges (obn_list, obn_range, MAX_RANGES, nfiles) == ERR)
		call error (1, "OBNOBJ: Illegal reduced file number list")

	    # Read successive archived IRAF files, convert and write into a
	    # succession of local IRAF files.
	    obn = 0
	    nfiles = 0
	    while (get_next_number (obn_range, obn) != EOF) {

		# Get input pathname and file name
		call obs_name (rootdir, obsdate, obn, arcdir, arcfile, verbose)

	# Remove image extension
		if (arcfile[1] != EOS) {
		    idot = strldx (jdot,arcfile)
		    if (idot > 0)
			arcfile[idot] = EOS
		    }

		if (verbose) {
		    if (arcfile[1] == EOS) {
			call printf ("OBNOBJ:  %s obs %d not in TDC archive\n")
			    call pargstr (arcdir)
			    call pargi (obn)
			}
		    else if (listdir==YES) {
			call printf ("OBNOBJ:  %s%s\n")
			    call pargstr (arcdir)
			    call pargstr (arcfile)
			}
		    else {
			call printf ("OBNOBJ:  %s\n")
			    call pargstr (arcfile)
			}
		    }
		if (fd > 0 && arcfile[1] != EOS) {
		    if (listdir==YES) {
			call fprintf (fd,"%s%s\n")
			    call pargstr (arcdir)
			    call pargstr (arcfile)
			}
		    else {
			call fprintf (fd,"%s\n")
			    call pargstr (arcfile)
			}
		    nfiles = nfiles + 1
		    }
		}
	    }

	call clpstr ("arcdir",arcdir)
	call clpstr ("arcfile",arcfile)
	call clputi ("nfiles",nfiles)
	if (fd > 0)
	    call close (fd)

end
# Jan 18 1995	New program
# Nov 20 1995	Clean up code; return file name without extension
