#*** File saotdc/archive/src/t_rarc.x
#*** June 25, 1991
#*** By Doug Mink, Center for Astrophysics

# RARC -- Read SAO TDC archive format data into IRAF image format
#         Further documentation given in rarc.hlp

include <error.h>
include <fset.h>

define	MAX_RANGES	100
define	NTYPES	7
define  SZ_LIST 1000

procedure t_rarc()

char	rfn_list[SZ_LINE]		# list of reduced file numbers
char	rootdir[SZ_FNAME]		# archive root directory
char	arcdir[SZ_FNAME]		# archive directory
char	arcfile[SZ_FNAME]		# archive file
char	in_fname[SZ_FNAME]		# input file name
char	out_fname[SZ_FNAME]		# output file name

char    next_rfname[SZ_FNAME], next_rfn[SZ_FNAME]
int	stat,nfiles,rfn,i,idot,stridx()
int	rfn_range[MAX_RANGES*2+1]
pointer fntopnb(), list_ptr
int     fntlenb(),fntgfnb(),junk, nlist, nc

bool	clgetb(), verbose
int	lamlog, arcout
char	clgetc(), jdot, jlt
int	arc_get_image_type(), btoi()
int	arc_read()
int	decode_ranges(),get_next_number()
include	"rarc.com"

begin
	jdot = '.'
	jlt = '<'

	# Set up the standard output to flush on a newline
	call fseti (STDOUT, F_FLUSHNL, YES)

	# Get RARC parameters.
	call clgstr ("rfn_list", rfn_list, SZ_LINE)

	# Get reduced file numbers from file or input list
	if (rfn_list[1] == '@') {
	    list_ptr = fntopnb (rfn_list,NO)
	    nlist = fntlenb (list_ptr)
	    rfn_list[1] = EOS
	    for (i=1; i <= nlist; i=i+1) {
		if (i > 1)
		    call strcat (",",rfn_list,SZ_LIST)
		junk = fntgfnb (list_ptr,next_rfname,SZ_LINE)
		nc = stridx (jdot, next_rfname)
		If (nc <= 0) nc = stridx (jlt, next_rfname)
		if (nc > 2) {
		    next_rfn[1] = EOS
		    call strcat (next_rfname,next_rfn,nc-2)
		    call strcat (next_rfn,rfn_list,SZ_LIST)
		    }
		else
		    call strcat (next_rfname,rfn_list,SZ_LIST)
		}
	    call fntclsb (list_ptr)
	    }

	verbose = clgetb ("verbose")

	# Zero the ranges
	do i = 1, MAX_RANGES*2+1 {
	    rfn_range[i] = 0
	    }

	# If list is file name, keep number only
	idot = stridx (jdot,rfn_list)
	if (idot <= 0) idot = stridx (jlt,rfn_list)
	if (idot > 0)
	    rfn_list[idot] = EOS
	if (verbose) {
	    call printf ("rfns: %s %d\n")
	    call pargstr (rfn_list)
	    call pargi (idot)
	    }

	# Decode the ranges
	if (decode_ranges (rfn_list, rfn_range, MAX_RANGES, nfiles) == ERR)
	    call error (1, "T_RARC: Illegal reduced file number list")

	call clgstr ("instrument", instrument, SZ_FNAME)
	call clgstr ("arc_dir", rootdir, SZ_FNAME)
	if (rootdir[1] == EOS) {
	    call clgstr ("root_dir", rootdir, SZ_FNAME)
	    arcout = NO
	    }
	else
	    arcout = YES
	long_header = btoi (clgetb ("long_header"))
	short_header = btoi (clgetb ("short_header"))
	debug = btoi (clgetb ("debug"))
	dispaxis = 1
	lamlog = NO
	lamlog = btoi (clgetb ("log"))
	if (lamlog == YES) {
	    call strcpy ("log lambda", disptype, SZ_LINE)
	    call strcpy ("log angstroms", dispunit, SZ_LINE)
	    }
	else {
	    call strcpy ("lambda", disptype, SZ_LINE)
	    call strcpy ("angstroms", dispunit, SZ_LINE)
	    }

	make_image = btoi (clgetb ("make_image"))
	if (make_image == YES)
	    data_type = arc_get_image_type (clgetc ("datatype"))

	# Read successive archive files, convert and write into a
	# succession of output IRAF files.

	rfn = 0
	while (get_next_number (rfn_range, rfn) != EOF) {

	    # Get input pathname
	    call arc_name (rootdir,instrument,rfn,arcout,arcdir,arcfile,debug)

	    # Set input path name
	    call strcpy (arcdir,in_fname,SZ_FNAME)
	    call strcat (arcfile,in_fname,SZ_FNAME)

	    # Set output file name
	    call arc_outfile (arcfile,rfn,out_fname,SZ_FNAME)
	    if (verbose) {
		call printf ("TDC archive file %s -> %s\n")
		call pargstr (in_fname)
	    	call pargstr (out_fname)
		}

	    # Convert SAO archive file to the output IRAF file.
	    # If EOT is reached then exit.
	    # If an error is detected then print a warning and continue with
	    # the next file.

	    if (arcfile[1] != EOS) {
		iferr (stat = arc_read (in_fname, out_fname))
		    call erract (EA_FATAL)
#		if (stat == EOF)
#		    break
		}
	    }
end


# ARC_GET_IMAGE_TYPE -- Convert a character to an IRAF image type.

int procedure arc_get_image_type (c)

char	c
int	type_codes[NTYPES], i
int	stridx()
string	types "usilrdx"			# supported image data types
data	type_codes /TY_USHORT, TY_SHORT, TY_INT, TY_LONG, TY_REAL,
		    TY_DOUBLE, TY_COMPLEX/
begin
	i = stridx (c, types)
	if (i == 0)
	    i = 5
	return (type_codes[i])
end


# ARC_OUTFILE -- Make output filename from input filename

int procedure arc_outfile (arcfile,rfn,outfile,maxchars)
char	arcfile[ARB]	# Reduced file name
int	rfn		# Reduced file number
char	outfile[ARB]	# IRAF file name
int	maxchars	# Length of IRAF file name

char	jsp
bool	keepname,clgetb()
int	lfn,strlen(),i
begin
	jsp = ' '
	keepname = FALSE
	keepname = clgetb ("keep_name")
	if (keepname) {
	    call strcpy (arcfile,outfile,maxchars)
	    lfn = strlen (outfile)
	    for (i=1; i<=lfn; i=i+1) {
		if (outfile[i] == jsp) outfile[i] = '_'
		}
	    }
	else {
	    call itoc (rfn,outfile,maxchars)
	    }
	return
end

# Jan 10 1989	Add parameter for archive directory

# Mar  7 1990	Return 0 for data type set from file

# Aug  9 1990	Return TY_REAL if no type specified

# Jun 25 1991	Allow output file to be reduced file number without object
