#*** File saotdc/archive/src/t_rarcd.x
#*** March 21, 2005
#*** By Doug Mink, Center for Astrophysics

# RARCD -- Read SAO TDC archive format data into IRAF image format
#          writing to a specified directory, 0-padded rfns
#          Further documentation given in rarc.hlp

include <syserr.h>
include <error.h>
include <fset.h>

define	MAX_RANGES	100
define	NTYPES	7
define  SZ_LIST 1000
define	SZ_EXT	8

procedure t_rarcd()

char	rfn_list[SZ_LINE]		# list of reduced file numbers
char	rootdir[SZ_FNAME]		# archive root directory
char	arcdir[SZ_FNAME]		# archive directory
char	arcfile[SZ_FNAME]		# archive file
char	in_fname[SZ_FNAME]		# input file name
char	outfile[SZ_FNAME]		# output file name
char	outdir[SZ_FNAME]		# output directory
char	out_fpath[SZ_FNAME]		# output file path
char	out_fname[SZ_FNAME]		# output file name
char	out_ext[SZ_EXT]			# output file extension from imtype

char    next_rfname[SZ_FNAME], next_rfn[SZ_FNAME]
int	stat,nfiles,rfn,i,idot,stridx()
int	rfn_range[MAX_RANGES*2+1]
pointer fntopnb(), list_ptr
int     fntlenb(),fntgfnb(),junk, nlist, nc, ncext

bool	clgetb(), verbose, overwrite
int	access()
int	lamlog, arcout, ldir
char	clgetc(), jdot, jlt
int	arc_get_image_type(), btoi()
int	arc_reade(), envgets()
int	decode_ranges(),get_next_number()
int	strlen()
include	"rarc.com"

begin
	jdot = '.'
	jlt = '<'

	# Set up the standard output to flush on a newline
	call fseti (STDOUT, F_FLUSHNL, YES)

	# Get RARCD parameters.
	call clgstr ("rfn_list", rfn_list, SZ_LINE)

	# Get reduced file numbers from file or input list
	if (rfn_list[1] == '@') {
	    list_ptr = fntopnb (rfn_list,NO)
	    nlist = fntlenb (list_ptr)
	    rfn_list[1] = EOS
	    for (i=1; i <= nlist; i=i+1) {
		if (i > 1)
		    call strcat (",",rfn_list,SZ_LIST)
		junk = fntgfnb (list_ptr,next_rfname,SZ_LINE)
		nc = stridx (jdot, next_rfname)
		If (nc <= 0) nc = stridx (jlt, next_rfname)
		if (nc > 2) {
		    next_rfn[1] = EOS
		    call strcat (next_rfname,next_rfn,nc-1)
		    call strcat (next_rfn,rfn_list,SZ_LIST)
		    }
		else
		    call strcat (next_rfname,rfn_list,SZ_LIST)
		}
	    call fntclsb (list_ptr)
	    }

	verbose = clgetb ("verbose")
	debug = btoi (clgetb ("debug"))
	overwrite = clgetb ("overwrite")

	# Zero the ranges
	do i = 1, MAX_RANGES*2+1 {
	    rfn_range[i] = 0
	    }

	# If list is file name, keep number only
	idot = stridx (jdot,rfn_list)
	if (idot <= 0) idot = stridx (jlt,rfn_list)
	if (idot > 0)
	    rfn_list[idot] = EOS
	if (debug == YES) {
	    call printf ("rfns: %s %d\n")
	    call pargstr (rfn_list)
	    call pargi (idot)
	    }

	# Decode the ranges
	if (decode_ranges (rfn_list, rfn_range, MAX_RANGES, nfiles) == ERR)
	    call error (1, "T_RARCD: Illegal reduced file number list")

	call clgstr ("instrument", instrument, SZ_FNAME)

	# Set input directory or base of archive tree 
	call clgstr ("arc_dir", rootdir, SZ_FNAME)
	if (rootdir[1] == EOS) {
	    call clgstr ("root_dir", rootdir, SZ_FNAME)
	    arcout = NO
	    }
	else
	    arcout = YES

	# Set output directory, if not current working directory
	call clgstr ("out_dir", outdir, SZ_FNAME)
	ldir = strlen (outdir)
	if (outdir[1] != EOS && outdir[ldir] != '/') {
	    outdir[ldir+1] = '/'
	    outdir[ldir+2] = EOS
	    }

	long_header = btoi (clgetb ("long_header"))
	short_header = btoi (clgetb ("short_header"))
	dispaxis = 1
	lamlog = NO
	lamlog = btoi (clgetb ("log"))
	if (lamlog == YES) {
	    call strcpy ("log lambda", disptype, SZ_LINE)
	    call strcpy ("log angstroms", dispunit, SZ_LINE)
	    }
	else {
	    call strcpy ("lambda", disptype, SZ_LINE)
	    call strcpy ("angstroms", dispunit, SZ_LINE)
	    }

	make_image = btoi (clgetb ("make_image"))
	if (make_image == YES)
	    data_type = arc_get_image_type (clgetc ("datatype"))

	ncext = envgets ("imtype", out_ext, SZ_EXT)

	# Read successive archive files, convert and write into a
	# succession of output IRAF files.
	rfn = 0
	while (get_next_number (rfn_range, rfn) != EOF) {

	    # Get input pathname
	    call arc_name (rootdir,instrument,rfn,arcout,arcdir,arcfile,debug)

	    # Set input path name
	    call strcpy (arcdir,in_fname,SZ_FNAME)
	    call strcat (arcfile,in_fname,SZ_FNAME)

	    # Set output file name
	    call arc_newfile (arcfile,rfn,outfile,SZ_FNAME)
	    call strcpy (outdir, out_fpath, SZ_FNAME)
	    call strcat (outfile, out_fpath, SZ_FNAME)
	    call strcpy (outfile, out_fname, SZ_FNAME)
	    if (ncext > 0) {
		call strcat (".", out_fname, SZ_FNAME)
		call strcat (out_ext, out_fname, SZ_FNAME)
		call strcat (".", out_fpath, SZ_FNAME)
		call strcat (out_ext, out_fpath, SZ_FNAME)
		}
	    if (debug == YES) {
		call printf ("TDC archive file %s -> %s\n")
		call pargstr (in_fname)
	    	call pargstr (out_fname)
		}
	    if (overwrite || access (out_fpath, 0, 0) == NO) {
		if (access (out_fname, 0, 0) == YES) {
		    iferr (call delete (out_fpath))
			call filerr (out_fpath, SYS_FCANTCLOB)
		    }

		# Convert SAO archive file to the output IRAF file.
		# If EOT is reached then exit.
		# If an error is detected then print a warning and continue with
		# the next file.

		if (arcfile[1] != EOS) {
		    iferr (stat = arc_reade (in_fname, out_fpath))
			call erract (EA_FATAL)
		    }
		call clpstr ("iraffile", out_fname)
		}
	    else if (access (out_fpath, 0, 0) == YES) {
		if (short_header == YES) {
		    call printf ("%s\n")
			call pargstr (out_fpath)
		    }
		call clpstr ("iraffile", out_fname)
		}
	    }
end

# Jan 10 1989	Add parameter for archive directory

# Mar  7 1990	Return 0 for data type set from file
# Aug  9 1990	Return TY_REAL if no type specified

# Jun 25 1991	Allow output file to be reduced file number without object

# Jun 17 1992	Use alternate reading subroutine

# Aug 18 1993	Fix bug when reading RFN's from list of reduced file names

# Feb  2 1994	Pass debug to arcname

# Aug  2 2001	Add outdir so output file can be anywhere
# Aug  2 2001	Add overwrite to control reaction if file exists
# Aug  3 2001	Write output IRAF file name to environment "spectrum"

# Apr  4 2002	Call arc_newfile() instead of arc_outfile() for zero-padding

# Mar 21 2005	Use debug internally as IRAF YES/NO, not boolean; fix access()
