#*** File archive/Src/t_rfnsky.x
#*** November 16, 1998
#*** By Doug Mink, Center for Astrophysics

# RFNSKY -- Get SKY reduced file name(s) from reduced file number ranges

include <error.h>
include <fset.h>

define	MAX_RANGES	100
define	NTYPES	7
define	SZ_LIST	1000

procedure t_rfnsky()

char	rfn_list[SZ_LIST]		# list of rfn ranges
char	inst[SZ_FNAME]			# instrument (echelle, zmach or mmt)
char	rootdir[SZ_FNAME]		# archive root directory
char	arcdir[SZ_FNAME]		# archive directory
char	arcfile[SZ_FNAME]		# archive file
char	sstring[SZ_FNAME]		# string for which to search (LC)
char	arclc[SZ_FNAME]		   	# lowercase version of archive file
int	rfn				# reduced file number
char	next_rfname[SZ_FNAME], next_rfn[SZ_FNAME]
int	nfiles, fd, open(), i
int	get_next_number(), decode_ranges()
#int	btoi()
int	rfn_range[MAX_RANGES*2+1]
char	listfile[SZ_FNAME]		# file with list of archive file names
bool	verbose				# true to echo filename
bool	debug				# true to print directory, file number
int	listdir				# true to include directory pathnames
int	arcout				# YES to use arc_dir
bool	sky				# true if SKY in object name
bool	clgetb()
pointer	fntopnb(), list_ptr
int	fntlenb(),fntgfnb(),junk, strmatch(), nlist, nc

begin
	# Set up the standard output to flush on a newline
	call fseti (STDOUT, F_FLUSHNL, YES)

	# Get reduced file numbers from file or input list
	call clgstr ("rfn_list", rfn_list, SZ_LINE)
	if (rfn_list[1] == '@') {
	    list_ptr = fntopnb (rfn_list,NO)
	    nlist = fntlenb (list_ptr)
	    rfn_list[1] = EOS
	    for (i=1; i <= nlist; i=i+1) {
		call strcat (",",rfn_list,SZ_LIST)
		junk = fntgfnb (list_ptr,next_rfname,SZ_LINE)
		nc = strmatch (next_rfname,".")
		if (nc > 2) {
		    next_rfn[1] = EOS
		    call strcat (next_rfname,next_rfn,nc-2)
		    call strcat (next_rfn,rfn_list,SZ_LIST)
		    }
		else
		    call strcat (next_rfname,rfn_list,SZ_LIST)
		}
	    call fntclsb (list_ptr)
	    }
	else {
	    nc = strmatch (rfn_list,".")
	    if (nc > 1) {
		for (i=nc; i<=SZ_LIST; i=i+1) {
		    rfn_list[i] = EOS
		    }
		}
	    }

#	listdir = btoi (clgetb ("listdir"))
	listdir = NO

	# Zero and decode reduced file number ranges
	do i = 1, MAX_RANGES*2+1 {
	    rfn_range[i] = 0
	    }
	if (decode_ranges (rfn_list, rfn_range, MAX_RANGES, nfiles) == ERR)
	    call error (1, "RFNSKY: Illegal reduced file number list")

	call clgstr ("root_dir", rootdir, SZ_FNAME)
	if (rootdir[1] == EOS) {
	    arcout = YES
	    call clgstr ("arc_dir", rootdir, SZ_FNAME)
	    }
	else
	    arcout = NO
	call clgstr ("list_file", listfile, SZ_FNAME)
	fd = 0
	if (listfile[1] != NULL)
	    fd = open (listfile, WRITE_ONLY, TEXT_FILE)
	call clgstr ("search", sstring, SZ_FNAME)
	call strlwr (sstring)
	call clgstr ("instrument", inst, SZ_FNAME)
	verbose = clgetb ("verbose")
	debug = FALSE
	if (debug) {
	    call printf ("RFNSKY: Searching for object %s in %s archive\n")
		call pargstr (sstring)
		call pargstr (inst)
	    }

	# Read successive archive files, convert and write into a
	# succession of output IRAF files.
	rfn = 0
	nfiles = 0
	while (get_next_number (rfn_range, rfn) != EOF) {

	    # Get input pathname and file name
	    call arc_name (rootdir,inst,rfn,arcout,arcdir,arcfile,debug)

	    sky = FALSE
	    if (arcfile[1] != EOS) {
		call strcpy (arcfile,arclc,SZ_FNAME)
		call strlwr (arclc)
		if (strmatch (arclc,sstring) != 0)
		    sky = TRUE
		}
	    if (verbose) {
		if (arcfile[1] == EOS) {
		    call printf ("RFNSKY:  %s rfn %d not in TDC archive\n")
			call pargstr (inst)
			call pargi (rfn)
		    }
		else if (listdir==YES && sky) {
		    call printf ("%s%s\n")
			call pargstr (arcdir)
			call pargstr (arcfile)
		    }
		else if (sky) {
		    call printf ("%s\n")
			call pargstr (arcfile)
		    }
		}
	    if (fd > 0 && sky) {
		if (listdir==YES) {
		    call fprintf (fd,"%s%s\n")
			call pargstr (arcdir)
			call pargstr (arcfile)
		    }
		else {
		    call fprintf (fd,"%s\n")
			call pargstr (arcfile)
		    }
		nfiles = nfiles + 1
		}
	    }

	call clputi ("nfiles",nfiles)
	if (fd > 0)
	    call close (fd)

end

# Jan 14 1993	Add case-insensitivity
# May  3 1993	Convert to lower case before testing to catch mixed cases

# Feb  2 1994	Pass verbose to arcname

# Nov 16 1998	Fix bug so correct root directory parameter is read
