	subroutine mosiac

	call mosaic

	return
	end

	subroutine mosaic
C-----------------------------------------------------------------------------
C
C	mosaicing program. This tries to be a bit more sophisticated
C	than the current IRAF irmosaic program. It makes one big frame
C	from an input list of files, taking into account bad pixels.
C	a database file has to indicate x,y positions of corresponding
C	stars or positions. An option is given that the program finds
C	the positions of the stars itself, from crude starting
C	coordinates supplied by the user. The frames are scaled in
C	background and intensity (to account for non-photometric weather).
C
C-----------------------------------------------------------------------------

	Parameter (MAXNPIX=256*256*30,MAXPIX=512*512,MAX1PIX=512*512)
	Parameter (MAXIM2=60)

	Integer Nx,Ny,unit,nimrows,nimcols
	Real*4 pixi(MAXNPIX),outf(MAXPIX),wt(MAXPIX)
	Real*4 maski(MAXNPIX),xc(MAXIM2),yc(MAXIM2),pixf(MAX1PIX),pixf2(MAX1PIX)
	Real*4 rat(100),wei(100),bsky(100),maskp(MAX1PIX)
	Integer scr(MAX1PIX),method
	Character*10 Ans
	Character*128 inlist,outname,maskfile,dbfile,masklist
	Character*128 ilist(100),string,omaskname,mlist(100)
	Character*60 obj(100),objdark,objmask,objflat,mas(100)
	Logical coappr,phot

	call gproms('inlist',' ',' ',inlist,ierr)
	call gproms('masklist',' ',' ',masklist,ierr)
	call gproms('outfile',' ',' ',outname,ierr)
	call gproms('outmaskname',' ',' ',omaskname,ierr)
	call gproms('dbfile',' ',' ',dbfile,ierr)
	call gpromi('nimcols',256,' ',nimcols,ierr)
	call gpromi('nimrows',256,' ',nimrows,ierr)
	call gproml('coappr',.TRUE.,' ',coappr,ierr)
	call gproml('photometric',.TRUE.,' ',phot,ierr)
	call gpromi('method',1,' ',method,ierr)

C	
C	first unravel the list of input files
C
	L=Len(inlist)
	ipos=1
10	if (inlist(ipos:ipos).eq.'@') then	
	   inlist(1:(128-ipos))=inlist((ipos+1):128)
	   call unravel(inlist,ilist,N)
	else
	   if (inlist(ipos:ipos).eq.' ') then
	      ipos=ipos+1
	      if (ipos.le.L) goto 10
	   else
	      ilist(1)=inlist
	      N=1
	   end if
	end if
	ipos=1
	L=Len(masklist)
20	if (masklist(ipos:ipos).eq.'@') then	
	   masklist(1:(128-ipos))=masklist((ipos+1):128)
	   call unravel(masklist,mlist,NN)
	else
	   if (masklist(ipos:ipos).eq.' ') then
	      ipos=ipos+1
	      if (ipos.le.L) goto 20
	   else
	      mlist(1)=masklist
	      NN=1
	   end if
	end if

	unit=11
	do i=1,N
	   call gimopen(unit,ilist(i),ierr)
	   if (ierr.ne.0) then
	    write (string,'('' MOSAIC: Error opening '',a)') ilist(i)
	    call gwrits(string,2)
	    return
	   end if
	   call gimhead(unit,obj(i),nx,ny,ierr)
	   if (ierr.ne.0) then
	    write (string,'('' MOSAIC: Error opening header of'',a)') ilist(i)
	    call gwrits(string,2)
	   end if
	   unit=unit+1
	end do
	call read3d(11,nx,ny,N,pixi,ierr)
C
C	now read the maskfiles
C
	nu=unit
	do i=1,NN
	   call gimopen(unit,mlist(i),ierr)
	   if (ierr.ne.0) then
	    write (string,'('' MOSAIC: Error opening '',a)') mlist(i)
	    call gwrits(string,2)
	    return
	   end if
	   call gimhead(unit,mas(i),nx,ny,ierr)
	   if (ierr.ne.0) then
	    write (string,'('' MOSAIC: Error opening header of'',a)') 
     1            mlist(i)
	    call gwrits(string,2)
	   end if
	   unit=unit+1
	end do
	call read3d(nu,nx,ny,NN,maski,ierr)
	if (NN.eq.1) then
	   call copypl(maski,maskp,nx,ny,1)
	end if
C
C	open database file and read coordinates
C
	open(unit=41,file=dbfile,status='old')
	rewind(41)
	do i=1,(2*N-2)
	   read (41,*) xc(i),yc(i)
	enddo
	if (phot) then
	   do i=1,N
	      read (41,*) wei(i),rat(i),bsky(i)
	   enddo
	else
	   do i=1,N
	      read (41,*) wei(i)
	   enddo
	end if
	close(41)
C
C	Now call subroutine to do all the work
C
	call domosaic(pixi,nx,ny,N,maski,NN,outf,nimcols,nimrows,wt,
     1	     xc,yc,coappr,pixf,pixf2,phot,rat,wei,bsky,scr,maskp,
     2       method)
	call gimwrit(outname,nimcols,nimrows,outf,obj(1),ierr)
	call gimwrit(omaskname,nimcols,nimrows,wt,obj(1),ierr)
	return
	end

	subroutine copypl(maski,maskp,nx,ny,nz)
	real maski(nx,ny,nz),maskp(nx,ny)
	do i=1,nx
	   do j=1,ny
	      maskp(i,j)=maski(i,j,1)
	   enddo
	enddo
	return
	end

	subroutine domosaic(data,nx,ny,nz,mask,NN,outf,nxo,nyo,wt,
     1         xc,yc,coappr,pixf,pixf2,phot,rat,wei,bsky,scr,maskp,
     2         method)

	real data(nx,ny,nz),mask(nx,ny,nz),wt(nxo,nyo),outf(nxo,nyo)
	real maskp(nx,ny)
	integer nx,ny,nz,imap(4),scr(nx,ny),method
	logical coappr,phot
	real xc(*),yc(*),pixf(nx,ny),fr(2),pixf2(nx,ny),rat(100)
	real yf(200000),sig(200000),wei(100),bsky(100)
	real xf(200000)
	integer ixl(4000),ixh(4000),iyl(4000),iyh(4000),ipt(4000)
	real xv(10000),yv(10000),zv(10000),sigv(10000),fit(3)
	character*128 string

C	do i=1,nz
C	   write (string,'(''pixel 28  45 nz '',F,I)') data(28,45,i),
C     1            nz
C	   call gwrits(string,2)
C	enddo
	if (NN.eq.1) then
	do k=2,nz
	   do i=1,nx
	      do j=1,ny
		 mask(i,j,k)=maskp(i,j)
	      enddo
	   enddo
	enddo
	   do i=1,nx
	      do j=1,ny
	         maskp(i,j)=1.-mask(i,j,k)
	      enddo
	   enddo
C	   if (k.le.6) then
	      call findimage(nx,ny,maskp,scr,0.5)
C	   end if
	   do i=1,1000
	      ixl(i)=9999
	      ixh(i)=-999
	      iyl(i)=9999
	      iyh(i)=-999
	      np=0
	      do j=1,nx
	         do kk=1,ny
		    if (scr(j,kk).eq.i) then
		       if (j.lt.ixl(i)) ixl(i)=j
		       if (j.gt.ixh(i)) ixh(i)=j
		       if (kk.lt.iyl(i)) iyl(i)=kk
		       if (kk.gt.iyh(i)) iyh(i)=kk
		       np=np+1
		    end if
	         enddo
	      enddo
	      ipt(i)=np
	      if (np.eq.0) goto 210
	   enddo
210	   ni=i-1
	end if
C
C	in order not to get interpolation errors we have to
C	interpolate across the bad areas to get a good estimate
C
C
C	first analyze the mask-file, and find the blotted areas
C	identify them as rectangles, fit a plane through the pixels
C	in that rectangle that are not blotted and in the 
C	border region of max. 2 pixels on each side
C
	do k=1,nz
	   if (NN.gt.1) then
	   do i=1,nx
	      do j=1,ny
	         maskp(i,j)=1.-mask(i,j,k)
	      enddo
	   enddo
C	   if (k.le.6) then
	      call findimage(nx,ny,maskp,scr,0.5)
C	   end if
	   do i=1,1000
	      ixl(i)=9999
	      ixh(i)=-999
	      iyl(i)=9999
	      iyh(i)=-999
	      np=0
	      do j=1,nx
	         do kk=1,ny
		    if (scr(j,kk).eq.i) then
		       if (j.lt.ixl(i)) ixl(i)=j
		       if (j.gt.ixh(i)) ixh(i)=j
		       if (kk.lt.iyl(i)) iyl(i)=kk
		       if (kk.gt.iyh(i)) iyh(i)=kk
		       np=np+1
		    end if
	         enddo
	      enddo
	      ipt(i)=np
	      if (np.eq.0) goto 211
	   enddo
211	   ni=i-1
	   write (string,'('' # bad areas: '',I)') ni
	   call gwrits(string,2)
	   end if
	   do i=2,ni
C	don't use the 1st area, because then you will interpolate all
C	the data
	      if (ixl(i).gt.2) then
		 nxl=ixl(i)-2
	      else
		 nxl=1
	      end if
	      if (ixh(i).lt.(nx-1)) then
		 nxh=ixh(i)+2
	      else
		 nxh=nx
	      end if
	      if (iyl(i).gt.2) then
		 nyl=iyl(i)-2
	      else
		 nyl=1
	      end if
	      if (iyh(i).lt.(ny-1)) then
		 nyh=iyh(i)+2
	      else
		 nyh=ny
	      end if
	      nt=0
	      do m=nxl,nxh
		 do l=nyl,nyh
		    if (maskp(m,l).lt.0.5) then
		       nt=nt+1
		       xv(nt)=m
		       yv(nt)=l
		       zv(nt)=data(m,l,k)
		       sigv(nt)=1.
		    end if
		 enddo
	      enddo
	      ndecl=1000
	      call planefit(xv,yv,zv,sigv,nt,ndecl,fit,ierr)
	      if (ierr.ne.0) then
	         write (string,'(''planefit did not work '',2I)') k,ierr
	         call gwrits(string,2)
		 goto 444
	      else
		 do l=nxl,nxh
		    do m=nyl,nyh
		       if (maskp(l,m).gt.0.5) then
			  data(l,m,k)=fit(1)+fit(2)*l+fit(3)*m
	                  maskp(l,m)=0.
		       end if
		    enddo
		 enddo
	      end if
444	   enddo
	enddo

C
C	first zero output frame
C
	do i=1,nxo
	   do j=1,nyo
	      outf(i,j)=0.
	      wt(i,j)=0.
	   enddo
	enddo
C
C	if coappr is true, then calculate the real position of the
C	matching objects
C
	if (coappr) then
	   ictr=1
	   do i=1,nz
	      do j=1,nx
	         do k=1,ny
		    pixf(j,k)=data(j,k,i)
		 enddo
	      enddo
	      imap(1)=int(xc(ictr)-3.+0.5)
	      imap(2)=int(xc(ictr)+3.+0.5)
	      imap(3)=int(yc(ictr)-3.+0.5)
	      imap(4)=int(yc(ictr)+3.+0.5)
	      if (imap(1).lt.1) imap(1)=1
	      if (imap(2).gt.nx) imap(2)=nx
	      if (imap(3).lt.1) imap(3)=1
	      if (imap(4).gt.ny) imap(4)=ny
	      ierr=0
	      call starcen(pixf,nx,ny,'M',imap,xout,yout,xerr,yerr,
     1                     xsig,ysig,ierr)
	      if ((ierr.eq.0).or.(abs(xout-xc(ictr)).le.3.).or.
     1                     (abs(yout-yc(ictr)).le.3.)) then
		 xc(ictr)=xout
	         yc(ictr)=yout
	    write (string,'('' stars: width '',I4,2F10.3)') ictr,
     1             xsig,ysig
	    call gwrits(string,2)
	      end if
	      ictr=ictr+1
	      if ((i.ne.1).and.(i.ne.nz)) then
	         imap(1)=int(xc(ictr)-3.+0.5)
	         imap(2)=int(xc(ictr)+3.+0.5)
	         imap(3)=int(yc(ictr)-3.+0.5)
	         imap(4)=int(yc(ictr)+3.+0.5)
	         if (imap(1).lt.1) imap(1)=1
	         if (imap(2).gt.nx) imap(2)=nx
	         if (imap(3).lt.1) imap(3)=1
	         if (imap(4).gt.ny) imap(4)=ny
	         ierr=0
	         call starcen(pixf,nx,ny,'M',imap,xout,yout,xerr,yerr,
     1                     xsig,ysig,ierr)
	         if ((ierr.eq.0).or.(abs(xout-xc(ictr)).le.3.).or.
     1                     (abs(yout-yc(ictr)).le.3.)) then
		    xc(ictr)=xout
	            yc(ictr)=yout
	    write (string,'('' stars: width '',I4,2F10.3)') ictr,
     1             xsig,ysig
	    call gwrits(string,2)
	         end if
	         ictr=ictr+1
	      end if
	   enddo
	end if
	xmax=-10000.
	xmin=10000.
	ymax=-10000.
	ymin=10000.
	dx=0.
	dy=0.
	do i=1,nz-1
	   dx=dx+xc(2*i)-xc(2*i-1)
	   dy=dy+yc(2*i)-yc(2*i-1)
	   if (dx.gt.xmax) xmax=dx
	   if (dx.lt.xmin) xmin=dx
	   if (dy.gt.ymax) ymax=dy
	   if (dy.lt.ymin) ymin=dy
	enddo
	if (xmax.lt.0.) xmax=0.
	if (ymax.lt.0.) ymax=0.
	ndx1=2.-int(1.-xmax)
	ndy1=2.-int(1.-ymax)
	do i=1,nx
	   do j=1,ny
	      if (mask(i,j,1).gt.0.5) then
		 outf(i+ndx1,j+ndy1)=data(i,j,1)*wei(1)
		 wt(i+ndx1,j+ndy1)=wei(1)
	      end if
	   enddo
	enddo
	shx=ndx1
	shy=ndy1
C
C	now add all other frames
C
	do k=2,nz
C
C	first determine position offset
C
	write (string,'('' adding frame  '',i)') k 
	call gwrits(string,2)
	   shx=shx-xc(2*k-2)+xc(2*k-3)
	   shy=shy-yc(2*k-2)+yc(2*k-3)
	   intshx=int(shx+0.5)
	   intshy=int(shy+0.5)
	   fr(1)=shx-intshx
	   fr(2)=shy-intshy
	   s=0
	   ns=0
	   do i=1,nx
	      do j=1,ny
	         if (mask(i,j,k).gt.0.5) then
	            pixf(i,j)=data(i,j,k)
		    s=s+data(i,j,k)
		    ns=ns+1
		 end if
	      enddo
	   enddo
	   if (ns.gt.0) then
	      s=s/ns
	   else
	      s=0.
	    write (string,'(''no valid pixels '')') 
	    call gwrits(string,2)
	   end if
	   do i=1,nx
	      do j=1,ny
	         if (mask(i,j,k).lt.0.5) pixf(i,j)=s
	      enddo
	   enddo
	   call xyshft(pixf,pixf2,nx,ny,fr,method)
	   sum=0.
	   npsum=0
	   do i=1,nx
	      do j=1,ny
	         maskp(i,j)=mask(i,j,k)
	         if (mask(i,j,k).gt.0.5) then
	            nxmin=i-2
		    nxmax=i+2
	            nymin=j-2
		    nymax=j+2
	            if ((nxmin.lt.1).or.(nxmax.gt.nx).or.
     1                  (nymin.lt.1).or.(nymax.gt.ny)) then
	                maskp(i,j)=0.
	            else
	               do l=nxmin,nxmax
		          do m=nymin,nymax
		             if (mask(l,m,k).lt.0.5) then
	                        maskp(i,j)=0.
	                        goto 7777
	                     end if
	                  enddo
	               enddo
	            end if
	         end if
7777             kk=i
	      enddo
	   enddo
	   do i=1,nx
	      do j=1,ny
	         mask(i,j,k)=maskp(i,j)
	      enddo
	   enddo
	   nt=1
	   do i=1,nx
	      do j=1,ny
	         if ((mask(i,j,k).gt.0.5).and.(wt(i+intshx,j+intshy)
     1                .gt.0.)) then
		    xf(nt)=pixf2(i,j)
		    yf(nt)=outf(i+intshx,j+intshy)/wt(i+intshx,j+intshy)
		    sig(nt)=1.
		    nt=nt+1
		 end if
	      enddo
	   enddo
	   nt=nt-1
	   nd=200000
	   call ffit(xf,yf,nd,nt,sig,0,a,b,siga,sigb,chi2,q)
	   if ((phot).and.(rat(1).gt.0.)) then
	      b=rat(k)/rat(1)
	      if (abs(bsky(1)).gt.0.0001) then
		 a=bsky(k)-bsky(1)
	      else
	         sum=0.
	         do i=1,nt
	            sum=sum+yf(i)-b*xf(i)
	         enddo
	         a=sum/(1.*nt)
	      end if
	   end if
	   write (string,'('' FIT: '',2F)') a,b
	   call gwrits(string,2)
	   do i=1,nx
	      do j=1,ny
		 if (mask(i,j,k).gt.0.5) then
	          outf(i+intshx,j+intshy)=outf(i+intshx,j+intshy)+
     1		      wei(k)*(pixf2(i,j)*b+a)
		  wt(i+intshx,j+intshy)=wt(i+intshx,j+intshy)+wei(k)
		 end if
	      enddo
	   enddo
	enddo
	sum=0.
	nt=0
	do i=1,nxo
	   do j=1,nyo
	      if (wt(i,j).gt.0.) then
	         outf(i,j)=outf(i,j)/wt(i,j)
	         sum=sum+outf(i,j)
	         nt=nt+1
	      end if
	   enddo
	enddo
	sum=sum/nt
	do i=1,nxo
	   do j=1,nyo
	      if (wt(i,j).le.0.) then
		 outf(i,j)=sum
	         wt(i,j)=0.
	      else
	         wt(i,j)=1.
	      end if
	   enddo
	enddo
	return
	end
	

        SUBROUTINE KAPSIG(VAL,NVAL,ITER,AKAP,CONS,RMS,NPTS,VSQ,ISTAT)
C
C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C.IDENTIFICATION             KAPSIG                   NOV-82
C
C.TYPE           subroutine
C
C.KEYWORDS       mathematics
C
C.PURPOSE        selects a constant mean through a kap*sig clipping
C
C.ALGORITHM      1.) calculate mean and rms
C                2.) delete pixels beyond AKAP*RMS from mean
C                3.) GO TO 1.)
C
C.MODULES CALL   none
C
C.INPUT/OUTPUT   parameters
C
C       -> VAL   input values
C       -> NVAL  number of input values
C       -> ITER  number of iterations (min : 1)
C       -> AKAP  AKAP*RMS
C       <- CONS  derived mean value
C       <- RMS   the rms of CONS
C       <- NPTS  number of points used to derived CONS
C       -> VSQ   work array same dimension as VAL
C       <- ISTAT status of operation
C       :  0  O.K.
C       : -1  NVAL .LT. 2
C
C------------------------------------------------------------------------
C
        DIMENSION VAL(1),VSQ(1)
        REAL*4    CONS,RMS,AKAP
        INTEGER*4 NVAL,NPTS,ISTAT,ITER
C
C.. initialize CONS
        ISTAT = -1
        IF(NVAL.LT.2 ) RETURN
        ISTAT = 0
        CONS = 0.
        DO I=1,NVAL
        CONS = CONS + VAL(I)
        END DO
        CONS = CONS/FLOAT(NVAL)
        CSQ = CONS*CONS
C
C.. initialize RMS
        DELS = 0.
        DO I=1,NVAL
        VSQ(I) = VAL(I)*VAL(I)
        DELV = VSQ(I) + CSQ - 2.*VAL(I)*CONS
        IF(DELV.LT.0.) DELV = 0.
        DELS = DELS + DELV
        END DO
C
        DELA = DELS/FLOAT(NVAL-1)
        ARGU = MAX(1.E-37,DELA)
        RMS  = SQRT(ARGU)
        CLIP = AKAP*RMS
        IF(ITER.LE.0) RETURN
C
C.. iterate
        NPOLD = 0
        DO L=1,ITER
        CON = 0.
        NPTS = 0
        DELS = 0.
         DO I=1,NVAL
           IF( ABS(VAL(I)-CONS) .LT. CLIP) THEN
            NPTS = NPTS + 1
             DELV = VSQ(I) + CSQ - 2.*VAL(I)*CONS
              IF(DELV.LT.0.) DELV = 0.
              DELS = DELS + DELV
             CON = CON + VAL(I)
            NPNEW = NPTS
           END IF
         END DO
C
        IF(NPNEW.LE.2) RETURN
        IF(NPNEW.EQ.NPOLD) RETURN
        NPOLD = NPNEW
C
C..  define new rms and CONS
        DELA = DELS/FLOAT(NPTS-1)
        ARGU = MAX(1.E-37,DELA)
        RMS  = SQRT(ARGU)
        CLIP = AKAP*RMS
        CONS = CON/FLOAT(NPOLD)
        CSQ = CONS*CONS
C
        END DO
C
C..  exit
        RETURN
        END


        SUBROUTINE STARCEN(AIMG,NPIX,NPIY,MET,IMAP,
     +                XOUT,YOUT,XERR,YERR,XSIG,YSIG,ISTAT)
C
C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C.IDENTIFICATION
C subroutine STARCEN            version 2.10    840322
C initially written by H.Waldthausen    ESO - Garching
C cleaned up to be legible by K. Banse
C                                       2.20    851106
C
C.KEYWORDS
C  photometry
C
C.PURPOSE
C calculates the center position of an object
C
C.ALGORITHM
C  according to P.B.Stetson for GAUSS-method and H.Waldthausen for MOM-method
C
C.INPUT/OUTPUT   param
C
C       -> AIMG   input image
C       -> NPIX   x-pixels and
C       -> NPIY   y-pixels of AIMG
C       -> MET    method of centering
C                 MOM   moment centering
C                 GAU   gaussian centering
C       -> IMAP   area with star x-st,x-nd,y-st,y-nd
C       <- XOUT   x- position in pixels
C       <- YOUT   y- position in pixels
C       <- XERR   x- error estimate of XOUT
C       <- YERR   y- error estimate of YOUT
C       <- XSIG   x- with of source
C       <- YSIG   y- with of source
C       <->ISTAT  output/input  status
C                   0 O.K.
C                   1 crowded, or weak source
C                   2 no pos. source found
C                   3 iteration failed
C                 < 0 input error
C                  -1 dimensions larger than 90
C                  -2 MET not MOM or GAU
C
C---------------------------------------------------------------------
C
        CHARACTER*(*)   MET
        REAL*8     KRX(90),KRY(90),SWORK(90)
        DIMENSION
     +  AIMG(NPIX,NPIY),IMAP(4),EDVAL(400),WORK(400)
C
C..  initialize
        IXA = IMAP(1)
        IXE = IMAP(2)
        IYA = IMAP(3)
        IYE = IMAP(4)
        NX = IXE - IXA + 1
        NY = IYE - IYA + 1
C
        ISTAT = 0
        XSIG =  0.
        YSIG =  0.
        XOUT = -1.
        YOUT = -1.
        XERR = 0.
        YERR = 0.
C
C..  *** MOMENT centering
C
        XOLD = -1.
        YOLD = -1.
C
C..  find bgval and rms from edge pixels
        I = 1
        DO IY=IYA,IYE
           IF (IY.EQ.IYA .OR. IY.EQ.IYE) THEN
               DO IX=IXA,IXE
                  EDVAL(I) = AIMG(IX,IY)
                  I = I + 1
               ENDDO
            ELSE
               EDVAL(I) = AIMG(IXA,IY)
               I = I + 1
               EDVAL(I) = AIMG(IXE,IY)
               I = I + 1
            ENDIF
        ENDDO
C
        NVAL = I - 1
        AKAP = 2.
        NTER = 5
        CALL KAPSIG(EDVAL,NVAL,NTER,AKAP,BGVAL,RMS,NPB,WORK,IOK)
C
C.. calculate moment for pixel values > 3*RMS above BGVAL
        CLIP = 3.*RMS
        ITER = 8
C
        DO I=1,ITER               
           XMOM = 0.
           YMOM = 0.
           SUMI = 0.
           DO IY=IYA,IYE
              DO IX=IXA,IXE
                 SOUR = AIMG(IX,IY) - BGVAL
                 IF ( SOUR .GT. CLIP) THEN
                    SUMI = SUMI + SOUR
                    XMOM = XMOM + FLOAT(IX)*SOUR
                    YMOM = YMOM + FLOAT(IY)*SOUR
                 ENDIF
              ENDDO
           ENDDO
C
C..  check status, crowded or weak source
           IXS = IXE - IXA + 1
           IYS = IYE - IYA + 1
           IF (IXS.LE.2 .OR. IYS.LE.2) THEN
              XSIG = FLOAT(IXS)
              YSIG = FLOAT(IYS)
              ISTAT = 1
              IF (SUMI .GT. 0.0) THEN
                 XOUT = XMOM/SUMI
                 YOUT = YMOM/SUMI
              ELSE
                 XOUT = FLOAT(IXA+IXE)/2.
                 YOUT = FLOAT(IYA+IYE)/2.
                 ISTAT = 2
                 RETURN
              ENDIF
              RETURN
           ENDIF
C
C.. only positive sources assumed
           IF (SUMI .GT. 0.0) THEN
              XOUT = XMOM/SUMI
              YOUT = YMOM/SUMI
              XSIG = FLOAT(IXS)
              YSIG = FLOAT(IYS)
C
C.. calculate sigma and errors if convergence
              IF (XOUT.EQ.XOLD .AND. YOUT.EQ.YOLD) THEN
                 XRMS = 0.
                 YRMS = 0.
                 XERR = 0.
                 YERR = 0.
                 APIX = 0.
                 SUMI = 0.
                 DO IY=IYA,IYE
                    DO IX = IXA,IXE
                       SOUR = AIMG(IX,IY) - BGVAL
                       IF ( SOUR .GT. CLIP ) THEN
                          XDIF = FLOAT(IX) - XOUT
                          YDIF = FLOAT(IY) - YOUT
                          XRMS = XRMS + ABS(SOUR)*XDIF*XDIF
                          YRMS = YRMS + ABS(SOUR)*YDIF*YDIF
                          SUMI = SUMI + SOUR
                          APIX = APIX + 1.
                       ENDIF
                    ENDDO
                 ENDDO
C
                 IXN = XOUT + 0.5
                 IYN = YOUT + 0.5
                 SUMF = SUMI + AIMG(IXN,IYN) - BGVAL
                 XSIG = SQRT(XRMS/SUMF)
                 YSIG = SQRT(YRMS/SUMF)
                 XERR = XSIG/SQRT(APIX-1.)
                 YERR = YSIG/SQRT(APIX-1.)
                 RETURN
              ENDIF
C
              YOLD = YOUT
              XOLD = XOUT
              ISTAT = 0
           ELSE
              XOUT = FLOAT(IXA+IXE)/2.
              YOUT = FLOAT(IYA+IYE)/2.
              ISTAT = 2
              RETURN
           ENDIF
C
C..  crowded  or week source conditions
           LX = XOUT
           LY = YOUT
           SOUR = AIMG(LX,LY) - BGVAL
           IF (SOUR .LE. CLIP) THEN
              XSIG = 0.
              YSIG = 0.
              ISTAT = 1
              RETURN
           ENDIF
C
C.. find extent of source i.e. delete spikes and weak sources
           LXA = IXA
           DO WHILE ( SOUR.GT.CLIP .AND. LX.GE.LXA)
              IXA = LX
              LX = LX - 1
              SOUR = AIMG(LX,LY) - BGVAL
           ENDDO
C
           LY = YOUT
           LX = XOUT
           LXE = IXE
           SOUR = AIMG(LX,LY) - BGVAL
           DO WHILE ( SOUR.GT.CLIP .AND. LX.LE.LXE)
              IXE = LX
              LX = LX + 1
              SOUR = AIMG(LX,LY) - BGVAL
           ENDDO
C
           LY = YOUT
           LX = XOUT
           LYA = IYA
           SOUR = AIMG(LX,LY) - BGVAL
           DO WHILE ( SOUR.GT.CLIP .AND. LY.GE.LYA)
              IYA = LY
              LY = LY - 1
              SOUR = AIMG(LX,LY) - BGVAL
           ENDDO
C
           LY = YOUT
           LX = XOUT
           LYE = IYE
           SOUR = AIMG(LX,LY) - BGVAL
           DO WHILE ( SOUR.GT.CLIP .AND. LY.LE.LYE)
              IYE = LY
              LY = LY + 1
              SOUR = AIMG(LX,LY) - BGVAL
           ENDDO
C
        ENDDO          
        ISTAT = 3
        RETURN
        END

	SUBROUTINE FFIT(ZX,Y,NDECL,NDATA,SIG,MWT,A,B,SIGA,SIGB,CHI2,Q)
	DIMENSION ZX(NDECL),Y(NDECL),SIG(NDECL)
	SX=0.
	SY=0.
	ST2=0.
	B=0.
	IF(MWT.NE.0) THEN
		SS=0.
		DO I=1,NDATA
			WT=1./(SIG(I)**2)
			SS=SS+WT
			SX=SX+ZX(I)*WT
			SY=SY+Y(I)*WT
		ENDDO
	ELSE
		DO I=1,NDATA
			SX=SX+ZX(I)
			SY=SY+Y(I)
		ENDDO
		SS=FLOAT(NDATA)
	ENDIF
	SXOSS=SX/SS
	IF(MWT.NE.0) THEN
		DO I=1,NDATA
			T=(ZX(I)-SXOSS)/SIG(I)
			ST2=ST2+T*T
			B=B+T*Y(I)/SIG(I)
		ENDDO
	ELSE
		DO I=1,NDATA
			T=ZX(I)-SXOSS
			ST2=ST2+T*T
			B=B+T*Y(I)
		ENDDO
	ENDIF
	B=B/ST2
	A=(SY-SX*B)/SS
	SIGA=SQRT((1.+SX*SX/(SS*ST2))/SS)
	SIGB=SQRT(1./ST2)
	CHI2=0.
	IF(MWT.EQ.0) THEN
		DO I=1,NDATA
			CHI2=CHI2+(Y(I)-A-B*ZX(I))**2
		ENDDO
		Q=1.
		SIGDAT=SQRT(CHI2/(NDATA-2))
		SIGA=SIGA*SIGDAT
		SIGB=SIGB*SIGDAT
	ELSE
		DO I=1,NDATA
			CHI2=CHI2+((Y(I)-A-B*ZX(I))/SIG(I))**2
		ENDDO
		Q=GAMMQ(0.5*(NDATA-2),0.5*CHI2)
	ENDIF
	RETURN
	END

	FUNCTION GAMMQ(A,X)
	IF(X.LT.0..OR.A.LE.0.)PAUSE
	IF(X.LT.A+1.)THEN
		CALL GSER(GAMSER,A,X,GLN)
		GAMMQ=1.-GAMSER
	ELSE
		CALL GCF(GAMMCF,A,X,GLN)
		GAMMQ=GAMMCF
	ENDIF
	RETURN
	END

	SUBROUTINE GCF(GAMMCF,A,X,GLN)
	PARAMETER (ITMAX=100,EPS=3.E-7)
	GLN=GAMMLN(A)
	GOLD=0.
	A0=1.
	A1=X
	B0=0.
	B1=1.
	FAC=1.
	DO N=1,ITMAX
		AN=FLOAT(N)
		ANA=AN-A
		A0=(A1+A0*ANA)*FAC
		B0=(B1+B0*ANA)*FAC
		ANF=AN*FAC
		A1=X*A0+ANF*A1
		B1=X*B0+ANF*B1
		IF(A1.NE.0.)THEN
			FAC=1./A1
			G=B1*FAC
			IF(ABS((G-GOLD)/G).LT.EPS)GO TO 1
			GOLD=G
		ENDIF
	ENDDO
	PAUSE 'A too large, ITMAX too small'
1	GAMMCF=EXP(-X+A*ALOG(X)-GLN)*G
	RETURN
	END

	SUBROUTINE GSER(GAMSER,A,X,GLN)
	PARAMETER (ITMAX=100,EPS=3.E-7)
	GLN=GAMMLN(A)
	IF(X.LE.0.)THEN
		IF(X.LT.0.)PAUSE
		GAMSER=0.
		RETURN
	ENDIF
	AP=A
	SUM=1./A
	DEL=SUM
	DO N=1,ITMAX
		AP=AP+1.
		DEL=DEL*X/AP
		SUM=SUM+DEL
		IF(ABS(DEL).LT.ABS(SUM)*EPS)GO TO 1
	ENDDO
	PAUSE 'A too large, ITMAX too small'
1	GAMSER=SUM*EXP(-X+A*LOG(X)-GLN)
	RETURN
	END

	FUNCTION GAMMLN(XX)
	REAL*8 COF(6),STP,HALF,ONE,FPF,X,TMP,SER
	DATA COF,STP/76.18009173D0,-86.50532033D0,24.01409822D0,
     *		-1.231739516D0,.120858003D-2,-.536382D-5,2.50662827465D0/
	DATA HALF,ONE,FPF/0.5D0,1.0D0,5.5D0/
	X=XX-ONE
	TMP=X+FPF
	TMP=(X+HALF)*LOG(TMP)-TMP
	SER=ONE
	DO J=1,6
		X=X+ONE
		SER=SER+COF(J)/X
	ENDDO
	GAMMLN=TMP+LOG(STP*SER)
	RETURN
	END

        SUBROUTINE XYSHFT(data,a,ni,nj,shift,method)
C
C   to shift a picture in DATA
C
C   SHIFT(2)    +ve moves data to right, or to increasing rownumber
C
C       RFP 870119
C
        REAL*4 DATA(NI,NJ),A(NI,NJ),SHIFT(2)
        INTEGER NI,NJ,METHOD
        REAL*4 OLD(1024),NEW(1024)
        REAL*4 DAT1(1024),DAT2(1024),Y2(1024)
C
        XSHIFT=SHIFT(1)
        YSHIFT=SHIFT(2)
C
        IF(XSHIFT.NE.0.) THEN
C          CALL MESSAGE('shifting in X')
           if (xshift.lt.0) then
              ILX=INT(NI-XSHIFT)
           else
              ILX=NI
           end if
           DO I=1,ILX
              OLD(I)=I
              NEW(I)=FLOAT(I)-XSHIFT
           END DO
           DO J=1,NJ
	      IF (METHOD.EQ.1) THEN
                 CALL TSINC(OLD,DATA(1,J),NEW,DAT2,ILX,ILX)
	      ELSE
	         CALL SPLI(OLD,DATA(1,J),NEW,DAT2,ILX,ILX,Y2)
	      END IF
              DO I=1,ILX
                 A(I,J)=DAT2(I)
              END DO
              DO I=ILX+1,NI
                 A(I,J)=0.
              END DO
           END DO
        END IF
        IF(YSHIFT.NE.0.) THEN
           IF (YSHIFT.GT.0) then
              ILY=INT(NJ-YSHIFT)
           else
              ILY=NJ
           end if
           DO J=1,ILY
              OLD(J)=J
              NEW(J)=FLOAT(J)-YSHIFT
           END DO
           DO I=1,NI
              DO J=1,ILY
                 DAT1(J)=A(I,J)
              END DO
	      IF (METHOD.EQ.1) THEN
                 CALL TSINC(OLD,DAT1,NEW,DAT2,ILY,ILY)
	      ELSE
	         CALL SPLI(OLD,DAT1,NEW,DAT2,ILY,ILY,Y2)
	      END IF
              DO J=1,ILY
                 A(I,J)=DAT2(J)
              END DO
              DO j=ILY+1,NJ
                 A(I,J)=0.
              END DO
           END DO
        END IF
	IF ((XSHIFT.EQ.0.).AND.(YSHIFT.EQ.0.)) then
	   do i=1,NI
	      do j=1,NJ
	         A(I,J)=DATA(I,J)
	      enddo
	   enddo
	END IF
C   write the section

C
        RETURN
        END
	SUBROUTINE SPLI(XOLD,YOLD,XNEW,YNEW,NOLD,NNEW,Y2)
C
C	subroutine to interploate from YOLD to YNEW using
C	spline interpolation and linear extrapolation
C
C   here
C   XOLD are the original X-values and XNEW the new ones.
C   NNEW is the size of arrays XNEW, YNEW
C   NOLD "	"	"      XOLD, YOLD
C
	DIMENSION XOLD(1),YOLD(1),XNEW(1),YNEW(1),Y2(1)

	yp1=(yold(2)-yold(1))/(xold(2)-xold(1))
	ypn=(yold(nold)-yold(nold-1))/(xold(nold)-xold(nold-1))
	call spline(xold,yold,nold,yp1,ypn,y2)
	do i=1,nnew
	   if ((xnew(i).ge.xold(1)).and.(xnew(i).le.xold(nold))) then
	      call splint(xold,yold,y2,nold,xnew(i),ynew(i))
	   else
	      if (xnew(i).lt.xold(1)) then
	         c=(yold(2)-yold(1))/(xold(2)-xold(1))
	         ynew(i)=c*xnew(i)+yold(1)-a*xold(1)
	      else
	         c=(yold(nold)-yold(nold-1))/(xold(nold)-xold(nold-1))
		 ynew(i)=c*xnew(i)+yold(nold)-a*xold(nold)
	      end if
	   end if
	enddo

	return
	end


	SUBROUTINE SPLINE(X,Y,N,YP1,YPN,Y2)
	PARAMETER (NMAX=100)
	DIMENSION X(N),Y(N),Y2(N),U(NMAX)
	IF (YP1.GT..99E30) THEN
		Y2(1)=0.
		U(1)=0.
	ELSE
		Y2(1)=-0.5
		U(1)=(3./(X(2)-X(1)))*((Y(2)-Y(1))/(X(2)-X(1))-YP1)
	ENDIF
	DO I=2,N-1
		SIG=(X(I)-X(I-1))/(X(I+1)-X(I-1))
		P=SIG*Y2(I-1)+2.
		Y2(I)=(SIG-1.)/P
		U(I)=(6.*((Y(I+1)-Y(I))/(X(I+1)-X(I))-(Y(I)-Y(I-1))
     *			/(X(I)-X(I-1)))/(X(I+1)-X(I-1))-SIG*U(I-1))/P
	ENDDO
	IF (YPN.GT..99E30) THEN
		QN=0.
		UN=0.
	ELSE
		QN=0.5
		UN=(3./(X(N)-X(N-1)))*(YPN-(Y(N)-Y(N-1))/(X(N)-X(N-1)))
	ENDIF
	Y2(N)=(UN-QN*U(N-1))/(QN*Y2(N-1)+1.)
	DO K=N-1,1,-1
		Y2(K)=Y2(K)*Y2(K+1)+U(K)
	ENDDO
	RETURN
	END

	SUBROUTINE SPLINT(XA,YA,Y2A,N,X,Y)
	DIMENSION XA(N),YA(N),Y2A(N)
	KLO=1
	KHI=N
1	IF (KHI-KLO.GT.1) THEN
		K=(KHI+KLO)/2
		IF(XA(K).GT.X)THEN
			KHI=K
		ELSE
			KLO=K
		ENDIF
	GOTO 1
	ENDIF
	H=XA(KHI)-XA(KLO)
	IF (H.EQ.0.) PAUSE 'Bad XA input.'
	A=(XA(KHI)-X)/H
	B=(X-XA(KLO))/H
	Y=A*YA(KLO)+B*YA(KHI)+
     *			((A**3-A)*Y2A(KLO)+(B**3-B)*Y2A(KHI))*(H**2)/6.
	RETURN
	END

	SUBROUTINE TSINC(XOLD,YOLD,XNEW,YNEW,NOLD,NNEW)
C
C   sub to interpolate from YOLD to YNEW where
C   XOLD are the original X-values and XNEW the new ones.
C   NNEW is the size of arrays XNEW, YNEW
C   NOLD "	"	"      XOLD, YOLD
C
C   the new value is the weighted sum of the 4 nearest old values
C   the weights depend on a 10-fold partition of XOLD values by XNEW
C   and are a tabulated SINC function.
C
C   the XOLD values must be evenly distributed in x
C
C   altered 10-JUL-81 AJP to test within grid outside loop
C   minor mods to handle ends of arrays, 15 - dec - 81 a.j.p.
C
	DIMENSION XOLD(1),YOLD(1),XNEW(1),YNEW(1)
	DIMENSION V(4,0:9)
	DATA V/
     *	0.0,		1.0,		0.0,		0.0,
     *	-3.470400E-2,	0.9796827,	5.8072016E-2,	-3.0506481E-3,
     *	-5.0847784E-2,	0.9161252,	0.1429679,	-8.2453936E-3,
     *	-5.3494219E-2,	0.8144783,	0.2549558,	-1.5939996E-2,
     *	-4.7525361E-2,	0.6838531,	0.3896306,	-2.5958365E-2,
     *	-3.7226114E-2,	0.5372261,	0.5372261,	-3.7226114E-2,
     *	-2.5958337E-2,	0.3896302,	0.6838535,	-4.7525384E-2,
     *	-1.5939996E-2,	0.2549558,	0.8144783,	-5.3494219E-2,
     *	-8.2453946E-3,	0.1429679,	0.9161253,	-5.0847787E-2,
     *	-3.0506481E-3,	5.8072012E-2,	0.9796826,	-3.4703996E-2/
C
C   EXAMPLE 					RANGES ARE
C	LOW	XCHAN		IX		-.05	.04	 0
C	549	550.0		0		.05	.14	 1
C	549	550.5		5			etc.
C	549	550.54		5		
C	549	550.55		6		
C	549	550.94	        9		
C	550	550.95		0		.84	.94	 9
C
	XSTEP=(XOLD(NOLD)-XOLD(1))/FLOAT(NOLD-1)
	TEST=ABS(XOLD(2)-XOLD(1)-XSTEP)
C	IF(TEST.GT.ABS(0.001*XSTEP)) THEN
C		CALL ERROR(-3,'XOLD unevenly spaced in TSINC')
C	END IF
C
	HIGH=NOLD-1.06
	DO 50 I=1,NNEW
C   find first of 4 channels bracketing XNEW
	XCHAN=(XNEW(I)-XOLD(1))/XSTEP+1.
C   test within grid of old values
	IF(XCHAN.LT.1.96) THEN
           YNEW(I)=YOLD(1)
C		YNEW(I)=0.
		GO TO 50
	ELSE IF(XCHAN.GT.HIGH) THEN
           YNEW(I)=YOLD(NOLD)
C		YNEW(I)=0.
		GOTO 50
	END IF
	LOW=XCHAN-0.95		
C first of 4 old chans bracketing XNEW
	IX=(XCHAN-LOW)*10.0+0.5001	
C 10th part of interval
	IX=MOD(IX,10)
C
	SUM=0.
	DO 40 J=1,4
	LL=LOW+J-1
C	IF(LL.GE.1.AND.LL.LE.NOLD) THEN	! removed 10-JUL-81 AJP
		SUM=SUM+V(J,IX)*YOLD(LL)
c	END IF
40	CONTINUE
	YNEW(I)=SUM
50	CONTINUE
	RETURN
	END
