        SUBROUTINE KAPSIG(VAL,NVAL,ITER,AKAP,CONS,RMS,NPTS,VSQ,ISTAT)
C
C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C.IDENTIFICATION             KAPSIG                   NOV-82
C
C.TYPE           subroutine
C
C.KEYWORDS       mathematics
C
C.PURPOSE        selects a constant mean through a kap*sig clipping
C
C.ALGORITHM      1.) calculate mean and rms
C                2.) delete pixels beyond AKAP*RMS from mean
C                3.) GO TO 1.)
C
C.MODULES CALL   none
C
C.INPUT/OUTPUT   parameters
C
C       -> VAL   input values
C       -> NVAL  number of input values
C       -> ITER  number of iterations (min : 1)
C       -> AKAP  AKAP*RMS
C       <- CONS  derived mean value
C       <- RMS   the rms of CONS
C       <- NPTS  number of points used to derived CONS
C       -> VSQ   work array same dimension as VAL
C       <- ISTAT status of operation
C       :  0  O.K.
C       : -1  NVAL .LT. 2
C
C------------------------------------------------------------------------
C
        DIMENSION VAL(1),VSQ(1)
        REAL*4    CONS,RMS,AKAP
        INTEGER*4 NVAL,NPTS,ISTAT,ITER
C
C.. initialize CONS
        ISTAT = -1
        IF(NVAL.LT.2 ) RETURN
        ISTAT = 0
        CONS = 0.
        DO I=1,NVAL
        CONS = CONS + VAL(I)
        END DO
        CONS = CONS/FLOAT(NVAL)
        CSQ = CONS*CONS
C
C.. initialize RMS
        DELS = 0.
        DO I=1,NVAL
        VSQ(I) = VAL(I)*VAL(I)
        DELV = VSQ(I) + CSQ - 2.*VAL(I)*CONS
        IF(DELV.LT.0.) DELV = 0.
        DELS = DELS + DELV
        END DO
C
        DELA = DELS/FLOAT(NVAL-1)
        ARGU = MAX(1.E-37,DELA)
        RMS  = SQRT(ARGU)
        CLIP = AKAP*RMS
        IF(ITER.LE.0) RETURN
C
C.. iterate
        NPOLD = 0
        DO L=1,ITER
        CON = 0.
        NPTS = 0
        DELS = 0.
         DO I=1,NVAL
           IF( ABS(VAL(I)-CONS) .LT. CLIP) THEN
            NPTS = NPTS + 1
             DELV = VSQ(I) + CSQ - 2.*VAL(I)*CONS
              IF(DELV.LT.0.) DELV = 0.
              DELS = DELS + DELV
             CON = CON + VAL(I)
            NPNEW = NPTS
           END IF
         END DO
C
        IF(NPNEW.LE.2) RETURN
        IF(NPNEW.EQ.NPOLD) RETURN
        NPOLD = NPNEW
C
C..  define new rms and CONS
        DELA = DELS/FLOAT(NPTS-1)
        ARGU = MAX(1.E-37,DELA)
        RMS  = SQRT(ARGU)
        CLIP = AKAP*RMS
        CONS = CON/FLOAT(NPOLD)
        CSQ = CONS*CONS
C
        END DO
C
C..  exit
        RETURN
        END


        SUBROUTINE STARCEN(AIMG,NPIX,NPIY,MET,IMAP,
     +                XOUT,YOUT,XERR,YERR,XSIG,YSIG,ISTAT)
C
C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C.IDENTIFICATION
C subroutine STARCEN            version 2.10    840322
C initially written by H.Waldthausen    ESO - Garching
C cleaned up to be legible by K. Banse
C                                       2.20    851106
C
C.KEYWORDS
C  photometry
C
C.PURPOSE
C calculates the center position of an object
C
C.ALGORITHM
C  according to P.B.Stetson for GAUSS-method and H.Waldthausen for MOM-method
C
C.INPUT/OUTPUT   param
C
C       -> AIMG   input image
C       -> NPIX   x-pixels and
C       -> NPIY   y-pixels of AIMG
C       -> MET    method of centering
C                 MOM   moment centering
C                 GAU   gaussian centering
C       -> IMAP   area with star x-st,x-nd,y-st,y-nd
C       <- XOUT   x- position in pixels
C       <- YOUT   y- position in pixels
C       <- XERR   x- error estimate of XOUT
C       <- YERR   y- error estimate of YOUT
C       <- XSIG   x- with of source
C       <- YSIG   y- with of source
C       <->ISTAT  output/input  status
C                   0 O.K.
C                   1 crowded, or weak source
C                   2 no pos. source found
C                   3 iteration failed
C                 < 0 input error
C                  -1 dimensions larger than 90
C                  -2 MET not MOM or GAU
C
C---------------------------------------------------------------------
C
        CHARACTER*(*)   MET
        REAL*8     KRX(90),KRY(90),SWORK(90)
        DIMENSION
     +  AIMG(NPIX,NPIY),IMAP(4),EDVAL(400),WORK(400)
C
C..  initialize
        IXA = IMAP(1)
        IXE = IMAP(2)
        IYA = IMAP(3)
        IYE = IMAP(4)
        NX = IXE - IXA + 1
        NY = IYE - IYA + 1
C
        ISTAT = 0
        XSIG =  0.
        YSIG =  0.
        XOUT = -1.
        YOUT = -1.
        XERR = 0.
        YERR = 0.
C
C..  *** MOMENT centering
C
        XOLD = -1.
        YOLD = -1.
C
C..  find bgval and rms from edge pixels
        I = 1
        DO IY=IYA,IYE
           IF (IY.EQ.IYA .OR. IY.EQ.IYE) THEN
               DO IX=IXA,IXE
                  EDVAL(I) = AIMG(IX,IY)
                  I = I + 1
               ENDDO
            ELSE
               EDVAL(I) = AIMG(IXA,IY)
               I = I + 1
               EDVAL(I) = AIMG(IXE,IY)
               I = I + 1
            ENDIF
        ENDDO
C
        NVAL = I - 1
        AKAP = 2.
        NTER = 5
        CALL KAPSIG(EDVAL,NVAL,NTER,AKAP,BGVAL,RMS,NPB,WORK,IOK)
C
C.. calculate moment for pixel values > 3*RMS above BGVAL
        CLIP = 3.*RMS
        ITER = 8
C
        DO I=1,ITER               ! iteration loop
           XMOM = 0.
           YMOM = 0.
           SUMI = 0.
           DO IY=IYA,IYE
              DO IX=IXA,IXE
                 SOUR = AIMG(IX,IY) - BGVAL
                 IF ( SOUR .GT. CLIP) THEN
                    SUMI = SUMI + SOUR
                    XMOM = XMOM + FLOAT(IX)*SOUR
                    YMOM = YMOM + FLOAT(IY)*SOUR
                 ENDIF
              ENDDO
           ENDDO
C
C..  check status, crowded or weak source
           IXS = IXE - IXA + 1
           IYS = IYE - IYA + 1
           IF (IXS.LE.2 .OR. IYS.LE.2) THEN
              XSIG = FLOAT(IXS)
              YSIG = FLOAT(IYS)
              ISTAT = 1
              IF (SUMI .GT. 0.0) THEN
                 XOUT = XMOM/SUMI
                 YOUT = YMOM/SUMI
              ELSE
                 XOUT = FLOAT(IXA+IXE)/2.
                 YOUT = FLOAT(IYA+IYE)/2.
                 ISTAT = 2
                 RETURN
              ENDIF
              RETURN
           ENDIF
C
C.. only positive sources assumed
           IF (SUMI .GT. 0.0) THEN
              XOUT = XMOM/SUMI
              YOUT = YMOM/SUMI
              XSIG = FLOAT(IXS)
              YSIG = FLOAT(IYS)
C
C.. calculate sigma and errors if convergence
              IF (XOUT.EQ.XOLD .AND. YOUT.EQ.YOLD) THEN
                 XRMS = 0.
                 YRMS = 0.
                 XERR = 0.
                 YERR = 0.
                 APIX = 0.
                 SUMI = 0.
                 DO IY=IYA,IYE
                    DO IX = IXA,IXE
                       SOUR = AIMG(IX,IY) - BGVAL
                       IF ( SOUR .GT. CLIP ) THEN
                          XDIF = FLOAT(IX) - XOUT
                          YDIF = FLOAT(IY) - YOUT
                          XRMS = XRMS + ABS(SOUR)*XDIF*XDIF
                          YRMS = YRMS + ABS(SOUR)*YDIF*YDIF
                          SUMI = SUMI + SOUR
                          APIX = APIX + 1.
                       ENDIF
                    ENDDO
                 ENDDO
C
                 IXN = XOUT + 0.5
                 IYN = YOUT + 0.5
                 SUMF = SUMI + AIMG(IXN,IYN) - BGVAL
                 XSIG = SQRT(XRMS/SUMF)
                 YSIG = SQRT(YRMS/SUMF)
                 XERR = XSIG/SQRT(APIX-1.)
                 YERR = YSIG/SQRT(APIX-1.)
                 RETURN
              ENDIF
C
              YOLD = YOUT
              XOLD = XOUT
              ISTAT = 0
           ELSE
              XOUT = FLOAT(IXA+IXE)/2.
              YOUT = FLOAT(IYA+IYE)/2.
              ISTAT = 2
              RETURN
           ENDIF
C
C..  crowded  or week source conditions
           LX = XOUT
           LY = YOUT
           SOUR = AIMG(LX,LY) - BGVAL
           IF (SOUR .LE. CLIP) THEN
              XSIG = 0.
              YSIG = 0.
              ISTAT = 1
              RETURN
           ENDIF
C
C.. find extent of source i.e. delete spikes and weak sources
           LXA = IXA
           DO WHILE ( SOUR.GT.CLIP .AND. LX.GE.LXA)
              IXA = LX
              LX = LX - 1
              SOUR = AIMG(LX,LY) - BGVAL
           ENDDO
C
           LY = YOUT
           LX = XOUT
           LXE = IXE
           SOUR = AIMG(LX,LY) - BGVAL
           DO WHILE ( SOUR.GT.CLIP .AND. LX.LE.LXE)
              IXE = LX
              LX = LX + 1
              SOUR = AIMG(LX,LY) - BGVAL
           ENDDO
C
           LY = YOUT
           LX = XOUT
           LYA = IYA
           SOUR = AIMG(LX,LY) - BGVAL
           DO WHILE ( SOUR.GT.CLIP .AND. LY.GE.LYA)
              IYA = LY
              LY = LY - 1
              SOUR = AIMG(LX,LY) - BGVAL
           ENDDO
C
           LY = YOUT
           LX = XOUT
           LYE = IYE
           SOUR = AIMG(LX,LY) - BGVAL
           DO WHILE ( SOUR.GT.CLIP .AND. LY.LE.LYE)
              IYE = LY
              LY = LY + 1
              SOUR = AIMG(LX,LY) - BGVAL
           ENDDO
C
        ENDDO          ! iteration loop
        ISTAT = 3
        RETURN
        END
