.TH XIMAGE 1 "1 Nov 1986" "X Version 10"
.SH NAME
ximage \- X false color image display program
.SH SYNOPSIS
.B ximage
[ image_file ] [ options ]
.SH DESCRIPTION
.I Ximage
is an
.I X
program which displays image data in false color.  It has the ability
to change the color map, zoom and pan around the image, and manipulate cursors.
.I Ximage
responds to mouse commands and uses a button-based user interface.
.I Ximage
will also be accessible through the IRAF environment.
.SH IMAGE FILES

.I Ximage
can display  several types of astronomical data files, including
FITS, Einstein HRI and IPC, SAO CCD, ROSAT HRI and **NOT YET IMPLEMENTED**
IRAF.  In addition, it can display array files of various formats (char,
short int, long int, float and double), with and without a header.
The file name is always the first argument on the command line.
.SH STANDARD OPTIONS
Options follow the filename on the
.I ximage
command line.  Generally, you must include a keyword to specify the file type.
For example, to display a FITS-style file, a "-fits" keyword is
used.  Some of the file type keywords take optional arguments.  For example,
the "-ipc" keyword can accept 0 or 4 optional arguments to specify center
(y,z), blocking factor (zoom) and energy range.  If no file type keyword is
specified, the program looks to see if the size of the file is consistent with
it being a square array of either char or short int format.  If so, it displays
the file as such.  Otherwise, an error is reported.
.PP
Additional keywords may be required under certain
circumstances.  For example, if a non-square array file is being displayed,
you must use the "-nrows" and "-ncols" keywords to specify the array size.
.PP
.I Ximage
understands the following options.
.TP 12
.B \-fits
The specified file is in standard FITS format.  Only 8 and 16 bit integer
array coding is accepted.
.TP
.B \-dfits
Same as above, except that the bytes are reversed from standard order.  This
occurs when the machine (i.e. VAX) uses reverse byte order and the file is
in native (non-standard) order. (Same as
.B \-fits \-byteswap).
.TP
.B \-saoccd
The specified file is in SAO CCD format.
.TP
.BI \-ipc " [y, z, zoom, energy]"
The specified file is in Einstein IPC format.  This flag accepts 0 or 4
arguments.  The optional arguments are center y, center z, blocking
factor (zoom) and energy.  The energy is input as an octal number.
.TP
.BI \-hri " [y, z, zoom]"
The specified file is in Einstein HRI format.  This flag accepts 0 or 3
arguments.  The optional arguments are center y, center z, and zoom.
.TP
.BI \-ros " [y, z, zoom]"
The specified file is in ROSAT HRI format.  This flag accepts 0 or 3
arguments.  The optional arguments are center y, center z, and zoom.
.TP
.BI \-arr " type"
The specified file is an array.
The data type must be specified as an argument.  Valid types are:
.B i1, char, i2, short, i4, long,
.B r4, float, r8, double.
With arrays, it is also necessary to specify the number of rows and columns
in the data unless the array is perfectly square. In this case,the
dimension can
be calculated.  An array which is square and which is either in
.B i1
or
.B i2
format does not require the
.B \-arr
switch.
.TP
.BI \-filecols " columns  " \-filerows " rows"
Specifies the number of columns and rows in the data file, for array files.
.TP
.BI \-fc " cols  " \-fr " rows"
Same as above.
.TP
.BI \-header " bytes"
Specifies the number of header bytes to skip, for array files.
.TP
.BI \-sk " bytes"
Same as above.
.TP
.BI \-skip " bytes"
Same as above.
.TP
.B \-byteswap
Swap bytes before scaling the data.  This might be required in order to
display data created on a different type of CPU, e.g., to display VAX
data on the Sun.
.TP
.B \-bswap
Same as above.
.TP
.BI \-dispcols " columns  " \-disprows " rows"
Specifies the size of the main display window.
For each dimension the default is the image dimension or 512,
whichever is smaller.
.TP
.BI \-dc " cols  " \-dr " rows"
Same as above.
.TP
.BI \-dispX " column  " \-dispY " row"
Specifies the file coordinates of the left corner of the initial display.
The default centers the initial display within the image field.
.TP
.BI \-dx " column  " \-dy " row"
Same as above.
.TP
.BI \-dispXcen " column  " \-dispYcen " row"
Specifies the file coordinates of the center of the initial display.
.TP
.BI \-dxc " column  " \-dyc " row"
Same as above.
.TP
.BI \-dispexpand " integer"
Expand factor for initial display.  Positive values cause pixel replication,
negative values cause sub-sampling from the image buffer (or, for large files,
whatever equivalent the image reading library for that file type provides).
.TP
.BI \-de " integer"
Same as above.
.TP
.BI \-readcols " columns  " \-readrows " rows"
Restricts the number of columns and rows for data file access. The
.B \-read
params define a restricted area within the image file.  The restricted
area affects the contents of the "pan window" and the ability to pan
and zoom.  It is useful for very large image files.  If
.B \-readX, \-readY
or
.B \-readXcen, \-readYcen
are not given, the restricted area defaults to being centered in
the image file.
.TP
.BI \-rc " columns  " \-rr " rows"
Same as above.
.TP
.BI \-readX " column  " \-readY " row"
Specifies the left corner of the ristricted area within the file.
.TP
.BI \-rx " column  " \-ry " row"
Same as above.
.TP
.BI \-readXcen " column  " \-readYcen " row"
Specifies the center of the ristricted area within the file.
.TP
.BI \-rxc " column  " \-ryc " row"
Same as above.
.TP
.BI \-readsample " positive-integer"
Minimum sample rate for reading from the image file.  The default allows
reading every pixel (1).  Specifying a higher sampling rate can save
time-consuming re-reading of large image files when the user intends
to pan the display many times.
.B Ximage
generally attempts to load its image buffer with the highest-resolution
(minimum sampling rate) data that can cover the desired display field.
.TP
.BI \-rs " positive-integer"
Same as above.
.TP
.BI \-planes  " planes"
Sets the maximum number of planes to use for the image on color machines.
The actual number of planes used depends on availability.  When only one
plane is used, ximage goes into its halftone bitmap mode.
.TP
.BI \-p  " planes"
Same as above.
.TP
.B \-asis
No rescaling is performed.  Values are used as read and may exceed the
allowable color values for display. (see
.B Scale
menu).
.TP
.B \-linear
Linear scaling is performed in order to scale image data between the
minimum and maximum allowable color values for display. (see
.B Scale
menu).
.TP
.B \-sqrt
Square root scaling is performed in order to scale image data between the
minimum and maximum allowable color range value for display. (see
.B Scale
menu).
.TP
.B \-log
Log scaling is performed in order to scale image data between the
minimum and maximum allowable color range value for display.  *NOT YET
IMPLEMENTED* (see
.B Scale
menu).
.TP
.B \-histeq
Histogram equalization scaling is performed to equalize displayed pixels
mapped at each color level (maximize visible image detail).  (see
.B Scale
menu).
.TP
.BI \-threshold " integer  " \-saturate " integer"
Sets image pixel value limits for scaling of data to the color map range.
Should not be used with
.B histeq
scaling.
.TP
.B \-thr " integer  " \-sat " integer"
Same as above.
.TP
.B \-neg
Puts the color map in inverse mode (see
.B Color
menu).
.TP
.B \-n
Same as above.
.TP
.B \-rgb
Sets the color mode to be
.B r|g|b,
instead of the default
.B r&g&b
(see
.B Color
menu).
.TP
.BI \-bw " borderwidth"
Allows you to specify the width of the main image window border in pixels.
.SH THE DISPLAY
The
.I ximage
display consists of 5 independent windows.  Each one is separately resizable
and can be moved around like any X window.  (They can all be brought
back into a cluster - see the
.B group
command below).
.SH \ MAIN IMAGE WINDOW
The main (and largest) window
displays the image data at the currently selected blocking factor.
In this window one can use the mouse to change the color map, pan and zoom
the image, move and save cursors,
and display the value of patches of data. The mouse actions depend on which
button menu is "active".  The mouse icon in the main window changes according
to its action.  See below for an explanation of the  menus, and
button actions within the main window.
.SH \ BUTTON PANEL
To the left of the main display window is a button panel with two sections.
The upper section consists of a fixed group of menu choices.  The lower
section holds a varying group of sub-menu command choices.  The sub-menu
choices change
when the the mouse is clicked on one of the menu choices.
Menu and associated sub-menu choices are detailed below.

.SH \ PAN WINDOW
Above the button panel is the pan window.  This consists of a reduced display
of the entire image.  The part of the image currently being displayed is
outlined by a green rectangle.  The rectangle can be moved and resized with
the mouse, causing the image in the main window to pan and zoom.
Use the left button to move the display rectangle around the image. The mouse
position defines the center.
Use the middle
mouse button to resize the display rectangle about its center. This will
cause the region of display to "zoom" in (using pixel replication) or
out (using pixel sampling).  The mouse position defines the edge of the
zoomed region.  In either case, the image is not redrawn until the button
is released.  Moving the mouse out of the window while still holding the
button down cancels the selection.
The right mouse button displays a table of an 11x11 patch of the image
data centered around the current mouse position.
.PP
For large images,
.B ximage
may not be able to store the entire image in memory.  In this case, a second
box, normally yellow, appears in the pan window.  This box indicates the
portion of the image currently in memory.  When making a pan or zoom
selection, the color of this second box changes to red to indicate that your
selection will require a new section of the file to be read into memory.
The right button patch table does not work outside of this region.
When an image restriction has been selected on the command line, the pan
window displays only the the restricted field.
.SH \ ZOOM-SCOPE WINDOW
To the right of the main display window, at the top, is the zoom-scope
window.  This window is activated when the
.B track
toggle button is "on", or when the shift key or shift lock is pressed.
(The shift keys reverse the effect of the
.B track
button).  When activated, the window displays a blow-up of the image,
centered on the mouse. The zoomed image
is redrawn as the mouse moves across the main display.
The zoom-scope can be frozen (or activated) temporarily with the shift key.
While useful for fine adjustment of postion, the zoom function slows cursor
control interactions.  A pop-up menu controling the blow-up factor can
be activated by holding both the shift and control keys, while pressing
a mouse button in the zoom-scope window.
.SH \ COLOR MAP WINDOW
Under the zoom box is the color map window, which displays the relative
intensity of red, green, and blue vs. levels in the color map.  The color
map is made from a table.  Each entry in the table is a "break-point" with
a specific level and intensity.  The map is interpolated between these
"break-points."  "Break-points" are represented by little squares in the
graph.  The correspondence between image values and color map
levels depends on the scaling (see
.B Scaling
menu below). When the mouse is
used in the main window to change the color map (see the
.B Color
menu below), the graph changes to reflect the new color map.  Normally this
is done only when a button is released, but the color graph
can be made to track continually with the
.B track
sub-menu button, or
by pressing the shift or shift lock key.  This scheme is similar to that of
zoom-scope window tracking.
.PP
The color graph can be directly altered in the color map window.
The colors of the image display change accordingly. The red, green, and
blue graphs can be changed using the left, middle, or right mouse buttons,
respectively.  Pressing a mouse button on a "break-point" (identified by small
boxes on the graph) grabs the graph, which can then be stretched or moved as
desired.  Pressing the mouse anywhere else creates a
new break-point, which can be stretched, etc. Pressing the control key along with a
mouse button on a break-point will delete that break-point.  The appropriate
mouse button must be used when stretching each graph color.
.SH MENUS AND COMMANDS
The button panel is the main user interface to
.I ximage.
It is implemented as a two-layer menu/command interface.  At the top of the
panel are fixed menu choices.  The currently activated menu choice is
highlighted.  The commands for the active menu are displayed in the
sub-menu or command panel under the menu panel.
.PP
As the mouse moves through the button choices, the current button is drawn
with a double border.  If the mouse is clicked on a menu button, the previous
menu choice becomes de-activated and the current choice becomes activated, and
highlighted.  The newly activated sub-menu command choices are displayed.
If the mouse is clicked on a command button, the command is executed.
.PP
The menus and associated commands are detailed below.  Some menus
change the action of the mouse within the main display window.  This is
noted below, where applicable.
.SH \ COLOR MENU (for color monitors)
The
.B Color
menu differs depending on whether a color monitor or bitmapped monitor
is being used.  With a color monitor, the
.B Color
menu has two parts, the controls menu and the map menu.  To get from the
controls to the map menu, press the
.B CMAP
button at the bottom.  To get from the map menu to the controls menu,
press the
.B color
button in the lower right corner.
.PP
In
.B Color
mode, the three mouse  buttons can be used within the main window
to manipulate the color map, according to which command modes are
activated.  In
.B r&g&b
mode, all three buttons have the identical effect of changing the red and green
and blue colors simultaneously.  In
.B r|g|b
mode, the red or green or blue color maps can be changed individually by
pressing the left, middle, or right mouse button and moving the mouse across
the main window.  More than one button can be pressed at the same time
when moving the mouse.
.PP
How the color map changes as the mouse moves (and buttons are pressed)
is determined by the next three buttons.  In the
.B threshold/saturation
mode, (the button with up and down arrows)
moving the mouse left and right shifts the threshold or low end of the
graph, while moving the mouse up and down shifts the saturation or high
end of the graph.  If saturation drops below threshold, the intensities are
inverted (not the graph).  In
.B contrast/bias
mode (the button with four arrows) moving the mouse left
and right changes the slope of the graph from the middle, while moving
the mouse up and down shifts the entire graph.  In power mode, moving
the mouse left and right remaps the intensities as x**(1/g) (the g, or
small gamma,
values are shown above the graph).  Up and down mouse movement has no
effect in power mode.  In the first two modes, the default position is
the center of the main window.  In power mode, the x**1 position is at
the left edge.
.PP
Finally, the
.B invert
toggle inverts the color intensities.
.PP
In the
.B CMAP
menu, the color map may be initialed to any of five choices:
.B Grey-scale,
.B HEAO
(default Xray image color map),
.B IMP8
(used by AIPS), and two arbitrary color maps, designated A and B.
Pressing any of these one-shot buttons sets the color map according to the
predefined color scheme associated with the button.  The color map can then
be further manipulated according to the selection of the color control menu.
.PP
The **NOT YET IMPLEMENTED**
.B read
and
.B write
buttons allow one to write out the color map and read in a color map from
a disc file.
.SH \ COLOR MENU (for bitmapped monitors)
With a monochrome bitmapped monitor, the
.B Color
menu the method of drawing the bitmap.  In
.B dither,
a rigid, repeating 8x8x2 weighting pattern, or matrix, is used.  Two matrices
are available.
.B Matrix1
maximizes diffusion while minizing patterns, allowing for some clumping.
.B Matrix2
simply maximizes diffusion, allowing obvious patterns in areas of similar
values and an apparent change in contrast due to size differences between
black dots and white dots.  The
.B diffuse
button draws the bitmap using a method called error diffusion.  By default,
the bitmaps are displayed as black dots on a white background.
.B Invert causes the display to be white dots on a black background.
Cursors do not track cleanly in inverted displays.
.SH \ SCALE MENU
This menu determines how the data is scaled when transforming image data values
to color map levels for display.  It does not effect values shown in the patch
print table (see
.B pan
menu).
.B Direct
scaling performs no translation.  The image values are assumed to be
directly mappable (0 to maximum color map level).
.B Linear
scaling scales the data linearly so as to lie just within the
available dynamic range of the color map,
.B Sqrt
or square root scaling scales the data non-linearly, favoring values
closer to the minimum, to fit within the range of the color map.
.B Histeq
or histogram equalization scaling which maximizes the information by
equalizing the number of pixels mapped at each color level.
.B Log
scaling, *NOT YET IMPLEMENTED* uses the log of data values for scaling
to fit the range of color map levels.
.B Hist
or histogram *NOT YET IMPLEMENTED* is similar to histogram equalization
but allows the user to control the allocation of color map levels to
ranges of image values.
Scaling calculations are performed on the image as displayed in the main
window at the time the calculation is made.  Features in the file, but not
in the current display, are excluded from the calculation.
.SH \ PAN MENU
The
.B Pan
menu provides two related pan/zoom facilities.
The
.B x1/4,
.B x1/2,
.B x2,
and
.B x4
buttons are one-shot commands which change the image
zoom about the central pixel by the factor displayed on each button.
The factors refer to the change in linear dimension (x2 will make a
feature twice as wide and twice as high).
.I Ximage
uses pixel sampling to make a smaller image and pixel replication
to blow the image up beyond the original blocking factor.
.B Zoom 1
displays the image at the original blocking factor.
.PP
Panning and zooming can also be accomplished by clicking the the mouse
buttons in the
main window.  In this mode, mouse control in the
.B main window
works the same as mouse control in the
.B pan window
(see 
.B pan
window, above).
.SH \ CURSOR MENU
The 
.B Cursor menu has two parts, cursor control and region control.  Ximage
allows the use of cursors to define regions for later analysis applications.
These regions are kept in list.
The
.B Cursor
menu allows you to define one of four cursors:
.B box, rotated box, circle,
and *NOT YET IMPLEMENTED*
.B ellipse.  In addition, *NOT YET IMPLEMENTED* a pie slice of one of the
above cursors can be defined.
Once chosen, a cursor can be moved around or resized with the mouse.  The left
mouse button centers the cursor on the current mouse position. The middle button
keeps the center fixed and changes the size of the cursor to reflect both
the X and Y offsets to the mouse.
The right button rotates the axes of the 
.B rot box
and
.B ellipse
and is a no-op for the other cursors.
The current size, shape and position of the cursor is displayed when the
mouse button is released.
.B None
erases the current cursor.
.SH REGION MENU
The
.B Region
menu allows you to save cursors in a list for use as region descriptors,
write the list to disk, read a list from disk, edit the list, and restore
a saved cursor from the list as the current cursor,  Up to 32 cursors or
regions can be saved in the same list.
.PP
The
.B reset
command clears the saved region list. The
.B view
toggle button selects whether or not to display currently saved
regions.  The
.B label
toggle adds text labels to the display which identify region centers,
and the height (for box and rot box) or radius (for circle).
.PP
The
.B incr
toggle automates making uniformly incremented concentric regions
When
.B incr
is "on", the cursor size is incremented by the difference
between the size of the two previously saved cursors (or twice the original
size, if only one cursor has been saved).
.B Incr
can be used, for
example, to save concentric circle cursors with constantly increasing radii.
.PP
The
.B read
and
.B write
modes allow you to read and write saved regions to disk.
In both cases, you are prompted for a file name.   When writing to disk,
if the specified file already exists, you are prompted to choose
appending to the existing file, overwriting the old file, or quitting the
command.  Reading a saved file always appends the contents to the currently
stored region list.
.PP
The
.B cycle
command cycles backwards through the region list, making the current cursor
the same size, shape and position as each of the saved cursors in turn.  You
can recycle back to a desired cursor, and then go to 
.B Cursor
mode to install that particular cursor as the current one.
.PP
To save cursors in the saved region list, you must be in
.B Cursor
mode. Holding the control key and pressing the left
mouse button saves the cursor as an "include" region.  Include regions are
displayed in yellow. Holding the control key and pressing the middle mouse
button to saves the cursor as an "exclude" region.  Exclude regions are
displayed in red.   Holding the control key and pressing the right
mouse button deletes a saved region.  The region deleted is the one
with the smallest area, of all regions containing the mouse cursor icon.
By carefully positioning the mouse cursor icon, any desired region can
be deleted.
.PP
The saved region file is an ASCII file.  It contains one line for each
saved region, detailing whether the region is an include or exclude
region, the shape and center, and the radius, length, width and/or rotation
angle, as necessary.
.SH ENV MENU
The
.B ENV
(environment) menu contains some general commands. The
.B group
command will group all five windows in a cluster, in the event that you
had moved one or more window. The
.B track
toggle controls tracking of the zoom box and color graph as the mouse is
moved in the main window (and in the color graph window).  The shift and
shift lock keys have the same control (and are exclusive or'd with
.B track).
The
.B raise
command raises
.I ximage
to the top of a window stack if it has been obscured by other windows (as
often happens with "auto-raising" xterms).  Typing the "a" key with the
mouse in a visible part of any of the five windows has the same effect.
.PP
The
.B exit
command exits from
.I ximage.
.SH BUGS
.I Ximage
cannot  display images on a monochrome monitor.  The rle algorithm
causes the program to crash if the rle file has less than 3 colors.
The display of coordinates and the writing of coordinates to region files
do not convert correctly between float and int coordinates.  The pan box
does not immediately represent the correct display area when the main
window is resized.  Centering of the panned image is limited by the edge
of the stored image.  The reported cursor rotation angle is off by pi/2
from standard astronomical conventions.  All of these and more
will be fixed in later revisions.
.SH AUTHORS
William Wyatt (SAO-OIR) wrote the original
.I ximage
as
.I showimg,
implementing the
main display, some color map manipulation, and the zoom box.
Mike VanHilst (SAO-HEAD) and Eric Mandel (SAO-HEAD) redesigned the program
(daring to call it
.I ximage),
adding pan box, color graphs, cursors,
and button interface.  MVH implemented nearly
all of this second sketchy design, generalizing the button interface so as
to be generally usable.  EGM is identified with the saved region code.




