# FDISP --- display folded spectrum image using saoimage
# By Doug Mink, Center for Astrophysics
# October 30, 1995

procedure fdisp (image)

file image=""		{prompt="image to fold and display"}

file imdir=""		{prompt="image directory for input image"}
int nfold=1		{prompt="Number of segments into which to fold image"}
file fdir=""		{prompt="image directory for folded image"}
char display=""		{prompt="Output X display"}
char geometry=""	{prompt="Image window geometry"}
char scaling="histeq"	{prompt="Image scaling (histeq or lfrac .01"}
real floor=0		{prompt="Minimum value to be plotted (0 for min)"}
real ceiling=0		{prompt="Maximum value to be plotted (0 for max)"}
int hbytes=0		{prompt="Number of bytes to skip before image"}
int nrows=0		{prompt="Number of row (0 if from header)"}
int ncols=0		{prompt="Number of columns (0 if from header)"}
char datatype=""	{prompt="Input data type"}
bool delfold=yes	{prompt="Delete folded image (yes or no)"}
bool verbose=yes	{prompt="Log progress (yes or no)"}
bool background=yes	{prompt="Run ximage in background (yes or no)"}

begin
    file imf, fimage, imin, imout
    char arg, imt, dt
    int nc, nr, min, max
    char imtype		# Image file type (imh,fits,dfits)

    imtype="imh"
    dt = datatype
    nc = ncols
    nr = nrows
    min = floor
    max = ceiling
    delf = delfold
    imin = image
    imout = imin // ".fold"

    ifold.indir = imdir
    ifold.imout = imout
    ifold.outdir = fdir
    ifold.nfold = nfold
    ifold.verbose = verbose
    ifold (imin)

    # Display for X
    if (display != "")
	arg = " -d " // display
    else
	arg = ""

    # Geometry for X (X*Y+XDIM+YDIM)
    if (geometry != "")
	arg = arg // " -gd " // geometry

# Floor and ceiling should be set of one is different from zero

    if (min != 0 || max != 0) {
	arg = arg // " -min " // str(min)
	arg = arg // " -max " // str(max)
	}

# Otherwise, set limits at fractions of histogram maximum
    else {
	arg = arg // " -" // scaling
	}

    if (hbytes > 0) {
	arg = arg // " -sk " // str(hbytes)
	}

# Set limits in image of displayed section

    if (nc > 0 && nr > 0) {
	if (dt == "i" || dt == "l")
	    arg = arg // " -i4 " // str(nc) // " " // str(nr)
	else if (dt == "s")
	    arg = arg // " -i2 " // str(nc) // " " // str(nr)
	else if (dt == "r")
	    arg = arg // " -r4 " // str(nc) // " " // str(nr)
	else if (dt == "d")
	    arg = arg // " -r8 " // str(nc) // " " // str(nr)
	else if (dt == "c")
	    arg = arg // " -i1 " // str(nc) // " " // str(nr)
	}

    # Image name and format
    imt = imtype
    if (fdir != "")
	imf = osfn (fdir) // imout
    else
	imf = osfn (imout)
    if (imtype == "imh")
	arg = arg // " - " // imf // ".imh"
    else
	arg = arg // " -" // imt // " - " // imf

    # Run in background if requested
    if (background)
      arg = arg // " &"

    # Execute SAOimage
    print ("saoimage",arg)
    saoimage (arg)

    if (delf)
	imdel (imf)

end
