include	<imhdr.h>

task mkv1 = t_mkv1

define	V1_VERSION	1

# MKV1 -- Make a V1 IRAF image from a V2 IRAF image.  The conversion is
# done in-place.  This should be harmless since IRAF V2.11 can handle either
# format image.  The only requirement is that this program be compiled in
# a V2.11 system and linked with '-z' to avoid IRAF shared library conflicts
# when the binary is run under a V2.10.4 system. Input may be a list of images.
#
#****************************************************************************
#  WARNING  WARNING  WARNING  WARNING  WARNING  WARNING  WARNING  WARNING  
#****************************************************************************
# NOTE:  The V1 image format used prior to V2.11 has a limit of ~63 chars on
# 	 i_pixfile pathname (include the node! prefix).  This task will trun-
# 	 cate the pathname if that limit is exceeded, rendering the image an
#	 orphan.  V2 images should be copied to a directory higher up in the
#	 tree prior to running this program if this will be a problem.
#****************************************************************************
#  WARNING  WARNING  WARNING  WARNING  WARNING  WARNING  WARNING  WARNING  
#****************************************************************************
#
# Program should be compiled and declared as
#
#	cl> xc -z t_mkv1.x -lxtools		# compile the task
#	cl> task mkv1 = /<path>/x_mkv1.e	# declare the task


procedure t_mkv1()

char	imtlist[SZ_LINE]			# Input image list
bool	verbose					# Print operations?

char	image1[SZ_PATHNAME]			# Input image name
int	list

int	imtopen(), imtgetim()
bool	clgetb()

begin
	# Get input and output image template lists.

	call clgstr ("input", imtlist, SZ_LINE)
	verbose = clgetb ("verbose")

	# Expand the input and output image lists.
	list = imtopen (imtlist)

	# Do each set of input/output images.

	while ((imtgetim (list, image1, SZ_PATHNAME) != EOF)) {
	    call img_imcopy (image1, image1, verbose)
	}

	call imtclose (list)
end


# IMG_IMCOPY -- Copy an image.  Use sequential routines to permit copying
# images of any dimension.  Perform pixel i/o in the datatype of the image,
# to avoid unnecessary type conversion.

procedure img_imcopy (image1, image2, verbose)

char	image1[ARB]			# Input image
char	image2[ARB]			# Output image
bool	verbose				# Print the operation

int	npix, junk
pointer	buf1, buf2, im1, im2
pointer	sp, imtemp
long	v1[IM_MAXDIM], v2[IM_MAXDIM]

int	imgnls(), imgnll(), imgnlr(), imgnld(), imgnlx()
int	impnls(), impnll(), impnlr(), impnld(), impnlx()
pointer	immap()

begin
	call smark (sp)
	call salloc (imtemp, SZ_PATHNAME, TY_CHAR)

	# If verbose print the operation.
	if (verbose) {
	    call printf ("%s -> %s\n")
		call pargstr (image1)
		call pargstr (image2)
	    call flush (STDOUT)
	}

	# Map the input image.
	im1 = immap (image1, READ_ONLY, 0)

	# Copy the input image to the temporary output image and unmap
	# the images.  Release the temporary image name.

	call xt_mkimtemp (image1, image2, Memc[imtemp], SZ_PATHNAME)
	im2 = immap (image2, NEW_COPY, im1)
	IM_HDRVER(im2) = V1_VERSION

	# Setup start vector for sequential reads and writes.
	call amovkl (long(1), v1, IM_MAXDIM)
	call amovkl (long(1), v2, IM_MAXDIM)

	# Copy the image.
	npix = IM_LEN(im1, 1)
	switch (IM_PIXTYPE(im1)) {
	case TY_SHORT:
	    while (imgnls (im1, buf1, v1) != EOF) {
		junk = impnls (im2, buf2, v2)
		call amovs (Mems[buf1], Mems[buf2], npix)
	    }
	case TY_USHORT, TY_INT, TY_LONG:
	    while (imgnll (im1, buf1, v1) != EOF) {
		junk = impnll (im2, buf2, v2)
		call amovl (Meml[buf1], Meml[buf2], npix)
	    }
	case TY_REAL:
	    while (imgnlr (im1, buf1, v1) != EOF) {
		junk = impnlr (im2, buf2, v2)
		call amovr (Memr[buf1], Memr[buf2], npix)
	    }
	case TY_DOUBLE:
	    while (imgnld (im1, buf1, v1) != EOF) {
		junk = impnld (im2, buf2, v2)
		call amovd (Memd[buf1], Memd[buf2], npix)
	    }
	case TY_COMPLEX:
	    while (imgnlx (im1, buf1, v1) != EOF) {
	        junk = impnlx (im2, buf2, v2)
		call amovx (Memx[buf1], Memx[buf2], npix)
	    }
	default:
	    call error (1, "unknown pixel datatype")
	}

	# Unmap the images.

	call imunmap (im2)
	call imunmap (im1)
	call xt_delimtemp (image2, Memc[imtemp])
end
