# File itools/Src/t_cmask.x
# December 16, 1993
# By Doug Mink and Nelson Caldwell

# mask code, takes one mask to another, expands by one pixel

include <imhdr.h>

procedure t_cmask

int	clgeti(), imask
real	clgetr(), rmask
bool	clgetb(), verbose

int	ncols,nrows
int	mcols,mrows
int	ocols,orows
int	wone
pointer sp
pointer	infile
pointer	outfile
pointer	maskfile
pointer	imi
pointer	imm
pointer	imo
pointer	indata
pointer	maskdata
pointer	outdata
pointer	imgs2r(), imgs2i()
pointer	imps2r(), imps2i()
pointer	immap()
char    hstring[SZ_LINE]
char    dstring[SZ_LINE]
long    ltime, clktime()

begin
	call smark (sp)
	call salloc (infile, SZ_FNAME, TY_CHAR)
	call salloc (outfile, SZ_FNAME, TY_CHAR)
	call salloc (maskfile, SZ_FNAME, TY_CHAR)
	verbose = clgetb ("verbose")

#  Open input image
	call clgstr ("infile", Memc[infile], SZ_FNAME)
	imi = immap (Memc[infile], READ_WRITE, 0)
	wone = 1
	ncols = IM_LEN(imi,1)
	nrows = IM_LEN(imi,2)
	if (verbose) {
	    call printf ("CMASK: input file %s: %s %dx%d\n")
		call pargstr (Memc[infile])
		call pargstr (IM_TITLE(imi))
		call pargi (ncols)
		call pargi (nrows)
	    call flush (STDOUT)
	    }

# Open output image
	call clgstr ("outfile", Memc[outfile], SZ_FNAME)
	imo = immap (memc[outfile], NEW_COPY, imi)
	ocols = IM_LEN(imo,1)
	orows = IM_LEN(imo,2)
	if (verbose) {
	    call printf ("CMASK: output file %s: %s %dx%d\n")
		call pargstr (Memc[outfile])
		call pargstr (IM_TITLE(imo))
		call pargi (ocols)
		call pargi (orows)
	    call flush (STDOUT)
	    }

# Read input image and allocate output image space
	switch (IM_PIXTYPE(imi)) {

	# Integer data
	case TY_SHORT, TY_USHORT, TY_INT, TY_LONG:
	    indata = imgs2i (imi,wone,ncols,wone,nrows)
	    outdata = imps2i (imo,wone,ncols,wone,nrows)
	    if (verbose) {
		call printf ("CMASK: %dx%d integer output array allocated\n")
		    call pargi (ncols)
		    call pargi (nrows)
		call flush (STDOUT)
		}

	# Floating point data
	default:
	    indata = imgs2r (imi,wone,ncols,wone,nrows)
	    outdata = imps2r (imo,wone,ncols,wone,nrows)
	    if (verbose) {
		call printf ("CMASK: %dx%d real output array allocated\n")
		    call pargi (ncols)
		    call pargi (nrows)
		call flush (STDOUT)
		}
	}

# Open mask image
	call clgstr ("maskfile", Memc[maskfile], SZ_FNAME)
	imm = immap (Memc[maskfile], READ_WRITE, 0)
	mcols = IM_LEN(imm,1)
	mrows = IM_LEN(imm,2)
	if (verbose) {
	    call printf ("CMASK: mask file %s: %s %dx%d\n")
		call pargstr (Memc[maskfile])
		call pargstr (IM_TITLE(imm))
		call pargi (mcols)
		call pargi (mrows)
	    call flush (STDOUT)
	    }

#  Put date into HISTORY
	ltime = 0
	ltime = clktime (ltime)
	call cnvtime (ltime, dstring, SZ_LINE)

# Read mask image and transfer masked areas and input image
	switch (IM_PIXTYPE(imi)) {

	# Integer data
	case TY_SHORT, TY_USHORT, TY_INT, TY_LONG:
	    maskdata = imgs2i (imm,wone,ncols,wone,nrows)
	    if (verbose) {
		call printf ("CMASK: mask data read\n")
		call flush (STDOUT)
		}
	    imask = clgeti ("ivmask")
	    call imgipar (imm,"BLANK",imask)
	    if (verbose) {
		call printf ("CMASK: mask value is %d\n")
		    call pargr (imask)
		call flush (STDOUT)
		}
	    call mediani (Memi[indata],Memi[maskdata],Memi[outdata],ncols,nrows,imask,verbose)
	    call imaddi (imo,"BLANK",imask)
	    call sprintf (hstring,SZ_LINE,"Cosmic rays set to %d by IRAF saotdc.cmask %s")
		call pargi (imask)
	    	call pargstr (dstring)
	    call imputh (imo, "HISTORY", hstring)
	    call imflsi (imo)

	# Floating point data
	default:
	    maskdata = imgs2r (imm,wone,ncols,wone,nrows)
	    if (verbose) {
		call printf ("CMASK: mask data read\n")
		call flush (STDOUT)
		}
	    rmask = clgetr ("vmask")
	    call imgrpar (imm,"BLANK",rmask)
	    if (verbose) {
		call printf ("CMASK: mask value is %g\n")
		    call pargr (rmask)
		call flush (STDOUT)
		}
	    call medianr (Memr[indata],Memr[maskdata],Memr[outdata],ncols,nrows,rmask,verbose)
	    call imaddr (imo, "BLANK", rmask)
	    call sprintf (hstring,SZ_LINE,"Cosmic rays set to %g by IRAF saotdc.cmask %s")
		call pargr (rmask)
	    	call pargstr (dstring)
	    call imputh (imo, "HISTORY", hstring)
	    call imflsr (imo)
	}
	if (verbose) {
	    call printf ("CMASK: %s\n")
		call pargstr (hstring)
	    call flush (STDOUT)
	    }

	call imunmap (imi)
	call imunmap (imm)
	call imunmap (imo)
	call sfree (sp)
end


procedure medianr (iar,mar,oar,ncols,nrows,rmask,verbose)

real	iar[ncols,nrows]
real	mar[ncols,nrows]
real	oar[ncols,nrows]
int	ncols,nrows
real	rmask
bool	verbose

int	i,j,im1,jm1,ip1,jp1

begin
	do i = 1, nrows {
	    do j = 1, ncols {
		if (mar[j,i] == rmask) {
        	    oar[j,i] = rmask
        	    jm1 = max0 (1,j-1)
        	    im1 = max0 (1,i-1)
        	    jp1 = min0 (ncols,j+1)
        	    ip1 = min0 (nrows,i+1)
        	    oar[jm1,im1] = rmask
        	    oar[jm1,i] = rmask
        	    oar[jm1,ip1] = rmask
        	    oar[j,im1] = rmask
        	    oar[j,ip1] = rmask
        	    oar[jp1,im1] = rmask
        	    oar[jp1,i] = rmask
        	    oar[jp1,ip1] = rmask
		    }
		else
        	    oar[j,i] = iar[j,i]
		}
#	    if (verbose) {
#		call printf ("CMASK: median done for row %d/%d\n")
#		    call pargi (i)
#		    call pargi (nrows)
#		call flush (STDOUT)
#		}
	    }
	return
end

procedure mediani (iar,mar,oar,ncols,nrows,imask,verbose)

int	iar[ncols,nrows]
int	mar[ncols,nrows]
int	oar[ncols,nrows]
int	ncols,nrows
int	imask
bool	verbose

int	i,j,im1,jm1,ip1,jp1

begin
	do i = 1, nrows {
	    do j = 1, ncols {
		if (mar[j,i] == imask) {
        	    oar[j,i] = imask
        	    jm1 = max0 (1,j-1)
        	    im1 = max0 (1,i-1)
        	    jp1 = min0 (ncols,j+1)
        	    ip1 = min0 (nrows,i+1)
        	    oar[jm1,im1] = imask
        	    oar[jm1,i] = imask
        	    iar[jm1,ip1] = imask
        	    oar[j,im1] = imask
        	    oar[j,ip1] = imask
        	    oar[jp1,im1] = imask
        	    oar[jp1,i] = imask
        	    oar[jp1,ip1] = imask
		    }
		else
        	    oar[j,i] = iar[j,i]
		}
#	    if (verbose) {
#		call printf ("CMASK: median done for row %d/%d\n")
#		    call pargi (i)
#		    call pargi (nrows)
#		call flush (STDOUT)
#		}
	    }
	return
end
