#*** File itools/Src/t_ifold.x
#*** October 30, 1995
#*** By Doug Mink

include	<mach.h>
include	<imhdr.h>
include	<gset.h>

define	MAX_FOLD 10

# IFOLD -- Fold a two-dimensional spectrum image

procedure t_ifold()

int	i, ncol, nrow, npix
int	ifold, nfold, ncfold, nrfold, dispaxis, ip, x2, y2, step
pointer imi, imo, sp, inbuf, outbuf, imin, imout, inpath, outpath
char    hstring[SZ_LINE]
char    dstring[SZ_LINE]
char	imsect[SZ_LINE]
int	nlen1,nlen2,nplen1,nplen2, lpath
int	xin1,xin2,yin1,yin2,xout1,xout2,yout1,yout2
bool	verbose, clgetb()

int	clgeti(), strlen()
int	imgs2r(), imgs2i()
int	imps2r(), imps2i()
pointer	immap()

begin
	call smark (sp)
	call salloc (imin,   SZ_FNAME,  TY_CHAR)
	call salloc (inpath,   SZ_PATHNAME,  TY_CHAR)
	call salloc (imout,   SZ_FNAME,  TY_CHAR)
	call salloc (outpath,   SZ_PATHNAME,  TY_CHAR)
	verbose = clgetb ("verbose")

	# Get the image input name.
	call clgstr ("imin", Memc[imin], SZ_LINE)
	call clgstr ("indir", Memc[inpath], SZ_LINE)
	lpath = strlen (Memc[inpath])
	if (lpath > 0) {
	    if (Memc[inpath+lpath-1] != '/') {
		Memc[inpath+lpath] = '/'
		Memc[inpath+lpath+1] = EOS
		}
	    call strcat (Memc[imin], Memc[inpath], SZ_FNAME)
	    }
	else
	    call strcpy (Memc[imin], Memc[inpath], SZ_FNAME)
	imi = immap (Memc[inpath], READ_ONLY, 0)

	# Get image dimensions
	nlen1 = IM_LEN(imi,1)
	nlen2 = IM_LEN(imi,2)
	nplen1 = IM_PHYSLEN(imi,1)
	nplen2 = IM_PHYSLEN(imi,2)

	# Get number of times to fold image
	nfold = clgeti ("nfold")
	if (nfold > MAX_FOLD)
	    nfold = MAX_FOLD
	if (nfold < 1)
	    nfold = 1

	# Get image section, if any
	call imgsection (imin, imsect, SZ_LINE)
	if (strlen(imsect) > 0) {
	    ip = 1
	    call im_decode_subscript (imsect, ip, xin1, x2, step)
	    if (xin1 < 1)
		xin1 = 1
	    if (x2 > nlen1)
		x2 = nlen1
	    ncol = x2 - xin1 + 1
	    call im_decode_subscript (imsect, ip, yin1, y2, step)
	    if (yin1 < 1)
		yin1 = 1
	    if (y2 > nlen2)
		y2 = nlen2
	    nrow = y2 - yin1 + 1
	    }
	else {
	    ncol = nlen1
	    nrow = nlen2
	    xin1 = 1
	    yin1 = 1
	    }
	xout1 = 1
	yout1 = 1 + (nrow * (nfold - 1))

	# Set dispersion axis
	if (ncol > nrow)
	    dispaxis = 1
	else
	    dispaxis = 2
	i = 0
	call imgipar (imi,"DISPAXIS",i)
	if (i == 0)
	    call imputi (imi,"DISPAXIS",dispaxis)
	else
	    dispaxis = i

	if (verbose) {
	    call printf ("IFOLD: %d x %d input image %s opened, fold axis %d\n")
		call pargi (ncol)
		call pargi (nrow)
		call pargstr (Memc[imin])
		call pargi (dispaxis)
	    call flush (STDOUT)
	    }

	# Compute locations from which to get folds 
	if (dispaxis == 1) {
	    nrfold = nrow * nfold
	    ncfold = ncol / nfold
	    if (ncfold * nfold < ncol)
		ncfold = ncfold + 1
	    xin2 = xin1 + ncfold - 1
	    yin2 = yin1 + nrow - 1
	    xout2 = ncfold
	    yout2 = yout1 + nrow - 1
	    }
	else {
	    ncfold = ncol * nfold
	    nrfold = nrow / nfold
	    if (nrfold * nfold < nrow)
		nrfold = nrfold + 1
	    xin2 = xin1 + ncol - 1
	    yin2 = yin1 + nrfold - 1
	    xout2 = ncol
	    yout2 = nrfold
	    }

	# Output filename can be either input or created
	call clgstr ("imout", Memc[imout], SZ_LINE)
	if (Memc[imout] == EOS) {
	    call strcpy (Memc[imin],Memc[imout],SZ_LINE)
	    call strcat (".fold", Memc[imout], SZ_LINE)
	    }
	call clgstr ("outdir", Memc[outpath], SZ_LINE)
	lpath = strlen (Memc[outpath])
	if (lpath > 0) {
	    if (Memc[outpath+lpath-1] != '/') {
		Memc[outpath+lpath] = '/'
		Memc[outpath+lpath+1] = EOS
		}
	    call strcat (Memc[imout], Memc[outpath], SZ_FNAME)
	    }
	else
	    call strcpy (Memc[imout], Memc[outpath], SZ_FNAME)

	# Set output file dimensions
	IM_LEN(imi,1) = ncfold
	IM_LEN(imi,2) = nrfold
	IM_PHYSLEN(imi,1) = ncfold
	IM_PHYSLEN(imi,2) = nrfold

	# Open output image
	imo = immap (Memc[outpath], NEW_COPY, imi)

	if (verbose) {
	    call printf ("IFOLD: %d x %d output image %s folded %d times\n")
		call pargi (ncfold)
		call pargi (nrfold)
		call pargstr (Memc[imout])
		call pargi (nfold)
	    call flush (STDOUT)
	    }

	# Reset input file dimensions
	IM_LEN(imi,1) = nlen1
	IM_LEN(imi,2) = nlen2
	IM_PHYSLEN(imi,1) = nplen1
	IM_PHYSLEN(imi,2) = nplen2

	# Save original dimensions for unfolding
	call imaddi (imo,"PFNAXIS1",ncol)
	call imaddi (imo,"PFNAXIS2",nrow)
	call imaddi (imo,"NFOLD",nfold)

	#  Put date into HISTORY
	call logtime (dstring, SZ_LINE)
	call sprintf (hstring,SZ_LINE,"Folded into %d segments by saotdc.itools.ifold %s")
	    call pargi (nfold)
	    call pargstr (dstring)
	call imputh (imo, "HISTORY", hstring)

	# Fill in folded image
	switch (IM_PIXTYPE(imi)) {
	case TY_SHORT, TY_USHORT, TY_INT, TY_LONG:
	    do ifold = 1, nfold {
		if (verbose) {
		    call printf ("IFOLD: %d:%d,%d:%d -> %d:%d,%d:%d\n")
			call pargi (xin1)
			call pargi (xin2)
			call pargi (yin1)
			call pargi (yin2)
			call pargi (xout1)
			call pargi (xout2)
			call pargi (yout1)
			call pargi (yout2)
		    call flush (STDOUT)
		    }
		inbuf = imgs2i (imi, xin1,xin2,yin1,yin2)
		outbuf = imps2i (imo, xout1,xout2,yout1,yout2)
		npix = (xin2 - xin1 + 1) * (yin2 - yin1 + 1)
		call amovi (Memi[inbuf],Memi[outbuf],npix)
		call imflsi (imo)
		if (dispaxis == 1) {
		    xin1 = xin1 + ncfold
		    xin2 = xin2 + ncfold
		    if (xin2 > nlen1) {
			xout2 = xout2 - (xin2 - nlen1)
			xin2 = nlen1
			}
		    yout1 = yout1 - nrow
		    yout2 = yout2 - nrow
		    }
		else {
		    yin1 = yin1 + nrfold
		    yin2 = yin2 + nrfold
		    if (yin2 > nlen2) {
			yout2 = yout2 - (yin2 - nlen2)
			yin2 = nlen2
			}
		    xout1 = xout1 + ncol
		    xout2 = xout2 + ncol
		    }
		}

	#  Floating point data

	default:
	    do ifold = 1, nfold {
		if (verbose) {
		    call printf ("IFOLD: %d:%d,%d:%d -> %d:%d,%d:%d\n")
			call pargi (xin1)
			call pargi (xin2)
			call pargi (yin1)
			call pargi (yin2)
			call pargi (xout1)
			call pargi (xout2)
			call pargi (yout1)
			call pargi (yout2)
		    call flush (STDOUT)
		    }
		inbuf = imgs2r (imi, xin1,xin2,yin1,yin2)
		outbuf = imps2r (imo, xout1,xout2,yout1,yout2)
		npix = (xin2 - xin1 + 1) * (yin2 - yin1 + 1)
		call amovr (Memr[inbuf],Memr[outbuf],npix)
		call imflsr (imo)
		if (dispaxis == 1) {
		    xin1 = xin1 + ncfold
		    xin2 = xin2 + ncfold
		    if (xin2 > nlen1) {
			xout2 = xout2 - (xin2 - nlen1)
			xin2 = nlen1
			}
		    yout1 = yout1 - nrow
		    yout2 = yout2 - nrow
		    }
		else {
		    yin1 = yin1 + nrfold
		    yin2 = yin2 + nrfold
		    if (yin2 > nlen2) {
			yout2 = yout2 - (yin2 - nlen2)
			yin2 = nlen2
			}
		    xout1 = xout1 + ncol
		    xout2 = xout2 + ncol
		    }
		}
	}


	# Shutdown.
	call imunmap (imi)
	call imunmap (imo)
	call sfree (sp)
end

# Feb 11 1993	New program

# Oct 30 1995	Add directories as parameters
# Oct 30 1995	Start at upper left instead of lower left
