#*** File itools/Src/t_ifold.x
#*** February 5, 1993
#*** By Doug Mink

include	<mach.h>
include	<imhdr.h>
include	<gset.h>

define	MAX_FOLD 10

# IFOLD -- Fold a spectrum image

procedure t_ifold()

long	vi[IM_MAXDIM], vo[IM_MAXDIM], vl
int	i, npix, nrow, ip0, ir0, ir1, ip[MAX_FOLD], np[MAX_FOLD]
int	ifold, nfold, npfold, nrfold, dispaxis, nppix, nprow, nlog
pointer imi, imo, sp, inbuf, outbuf, outbuf0, image, imout, inbuf0
char    hstring[SZ_LINE]
char    dstring[SZ_LINE]
long    ltime, clktime()

int	clgeti()
int	imgnlr(), imgnli()
int	impnlr(), impnli()
pointer	immap()

begin
	call smark (sp)
	call salloc (image,   SZ_LINE,  TY_CHAR)
	call salloc (imout,   SZ_LINE,  TY_CHAR)
	nlog = clgeti ("nlog")

	# Get the image input name.
	call clgstr ("image", Memc[image], SZ_LINE)
	imi = immap (Memc[image], READ_ONLY, 0)

	# Get image dimensions and dispersion axis
	npix = IM_LEN(imi,1)
	nrow = IM_LEN(imi,2)
	nppix = IM_PHYSLEN(imi,1)
	nprow = IM_PHYSLEN(imi,2)

	# Get dispersion axis
	dispaxis = clgeti ("dispaxis")
	i = 0
	call imgipar (imi,"DISPAXIS",i)
	if (i == 0)
	    call imputi (imi,"DISPAXIS",dispaxis)
	else
	    dispaxis = i

	if (nlog > 0) {
	    call printf ("IFOLD: %d x %d input image %s opened\n")
		call pargi (npix)
		call pargi (nrow)
		call pargstr (Memc[image])
	    }

	# Get number of times to fold image
	nfold = clgeti ("nfold")
	if (nfold > MAX_FOLD)
	    nfold = MAX_FOLD
	if (nfold < 1)
	    nfold = 1

	# Compute locations from which to get folds 
	if (dispaxis == 1) {
	    nrfold = nrow * nfold
	    npfold = npix / nfold
	    if (npfold * nfold < npix)
		npfold = npfold + 1
	    ip0 = 0
	    do ifold = 1, nfold {
		ip[ifold] = ip0
		ip0 = ip0 + npfold
		if (ip0 > npix)
		    ip0 = npix
		np[ifold] = ip0 - ip[ifold]
		}
	    }
	else {
	    npfold = npix * nfold
	    nrfold = nrow / nfold
	    if (nrfold * nfold < nrow)
		nrfold = nrfold + 1
	    ir0 = 0
	    ip0 = 1
	    do ifold = 1, nfold {
		ir1 = ir0 + nrfold
		if (ir1 > nrow)
		    ir1 = nrow
		np[ifold] = ir1 - ir0
		ir0 = ir1 + 1
		ip[ifold] = ip0
		ip0 = ip0 + npix
		}
	    }

	# Output filename can be either input or created
	call clgstr ("imout", Memc[imout], SZ_LINE)
	if (Memc[imout] == EOS) {
	    call strcpy (Memc[image],Memc[imout],SZ_LINE)
	    call strcat (".fold", Memc[imout], SZ_LINE)
	    }

	# Set output file dimensions
	IM_LEN(imi,1) = npfold
	IM_LEN(imi,2) = nrfold
	IM_PHYSLEN(imi,1) = npfold
	IM_PHYSLEN(imi,2) = nrfold

	# Open output image
	imo = immap (Memc[imout], NEW_COPY, imi)
	IM_PHYSLEN(imo,1) = npfold
	IM_PHYSLEN(imo,2) = nrfold

	if (nlog > 0) {
	    call printf ("IFOLD: %d x %d output image %s opened\n")
		call pargi (npfold)
		call pargi (nrfold)
		call pargstr (Memc[imout])
	    }

	# Reset input file dimensions
	IM_LEN(imi,1) = npix
	IM_LEN(imi,2) = nrow
	IM_PHYSLEN(imi,1) = nppix
	IM_PHYSLEN(imi,2) = nprow

	# Save original dimensions for unfolding
	call imaddi (imo,"PFNAXIS1",npix)
	call imaddi (imo,"PFNAXIS2",nrow)
	call imaddi (imo,"NFOLD",nfold)

	#  Put date into HISTORY
	ltime = 0
	ltime = clktime (ltime)
	call cnvtime (ltime, dstring, SZ_LINE)
	call sprintf (hstring,SZ_LINE,"Folded in %d by IRAF saotdc.ifold %s")
	    call pargi (nfold)
	    call pargstr (dstring)
	call imputh (imo, "HISTORY", hstring)

	# Initialize image line vectors
	call amovkl (long(1), vi, IM_MAXDIM)
	call amovkl (long(1), vo, IM_MAXDIM)

	# Fill in folded image

	switch (IM_PIXTYPE(imi)) {
	case TY_SHORT, TY_USHORT, TY_INT, TY_LONG:

	# Read successive lines of the image, writing out multiple lines

	    if (dispaxis == 1) {
		while (imgnli (imi, inbuf0, vi) != EOF) {
		    inbuf = inbuf0
		    vl = vi[2]
		    do ifold = 1, nfold {
			vo[2] = vl
			if (impnli (imo, outbuf, vo) != EOF) {
			    call amovi (Memi[inbuf],Memi[outbuf],np[ifold])
			    inbuf = inbuf + np[ifold]
			    }
			else {
			    call printf ("IFOLD: line %d->%d/%d out of bounds\n")
				call pargi (vi[2])
				call pargi (vo[2])
				call pargi (ifold)
			    }
			vl = vl + nrow
			}
		    if (mod (vo[2], nlog) == 0) {
			call printf ("IFOLD:  (%d,%d)->(%d,%d)\r")
			    call pargi (vi[1])
			    call pargi (vi[2])
			    call pargi (vo[1])
			    call pargi (vo[2])
			call flush (STDOUT)
			}
		    }
		}

	# Fill successive lines of the image, reading from multiple lines

	    else {
		while (impnli (imo, outbuf0, vo) != EOF) {
		    vl = vo[2]
		    outbuf = outbuf0
		    do ifold = 1, nfold {
			vi[2] = vl
			if (imgnli (imi, inbuf, vi) != EOF) {
			    call amovi (Memi[inbuf],Memi[outbuf],npix)
			    outbuf = outbuf + npix
			    }
			else {
			    call printf ("IFOLD: line %d->%d/%d out of bounds\n")
				call pargi (vi[2])
				call pargi (vo[2])
				call pargi (ifold)
			    }
			vl = vl + nrfold
			}
		    if (mod (vo[2], nlog) == 0) {
			call printf ("IFOLD:  (%d,%d)->(%d,%d)\r")
			    call pargi (vi[1])
			    call pargi (vi[2])
			    call pargi (vo[1])
			    call pargi (vo[2])
			call flush (STDOUT)
			}
		    }
		}

	#  Floating point data

	default:

	# Read successive lines of the image, writing out multiple lines

	    if (dispaxis == 1) {
		while (imgnlr (imi, inbuf0, vi) != EOF) {
		    inbuf = inbuf0
		    vl = vi[2]
		    do ifold = 1, nfold {
			vo[2] = vl
			if (impnlr (imo, outbuf, vo) != EOF) {
			    call amovr (Memr[inbuf],Memr[outbuf],np[ifold])
			    inbuf = inbuf + np[ifold]
			    }
			else {
			    call printf ("IFOLD: line %d->%d/%d out of bounds\n")
				call pargi (vi[2])
				call pargi (vo[2])
				call pargi (ifold)
			    }
			vl = vl + nrow
			}
		    if (mod (vo[2], nlog) == 0) {
			call printf ("IFOLD:  (%d,%d)->(%d,%d)\r")
			    call pargi (vi[1])
			    call pargi (vi[2])
			    call pargi (vo[1])
			    call pargi (vo[2])
			call flush (STDOUT)
			}
		    }
		}

	# Fill successive lines of the image, reading from multiple lines

	    else {
		while (impnlr (imo, outbuf0, vo) != EOF) {
		    vl = vo[2]
		    outbuf = outbuf0
		    do ifold = 1, nfold {
			vi[2] = vl
			if (imgnlr (imi, inbuf, vi) != EOF) {
			    call amovr (Memr[inbuf],Memr[outbuf],npix)
			    outbuf = outbuf + npix
			    }
			else {
			    call printf ("IFOLD: line %d->%d/%d out of bounds\n")
				call pargi (vi[2])
				call pargi (vo[2])
				call pargi (ifold)
			    }
			vl = vl + nrfold
			}
		    if (mod (vo[2], nlog) == 0) {
			call printf ("IFOLD:  (%d,%d)->(%d,%d)\r")
			    call pargi (vi[1])
			    call pargi (vi[2])
			    call pargi (vo[1])
			    call pargi (vo[2])
			call flush (STDOUT)
			}
		    }
		}
	    }

	# Shutdown.
	call imunmap (imi)
	call imunmap (imo)
	call sfree (sp)
end
