include	<mach.h>
include	<imhdr.h>
include	<gset.h>

# IHISTOGRAM -- Compute and plot the histogram of an image.

# modified to make log scaling optional  -Doug Mink, June 13, 1988
# modified to print file name optional   -Doug Mink, Sept 14, 1989

procedure t_ihistogram()

real	z1, z2, z1temp, z2temp
bool	listout,lognum
long	v[IM_MAXDIM]
int	npix, nbins, z1i, z2i, i
pointer gp, im, sp, hgm, hgmr, buf, image, device
char	title[SZ_LINE]

bool	clgetb(), fp_equalr()
int	clgeti()
real	clgetr()
int	imgnlr(), imgnli()
pointer	immap(), gopen()

begin
	call smark (sp)
	call salloc (image,   SZ_LINE,  TY_CHAR)
	call salloc (device,  SZ_FNAME, TY_CHAR)

	# Get the image name.
	call clgstr ("image", Memc[image], SZ_LINE)
	im = immap (Memc[image], READ_ONLY, 0)
	npix = IM_LEN(im,1)

	# Output can be either a list or a plot.
	listout = clgetb ("listout")
	if (!listout)
	    call clgstr ("device", Memc[device], SZ_FNAME)

	# Output can have optional log-scaled y axis
	lognum = clgetb ("logy")

	# Get histogram length and allocate buffer.
	nbins = clgeti ("nbins")
	call salloc (hgm,  nbins, TY_INT)
	call salloc (hgmr, nbins, TY_REAL)

	z1 = clgetr ("z1")
	z2 = clgetr ("z2")
	z1temp = INDEFR
	z2temp = INDEFR

	# Get histogram range.
	if (z1 == INDEF) {
	    if (IM_LIMTIME(im) >= IM_MTIME(im))
	        z1 = IM_MIN(im)
	    else {
	        if (IS_INDEFR(z1temp))
	            call im_minmax (im, z1temp, z2temp)
		z1 = z1temp
	    }
	}

	if (z2 == INDEF) {
	    if (IM_LIMTIME(im) >= IM_MTIME(im))
	        z2 = IM_MAX(im)
	    else {
	        if (IS_INDEFR(z2temp))
	            call im_minmax (im, z1temp, z2temp)
		z2 = z2temp
	    }
	}

	z1i = nint (z1)
	z2i = nint (z2)

	# Initialize histogram and image line vector.
	call aclri (Memi[hgm], nbins)
	call amovkl (long(1), v, IM_MAXDIM)

	# Read successive lines of the image into memory and accumulate the
	# histogram.

	switch (IM_PIXTYPE(im)) {
	case TY_SHORT, TY_USHORT, TY_INT, TY_LONG:
	    # Test for constant valued image, which causes zero divide in ahgm.
	    if (z1i == z2i) {
	        call eprintf (
		    "Warning: Constant valued image `%s' has no data range.\n")
		    call pargstr (Memc[image])
	        call imunmap (im)
	        call sfree (sp)
	        return
	    }

	    while (imgnli (im, buf, v) != EOF) 
		call ahgmi (Memi[buf], npix, Memi[hgm], nbins, z1i, z2i)

	default:
	    # Test for constant valued image, which causes zero divide in ahgm.
	    if (fp_equalr (z1, z2)) {
	        call eprintf (
		    "Warning: Constant valued image `%s' has no data range.\n")
		    call pargstr (Memc[image])
	        call imunmap (im)
	        call sfree (sp)
	        return
	    }

	    while (imgnlr (im, buf, v) != EOF)
		call ahgmr (Memr[buf], npix, Memi[hgm], nbins,  z1,  z2)
	}

	# List or plot the histogram.
	if (listout) {
	    do i = 1, nbins {
		call printf ("%3d %d\n")
		    call pargi (i)
		    call pargi (Memi[hgm+i-1])
	    }
	} else {
	    gp = gopen (Memc[device], NEW_FILE, STDGRAPH)
	    if (lognum) {
		call gseti (gp, G_YTRAN, GW_LOG) }
	    else {
		call gseti (gp, G_YTRAN, GW_LINEAR) }
	    call achtir (Memi[hgm], Memr[hgmr], nbins)
	    call sprintf (title,SZ_LINE,"%s: %s")
		call pargstr (Memc[image])
		call pargstr (IM_TITLE(im))
	    call gploth (gp, Memr[hgmr], nbins, z1, z2, title)
	    call gclose (gp)
	}

	# Shutdown.
	call imunmap (im)
	call sfree (sp)
end
