#*** File itools/Src/t_iunfold.x
#*** February 11, 1993
#*** By Doug Mink

include	<mach.h>
include	<imhdr.h>
include	<gset.h>

define	MAX_FOLD 10

# UNFOLD -- Unfold a spectrum image

procedure t_iunfold()

int	npix, nrow, ncol
int	ifold, nfold, ncfold, nrfold, dispaxis
int	xin1, xin2, yin1, yin2, xout1, xout2, yout1, yout2
pointer imi, imo, sp, inbuf, outbuf, imout, imin
char    hstring[SZ_LINE]
char    dstring[SZ_LINE]
long    ltime, clktime()

int	imgs2r(), imgs2i(), imgeti()
int	imps2r(), imps2i()
pointer	immap()
bool	verbose, clgetb()

begin
	call smark (sp)
	call salloc (imout, SZ_LINE, TY_CHAR)
	call salloc (imin,  SZ_LINE, TY_CHAR)
	verbose = clgetb ("verbose")

	# Get the output image name.
	call clgstr ("imout", Memc[imout], SZ_LINE)

	# Input filename can be either input or created
	call clgstr ("imin", Memc[imin], SZ_LINE)
	if (Memc[imin] == EOS) {
	    call strcpy (Memc[imout],Memc[imin],SZ_LINE)
	    call strcat (".fold", Memc[imin], SZ_LINE)
	    }
	imi = immap (Memc[imin], READ_ONLY, 0)
	ncfold = IM_LEN(imi,1)
        nrfold = IM_LEN(imi,2)

	if (verbose) {
	    call printf ("IFOLD: %d x %d input image %s opened\n")
		call pargi (ncfold)
		call pargi (nrfold)
		call pargstr (Memc[imin])
	    call flush (STDOUT)
	    }

	# Get unfolded image dimensions
	ncol = imgeti (imi,"PFNAXIS1")
	nrow = imgeti (imi,"PFNAXIS2")

	# Get dispersion axis
	call imgipar (imi,"DISPAXIS",dispaxis)

	# Get number of times to unfold image
	nfold = imgeti (imi,"NFOLD")

	# Open output image
	IM_LEN(imi,1) = ncol
	IM_LEN(imi,2) = nrow
	imo = immap (Memc[imout], NEW_COPY, imi)
	call imdelf (imo,"PFNAXIS1")
	call imdelf (imo,"PFNAXIS2")
	call imdelf (imo,"NFOLD")
	IM_LEN(imi,1) = ncfold
	IM_LEN(imi,2) = nrfold

	if (verbose) {
	    call printf ("IFOLD: %d x %d output image %s opened\n")
		call pargi (ncol)
		call pargi (nrow)
		call pargstr (Memc[imout])
	    call flush (STDOUT)
	    }

	# Compute locations from which to get folds 
	if (dispaxis == 1) {
	    nrfold = nrow * nfold
	    ncfold = ncol / nfold
	    if (ncfold * nfold < ncol)
		ncfold = ncfold + 1
	    }
	else {
	    ncfold = ncol * nfold
	    nrfold = nrow / nfold
	    if (nrfold * nfold < nrow)
		nrfold = nrfold + 1
	    }

	#  Put date into HISTORY
	ltime = 0
	ltime = clktime (ltime)
	call cnvtime (ltime, dstring, SZ_LINE)
	call sprintf (hstring,SZ_LINE,"Unfolded by IRAF saotdc.iunfold %s")
	    call pargstr (dstring)
	call imputh (imo, "HISTORY", hstring)

	# Set initial limits for input and output image sections
	xin1 = 1
	yin1 = 1
	xout1 = 1
	yout1 = 1
	if (dispaxis == 1) {
	    xin2 = ncfold
	    yin2 = nrow
	    xout2 = ncfold
	    yout2 = nrow
	    }
	else {
	    xin2 = ncol
	    yin2 = nrfold
	    xout2 = ncol
	    yout2 = nrfold
	    }

	# Fill in unfolded image
	switch (IM_PIXTYPE(imi)) {

	# Integer data
	case TY_SHORT, TY_USHORT, TY_INT, TY_LONG:
	    do ifold = 1, nfold {
		npix = (xin2 - xin1 + 1) * (yin2 - yin1 + 1)
		if (verbose) {
		    call printf ("IFOLD: %d:%d,%d:%d -> %d:%d,%d:%d\n")
			call pargi (xin1)
			call pargi (xin2)
			call pargi (yin1)
			call pargi (yin2)
			call pargi (xout1)
			call pargi (xout2)
			call pargi (yout1)
			call pargi (yout2)
		    call flush (STDOUT)
		    }
		inbuf = imgs2i (imi, xin1,xin2,yin1,yin2)
		outbuf = imps2i (imo, xout1,xout2,yout1,yout2)
		call amovi (Memi[inbuf],Memi[outbuf],npix)
		call imflsi (imo)
		if (dispaxis == 1) {
		    xout1 = xout1 + ncfold
		    xout2 = xout2 + ncfold
		    if (xout2 > ncol) {
			xin2 = xin2 - (xout1 - ncol)
			xout2 = ncol
			}
		    yin1 = yin1 + nrow
		    yin2 = yin2 + nrow
		    }
		else {
		    yout1 = yout1 + nrfold
		    yout2 = yout2 + nrfold
		    if (yout2 > nrow) {
			yin2 = yin2 - (yout2 - nrow)
			yout2 = nrow
			}
		    xin1 = xin1 + ncol
		    xin2 = xin2 + ncol
		    }
		}

	#  Floating point data
	default:
	    do ifold = 1, nfold {
		if (verbose) {
		    call printf ("IFOLD: %d:%d,%d:%d -> %d:%d,%d:%d\n")
			call pargi (xin1)
			call pargi (xin2)
			call pargi (yin1)
			call pargi (yin2)
			call pargi (xout1)
			call pargi (xout2)
			call pargi (yout1)
			call pargi (yout2)
		    call flush (STDOUT)
		    }
		npix = (xin2 - xin1 + 1) * (yin2 - yin1 + 1)
		inbuf = imgs2r (imi, xin1,xin2,yin1,yin2)
		outbuf = imps2r (imo, xout1,xout2,yout1,yout2)
		call amovr (Memr[inbuf],Memr[outbuf],npix)
		call imflsr (imo)
		if (dispaxis == 1) {
		    xout1 = xout1 + ncfold
		    xout2 = xout2 + ncfold
		    if (xout2 > ncol) {
			xin2 = xin2 - (xout1 - ncol)
			xout2 = ncol
			}
		    yin1 = yin1 + nrow
		    yin2 = yin2 + nrow
		    }
		else {
		    yout1 = yout1 + nrfold
		    yout2 = yout2 + nrfold
		    if (yout2 > nrow) {
			yin2 = yin2 - (yout2 - nrow)
			yout2 = nrow
			}
		    xin1 = xin1 + ncol
		    xin2 = xin2 + ncol
		    }
		}
	}

	# Shutdown.
	call imunmap (imi)
	call imunmap (imo)
	call sfree (sp)
end
