#*** File itools/Src/t_iunfold.x
#*** February 5, 1993
#*** By Doug Mink

include	<mach.h>
include	<imhdr.h>
include	<gset.h>

define	MAX_FOLD 10

# UNFOLD -- Unfold a spectrum image

procedure t_iunfold()

long	vi[IM_MAXDIM], vo[IM_MAXDIM], vl
int	npix, nrow, ir0, ir1, ip0, ip[MAX_FOLD], np[MAX_FOLD]
int	ifold, nfold, npfold, nlog, nrfold, dispaxis
pointer imi, imo, sp, inbuf, outbuf, imout, imin, inbuf0, outbuf0
char    hstring[SZ_LINE]
char    dstring[SZ_LINE]
long    ltime, clktime()

int	imgnlr(), imgnli(), imgeti()
int	impnlr(), impnli()
pointer	immap()

int	clgeti()

begin
	call smark (sp)
	call salloc (imout, SZ_LINE, TY_CHAR)
	call salloc (imin,  SZ_LINE, TY_CHAR)
	nlog = clgeti ("nlog")

	# Get the output image name.
	call clgstr ("image", Memc[imout], SZ_LINE)

	# Input filename can be either input or created
	call clgstr ("imin", Memc[imin], SZ_LINE)
	if (Memc[imin] == EOS) {
	    call strcpy (Memc[imout],Memc[imin],SZ_LINE)
	    call strcat (".fold", Memc[imin], SZ_LINE)
	    }
	imi = immap (Memc[imin], READ_ONLY, 0)
	npfold = IM_LEN(imi,1)
        nrfold = IM_LEN(imi,2)

	if (nlog > 0) {
	    call printf ("IFOLD: %d x %d input image %s opened\n")
		call pargi (npfold)
		call pargi (nrfold)
		call pargstr (Memc[imin])
	    call flush (STDOUT)
	    }

	# Get unfolded image dimensions
	npix = imgeti (imi,"PFNAXIS1")
	nrow = imgeti (imi,"PFNAXIS2")

	# Get dispersion axis
	call imgipar (imi,"DISPAXIS",dispaxis)

	# Get number of times to unfold image
	nfold = imgeti (imi,"NFOLD")

	# Open output image
	IM_LEN(imi,1) = npix
	IM_LEN(imi,2) = nrow
	imo = immap (Memc[imout], NEW_COPY, imi)
	call imdelf (imo,"PFNAXIS1")
	call imdelf (imo,"PFNAXIS2")
	call imdelf (imo,"NFOLD")
	IM_LEN(imi,1) = npfold
	IM_LEN(imi,2) = nrfold

	if (nlog > 0) {
	    call printf ("IFOLD: %d x %d output image %s opened\n")
		call pargi (npix)
		call pargi (nrow)
		call pargstr (Memc[imout])
	    call flush (STDOUT)
	    }

	# Compute locations from which to get folds 
	if (dispaxis == 1) {
	    nrfold = nrow * nfold
	    npfold = npix / nfold
	    if (npfold * nfold < npix)
		npfold = npfold + 1
	    ip0 = 0
	    do ifold = 1, nfold {
		ip[ifold] = ip0
		ip0 = ip0 + npfold
		if (ip0 > npix)
		    ip0 = npix
		np[ifold] = ip0 - ip[ifold]
		}
	    }
	else {
	    npfold = npix * nfold
	    nrfold = nrow / nfold
	    if (nrfold * nfold < nrow)
		nrfold = nrfold + 1
	    ir0 = 0
	    ip0 = 1
	    do ifold = 1, nfold {
		ir1 = ir0 + nrfold
		if (ir1 > nrow)
		    ir1 = nrow
		np[ifold] = ir1 - ir0
		ir0 = ir1 + 1
		ip[ifold] = ip0
		ip0 = ip0 + npix
		}
	    }

#  Put date into HISTORY
	ltime = 0
	ltime = clktime (ltime)
	call cnvtime (ltime, dstring, SZ_LINE)
	call sprintf (hstring,SZ_LINE,"Unfolded by IRAF saotdc.iunfold %s")
	    call pargstr (dstring)
	call imputh (imo, "HISTORY", hstring)

	# Initialize image line vectors
	call amovkl (long(1), vi, IM_MAXDIM)
	call amovkl (long(1), vo, IM_MAXDIM)

	# Fill in unfolded image

	switch (IM_PIXTYPE(imi)) {

	# Integer data
	case TY_SHORT, TY_USHORT, TY_INT, TY_LONG:

	# Write successive lines of the image, reading from multiple lines
	    if (dispaxis == 1) {
		while (impnli (imo, outbuf0, vo) != EOF) {
		    outbuf = outbuf0
		    vl = vo[2]
		    do ifold = 1, nfold {
			vi[2] = vl
			if (imgnli (imi, inbuf, vi) != EOF) {
			    call amovi (Memi[inbuf],Memi[outbuf],npfold)
			    outbuf = outbuf + npfold
			    }
			else {
			    call printf ("IUNFOLD: line %d->%d/%d out of bounds\n")
				call pargi (vi[2])
				call pargi (vo[2])
				call pargi (ifold)
			    }
			vl = vl + nrow
			}
		    if (mod (vo[2], nlog) == 0) {
			call printf ("IUNFOLD:  (%d,%d)->(%d,%d)\r")
			    call pargi (vi[1])
			    call pargi (vi[2])
			    call pargi (vo[1])
			    call pargi (vo[2])
			call flush (STDOUT)
			}
		    }
		}

	# Read successive lines of the image, writing out multiple lines
	    else {
		while (imgnli (imi, inbuf0, vi) != EOF) {
		    vl = vi[2]
		    inbuf = inbuf0
		    do ifold = 1, nfold {
			vo[2] = vl
			if (impnli (imo, outbuf, vo) != EOF) {
			    call amovi (Memi[inbuf],Memi[outbuf],npix)
			    inbuf = inbuf + npix
			    }
			else {
			    call printf ("IUNFOLD: line %d->%d/%d out of bounds\n")
				call pargi (vi[2])
				call pargi (vo[2])
				call pargi (ifold)
			    }
			vl = vl + nrfold
			}
		    if (mod (vi[2], nlog) == 0) {
			call printf ("IUNFOLD:  (%d,%d)->(%d,%d)\r")
			    call pargi (vi[1])
			    call pargi (vi[2])
			    call pargi (vo[1])
			    call pargi (vo[2])
			call flush (STDOUT)
			}
		    }
		}

	# Floating point data
	default:

	# Write successive lines of the image, reading from multiple lines
	    if (dispaxis == 1) {
		while (impnlr (imo, outbuf0, vo) != EOF) {
		    outbuf = outbuf0
		    vl = vo[2]
		    do ifold = 1, nfold {
			vi[2] = vl
			if (imgnlr (imi, inbuf, vi) != EOF) {
			    call amovr (Memr[inbuf],Memr[outbuf],np[ifold])
			    outbuf = outbuf + npfold
			    }
			else {
			    call printf ("IUNFOLD: line %d->%d/%d out of bounds\n")
				call pargi (vi[2])
				call pargi (vo[2])
				call pargi (ifold)
			    }
			vl = vl + nrow
			}
		    if (mod (vo[2], nlog) == 0) {
			call printf ("IUNFOLD:  (%d,%d)->(%d,%d)\r")
			    call pargi (vi[1])
			    call pargi (vi[2])
			    call pargi (vo[1])
			    call pargi (vo[2])
			call flush (STDOUT)
			}
		    }
		}

	# Read successive lines of the image, writing out multiple lines
	    else {
		while (imgnlr (imi, inbuf0, vi) != EOF) {
		    vl = vi[2]
		    inbuf = inbuf0
		    do ifold = 1, nfold {
			vo[2] = vl
			if (impnlr (imo, outbuf, vo) != EOF) {
			    call amovr (Memr[inbuf],Memr[outbuf],npix)
			    inbuf = inbuf + npix
			    }
			else {
			    call printf ("IUNFOLD: line %d->%d/%d out of bounds\n")
				call pargi (vi[2])
				call pargi (vo[2])
				call pargi (ifold)
			    }
			vl = vl + nrfold
			}
		    if (mod (vi[2], nlog) == 0) {
			call printf ("IFOLD:  (%d,%d)->(%d,%d)\r")
			    call pargi (vi[1])
			    call pargi (vi[2])
			    call pargi (vo[1])
			    call pargi (vo[2])
			call flush (STDOUT)
			}
		    }
		}
	    }

	# Shutdown.
	call imunmap (imi)
	call imunmap (imo)
	call sfree (sp)
end
