include <imhdr.h>

define	MAXFILES	40

procedure t_mcombine

# Combine a list of images taking into account flagged pixels
# real input, real output, compensates for different exposure levels
# (by adding).  Mode input should be scaled to 1, e.g., if all levels
# are the same then all modes should be 1.0

pointer	sp
pointer	infiles, infile
pointer	outfile
pointer	imi[MAXFILES]
pointer	immap()
pointer	imo
pointer	idata[MAXFILES]
pointer	odata
pointer	imgl2r(), impl2r(), imgl2i(), impl2i()

int	low, high
int	ngdval[MAXFILES+1]
int	ngmax,ngmin,ngood,nkept
int	ifile,nfiles,icol,i,ncols,nrows,irow,n
int	list,imtopen(), imtgetim(), clgeti()

real	maskval0, maskval[MAXFILES]
real	rdata, rval, gdata[MAXFILES]
real	scale, scli[MAXFILES],scltot,sclg[MAXFILES]
real	clgetr()

bool	verbose, clgetb()

begin
	call smark (sp)
	call salloc (infile, SZ_FNAME, TY_CHAR)
	call salloc (infiles, SZ_LINE, TY_CHAR)
	call salloc (outfile, SZ_FNAME, TY_CHAR)

	verbose = clgetb ("verbose")

# Open the list of input images
        call clgstr ("infiles", Memc[infiles], SZ_FNAME)
	list = imtopen (Memc[infiles])

# Number of high and low values to reject (0 or 1 for now)
	low = clgeti ("lowpix")
	high = clgeti ("highpix")

# Default bad pixel value for the output image
# (and for input images if BLANK is not set in headers)
	maskval0 = clgetr ("maskval")

# open all the files - assumed all the same size!
	nfiles = 0
	while (imtgetim (list, Memc[infile], SZ_FNAME) != EOF) {
	    nfiles = nfiles + 1
	    imi[nfiles] = immap (infile, READ_ONLY, 0)

	# Value for excluded pixels for this image
	    maskval[nfiles] = maskval0
	    call imgrpar (imi[nfiles],"BLANK", maskval[nfiles])

	# Exposure time for this image
	    scli[nfiles] = -1.
	    call imgrpar (imi[nfiles],"EXPOSURE", scli[nfiles])
	    if (scli[nfiles] <= 0.)
		call imgrpar (imi[nfiles],"EXPTIME", scli[nfiles])
	    scltot = scltot + scli[nfiles]

	    if (verbose) {
		call printf ("%d: %16s %8.1f sec\n")
		    call pargi (nfiles)
		    call pargstr (Memc[infile])
		    call pargr (scli[nfiles])
		}
	    }

	ncols = IM_LEN(imi[1],1)
	nrows = IM_LEN(imi[1],2)

# Open output image
	call clgstr ("outfile", Memc[outfile], SZ_FNAME)
	imo = immap (Memc[outfile], NEW_COPY, imi[1])
	call imputr (imo,"BLANK",maskval0)

# if this message occurs, then all files open 
	if (verbose) {
	    call printf ("MCOMBINE: %d input images, low rej = %d, high rej = %d\n")
		call pargi (nfiles)
		call pargi (low)
		call pargi (high)
	    }

# now work through all the images row by row
	do irow = 1, nrows {

	# Load row from each image
	    do ifile = 1, nfiles {
		switch (IM_PIXTYPE(imi[ifile])) {
		    case TY_SHORT, TY_USHORT, TY_INT, TY_LONG:
			idata[ifile] = imgl2i (imi[ifile],irow)
		    default:
			idata[ifile] = imgl2r (imi[ifile],irow)
		    }
		}

	# Get buffer for output of this line
	    switch (IM_PIXTYPE(imo)) {
        	case TY_SHORT, TY_USHORT, TY_INT, TY_LONG:
		    odata = impl2i (imo, icol)
        	default:
		    odata = impl2r (imo, icol)
		}

	# Work down each row
	    do icol = 1, ncols {
		i = icol - 1
		ngood = 0
		scale=0.0

	# Save good values for current pixel in vector
		ngood = 0
		scale = 0.
		ngmax = 1
		ngmin = 1
		do ifile = 1, nfiles {
		    switch (IM_PIXTYPE(imi[ifile])) {
        		case TY_SHORT, TY_USHORT, TY_INT, TY_LONG:
			    rdata = Memi[idata[ifile]+i]
			default:
			    rdata = Memr[idata[ifile]+i]
			}
		    if (rdata != maskval[ifile]) {
			ngood = ngood + 1
     			gdata[ngood] = rdata * scli[ifile]
			sclg[ngood] = scli[ifile]
			scale = scale + scli[ifile]
			if (gdata[ngood] > gdata[ngmax]) {
			    ngmax = ngood
			    }
			if (gdata[ngood] < gdata[ngmin]) {
			    ngmin = ngood
			    }
			}
		    }

	# Set value for current pixel
		rval = 0
		if (ngood == 0) {
		    rval = maskval0
		    nkept = 0
		    }
		    
	# Drop high and/or low values for current pixel
		else if (low+high > 0) {
		    nkept = ngood - low - high
		    if (low == 1)
			scale = scale - sclg[ngmin]
		    if (high == 1)
			scale = scale - sclg[ngmax]
		    if (nkept > 0) {
			do n = 1+low, ngood-high {
			    rval = rval + gdata[n]
			    }
			rval = (scltot / scale) * rval / scale
			}
		    else
			rval = maskval0
		    }

	# Save all good values for current pixel
		else {
		    do n = 1, ngood {
			rval = rval + gdata[n]
			}
		    rval = (scltot / scale) * rval / scale
		    nkept = ngood
		    }

	# Save value of current pixel to output file
		switch (IM_PIXTYPE(imo)) {
        	    case TY_SHORT, TY_USHORT, TY_INT, TY_LONG:
			Memi[odata+i] = nint (rval)
		    default:
			Memr[odata+i] = rval
		    }

	# Keep track of how many pixels at each value of nkept
		ngdval[nkept+1] = ngdval[nkept+1] + 1
		}


	# Save this line to image
	    call imflsr (imo)

	# Free line vectors
	    call mfree (odata)
	    do ifile = 1, nfiles {
		call mfree (idata[ifile])
		}
	    }

# close all images
	do ifile = 1, nfiles {
	    call imunmap (imi[ifile])
	    }
	call imunmap (imo)

# now output the statistics of good pixels
	if (verbose) {
	    do ifile = 1, nfiles+1 {
		call printf ("%d %8d pixels\n")
		    call pargi (ifile-1)
		    call pargi (ngdval(ifile))
		}
	    }

	call sfree (sp)
end
