.ds CH IR Array Data Reduction
.ds LH 3 January 1989
.ds RH page %
.DS C
.ps 14
\fBIR Array Data Reduction under IRAF

.ps 12
\fIDoug Mink, Center for Astrophysics
3 January 1989\fR

.DE
.PP
This is an outline for a method of reducing images taken
with the SAO SONIC Infrared Camera.  Image reduction will be done
using the NOAO IRAF image reduction system with a few additional functions.
This procedure is based on discussions with Steve Willner.

.in 0.3i
.ti -0.3i
\fBImages that should be taken
.LP
\fRIn addition to images of interesting objects, the following kinds of
images should be taken:
.in 0.5i
.LP
.ti -0.2i
\fR\(bu  Bias frames
.br
\fIA bunch of very short unexposed bias images should be taken at the beginning or
end of a night's observing.  The multiplicy of images allows for easy removal of
cosmic rays and good statistics on pixel by pixel bias level variations.\fR
.LP
.ti -0.2i
\(bu  Dark frames
.br
\fIThese should be taken at the same frame rates as data images.\fR
.LP
.ti -0.2i
\(bu  Flat fields
.br
\fIThese should be taken at the same frame rates and filter wavelengths
as data images.\fR
.in 0.3i


.KS
.ti -0.3i
\fBRead data from tape to FITS files
.LP
\fRtar xv *
.br
\fIRead FITS data from Unix tar tape onto disk.  Data can also be
transferred acros the network from the mountain to Cambridge to avoid
this step.  Data files are named with a sequence number and object,
i.e. \fR0001.DARK.

.ti -0.3i
\fBRead data from FITS files to IRAF files
.LP
\fRsao.iodata.rira <object name in> <object name out>
.br
\fIRead IR array data from FITS files into IRAF image files.
Headers reside in the user directory, while images reside elsewhere.
All parameters are carried, and the images may be optionally split into
their two components (nominally object and sky).  The new images are
named with the object, optional a or b, and the sequence number, i.e.
\fRdark.a.0001\fI.  When the image is to be split and \fREXPOSURE\fI is
set to 0, no \fR.a.\fI file is created; if \fREXP-SKY\fI is set to 0,
no \fR.b.\fI file is created.  This script also corrects the exposure and
frame rate values in the header for round-off error.
.KE

.KS
.ti -0.3i
\fBNormalize to single frame and linearize
.LP
\fRsao.procdata.ircorrect <iraf file or file list>
.br
\fINormalize image pixel values to counts per frame by dividing by
exposure/frame rate.  After normalization, perform a pixel-by-pixel polynomial
linearization correction.
.KE
.KS
.ti -0.3i
\fBReplace bad pixels
.LP
\fRnoao.proto.fixpix <file> <badpix file>
.br
\fIReplace regions of bad pixels on array chip by interpolation
from nearby good data.
.KE

.KS
.ti -0.3i
\fBFlag and replace cosmic rays
.LP
\fRsao.itools.sdisp <iraf file>
.br
\fIExamine image for cosmic rays.  This is an interface to Bill Wyatt's
\fRshowimg\fI program, which handles all cursor interaction and writes
out a file of regions.
.LP
\fRsao.itools.cosrep <iraf file>
.br
\fIReplace regions of bad pixels (usually cosmic rays) by interpolation
from nearby good data using noao.proto.fixpix.
.LP
\fBor
.LP
\fRnoao.imred.ccdred.cosmicrays <iraf file>
.br
\fIFind and replace regions of bad pixels (usually cosmic rays) by interpolation
from nearby good data.
.KE

.KS
.ti -0.3i
\fBCompute bias
.LP
\fRnoao.imred.ccdred.combine <bias file list> <bias image>
.br
\fIMake a bias image.

.ti -0.3i
\fBSubtract bias
.LP
\fRimages.imarith <iraf file> - <bias image> <iraf file>
.br
\fISubtract bias image from data, dark, and flat images.
.KE

.KS
.ti -0.3i
\fBCombine dark images
.LP
\fRsao.procdata.makedark
.br
\fICombine dark frames taken at each frame rate into average frames
with names of format
.br
\fRdarkavnnnnnn\fI, where \fRnnnnnn\fI is the frame
rate in milliseconds.

.ti -0.3i
\fBSubtract dark images
.LP
\fRsao.procdata.irdark <iraf file>
.br
\fISubtract appropriate dark file based on framerate of image.
.KE

.KS
.ti -0.3i
\fBCombine flat field images
.LP
\fRsao.procdata.makeflat
.br
\fICombine flat fields taken at each frame rate and filter into average frames
with names of format \fRffffffnnnnnn\fI, where \fRffffff\fI is the
filter name from the FILTER image header parameter and \fRnnnnnn\fI is
the frame rate in milliseconds.
.KE
.KS
.ti -0.3i
\fBProduce flat field calibration images
.LP
\fRimages.imsurfit <iraf flat field> <\fBresponse image\fR>
.br
\fIFit Legendre polynomial or cubic spline to flat field surface.
The "response" output image is effectively a high-pass-filtered flat field,
normalized pixel-to-pixel variation.
.KE

.KS
.ti -0.3i
\fBFlatten images
.LP
\fRimages.imarith <iraf file> / <response image> <\fBiraf file\fR>
.br
\fIProduce flattened 2-D images.  The image header should contain the
names of the flat field and bias which were used in the
reduction.
.KE

.KS
.ti -0.3i
\fBCombine sky images for each object \fI(optional)
.LP
\fRnoao.imred.combine <sky image list> <sky average image>
.br
\fITo improve noise, combine sky images for each object, weighting by
exposure and using avsigclip or median to remove stars.
.KE
.KS
.ti -0.3i
\fBNormalize sky image for each object \fI(optional)
.LP
\fRimages.imarith <sky image> * normfactor <sky image>
.br
\fINormalize sky image or averaged sky image to match a sourceless
region in the object image.
.KE

.KS
.ti -0.3i
\fBSubtract sky from object
.LP
\fRimages.imarith <object image> - <sky image> <object image>
.br
\fISubtract sky background from object image.
.KE

.KS
.ti -0.3i
\fBFind sources for aperture photometry
.LP
\fRsao.itools.sdisp <object image>
.br
\fIExamine image for objects of interest.  This is an interface to Bill Wyatt's
\fRshowimg\fI program, which handles all cursor interaction and writes
out a file of coordinate pairs.
.KE
.KS
.ti -0.3i
\fBAperture phometry
.LP
\fRlocal.apphot <object image> <center coordinate file>
.KE
