# File procdata/Scripts/biascheck.cl
# September 12, 1995
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# BIASCHECK -- Get statistics concerning bias frames

procedure biascheck (filename)

string	filename=""	{prompt="Filename or name of file with list of names"}
string	imagesect=""	{prompt="Image section to be used"}
string	dirpath=""	{prompt="Directory path to prepend to filename(s)"}
bool	plotstat=no	{prompt="Plot image statistics (y or n)"}
bool	hardcopy=no	{prompt="Print plot of image statistics (y or n)"}
bool	verbose=no	{prompt="Display what's happening (y or n)"}
real	maxdiff=2.0	{prompt="Maximum allowable difference of mean in sigma"}

begin

real	medsum, meansum, medsum2, meansum2, mxdiff
real	nfiles, mmean, mmedian, mndev, mddev, mndiff, mddiff
bool	vb, delf, hc
file	rfn_temp, biaspar
file	fname, listname, newlist, rejlist, oldlist
string	listfile, jd, filepath, filesect, outline
string	tchar, rfile, sect, crej

	rfile = filename
	vb = verbose
	sect = imagesect
	hc = hardcopy
	listname = "statbias"
	mxdiff = maxdiff
	onedspec
	setjd.observatory = "flwo1"
	setjd.date = "DATE-OBS"
	setjd.time = "UTOPEN"
	setjd.exposure="EXPTIME"
	setjd.jd="GJDN"
	setjd.hjd=""
	setjd.ljd=""
	imgetp.verbose = no

#  Process data files from command line or file of files

	tchar = substr (rfile,1,1)
	if (tchar != '@') {
	    rfn_temp = mktemp ("/tmp/bias")
	    print (rfile,>rfn_temp)
	    listfile = rfn_temp
	    }
	else
	    listfile = substr (rfile,2,64)

	if (vb)
	    print ("RFNs from ",listfile)
 
#  Loop through list of reduced files and compute mean mean and median
	istat.fields = "npix,mean,stddev,median,min,max"
	medsum = 0.0
	medsum2 = 0.0
	meansum = 0.0
	meansum2 = 0.0
	nfiles = 0.0
	list = listfile
	while (fscan (list,fname) != EOF) {
	    filepath = dirpath // "/" // fname
	    filesect = filepath // sect
	    if (vb) {
		print ("Filename is ",filesect)
		}
	    istat (filesect)
	    mmed = istat.median
	    medsum = medsum + mmed
	    medsum2 = medsum2 + (mmed * mmed)
	    mmean = istat.mean
	    meansum = meansum + mmean
	    meansum2 = meansum2 + (mmean * mmean)
	    nfiles = nfiles + 1.0
	    }
	mmean = meansum / nfiles
	mndev = sqrt ((meansum2 - (meansum * mmean)) / (nfiles - 1.0))
	mmedian = medsum / nfiles
	mddev = sqrt ((medsum2 - (medsum * mmedian)) / (nfiles - 1.0))
	
	print ("Mean of ",nfiles," files is ",mmean," +- ",mndev)
	print ("Median of ",nfiles," files is ",mmedian," +- ",mddev)

# Initialize output files
	list = listfile
	if (plotstat) {
	    if (access (listname))
		delete (listname)
	    touch (listname)
	    }
	newlist = listfile // ".new"
	if (access (newlist))
	    delete (newlist)
	touch (newlist)
	oldlist = listfile // ".old"
	rejlist = listfile // ".rej"
	if (access (rejlist))
	    delete (rejlist)
	touch (rejlist)

# Read bias frames again and compare to the mean, rejected bad images
	while (fscan (list,fname) != EOF) {
	    outline = fname // " "
	    filepath = dirpath // "/" // fname
	    filesect = filepath // sect
	    if (vb)
		print ("Filename is ",filesect)
	    istat (filesect)

	    imgetp (filepath,"HJDN")
	    jd = imgetp.value
	    if (jd == "0") {
		setjd (filepath)
		imgetp (filepath,"HJDN")
		jd = imgetp.value
		}
	    outline = outline // jd // " "

	    mndiff = (istat.mean - mmean) / mndev
	    mddiff = (istat.median - mmedian) / mddev
	    if (mndiff < mxdiff && mndiff > -mxdiff) {
		print (fname,>>newlist)
		crej = " "
		}
	    else {
		print (fname,>>rejlist)
		crej = "X"
		}
	    round (istat.mean,"%.2f")
	    outline = outline // round.value // " "
	    round (mndiff,"%.2f")
	    outline = outline // round.value // " "
	    round (istat.median,"%.2f")
	    outline = outline // round.value // " "
	    round (mddiff,"%.2f")
	    outline = outline // round.value // " "
	    outline = outline // istat.min // " "
	    outline = outline // istat.max // " "
	    outline = outline // crej
	    print (outline)
	    if (plotstat) {
		print (outline, >>listname)
		}
	    flpr
	    }

#  Plot bias mean levels versus time
	if (plotstat) {
	    biaspar = "biasplot.par"
	    if (access (biaspar))
		delete (biaspar)
	    copy ("procdata$biasplot.par",".")
	    biasplot
	    if (hc) {
		pstek ("biasplot.tek")
		}
	    }

# Drop rejected bias frames from bias list
	mv (listfile,oldlist)
	mv (newlist,listfile)

	if (tchar != '@')
	    delete (rfn_temp)
end

# Aug 25 1995	New task
# Sep 12 1995	Check for existence before deleting files
