# File procdata/Scripts/darkcheck.cl
# August 21, 1995
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# DARKCHECK -- Get statistics concerning dark frames

procedure darkcheck (filename)

string	filename=""	{prompt="Filename or name of file with list of names"}
string	imagesect=""	{prompt="Image section to be used"}
string	dirpath=""	{prompt="Directory path to prepend to filename(s)"}
real	bias=0		{prompt="Median or mean bias value to subtract"}
bool	verbose=no	{prompt="Print what's happening (y or n)"}

begin

real	medsum, meansum, medsum2, meansum2, zero
real	nfiles, mmean, mmedian, mndev, mddev, mndiff, mddiff
bool	vb, delf
file	rfn_temp
file	fname
string	listfile, jd, filepath, filesect
string	tchar, rfile, sect, exp

	rfile = filename
	vb = verbose
	sect = imagesect
	zero = bias
	onedspec
	setjd.observatory = "flwo1"
	setjd.date = "DATE-OBS"
	setjd.time = "UTOPEN"
	setjd.exposure="EXPTIME"
	setjd.jd="GJDN"
	setjd.hjd=""
	setjd.ljd=""
	imgetp.verbose = no

#  Process data files from command line or file of files

	tchar = substr (rfile,1,1)
	if (tchar != '@') {
	    rfn_temp = mktemp ("/tmp/dark")
	    print (rfile,>rfn_temp)
	    listfile = rfn_temp
	    }
	else
	    listfile = substr (rfile,2,64)

	if (vb)
	    print ("RFNs from ",listfile)
 
#  Loop through list of reduced files and compute mean mean and median
	istat.fields = "npix,mean,stddev,median,min,max"
	medsum = 0.0
	medsum2 = 0.0
	meansum = 0.0
	meansum2 = 0.0
	nfiles = 0.0
	list = listfile
	while (fscan (list,fname) != EOF) {
	    filepath = dirpath // "/" // fname
	    filesect = filepath // sect
	    if (vb)
		print ("Filename is ",filesect)
	    istat (filesect)
	    mmed = istat.median - zero
	    medsum = medsum + mmed
	    medsum2 = medsum2 + (mmed * mmed)
	    mmean = istat.mean - zero
	    meansum = meansum + mmean
	    meansum2 = meansum2 + (mmean * mmean)
	    nfiles = nfiles + 1.0
	    }
	mmean = meansum / nfiles
	mndev = sqrt ((meansum2 - (meansum * mmean)) / (nfiles - 1.0))
	mmedian = medsum / nfiles
	mddev = sqrt ((medsum2 - (medsum * mmedian)) / (nfiles - 1.0))
	
	print ("Mean of ",nfiles," files is ",mmean," +- ",mndev)
	print ("Median of ",nfiles," files is ",mmedian," +- ",mddev)

	list = listfile
	while (fscan (list,fname) != EOF) {
	    filepath = dirpath // "/" // fname
	    filesect = filepath // sect
	    if (vb)
		print ("Filename is ",filesect)
	    istat (filesect)
	    imgetp (filepath,"HJDN")
	    jd = imgetp.value
	    if (jd == "0") {
		setjd (filepath)
		imgetp (filepath,"HJDN")
		jd = imgetp.value
		}
	    imgetp (filepath,"EXPTIME")
	    exp = imgetp.value
	    mndiff = (istat.mean - mmean) / mndev
	    mddiff = (istat.median - mmedian) / mddev
	    print (fname," ",jd," ",istat.mean,mndiff,istat.median,mddiff,istat.min,istat.max," ",exp)
	    flpr
	    }

	if (tchar != '@')
	    delete (rfn_temp)
end
