# DOFAST- Process FAST spectrograph images
#         By Doug Mink, CfA
#	  March 10, 1995

procedure dofast (nobs,ncal)

int	nobs=0		{prompt="FAST image number to process"}
int	ncal=0		{prompt="Calibration file number"}

file	imdir="./"	{prompt="Image directory (root if obsdate set"}
file	caldir=""	{prompt="Directory for calibration spectrum"}
file	refcal=""	{prompt="Reference 1-D calibration spectrum"}
file	refdir=""	{prompt="Directory for reference spectrum"}
file	normspec="fastnorm"	{prompt="1-D normalization spectrum"}
file	normdir="procdata$Spectra/"	{prompt="Directory for normalization spectrum"}
string	obsdate=""	{prompt="Date of observation (yyyy/mm/dd/)"}
int	row1=INDEF	{prompt="First row for spectrum search"}
int	row2=INDEF	{prompt="Last row for spectrum search"}
int	nback=50	{prompt="Last row for spectrum search"}
bool	imdisp=no	{prompt="Display raw image (y or n)"}
bool	autofind=yes	{prompt="Find brightest aperture automatically (y or n)"}
bool	reidint=no	{prompt="Set reidentify interactive (y or n)"}
bool	apallint=no	{prompt="Set apall.interactive (y or n)"}
bool	spdisp=no	{prompt="Display spectrum (y or n)"}
bool	quickred=no	{prompt="Find redshift (y or n)?"}
int	contorder=20	{prompt="Order for continuum subtraction"}
bool	curmode=no	{prompt="Stop with cursor in EMSAO (y or n)?"}
bool	verbose=no	{prompt="Log progress (y or n)?"}


begin

	file	specfile, mspec, dspec, nspec
	file	obspath, calpath, obsdir, ndir, npath
	file	biasfile
	file	testfile
	file	calfile, calidpath, mscal, msref, mscal1
	file	filename, filename0
	bool	qr, vb, lspechead, lspecpix, lcalhead, lcalpix, apint
	int	nob, nca

	qr = quickred
	vb = verbose
	nob = nobs
	nca = ncal
	nspec = normspec
	ndir = normdir
	apint = apallint

#  Find image name given number
	obsname.imdir = imdir
	obsname.obsdate = obsdate
	obsname (nob)
	obsdir = obsname.obsdir
	if (obsdir == "") {
	    obspath = imdir // obsdate
	    print ("DOFAST:  Spectrum image directory ",obspath," not found")
	    return
	    }
	specfile = obsname.obsfile
	obspath = obsdir // specfile
	if (specfile == "") {
	    print ("DOFAST:  Spectrum image number ",nob," not found in ",obsdir)
	    return
	    }
	if (vb)
	    print ("DOFAST:  Spectrum image file is ",obspath)

#  Link image to working directory, if necessary
	filename = specfile // ".imh"
	lspechead = no
	lspecpix = no
	if (!access (filename)) {
	    filename0 = obspath // ".imh"
	    if (access (filename0)) {
		lns (filename0,filename)
		lspechead = yes
		}
		if (vb)
		    print ("DOFAST:  Spectrum file ",filename," linked from ",obsdir)
	    else {
		print ("DOFAST:  Spectrum file ",filename," not in ",obsdir)
		return
		}
	    }
	filename = specfile // ".pix"
	if (!access (filename)) {
	    filename0 = obspath // ".pix"
	    if (access (filename0)) {
		lns (filename0,filename)
		lspecpix = yes
		}
	    else {
		print ("DOFAST:  Pixel file ",filename," not in ",obsdir)
		return
		}
	    }

#  Display image, if requested
	if (imdisp) {
	    xdisp.geometry = "1100x528"
	    xdisp.background = no
	    xdisp (specfile)
	    }

#  Find spectrum and background in image
	filename = "database/ap" // specfile
	if (access (filename)) {
	    delete (filename)
	    if (vb)
		print ("DOFAST:  Deleting ",filename)
	    }
	if (autofind) {
	    findspec.row1 = row1
	    findspec.row2 = row2
	    findspec.nbackpix = nback
	    findspec.imdir="./"
	    findspec (specfile)
	    apall.lower = findspec.lower
	    apall.upper = findspec.upper
	    apall.b_sample = findspec.b_sample
	    apall.b_function = "legendre"
	    apall.b_order = 2
	    apall.b_niterate = 1
	    }
	else {
	    apfind.interactive=yes
	    apfind.find=yes
	    apfind.edit=yes
	    apfind.recenter=yes
	    apfind (specfile,1)
	    }

#  Extract spectrum and sky from image
	apall.format = "multispec"
	mspec = specfile // ".ms"
	filename = mspec // ".imh"
	if (access (filename))
	    imdelete (filename)
	apall.output = mspec
	apall.interactive = apint
	apall.find = no
	apall.edit = no
	apall.recenter = no
	apall.resize = no
	apall.trace = yes
	apall.fittrace = yes
	apall.extract = yes
	apall.extras = yes
	apall.review = no
	apall.background = "fit"
	apall.weights = "variance"
	apall.reference = specfile
	apall.profile = specfile
#	apall.gain = 1.2
#	apall.readnoise = 8.5
	apall (specfile, >"dev$null")

#  Find calibration image file name given number
	if (caldir != "")
	    obsname.obsdir = caldir
	obsname (nca)
	obsdir = obsname.obsdir
	if (obsdir == "") {
	    obspath = caldir // obsdate
	    print ("DOFAST:  Calibration image directory ",obspath," not found")
	    return
	    }
	calfile = obsname.obsfile
	obspath = obsname.obsdir
	if (calfile == "") {
	    print ("DOFAST:  Calibration image file ",nca," not found in ",obsdir)
	    return
	    }
	if (vb)
	    print ("DOFAST: Calibration file is ",calfile)
	calpath = obsdir // calfile

#  Link calibration image to working directory, if necessary
	filename = calfile // ".imh"
	lcalhead = no
	lcalpix = no
	if (!access (filename)) {
	    filename0 = calpath // ".imh"
	    if (access (filename0)) {
		lns (filename0,filename)
		lcalhead = yes
		}
	    else {
		print ("DOFAST:  Calibration file ",filename," not in ",obsdir)
		return
		}
	    }
	filename = calfile // ".pix"
	if (!access (filename)) {
	    filename0 = calpath // ".pix"
	    if (access (filename0)) {
		lns (filename0,filename)
		lcalpix = yes
		}
	    else {
		print ("DOFAST:  Pixel file ",filename," not in ",obsdir)
		return
		}
	    }

#  Extract 1-D calibration spectrum for this object
	mscal = calfile // ".ms"
	filename = mscal // ".imh"
	if (access (filename))
	    imdelete (filename)
	apall.output = mscal
	apall.trace = no
	apall.extras = no
	apall.review = yes
	apall.background = "none"
	apall (calfile)

#  Copy calibration reference file to current directory if necessary
	msref = refcal
	filename = "database/id" // msref
	if (!access (filename)) {
	    filename0 = refdir // "database/id" // msref
	    if (access (filename0)) {
		copy (filename0,filename)
		}
	    else {
		print ("DOFAST:  Reference file id",msref," not in ",refdir)
		return
		}
	    }
	filename = msref // ".imh"
	if (!access (filename)) {
	    filename0 = refdir // msref // ".imh"
	    if (access (filename0)) {
		imcopy.verbose = no
		imcopy (filename0,filename)
		}
	    else {
		print ("DOFAST:  Calibration spectrum ",msref," not in ",refdir)
		return
		}
	    }
	if (vb)
	    print ("DOFAST:  Reference calibration spectrum ",msref," copied")

#  Unlink full image file if it is symbolically linked 
	if (lspechead) {
	    filename = specfile // ".imh"
	    del (filename)
	    }
	if (lspecpix) {
	    filename = specfile // ".pix"
	    del (filename)
	    }

#  Unlink full calibration image file if it is symbolically linked 
	if (lcalhead) {
	    filename = calfile // ".imh"
	    del (filename)
	    }
	if (lcalpix) {
	    filename = calfile // ".pix"
	    del (filename)
	    }

#  Reidentify calibration lines based on reference calibration spectrum
	filename = "database/id" // mscal
	if (access(filename))
	    del (filename)
	if (vb)
	    print ("DOFAST:  1-D calibration reference file is ",msref)
	reidentify.coordlist = "procdata$fasthenear.dat"
	reidentify.maxfeatures = 80
	if (reidint)
	    reidentify.interactive = yes
	reidentify (msref, mscal, yes)

#  Set reference spectrum for wavelength dispersion correction
	if (vb)
	    print ("DOFAST:  Calibration file is ",mscal)
	refspec.references = mscal
	refspec.select = "match"
	refspec.group = "none"
	refspec.override = yes
	refspec.sort = ""
	refspec.verbose = yes
	refspec (mspec,yes)

#  Dispersion correct spectrum
	dispcor.linearize = yes
	dispcor.verbose = vb
	dspec = specfile // "d.ms"
#	dspec = mspec
	dispcor (mspec,dspec)
	imdelete (mspec)
	imrename (dspec,mspec)
	dspec = mspec

#  Normalize spectrum
	if (nspec != "") {
	    npath = ndir // nspec
	    filename = npath // ".imh"
	    sarith.clobber = yes
	    if (access (filename)) {
		sarith (dspec, "/", npath, mspec)
		if (vb)
		    print ("DOFAST: Divided by normalization file ",filename)
		}
	    else
		print ("DOFAST: Cannot find normalization file ",filename)
	    }

#  Display spectrum, if requested
	if (spd) {
	    splot (dspec,1,1)
	    }

#  Change name of wavelength calibration database to match object
	filename0 = "database/id" // mscal
	filename = "database/id" // dspec
	if (access(filename))
	    del (filename)
	rename (filename0, filename)

#  Find redshift, if requested
	if (qr) {
	    contpars.order = contorder
	    contpars.t_low_reject = 3.
	    contpars.t_high_reject = 3.
	    contpars.s_abs_reject = 5.
	    contpars.t_abs_reject = 20.
	    xcsao.save_vel = yes
	    xcsao (dspec)
	    emsao.curmode = curmode
	    emsao (dspec)
	    }
end

# Apr  4 1994	Add normalization spectrum option
# Apr  4 1994	Use curmode only for EMSAO
# Jun  7 1994	Use smoothed normalization spectrum
# Jul  5 1994	Change name of calibration ID database to match object
# Jul  7 1994	Do not load rvsao
# Aug  4 1994	Change background order from 3 to 2
# Dec 28 1994	Drop APALL.DISPAXIS setting

# Jan  3 1994	Do not set apall.gain and apall.readnoise; they change
# Feb  1 1995	Add switch to allow apall to be interactive
# Feb  8 1995	Turn off apall.review
# Mar 10 1995	Add parameter to set order of continuum fit
