# File archive/Scripts/findall.cl
# May 24, 2000
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# FINDALL -- Find and extract aperture for a list of FAST observation numbers

procedure findall (filename)

string	filename=""	{prompt="Filename or name of file with list of names"}
string	extension=""	{prompt="Image extension added to filename from list"}
bool	extract=yes	{prompt="Extract aperture using APALL (yes or no)"}
bool	interactive=no	{prompt="Run FINDSPEC interactively (yes or no)"}
bool	verbose=no	{prompt="Print what's happening (y or n)"}
bool	ainteractive=no	{prompt="Run APALL interactively (y or n)"}
bool	afind=no	{prompt="Find apertures in APALL (y or n)"}
bool	arecenter=yes	{prompt="Recenter apertures in APALL (y or n)"}
bool	aresize=yes	{prompt="Resize apertures in APALL (y or n)"}
bool	aedit=no	{prompt="Edit apertures in APALL (y or n)"}
bool	atrace=yes	{prompt="Trace apertures in APALL (y or n)"}
bool	afittrace=no	{prompt="Fit trace interactively in APALL (y or n)"}
bool	aextract=yes	{prompt="Extract spectrum in APALL (y or n)"}
bool	areview=yes	{prompt="Review extracted spectrum in APALL (y or n)"}
bool	aextras=yes	{prompt="Extract sky, sigma, etc. in APALL (y or n)"}

begin

bool	vb, delf
file	rfn_temp
file	fname
string	listfile
string	tchar, rfile, ext

	rfile = filename
	vb = verbose
	ext = extension
	findspec.verbose = vb

#  Process data files from command line or file of files

	tchar = substr (rfile,1,1)
	if (tchar != '@') {
	    rfn_temp = mktemp ("findall")
	    print (rfile,>rfn_temp)
	    listfile = rfn_temp
	    }
	else
	    listfile = substr (rfile,2,64)

	if (vb)
	    print ("RFNs from ",listfile)
	if (extract) {
	    findspec.dbap = yes
	    }
	findspec.interactive = interactive
 
#  Loop through list of reduced files
	list = listfile
	while (fscan (list,fname) != EOF) {
	    if (extension != "")
		fname = fname // ext
	    findspec (fname)
	    if (extract) {
		if (findspec.center != 0) {
		    apall.interactive = ainteractive
		    apall.find = afind
		    apall.recenter = arecenter
		    apall.resize = aresize
		    apall.edit = aedit
		    apall.trace = atrace
		    apall.fittrace = afittrace
		    apall.extract = aextract
		    apall.extras = aextras
		    apall.lower = findspec.lower
		    apall.upper = findspec.upper
		    apall.b_sample = findspec.b_sample
		    apall.b_function = findspec.b_function
		    apall.b_order = findspec.b_order
		    apall.b_niterate = findspec.b_niterate
		    apall.review = areview
		    apall (fname)
		    }
		else {
		    print ("Aperture not found in ",fname,>>"findall.errors")
		    if (vb)
			print ("FINDALL: Aperture not found in ",fname)
		    }
		}
	    }

	if (tchar != '@')
	    delete (rfn_temp)
end
# Apr 10 1995	Original script
# Jul 14 1995	Set APALL background parameters from FINDSPEC
# Oct  5 1995	Drop FLPR from loop
# Nov  3 1995	Add flags for optional extraction and interactive findspec

# Feb 28 1996	Add image extension; use verbose flag
# Sep 11 1996	Set additional APALL parameters from new FINDALL parameters
# Sep 12 1996	Set DBAP in FINDSPEC to yes if EXTRACT is yes

# May 24 2000	Set clobber environment variable to yes before running apall
