# LFAST  List exposure and filter for selected FAST images
#	 February 7, 1994
#        By Doug Mink, CfA

procedure lfast (start,finish)

int start=1		{prompt="Starting sequence number"}
int finish=1		{prompt="Ending sequence number"}
file obsdir="./"	{prompt="Directory for input files"}
string imgtype=""	{prompt="image type to list if not all"}
string imext=".imh"	{prompt="Image extension"}
string timepar="UT"	{prompt="Observation time header parameter"}
string exppar="EXPTIME"	{prompt="Exposure header parameter"}
string rapar="RA"	{prompt="Right ascension header parameter"}
string decpar="DEC"	{prompt="Declination header parameter"}
string epochpar="EPOCH"	{prompt="Epoch header parameter"}
bool verbose=no	{prompt="Log progress (y or n)?"}

begin
	int	nframe, iobs, firstobs, lastobs, i, nblank, nbobj
	real	exp,epoch
	string	filter, ccdtype, object, obstime, ra, dec, obsdate
	string	indir, in, val, sky, flat, pi, ltype, blank1, blank2
	file	infile,filename
	bool	vb, st

#   Set up initial parameters (prompt for those unset)
	firstobs = start
	lastobs = finish
	vb = verbose
	ltype = imgtype
	indir = obsdir
	if (indir != "") indir = indir//'$'

	print("")
	print("     File           Exp       Right                                                     Obs       Obs")
	print("     Name           Time    Ascension  Declination  Epoch    Object        PI           Date      Time")
	print("")

#   Read files one at a time
	for (iobs=firstobs; iobs<=lastobs; iobs+=1 ) {

#	Determine correct file extension
	    obsname.imdir = obsdir
	    obsname.obsdate = ""
	    obsname.verbose = vb
	    obsname (iobs)

	    if (obsname.obsfile == "")
		next

#	Set up file name
	    infile = obsname.obsdir // obsname.obsfile

#	Get exposure duration
	    imgetp (infile, exppar)
	    val = imgetp.value
	    exp = real (val)
	    if (vb) print("exp=",exp)

#	Object name
	    imgetp (infile, "title")
	    object = imgetp.value
	    i = stridx ("-",object)
	    if (i > 0)
		object = substr (object,1,i-2)
	    if (vb) print("obj=",object)

#	Object position
	    imgetp (infile, rapar)
	    ra = imgetp.value
	    imgetp (infile, decpar)
	    dec = imgetp.value
	    imgetp (infile, epochpar)
	    epoch = real (imgetp.value)
	    if (vb) print("ra=",ra," dec=",dec," (",epoch,")")

#	Filter name
	    imgetp (infile, "FILTER")
	    filter = imgetp.value
	    if (filter == "") filter = " "
	    if (vb) print("fil=",filter)

#	Get ccdtype or set it from object
	    imgetp (infile, "imagetyp")
	    ccdtype = imgetp.value
	    if (ccdtype == "0") {
		sky = filter//"SKY"
		flat = filter//"FLAT"
		if (object == "BIAS") ccdtype = " zero"
		else if (object == "DARK") ccdtype = " dark"
		else if (object == "SKY") ccdtype = "  sky"
		else if (object == sky) ccdtype = "  sky"
		else if (object == flat) ccdtype = " flat"
		}
	    if (vb) print("typ=",ccdtype)
	    if (ltype != "") {
		if (ccdtype != ltype)
		    next
		}

#	Get time of observation
	    imgetp (infile, "UT")
	    obstime = imgetp.value
	    imgetp (infile, "DATE-OBS")
	    obsdate = imgetp.value
	    if (vb) print("UT=",obstime," on ",obsdate)

#	Get principal investigator
	    imgetp (infile, "P.I.")
	    pi = imgetp.value

#	Print information
	    filename = obsname.obsfile
	    i = strlen (filename)
	    nblank = 18 - i
	    if (exp < 10.)
		nblank = nblank + 4
	    else if (exp < 100.)
		nblank = nblank + 3
	    else if (exp < 1000.)
		nblank = nblank + 2
	    else
		nblank = nblank + 1
	    blank1 = " "
	    for (i = 1; i < nblank; i = i + 1)
		blank1 = blank1 // " "
	    i = strlen (object)
	    nbobj = 14 - i
	    blank2 = " "
	    for (i = 1; i < nbobj; i = i + 1)
		blank2 = blank2 // " "
	    print (filename,blank1,exp," ",ra,"  ",dec,"  ",epoch,"  ",
			object,blank2,pi,"  ",obsdate,"  ",obstime)
	    }
end
# Feb  7 1994	Write task
