# LSCCD- List exposure and filter for selected CCD images
#         By Doug Mink, CfA

procedure lsccd (root)

string root	{prompt="Root for image files"}
char dirin	{prompt="Directory for input files"}
int start=1	{min=0, max=999, prompt="Starting sequence number"}
int finish=99	{min=0, max=999, prompt="Ending sequence number"}
bool settype=no	{prompt="Set image type if not set (y or n)?"}
bool verbose=no	{prompt="Log progress (y or n)?"}

begin

	int	nframe, iext, firstext, lastext
	real	exp
	char	filter, ccdtype, object, obstime
	char	indir, ext, in, val, sky, flat
	file	infile
	bool	vb, st

#   Set up initial parameters (prompt for those unset)

	indir = dirin
	in = root
	firstext = start
	lastext = finish
	st = settype
	vb = verbose
	if (indir != "") indir = indir//'$'

	print("")
	print("    image         exp                   image      obs")
	print("    header       time  filter  object    type     time")
	print("")

#   Read files one at a time

	for (iext=firstext; iext<=lastext; iext+=1 ) {

#	Determine correct file extension

	    if (iext < 10) { ext = '000'//iext }
	    else if (iext < 100) { ext = '00'//iext }
	    else { ext = '0'//iext }

#	Set up file name

	    infile = indir//in//'.'//ext//".imh"

	    if (vb) print("checking for ",infile)

#	If it exists, read parameters from file

	    if (access(infile)) {
		if (vb) print("reading from ",infile)

#	Get exposure duration

		imgetp (infile, "exposure")
		val = imgetp.value
		exp = real (val)
		if (vb) print("exp=",exp)

#	Get object name

		imgetp (infile, "title")
		object = imgetp.value
		if (vb) print("obj=",object)

#	Get filter name

		imgetp (infile, "filter")
		filter = imgetp.value
		if (filter == "") filter = " "
		if (vb) print("fil=",filter)

#	Get ccdtype or set it from object

		imgetp (infile, "imagetyp")
		ccdtype = imgetp.value
		if (ccdtype == "0") {
		    sky = filter//"SKY"
		    flat = filter//"FLAT"
		    if (object == "BIAS") ccdtype = " zero"
		    else if (object == "DARK") ccdtype = " dark"
		    else if (object == "SKY") ccdtype = "  sky"
		    else if (object == sky) ccdtype = "  sky"
		    else if (object == flat) ccdtype = " flat"
		    else ccdtype = "object"
		    if (st) {
			hedit.add = yes
			hedit (infile,"imagetyp",ccdtype)
			hedit.add = no
			}
		    }
		if (vb) print("typ=",ccdtype)

#	Get time of observation

		imgetp (infile, "ut-obs")
		obstime = imgetp.value
		if (vb) print("tim=",obstime)

	    if (exp < 10.)
		print (infile,"     ",exp,"  ",filter,"      ",object,
			"  ",ccdtype,"  ",obstime)
	    else if (exp < 100.)
		print (infile,"    ",exp,"  ",filter,"      ",object,
			"  ",ccdtype,"  ",obstime)
	    else if (exp < 1000.)
		print (infile,"   ",exp,"  ",filter,"      ",object,
			"  ",ccdtype,"  ",obstime)
	    else
		print (infile,"  ",exp,"  ",filter,"      ",object,
			"  ",ccdtype,"  ",obstime)
		}
	    }
end
