# QFAST- Quick look for FAST spectrograph images
#         By Doug Mink, CfA
#	  December 3, 1999

procedure qfast (nobs)

int	nobs=""		{prompt="FAST image number to process"}

file	rootdir="./"	{prompt="Spectum image root directory"}
string	obsdate=""		{prompt="Date of observation directory (yy/mm/dd)"}
file	fcal="fast.henear"	 {prompt="Calibration file name"}
file	caldir="procdata$Spectra/" {prompt="Calibration image directory"}
int	row1=INDEF	{prompt="First row for spectrum search"}
int	row2=INDEF	{prompt="Last row for spectrum search"}
bool	keepname=yes	{prompt="Keep object name in file name (y or n)"}
bool	apdisp=yes	{prompt="Display aperture and background (y or n"}
bool	imdisp=no	{prompt="Display raw image (y or n)"}
bool	spdisp=no	{prompt="Display spectrum (y or n)"}
bool	quickred=no	{prompt="Find redshift (y or n)?"}
bool	delquick=no	{prompt="Delete quick-look 1-D spectrum (y or n)?"}
bool	verbose=no	{prompt="Log progress (y or n)?"}

begin
	file	infile
	file	calfile
	file	lhfile, lpfile
	file	qfile
	file	filename, filename0,obspath
	file	mscal
	int	idot,nob
	bool	qr, vb, spd, qd, kn, linkp, linkh

	qr = quickred
	vb = verbose
	spd = spdisp
	qd = delquick
	calfile = fcal
	kn = keepname
	nob = nobs

#	Find image name given number
	obsname.imdir = rootdir
	obsname.obsdate = obsdate
	obsname (nob)
	if (obsname.obsdir == "") {
	    obspath = rootdir // obsdate
	    print ("QFAST:  Spectrum image directory ",obspath," not found")
	    return
	    }
	infile = obsname.obsfile
	obspath = obsname.obsdir // infile
	if (infile == "") {
	    print ("QFAST:  Spectrum image file ",nob," not found in ",obspath)
	    return
	    }
	if (vb)
	    print ("QFAST:  Spectrum image file is ",infile)

#	Link image to working directory, if necessary
	filename0 = obspath // ".imh"
	filename = infile // ".imh"
	linkh = no
	linkp = no
	if (!access (filename)) {
	    lns (filename0,filename)
	    linkh = yes
	    }
	filename0 = obspath // ".pix"
	filename = infile // ".pix"
	if (!access (filename)) {
	    lns (filename0,filename)
	    linkp = yes
	    }

#	Display image, if requested
	if (imdisp) {
	    xdisp.geometry = "1100x528"
	    xdisp.background = no
	    xdisp (infile)
	    }

#	Load aperture extraction package
#	noao
#	onedspec
#	twodspec
#	apextract
#	imred
#	specred

#	Find spectrum and background in image
	findspec.row1 = row1
	findspec.row2 = row2
	findspec.dispap = apdisp
	findspec.dbap = yes
	findspec.imdir = "./"
	findspec.verbose = verbose
	findspec (infile)
	apall.lower = findspec.lower
	apall.upper = findspec.upper
	apall.b_sample = findspec.b_sample
	apall.b_function = "legendre"
	apall.b_order = 3
	apall.b_niterate = 1

#	Remove old version of this image, if it exists
	if (kn)
	    qfile = infile // "q.ms"
	else {
	    qfile = nob
	    }
	filename = qfile // ".imh"
	if (access (filename)) {
	    imdel (filename)
	    if (vb)
		print ("QFAST:  Deleting old version of ",filename)
	    }

#	Extract spectrum and sky from image
	apall.format = "multispec"
	apall.output = qfile
	apall.interactive = no
	apall.find = no
	apall.recenter = no
	apall.resize = no
	apall.edit = no
	apall.trace = no
	apall.fittrace = no
	apall.extract = yes
	apall.extras = no
	apall.review = yes
#	apall.dispaxis = 1
	apall.background = "fit"
	apall.reference = infile
	apall.profile = infile
	apall (infile)
	if (linkh) {
	    filename = infile // ".imh"
	    del (filename)
	    }
	if (linkp) {
	    filename = infile // ".pix"
	    del (filename)
	    }

#	Copy calibration file to current directory if necessary
	mscal = calfile // ".ms"
	filename = "database/id" // mscal
	if (!access (filename)) {
	    filename0 = caldir // "database/id" // mscal
	    if (access (filename0)) {
		copy (filename0,filename)
		}
	    else {
		print ("QFAST:  Calibration file id",mscal," not in ",caldir)
		return
		}
	    }
	filename = mscal // ".imh"
	if (!access (filename)) {
	    filename0 = caldir // mscal // ".imh"
	    if (access (filename0)) {
		imcopy.verbose = no
		imcopy (filename0,filename)
		}
	    else {
		print ("QFAST:  Calibration spectrum ",mscal," not in ",caldir)
		return
		}
	    if (vb)
		print ("QFAST:  Calibration spectrum ",mscal," copied")
	    }

#	Set reference spectrum for wavelength dispersion correction
	if (vb)
	    print ("QFAST:  Spectrum file is ",qfile)
	refspec.references = mscal
	refspec.select = "match"
	refspec.group = "none"
	refspec.override = yes
	refspec.sort = ""
	refspec.verbose = no
	refspec (qfile,yes)
	if (vb)
	    print ("QFAST:  Calibration file is ",mscal)

#	Dispersion correct spectrum
	dispcor.linearize = yes
	dispcor.verbose = vb
	dispcor (qfile,qfile)

#	Display spectrum, if requested
	if (spd) {
	    splot (qfile,1,1)
	    }

#	Find redshift, if requested
	if (qr) {
	    rvsao
	    contpars.order = 20
	    contpars.t_low_reject = 3.
	    contpars.t_high_reject = 3.
	    xcsao.s_abs_reject = 5.
	    xcsao.t_abs_reject = 20.
	    xcsao.save_vel = yes
	    xcsao (qfile)
	    emsao (qfile,curmode=yes)
	    }
#	Delete spectrum, if requested
	if (qd)
	    imdel (qfile)
end

# Feb  8 1994	Check to see if image file is in current directory already
# Feb 10 1994	Avoid conflicts by presuming specred to be loaded already
# Dec 13 1994	Drop xcsao.emchop setting
# Dec  3 1999	Update to work with RVSAO 2.1 under IRAF 2.11.2
