# File archive/Scripts/qspec.cl
# May 25, 2000
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# QSPEC -- Find and extract aperture for a list of FAST observation numbers

procedure qspec (filename)

string	filename=""	{prompt="Filename or name of file with list of names"}
bool	ainteractive=no	{prompt="Run APALL interactively (y or n)"}
bool	afind=no	{prompt="Find apertures in APALL (y or n)"}
bool	arecenter=yes	{prompt="Recenter apertures in APALL (y or n)"}
bool	aresize=yes	{prompt="Resize apertures in APALL (y or n)"}
bool	aedit=no	{prompt="Edit apertures in APALL (y or n)"}
bool	atrace=yes	{prompt="Trace apertures in APALL (y or n)"}
bool	afittrace=no	{prompt="Fit trace interactively in APALL (y or n)"}
bool	aextract=yes	{prompt="Extract spectrum in APALL (y or n)"}
bool	aextras=yes	{prompt="Extract sky, sigma, etc. in APALL (y or n)"}
bool	verbose=no	{prompt="Print what's happening (y or n)"}

begin

bool	extract, isimh
bool	vb, delf
file	rfn_temp
file	fname
file	msname, mshead, mspix, msback
int	idot,idot1
string	listfile
string	tchar, rfile

	rfile = filename
	vb = verbose

#  Process data files from command line or file of files

	tchar = substr (rfile,1,1)
	if (tchar != '@') {
	    rfn_temp = mktemp ("findall")
	    print (rfile,>rfn_temp)
	    listfile = rfn_temp
	    }
	else
	    listfile = substr (rfile,2,64)

	if (vb)
	    print ("QSPEC: RFNs from ",listfile)
 
#  Loop through list of reduced files
	list = listfile
	while (fscan (list,fname) != EOF) {
	    findspec.dbap = no
	    findspec.verbose = vb
	    extract = no
	    findspec.lower = INDEF
	    findspec.upper = INDEF
	    findspec (fname, interactive=yes)
	    if (findspec.dbap) {

#	Drop file extension, if present
		idot = stridx (".",fname)
		idot1 = stridx (".",substr(fname,idot+1,100))
		while (idot1 > 0) {
		    idot = idot1 + idot
		    idot1 = stridx (".",substr(fname,idot+1,100))
		    }

#	Delete multispec version of file, if it is present
		if (idot > 0) {
		    if (substr(fname,idot+1,idot+3) == "imh") {
			msname = substr (fname,1,idot-1) // ".ms"
			mshead = msname // ".imh"
			isimh = yes
			}
		    else if (substr(fname,idot+1,idot+4) == "fits") {
			msname = substr (fname,1,idot-1) // ".ms"
			mshead = msname // ".fits"
			isimh = no
			}
		    else if (substr(fname,idot+1,idot+3) == "fit") {
			msname = substr (fname,1,idot-1) // ".ms"
			mshead = msname // ".fit"
			isimh = no
			}
		    else {
			msname = fname // ".ms"
			mshead = msname // ".imh"
			if (access (mshead))
			    isimh = yes
			else {
			    mshead = msname // ".fits"
			    if (access (mshead))
				isimh = no
			    else {
				mshead = msname // ".fit"
				if (access (mshead))
				    isimh = no
				else
				    print ("QSPEC: ",msname," not found")
				}
			    }
			}
		    }
		else {
		    msname = fname // ".ms"
		    mshead = msname // ".imh"
		    if (access (mshead))
			isimh = yes
		    else {
			mshead = msname // ".fits"
			if (access (mshead))
			    isimh = no
			else {
			    mshead = msname // ".fit"
			    if (access (mshead))
				isimh = no
			    else
				print ("QSPEC: ",msname," not found")
			    }
			}
		    }

		if (access (mshead)) {
		    if (vb)
			print ("QSPEC: ",mshead," being imdeleted")
		    imdel (msname,verify=no)
		    }
		else
		    print ("QSPEC: ",mshead," not found")
		if (access (mshead)) {
		    if (vb)
		    print ("QSPEC: ",mshead," not imdeleted, trying delete")
		    del (mshead)
		    }
		if (isimh) {
		    mspix = msname // ".pix"
		    if (access (mspix)) {
			if (vb)
			print ("QSPEC: ",mspix," not imdeleted, trying delete")
			del (mspix)
			}
		    msback = ".." // mshead
		    if (access (msback)) {
			if (vb)
			print ("QSPEC: ",msback," not imdeleted, trying delete")
			del (msback)
			}
		    }

		if (!access (mshead)) {
		    apall.lower = findspec.lower
		    apall.upper = findspec.upper
		    apall.b_sample = findspec.b_sample
		    apall.b_function = findspec.b_function
		    apall.b_order = findspec.b_order
		    apall.b_niterate = findspec.b_niterate
		    apall.interactive = ainteractive
		    apall.find = afind
		    apall.recenter = arecenter
		    apall.resize = aresize
		    apall.edit = aedit
		    apall.trace = atrace
		    apall.fittrace = afittrace
		    apall.extract = aextract
		    apall.extras = aextras
		    apall (fname)
		    }
		}
	    }

	if (tchar != '@')
	    delete (rfn_temp)
end

# Nov 27 1995	New task based on findall

# Sep 16 1996	Set additional APALL parameters from new QSPEC parameters
# Sep 16 1996	Do more checks to clear out old multispec files
# Nov 21 1996	Do NOT verify deletion of old .ms file
# Nov 22 1996	Allow lists to have .imh extensions

# May 25 2000	Delete .ms.fits and .ms.fit files, too
