# File procdata/Src/t_findspec.x
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
# October 13, 1995

# Based on plot.pcols
# Copyright(c) 1986 Association of Universities for Research in Astronomy Inc.

define	MAXBSECT	10

include	<gset.h>
include	<imhdr.h>
include	<imio.h>
include	<mwset.h>
include	<math/curfit.h>

# FSPLOT - Draw the projected vector to the screen.

procedure fsplot (image, xvec, yvec, row1,row2, zmin, zmax, col1, col2,
		  wcslab, format, imax,il,ir, nsect, bsect, ns)

char	image[SZ_FNAME]		# Image name
real	xvec[ARB], yvec[ARB]	# Vectors to be plotted
int	row1, row2		# Selected rows
real	zmin, zmax		# Y vector min max
int	col1, col2		# Selected columns
char	wcslab[ARB]		# WCS label
char	format[ARB]		# WCS format
int	imax			# Line of maximum peak
int	il,ir			# Left and right limits of peak
int	nsect			# Left background limits
int	bsect[MAXBSECT,2]	# Background sections
int	ns			# Number of times spectrum is smoothed

pointer	sp, gp, ypvec
pointer	device, marker, xlabel, ylabel, title, suffix, caption
real	wx1, wx2, wy1, wy2, szm, x, dx, y, y1, y2, dy
real	ydiff,y0, ymin, ymax, yoff
int	mode, imark, n, i
int	nlines				# Npts in vector
bool	pointmode, logplot
char	bsample[SZ_LINE] # Background sections as a string
char	temp[SZ_LINE]

pointer	gopen()
real	clgetr(), amedr()
bool	clgetb()
real	pfrac,bfrac

begin
	pointmode = FALSE
	logplot = clgetb ("logplot")

	nlines = row2 - row1 + 1
	call smark (sp)
	call salloc (device, SZ_FNAME, TY_CHAR)
	call salloc (marker, SZ_FNAME, TY_CHAR)
	call salloc (xlabel, SZ_LINE,  TY_CHAR)
	call salloc (ylabel, SZ_LINE,  TY_CHAR)
	call salloc (title,  SZ_LINE,  TY_CHAR)
	call salloc (caption,  SZ_LINE,  TY_CHAR)
	call salloc (suffix, SZ_FNAME, TY_CHAR)
	call salloc (ypvec, nlines, TY_REAL)

	call clgstr ("device", Memc[device], SZ_FNAME)
	mode = NEW_FILE

	gp = gopen (Memc[device], mode, STDGRAPH)

	call gclear (gp)
	call strcpy (wcslab, Memc[xlabel], SZ_LINE)
	call strcpy ("FINDSPEC: ", Memc[title], SZ_LINE)
	call strcat (image, Memc[title], SZ_LINE)
	if (col1 != col2) {
	    call sprintf (Memc[suffix], SZ_FNAME, ": columns %d to %d")
		call pargi (col1)
		call pargi (col2)
	    }
	else {
	    call sprintf (Memc[suffix], SZ_FNAME, ": column %d")
		call pargi (col1)
	    }
	call strcat (Memc[suffix], Memc[title], SZ_LINE)
	if (ns > 0) {
	    call sprintf (Memc[suffix],SZ_FNAME," %dx smoothed")
		call pargi (ns)
	    call strcat (Memc[suffix], Memc[title], SZ_LINE)
	    }

	Memc[ylabel] = EOS

	# Set window limits
	wx1 = x_vec[1]
	wx2 = x_vec[nlines]
	if (logplot) {
	    if (zmin < 0)
		yoff = -zmin + 1.0
	    else
		yoff = 0.0
	    do i = 1, nlines {
		Memr[ypvec+i-1] = alog10 (yvec[i] + yoff)
		}
	    call alimr (Memr[ypvec],nlines,ymin,ymax)
	    ydiff = (ymax - ymin) * 0.1
	    }
	else {
	    call amovr (yvec, Memr[ypvec], nlines)
	    ymin = zmin
	    ymax = zmax
	    ydiff = (zmax - zmin) * 0.1
	    }
	wy1 = ymin - ydiff
	wy2 = ymax + (ydiff * 2.)

	call gswind (gp, wx1, wx2, wy1, wy2)
    
	call gsets (gp, G_XTICKFORMAT, format)
	call gseti (gp, G_XNMAJOR, 5)
	call gseti (gp, G_XNMINOR, 5)
	call gseti (gp, G_YNMAJOR, 5)
	call gseti (gp, G_YNMINOR, 5)
	call gseti (gp, G_ROUND, NO)

# Draw axes using all this information.
	call glabax (gp, Memc[title], Memc[xlabel], Memc[ylabel])
    
# Now actually draw the data
        if (pointmode)
            call gpmark (gp, xvec, Memr[ypvec], nlines, imark, szm, szm)
        else
            call gpline (gp, xvec, Memr[ypvec], nlines)

# Mark the peak
	dy = 0.1 * (wy2 - wy1)
	x = imax
	y1 = Memr[ypvec+imax-row1] + dy
	y2 = y1 + dy
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)

# Mark the spectrum
	x = imax + il
	y0 = amedr (Memr[ypvec],nlines)
	y1 = y0 - dy
	y2 = Memr[ypvec+imax-row1] + dy
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)

	x = imax + ir
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)

# Mark the background
	y2 = y0 + dy
	do n = 1, nsect {
	    x = bsect[n,1]
	    call gamove (gp,x,y1)
	    call gadraw (gp,x,y2)
	    x = bsect[n,2]
	    call gamove (gp,x,y1)
	    call gadraw (gp,x,y2)
	    }
	call gflush (gp)

# Print results of search on plot
	dx = 0.1 * (wx2 - wx1)
	x = imax + dx
	y = wy2 - dy
	pfrac = clgetr ("fracpeak")
	call sprintf (Memc[caption],SZ_LINE,"Spectrum at %d, %d:%d [%4.2f]")
	    call pargi (imax)
	    call pargi (imax+il)
	    call pargi (imax+ir)
	    call pargr (pfrac)
	call gtext (gp, x,y,Memc[caption],"q=h;f=r;h=l;s=0.7")
	call gflush (gp)

# Save background sample range as a parameter
	bsample[1] = EOS
	do n = 1, nsect {
	    call sprintf (temp,SZ_LINE,"%d:%d")
		call pargi (bsect[n,1])
		call pargi (bsect[n,2])
	    call strcat (temp,bsample,SZ_LINE)
	    if (n < nsect)
		call strcat (",",bsample,SZ_LINE)
	    }
	y = y - (dy * 0.4)
	bfrac = clgetr ("fracback")
	call sprintf (Memc[caption],SZ_LINE,"Background from %s [%4.2f]")
	    call pargstr (bsample)
	    call pargr (bfrac)
	call gtext (gp, x,y, Memc[caption],"q=h;f=r;h=l;s=0.7")

	call gflush (gp)
        call gclose (gp)
	call sfree (sp)
end
# Oct 10 1995	New program
# Oct 13 1995	Add option to plot intensity as log
