# File procdata/Src/fsplot.x
# October 10, 1995
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# Copyright(c) 1995 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.

#  FSPLOT uses IRAF plot routines to plot a spectrum on STDGRAPH.
#  Adapted from splot in the IRAF onedspec package
 
include	<gset.h>
include <mach.h>
include <imhdr.h>
include <imio.h>
include	<pkg/gtools.h>

define	MAXBSECT	10

procedure fsplot (image, xvec, yvec, row1,row2, zmin, zmax, col1, col2,
		  wcslab, format, imax,il,ir, nsect, bsect, ns)

char	image[SZ_FNAME]		# Image name
real	xvec[ARB]	# Image rows (X coordinates) for plotted vector
real	yvec[ARB]	# Pixel medians (Y coordinates) for plotted vector
int	row1, row2	# Selected rows
real	zmin, zmax	# Vector min max
int	col1, col2	# Selected columns
char	wcslab[ARB]	# WCS label
char	format[ARB]	# WCS format
int	imax		# Line of maximum peak
int	il,ir		# Left and right limits of peak
int	nsect		# Left background limits
int	bsect[MAXBSECT,2]	# Background sections
int	ns		# Number of times spectrum is smoothed
 
pointer	gfd,gt		# Pointers for graphics control structures
int	npoints		# Number of points in spectrum
int	ipix1		# first pixel to plot
int	ipix2		# last pixel to plot
int	npix		# number of points to plot
char	command[SZ_FNAME]
int	wc, key, itemp
bool	newgraph, dfirst, zfirst, interact
int	i, ix, i1, i2, ix0
real	wx, wy, wx0
char    device[SZ_FNAME] # Display on which to plot data
char    plotter[SZ_FNAME] # Device on which to make hard copies
char    linebuf[SZ_LINE] # Line buffer for input
 
int	gt_gcur(), getline()
bool	clgetb()
pointer	gopen(), gt_init()
 
begin

# Open display and eliminate y-axis minor ticks.
	call clgstr("device", device, SZ_FNAME)
	gfd = gopen (device, NEW_FILE, STDGRAPH)
	If (gfd == ERR) {
	    call printf ("PLOTSPEC:  cannot open display %s\n")
		call pargstr (device)
	    return
	    }
	call gseti (gfd, G_YNMINOR, NO)
	call gseti (gfd, G_XNMINOR, YES)
	call gsetr (gfd, G_TICKLABELSIZE, 0.8)
 
# Initialize graph format
	gt = gt_init()
	call loghead ("FINDSPEC",linebuf)
	call gt_seti (gt, GTSYSID, NO)
	call gt_sets (gt, GTTITLE, linebuf)
	call gt_sets (gt, GTSUBTITLE, image)
	call gt_sets (gt, GTXLABEL, "Row")

# Initial plot of data array
	ipix1 = row1
	ipix2 = row2
	call fsaplot (gfd, gt, image, xvec, yvec, row1,row2, zmin, zmax, col1,
		      col2, wcslab, format, imax,il,ir, nsect, bsect, ns)
	newgraph = false
	dfirst = true
	zfirst = true
	interact = clgetb ("interactive")
	if (!interact) {
	    call gclear (gfd)
	    call gclose (gfd)
	    call gt_free (gt)
	    return
	    }
 
# Handle cursor interaction
	while (gt_gcur ("cursor", wx, wy, wc, key, command, SZ_FNAME) != EOF) {

# Find closest pixel within display limits
	    if (wx <= xvec[ipix1])
		ix = ipix1
	    else if (wx >= xvec[ipix2])
		ix = ipix2
	    else {
		i1 = ipix1
		i2 = ipix2 - 1
		do i = i1, i2 {
		    if (wx >= xvec[i] && wx < xvec[i+1]) {
			if (wx - xvec[i] > 0.5)
			    ix = i
			else
			    ix = i + 1
			break
			}
		    }
		}
	    switch (key) {
		case 'r':
		    newgraph = true

#	Make a hardcopy of the current display
		case '@':
		    call gflush (gfd)
		    call gclose (gfd)
		    call printf ("Making hard copy on stdplot\n")
		    call flush (STDOUT)
		    call clgstr("plotter", plotter, SZ_FNAME)
		    gfd = ERR
		    gfd = gopen (plotter, APPEND, STDGRAPH)
		    If (gfd == ERR) {
			call printf ("PLOTSPEC:  cannot open plotter %s\n")
			    call pargstr (plotter)
			next
			}
		    call fsaplot (gfd, gt, image, xvec[ipix1], yvec[ipix1],
				  ipix1,ipix2, zmin, zmax, col1, col2,
				  wcslab, format, imax,il,ir,
				  nsect, bsect, ns)
		    call gflush (gfd)
		    call gclose (gfd)
		    gfd = gopen (device, APPEND, STDGRAPH)
		    call gseti (gfd, G_YNMINOR, NO)
		    call gseti (gfd, G_XNMINOR, YES)
		    call gsetr (gfd, G_TICKLABELSIZE, 0.8)

#	Set number of times to smooth spectrum
		case 'g':
		    call gclose (gfd)
		    call printf("Number of times to smooth spectrum (%d): ")
			call pargi (ns)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 0)
			ns = i
		    gfd = gopen (device, APPEND, STDGRAPH)
		    newgraph = true

#	Reinitialize two-keystroke commands
		case 'x':
		    if (!zfirst) {
			call gclose (gfd)
			call printf("zoom cancelled\n")
			call flush (STDOUT)
			gfd = gopen (device, APPEND, STDGRAPH)
			zfirst = true
			}

#	Zoom in on section of graph
		case 'z':
		    if (zfirst) {
			ipix1 = ix
			zfirst = false
			call gclose (gfd)
			call printf("z again to zoom from %5.0f\n")
			    call pargr (xvec[ipix1])
			call flush (STDOUT)
			}
		    else {
			ipix2 = ix
			zfirst = true
			if (ipix2 < ipix1) {
			    itemp = ipix1
			    ipix1 = ipix2
			    ipix2 = itemp
			    }
			npix = ipix2 - ipix1 + 1
			call gclose (gfd)
			call printf("zooming from %5.0f to %5.0f\n")
			    call pargr (xvec[ipix1])
			    call pargr (xvec[ipix2])
			call flush (STDOUT)
			newgraph = true
			}
		    gfd = gopen (device, APPEND, STDGRAPH)

#	Unzoom
		case 'u':
		    ipix1 = 1
		    ipix2 = npoints
		    npix = npoints
		    newgraph = true

#	Print menu
		case 'h','?',' ':
		    call gclose (gfd)
		    call printf ("@  make hard copy of screen\n")
		    call printf ("a  select aperture center\n")
		    call printf ("c  show x and y coordinates\n")
		    call printf ("f  reset background automatically\n")
		    call printf ("g  change smoothing\n")
		    call printf ("h  this menu\n")
		    call printf ("q  leave plot\n")
		    call printf ("r  replot current graph\n")
		    call printf ("u  unzoom\n")
		    call printf ("x  cancel zoom after 1st z\n")
		    call printf ("z  zoom between 1st and 2nd positions\n")
		    call printf ("?  this menu\n")
		    call flush (STDOUT)
		    gfd = gopen (device, APPEND, STDGRAPH)

#	Set estimate of spectrum center to the cursor value
		case 'P','p':
		    call gclose (gfd)
		    ix0 = ix
		    wx0 = wx
		    call printf ("Set spectrum center at %d\n")
			call pargi (ix)
		    call flush (STDOUT)
		    gfd = gopen (device, APPEND, STDGRAPH)

		case 'Q','q':
		    break

#	Default is to print coordinates of cursor
		default:
		    call gclose (gfd)
		    call printf ("x,y: %10.3f %10.4g\n")
			call pargr (wx)
			call pargr (wy)
		    call flush (STDOUT)
		    gfd = gopen (device, APPEND, STDGRAPH)
		}
 
#	Replot graph if necessary
	    if (newgraph) {
		call fsaplot (gfd, gt, image, xvec[ipix1], yvec[ipix1],
			      ipix1,ipix2, zmin, zmax, col1, col2,
			      wcslab, format, imax,il,ir, nsect, bsect, ns)
		newgraph = false
		}
	    }
 
# Close graph window
 
	call gclear (gfd)
	call gclose (gfd)
	call gt_free (gt)
	return
end
