# File procdata/Src/setplot.x
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
# March 21, 1994

include <gset.h>
include <pkg/gtools.h>

# PLOT_SPEC - Handle drawing of vector to the screen.

procedure plot_spec (image, xvec, yvec, row1,row2, zmin, zmax, col1, col2,
		     wcslab, format, imax,il,ir, ilb1,ilb2,irb1,irb2, ns)

char	image[SZ_FNAME]			# Image name
real	xvec[ARB], yvec[ARB]		# Vectors to be plot
int	row1, row2			# Selected rows
real	zmin, zmax			# Vector min max
int	col1, col2			# Selected columns
char	wcslab[ARB]			# WCS label
char	format[ARB]			# WCS format
int	imax				# Line of maximum peak
int	il,ir				# Left and right limits of peak
int	ilb1,ilb2			# Left background limits
int	irb1,irb2			# Right background limits
int	ns				# Number of times spectrum is smoothed

bool	interact
pointer	sp, gp, gt
pointer	device
begin
	call smark (sp)
	call salloc (device, SZ_FNAME, TY_CHAR)

	call clgstr ("device", Memc[device], SZ_FNAME)
	mode = NEW_FILE
	gp = gopen (Memc[device], mode, STDGRAPH)

	call draw_find (gp,image, xvec, yvec, row1,row2, zmin, zmax, col1, col2,
		        wcslab, format, imax,il,ir, ilb1,ilb2,irb1,irb2, ns)

	interact = clgetb ("interactive")
	if (interact) {
	if (newgraph) {
	    call draw_spec (gp,gt,image,xvec,yvec,row1,row2,zmin,zmax,col1,col2,
		        wcslab, format, imax,il,ir, ilb1,ilb2,irb1,irb2, ns)
	    }
        call gclose (gp)
end

# DRAW_SPEC - Draw vector to the screen.
procedure draw_spec (gp,gt,image, xvec,yvec,row1,row2,zmin,zmax,col1,col2,
		     wcslab, format, imax,il,ir, ilb1,ilb2,irb1,irb2, ns)

char	image[SZ_FNAME]			# Image name
real	xvec[ARB], yvec[ARB]		# Vectors to be plot
int	row1, row2			# Selected rows
real	zmin, zmax			# Vector min max
int	col1, col2			# Selected columns
char	wcslab[ARB]			# WCS label
char	format[ARB]			# WCS format
int	imax				# Line of maximum peak
int	il,ir				# Left and right limits of peak
int	ilb1,ilb2			# Left background limits
int	ns				# Number of times spectrum is smoothed

pointer	sp, gp
pointer	device, xlabel, ylabel, title, suffix, caption
real	wx1, wx2, wy1, wy2, szm, x, dx, y, y1, y2, dy
real	zdiff,y0
int	mode, imark
int	nlines				# Npts in vector
bool	pointmode

pointer	gopen()
real	clgetr(), amedr()
real	pfrac,bfrac

begin
	call smark (sp)
	call salloc (device, SZ_FNAME, TY_CHAR)
	call salloc (xlabel, SZ_LINE,  TY_CHAR)
	call salloc (ylabel, SZ_LINE,  TY_CHAR)
	call salloc (title,  SZ_LINE,  TY_CHAR)
	call salloc (caption,  SZ_LINE,  TY_CHAR)
	call salloc (suffix, SZ_FNAME, TY_CHAR)

	call gclear (gp)
	call strcpy (wcslab, Memc[xlabel], SZ_LINE)
	call strcpy ("DRAWSPEC: ", Memc[title], SZ_LINE)
	call strcat (image, Memc[title], SZ_LINE)
	if (col1 != col2) {
	    call sprintf (Memc[suffix], SZ_FNAME, ": columns %d to %d")
		call pargi (col1)
		call pargi (col2)
	    }
	else {
	    call sprintf (Memc[suffix], SZ_FNAME, ": column %d")
		call pargi (col1)
	    }
	call strcat (Memc[suffix], Memc[title], SZ_LINE)
	if (ns > 0) {
	    call sprintf (Memc[suffix],SZ_FNAME," %dx smoothed")
		call pargi (ns)
	    call strcat (Memc[suffix], Memc[title], SZ_LINE)
	    }

	Memc[ylabel] = EOS

	# Set window limits
	nlines = row2 - row1 + 1
	wx1 = x_vec[1]
	wx2 = x_vec[nlines]

	zdiff = (zmax - zmin) * 0.1
	wy1 = zmin - zdiff
	wy2 = zmax + (zdiff * 2.)

	call gswind (gp, wx1, wx2, wy1, wy2)
    
	call gsets (gp, G_XTICKFORMAT, format)
	call gseti (gp, G_XNMAJOR, 5)
	call gseti (gp, G_XNMINOR, 5)
	call gseti (gp, G_YNMAJOR, 5)
	call gseti (gp, G_YNMINOR, 5)
	call gseti (gp, G_ROUND, NO)

# Draw axes using all this information.
	call glabax (gp, Memc[title], Memc[xlabel], Memc[ylabel])
    
# Now actually draw the data
        if (pointmode)
            call gpmark (gp, xvec, yvec, nlines, imark, szm, szm)
        else
            call gpline (gp, xvec, yvec, nlines)

# Mark the peak
	dy = 0.1 * (wy2 - wy1)
	x = imax
	y1 = yvec[imax-row1+1] + dy
	y2 = y1 + dy
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)

# Mark the spectrum
	x = imax + il
	y0 = amedr (yvec,nlines)
	y1 = y0 - dy
	y2 = yvec[imax-row1+1] + dy
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)

	x = imax + ir
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)

# Mark the background
	y2 = y0 + dy
	x = imax + ilb1
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)
	x = imax + ilb2
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)
	x = imax + irb1
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)
	x = imax + irb2
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)
	call gflush (gp)

# Print results of search on plot
	dx = 0.1 * (wx2 - wx1)
	x = imax + dx
	y = wy2 - dy
	pfrac = clgetr ("fracpeak")
	call sprintf (Memc[caption],SZ_LINE,"Spectrum at %d, %d:%d [%4.2f]")
	    call pargi (imax)
	    call pargi (imax+il)
	    call pargi (imax+ir)
	    call pargr (pfrac)
	call gtext (gp, x,y,Memc[caption],"q=h;f=r;h=l;s=0.7")
	call gflush (gp)

	y = y - (dy * 0.4)
	bfrac = clgetr ("fracback")
	call sprintf (Memc[caption],SZ_LINE,"Background from %d:%d,%d:%d [%4.2f]")
	    call pargi (imax+ilb2)
	    call pargi (imax+ilb1)
	    call pargi (imax+irb1)
	    call pargi (imax+irb2)
	    call pargr (bfrac)
	call gtext (gp, x,y, Memc[caption],"q=h;f=r;h=l;s=0.7")

	call gflush (gp)
	call sfree (sp)
end
