# File procdata/Src/t_setspec.x
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
# March 21, 1994

include	<gset.h>
include	<mach.h>
include	<imhdr.h>
include	<mwset.h>
include	<math/curfit.h>
include "apertures.h"

# T_SETSPEC -- Find the spectrum in a range of columns from an image.

procedure t_setspec ()

pointer	image, wcslab, fmt
pointer	im, mw, ct, sp, x_vec, y_vec, impath
int	col1, col2, ncols, nlines, row1, row2, itemp, nap, lpath
real	zmin, zmax, xmin, xmax
int	clgeti(), strlen()
pointer	immap(), mw_openim(), mw_sctran()
real	clgetr()
bool	verbose, clgetb()
bool	dispap		# Plot aperture and background limits
bool	dbap		# Save aperture and background limits in database

int	apid		# Aperture ID
int	apbeam		# Aperture beam number
int	apaxis		# Aperture axis
real	apcenter	# Center along the aperture axis
real	dispcenter	# Center along the dispersion axis
pointer	ap		# Aperture pointer

int	imax		# Bin containing maximum (computed if zero)
int	il		# first bin for peak fit (returned)
int	ir		# last bin for peak fit (returned)
int	ilb1,ilb2	# Left background limits
int	irb1,irb2	# Right background limits
int	nbackpix	# Number of pixels in background (on each side)
int	nrows		# Number of rows to check and plot
int	nsmooth		# Number of times to smooth vector before search
int	dispaxis	# Dispersion axis
char	bsample[SZ_LINE] # Pixel range for background
real	a, b, c[1]
pointer	ic, str

begin
	call smark (sp)
	call salloc (image, SZ_FNAME, TY_CHAR)
	call salloc (impath, SZ_FNAME, TY_CHAR)
	call salloc (wcslab, SZ_LINE, TY_CHAR)
	call salloc (fmt, SZ_LINE, TY_CHAR)

	# Open image
	call clgstr ("image", Memc[image], SZ_FNAME)
	call clgstr ("imdir", Memc[impath], SZ_FNAME)
	lpath = strlen (Memc[impath])
	if (Memc[impath+lpath-1] != '/') {
	    Memc[impath+lpath] = '/'
	    Memc[impath+lpath+1] = EOS
	    }
	call strcat (Memc[image], Memc[impath], SZ_FNAME)

	im = immap (Memc[impath], READ_ONLY, 0)
	call strcpy ("logical", Memc[wcslab], SZ_LINE)
	mw = mw_openim (im)
	call mw_seti (mw, MW_USEAXMAP, NO)
	ct = mw_sctran (mw, "logical", Memc[wcslab], 0)

	# Dimensions of image
	ncols  = IM_LEN(im,1)
	nlines = IM_LEN(im,2)

	# Range of columns within which to find median
	col1 = clgeti ("col1")
	col2 = clgeti ("col2")
	if (col2 < col1) {
	    itemp = col1
	    col1 = col2
	    col2 = itemp
	    }
	if (col1 == INDEFI && col2 == INDEFI) {
	    col1 = (ncols / 2) - 50
	    col2 = (ncols / 2) + 50
	    }
	else if (col1 != INDEFI && col2 == INDEFI) {
	    col2 = col1 + 100
	    }
	else if (col1 == INDEFI && col2 != INDEFI) {
	    col1 = col2 - 100
	    }
	if (min(col1,col2) < 1 || max(col1,col2) > ncols) {
	    call imunmap (im)
	    call error (2, "column index references outside image")
	    }

	# Range of rows within which to find object spectrum
	row1 = clgeti ("row1")
	row2 = clgeti ("row2")
	if (row1 == INDEFI && row2 == INDEFI) {
	    row1 = 1
	    row2 = nlines
	    }
	else if (row1 != INDEFI && row2 == INDEFI) {
	    row2 = nlines
	    }
	else if (row1 == INDEFI && row2 != INDEFI) {
	    row1 = 1
	    }
	if (row2 < row1) {
	    itemp = row1
	    row1 = row2
	    row2 = itemp
	    }
	nrows = row2 - row1 + 1

	# Get the median for the requested columns from the image.
	call malloc (x_vec, nlines, TY_REAL)
	call malloc (y_vec, nlines, TY_REAL)
	call plt_gcols (im, mw, ct, col1,col2, row1,row2,
	    Memr[x_vec], Memr[y_vec], zmin, zmax, Memc[wcslab], Memc[fmt],
	    SZ_LINE)

	# Smooth data, if requested
	nsmooth = 0
	nsmooth = clgeti ("nsmooth")
	if (nsmooth > 0)
	    call smooth (Memr[yvec],nrows,nsmooth)

	# Find peak in vector
	imax = 0
	call pkwidth (nrows, row1, Memr[y_vec], imax, il, ir, ilb1, irb1)
	nbackpix = clgeti ("nbackpix")
	ilb2 = ilb1 - nbackpix
	irb2 = irb1 + nbackpix
	ilb2 = ilb2 - imax
	ilb1 = ilb1 - imax
	irb1 = irb1 - imax
	irb2 = irb2 - imax

	# Set spectrum limits
	il = il - imax
	ir = ir - imax
	call clputi ("center",imax)
	call clputi ("lower",il)
	call clputi ("upper",ir)
	verbose = clgetb ("verbose")
	if (verbose) {
	    call printf ("SETSPEC: %s spectrum centered at %d, %d:%d\n")
		call pargstr (Memc[image])
		call pargi (imax)
		call pargi (imax+il)
		call pargi (imax+ir)
	    call flush (STDOUT)
	    }

	# Set background limits
	call sprintf (bsample,SZ_LINE,"%d:%d,%d:%d")
	    call pargi (ilb2)
	    call pargi (ilb1)
	    call pargi (irb1)
	    call pargi (irb2)
	call clpstr ("b_sample",bsample)

	if (verbose) {
	    call printf ("SETSPEC: Background sampled from %d:%d,%d:%d\n")
		call pargi (imax+ilb2)
		call pargi (imax+ilb1)
		call pargi (imax+irb1)
		call pargi (imax+irb2)
	    call flush (STDOUT)
	    }

	# Draw the vector to the screen.
	dispap = clgetb ("dispap")
	if (dispap) 
	    call plot_find (Memc[image], Memr[x_vec], Memr[y_vec],
			    row1,row2, zmin, zmax, col1, col2,
			    Memc[wcslab], Memc[fmt], imax,il,ir,
			    ilb1,ilb2,irb1,irb2,nsmooth)

# Create a default aperture.
	dbap = clgetb ("dbap")
	if (dbap) {
#	Specify aperture ID, beam, axis, and center in both dimensions.
#	Aperture limits along the dispersion axis are set to the full size
#	of the image while along the dispersion axis they are queried.
#	The default offset curve is a constant zero curve.
	    apid = 1
	    apbeam = 1
	    dispaxis = 1
	    call imgipar (im, "DISPAXIS", dispaxis)
	    apaxis = mod (dispaxis, 2) + 1
	    apcenter = imax
	    dispcenter = (col1 + col2) * 0.5

	    call ap_alloc (ap)
	    AP_ID(ap) = 1
	    AP_BEAM(ap) = 1
	    AP_AXIS(ap) = apaxis
	    AP_CEN(ap, apaxis) = imax
	    AP_LOW(ap, apaxis) = il
	    AP_HIGH(ap, apaxis) = ir
	    AP_CEN(ap, dispaxis) = dispcenter
	    AP_LOW(ap, dispaxis) = 1 - dispcenter
	    AP_HIGH(ap, dispaxis) = IM_LEN(im, dispaxis) - dispcenter

# Set the trace curve.
# If the input template aperture is NULL then the output trace curve
# is set to a constant zero otherwise a copy from the input template
# aperture is made.

	# Determine the limits over which the curve is defined.
	    a = AP_CEN(ap, dispaxis) + AP_LOW(ap, dispaxis)
	    b = AP_CEN(ap, dispaxis) + AP_HIGH(ap, dispaxis)
	    if (a == b)
		b = b + 1

	# Set the curve to a legendre polynomial of order 1 and value 0.
	    c[1] = 0.
	    call cvset (AP_CV(ap), LEGENDRE, a, b, c, 1)

# Set the background fitting ICFIT structure for an aperture.
# If the input template aperture is NULL then the output background fitting
# ICFIT pointer is initialized otherwise a copy from the input template
# aperture is made.

	    call ic_open (AP_IC(ap))
	    ic = AP_IC(ap)

	    call salloc (str, SZ_LINE, TY_CHAR)
	    call clgstr ("b_function", Memc[str], SZ_LINE)
	    call ic_pstr (ic, "function", Memc[str])
	    call ic_puti (ic, "order", clgeti ("b_order"))
	    call ic_pstr (ic, "sample", bsample)
	    call ic_puti (ic, "naverage", clgeti ("b_naverage"))
	    call ic_puti (ic, "niterate", clgeti ("b_niterate"))
	    call ic_putr (ic, "low", clgetr ("b_low"))
	    call ic_putr (ic, "high", clgetr ("b_high"))
	    call ic_putr (ic, "grow", clgetr ("b_grow"))
	    if (AP_AXIS(ap) == 1)
		call ic_pstr (ic, "xlabel", "Column")
	    else
		call ic_pstr (ic, "xlabel", "Line")

	# Set background min and max based on sample regions.
	    xmin = ilb2
	    xmax = irb2
	    call ic_putr (ic, "xmin", xmin)
	    call ic_putr (ic, "xmax", xmax)

	    nap = 1
	    call ap_dbwrite (Memc[image], ap, nap)
	    call apfree (ap)
	    }

	# Free resources.
	call mfree (x_vec, TY_REAL) 
	call mfree (y_vec, TY_REAL) 
	call imunmap (im) 
	call sfree (sp) 
end


# PLT_GCOLS -- Get median of specified columns from an image.  The average
# data vector is returned as y_vector; the line coordinates are returned in
# x_vector.  The data vector min and max are also returned.
 
procedure plt_gcols (im, mw, ct, col1,col2, row1,row2, x_vector, y_vector,
	zmin, zmax, wcslab, format, sz_wcslab)
 
pointer im              # Pointer to image header
pointer	mw		# MWCS  pointer
pointer	ct		# CT pointer
int     col1            # First column to extract
int     col2            # Last column to extract
int     row1            # First row to extract
int     row2            # Last row to extract
real    x_vector[ARB]   # The row ordinal values (returned)
real    y_vector[ARB]   # The column data values (returned)
real    zmin, zmax      # Minimum and maximum data values (returned)
char	wcslab[sz_wcslab]	# WCS label if present
char	format[sz_wcslab]	# WCS format if present
int	sz_wcslab	# String length
 
int     i, nrows, ncols, irow
pointer	sp, axvals, off, imgl2r()
real    amedr()
 
begin
	call smark (sp)
	call salloc (axvals, IM_MAXDIM, TY_REAL)

        # Fill x and y arrays.
        nrows = row2 - row1 + 1
        ncols = col2 - col1 + 1
 
	Memr[axvals] = (col1 + col2) / 2.
	call plt_wcs (im, mw, ct, 2, Memr[axvals], real(row1), real(row2),
	    x_vector, nrows, wcslab, format, sz_wcslab)

	i = 0
        do irow = row1, row2 {
            off = imgl2r (im, irow)
	    i = i + 1
	    x_vector[i] = real (irow)
            y_vector[i] = amedr (Memr[off+col1-1], ncols)
            }
 
        # Find min and max values in y array.
        call alimr (y_vector, nrows, zmin, zmax)

	call sfree (sp)
end


# PLOT_FIND - Handle drawing of vector to the screen.

procedure plot_find (image, xvec, yvec, row1,row2, zmin, zmax, col1, col2,
		     wcslab, format, imax,il,ir, ilb1,ilb2,irb1,irb2, ns)

char	image[SZ_FNAME]			# Image name
real	xvec[ARB], yvec[ARB]		# Vectors to be plot
int	row1, row2			# Selected rows
real	zmin, zmax			# Vector min max
int	col1, col2			# Selected columns
char	wcslab[ARB]			# WCS label
char	format[ARB]			# WCS format
int	imax				# Line of maximum peak
int	il,ir				# Left and right limits of peak
int	ilb1,ilb2			# Left background limits
int	irb1,irb2			# Right background limits
int	ns				# Number of times spectrum is smoothed

bool	interact
pointer	sp, gp, gt
pointer	device
begin
	call smark (sp)
	call salloc (device, SZ_FNAME, TY_CHAR)

	call clgstr ("device", Memc[device], SZ_FNAME)
	mode = NEW_FILE
	gp = gopen (Memc[device], mode, STDGRAPH)

	call draw_find (gp,image, xvec, yvec, row1,row2, zmin, zmax, col1, col2,
		        wcslab, format, imax,il,ir, ilb1,ilb2,irb1,irb2, ns)

	interact = clgetb ("interactive")
	if (interact) {
	if (newgraph) {
	    call draw_find (gp,gt,image,xvec,yvec,row1,row2,zmin,zmax,col1,col2,
		        wcslab, format, imax,il,ir, ilb1,ilb2,irb1,irb2, ns)
	    }
        call gclose (gp)
end

# DRAW_FIND - Draw vector to the screen.
procedure draw_find (gp,gt,image, xvec,yvec,row1,row2,zmin,zmax,col1,col2,
		     wcslab, format, imax,il,ir, ilb1,ilb2,irb1,irb2, ns)

char	image[SZ_FNAME]			# Image name
real	xvec[ARB], yvec[ARB]		# Vectors to be plot
int	row1, row2			# Selected rows
real	zmin, zmax			# Vector min max
int	col1, col2			# Selected columns
char	wcslab[ARB]			# WCS label
char	format[ARB]			# WCS format
int	imax				# Line of maximum peak
int	il,ir				# Left and right limits of peak
int	ilb1,ilb2			# Left background limits
int	ns				# Number of times spectrum is smoothed

pointer	sp, gp
pointer	device, xlabel, ylabel, title, suffix, caption
real	wx1, wx2, wy1, wy2, szm, x, dx, y, y1, y2, dy
real	zdiff,y0
int	mode, imark
int	nlines				# Npts in vector
bool	pointmode

pointer	gopen()
real	clgetr(), amedr()
real	pfrac,bfrac

begin
	call smark (sp)
	call salloc (device, SZ_FNAME, TY_CHAR)
	call salloc (xlabel, SZ_LINE,  TY_CHAR)
	call salloc (ylabel, SZ_LINE,  TY_CHAR)
	call salloc (title,  SZ_LINE,  TY_CHAR)
	call salloc (caption,  SZ_LINE,  TY_CHAR)
	call salloc (suffix, SZ_FNAME, TY_CHAR)

	call gclear (gp)
	call strcpy (wcslab, Memc[xlabel], SZ_LINE)
	call strcpy ("SETSPEC: ", Memc[title], SZ_LINE)
	call strcat (image, Memc[title], SZ_LINE)
	if (col1 != col2) {
	    call sprintf (Memc[suffix], SZ_FNAME, ": columns %d to %d")
		call pargi (col1)
		call pargi (col2)
	    }
	else {
	    call sprintf (Memc[suffix], SZ_FNAME, ": column %d")
		call pargi (col1)
	    }
	call strcat (Memc[suffix], Memc[title], SZ_LINE)
	if (ns > 0) {
	    call sprintf (Memc[suffix],SZ_FNAME," %dx smoothed")
		call pargi (ns)
	    call strcat (Memc[suffix], Memc[title], SZ_LINE)
	    }

	Memc[ylabel] = EOS

	# Set window limits
	nlines = row2 - row1 + 1
	wx1 = x_vec[1]
	wx2 = x_vec[nlines]

	zdiff = (zmax - zmin) * 0.1
	wy1 = zmin - zdiff
	wy2 = zmax + (zdiff * 2.)

	call gswind (gp, wx1, wx2, wy1, wy2)
    
	call gsets (gp, G_XTICKFORMAT, format)
	call gseti (gp, G_XNMAJOR, 5)
	call gseti (gp, G_XNMINOR, 5)
	call gseti (gp, G_YNMAJOR, 5)
	call gseti (gp, G_YNMINOR, 5)
	call gseti (gp, G_ROUND, NO)

# Draw axes using all this information.
	call glabax (gp, Memc[title], Memc[xlabel], Memc[ylabel])
    
# Now actually draw the data
        if (pointmode)
            call gpmark (gp, xvec, yvec, nlines, imark, szm, szm)
        else
            call gpline (gp, xvec, yvec, nlines)

# Mark the peak
	dy = 0.1 * (wy2 - wy1)
	x = imax
	y1 = yvec[imax-row1+1] + dy
	y2 = y1 + dy
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)

# Mark the spectrum
	x = imax + il
	y0 = amedr (yvec,nlines)
	y1 = y0 - dy
	y2 = yvec[imax-row1+1] + dy
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)

	x = imax + ir
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)

# Mark the background
	y2 = y0 + dy
	x = imax + ilb1
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)
	x = imax + ilb2
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)
	x = imax + irb1
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)
	x = imax + irb2
	call gamove (gp,x,y1)
	call gadraw (gp,x,y2)
	call gflush (gp)

# Print results of search on plot
	dx = 0.1 * (wx2 - wx1)
	x = imax + dx
	y = wy2 - dy
	pfrac = clgetr ("fracpeak")
	call sprintf (Memc[caption],SZ_LINE,"Spectrum at %d, %d:%d [%4.2f]")
	    call pargi (imax)
	    call pargi (imax+il)
	    call pargi (imax+ir)
	    call pargr (pfrac)
	call gtext (gp, x,y,Memc[caption],"q=h;f=r;h=l;s=0.7")
	call gflush (gp)

	y = y - (dy * 0.4)
	bfrac = clgetr ("fracback")
	call sprintf (Memc[caption],SZ_LINE,"Background from %d:%d,%d:%d [%4.2f]")
	    call pargi (imax+ilb2)
	    call pargi (imax+ilb1)
	    call pargi (imax+irb1)
	    call pargi (imax+irb2)
	    call pargr (bfrac)
	call gtext (gp, x,y, Memc[caption],"q=h;f=r;h=l;s=0.7")

	call gflush (gp)
	call sfree (sp)
end

# Mar 17 1994	Add interactive graphics
