# File procdata/Src/fsaplot.x
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
# February 21, 1996

define	MAXBSECT	10

include <gset.h>
include <imhdr.h>

# FSAPLOT - Draw and label an aperture and its associated background

procedure fsaplot (gfd,gt,image, xvec, yvec, row1,row2, col1, col2,
		   wcslab, format, imax,il,ir, ns, autoscale)

pointer	gfd,gt			# Pointers for graphics control
char	image[SZ_FNAME]		# Image name
real	xvec[ARB], yvec[ARB]	# Vectors to be plot
int	row1, row2		# Selected rows
int	col1, col2		# Selected columns
char	wcslab[ARB]		# WCS label
char	format[ARB]		# WCS format
int	imax			# Line of maximum peak
int	il,ir			# Left and right limits of peak
int	ns			# Number of times spectrum is smoothed
bool	autoscale		# If true, scale to data used, else all

pointer	sp
pointer	marker, xlabel, ylabel, title, suffix, caption
real	zmin, zmax		# Vector min max
real	wx1, wx2, wy1, wy2, szm, x, y, y1, y2, dy
real	zdiff,y0
int	imark, n, row0, i, i1, i2
int	nlines				# Npts in vector
bool	pointmode
char	bsample[SZ_LINE] # Background sections as a string
char	temp[SZ_LINE]

real	amedr()

include	"fspec.com"

begin
	pointmode = FALSE

	call smark (sp)
	call salloc (marker, SZ_FNAME, TY_CHAR)
	call salloc (xlabel, SZ_LINE,  TY_CHAR)
	call salloc (ylabel, SZ_LINE,  TY_CHAR)
	call salloc (title,  SZ_LINE,  TY_CHAR)
	call salloc (caption,  SZ_LINE,  TY_CHAR)
	call salloc (suffix, SZ_FNAME, TY_CHAR)

# Make image cross-section label
	call strcpy (wcslab, Memc[xlabel], SZ_LINE)
	call strcpy ("FINDSPEC: ", Memc[title], SZ_LINE)
	call strcat (image, Memc[title], SZ_LINE)
	if (col1 != col2) {
	    call sprintf (Memc[suffix], SZ_FNAME, ": columns %d to %d")
		call pargi (col1)
		call pargi (col2)
	    }
	else {
	    call sprintf (Memc[suffix], SZ_FNAME, ": column %d")
		call pargi (col1)
	    }
	call strcat (Memc[suffix], Memc[title], SZ_LINE)
	if (ns > 0) {
	    call sprintf (Memc[suffix],SZ_FNAME," %dx smoothed")
		call pargi (ns)
	    call strcat (Memc[suffix], Memc[title], SZ_LINE)
	    }

	Memc[ylabel] = EOS

	# Set window x-axis limits
	nlines = row2 - row1 + 1
	wx1 = x_vec[1]
	wx2 = x_vec[nlines]

	# Set window y-axis limits
	row0 = row1 - 1

	# Scale using only peak and background
	if (autoscale) {
	    zmin = yvec[imax-row0]
	    zmax = yvec[imax-row0]
	    i1 = imax + il - row0
	    i2 = imax + ir - row0
	    do i = i1, i2 {
		if (yvec[i] > zmax)
		    zmax = yvec[i]
		if (yvec[i] < zmin)
		    zmin = yvec[i]
		}
	    do n = 1, nsect {
		i1 = bsect[n,1] - row0
		i2 = bsect[n,2] - row0
		do i = i1, i2 {
		    if (yvec[i] > zmax)
			zmax = yvec[i]
		    if (yvec[i] < zmin)
			zmin = yvec[i]
		    }
		}
	    }

	# Scale using entire cross-dispersion vector
	else
	    call alimr (yvec, nlines, zmin, zmax)
	zdiff = (zmax - zmin) * 0.1
	wy1 = zmin - zdiff
	wy2 = zmax + (zdiff * 2.)

	call gclear (gfd)
	call gswind (gfd, wx1, wx2, wy1, wy2)
	call gt_swind (gfd,gt)
	call gsets (gfd, G_XTICKFORMAT, format)
	call gseti (gfd, G_XNMAJOR, 5)
	call gseti (gfd, G_XNMINOR, 5)
	call gseti (gfd, G_YNMAJOR, 5)
	call gseti (gfd, G_YNMINOR, 5)
	call gseti (gfd, G_ROUND, NO)

# Draw axes using all this information.
	call glabax (gfd, Memc[title], Memc[xlabel], Memc[ylabel])
    
# Now actually draw the data
        if (pointmode)
            call gpmark (gfd, xvec, yvec, nlines, imark, szm, szm)
        else
            call gpline (gfd, xvec, yvec, nlines)

# Mark the peak
	dy = 0.1 * (wy2 - wy1)
	x = imax
	y1 = yvec[imax-row0] + dy
	y2 = y1 + dy
	call gamove (gfd,x,y1)
	call gadraw (gfd,x,y2)

# Mark the spectrum
	x = imax + il
	y2 = yvec[imax-row0] + dy
	call gamove (gfd,x,wy1)
	call gadraw (gfd,x,y2)

	x = imax + ir
	call gamove (gfd,x,wy1)
	call gadraw (gfd,x,y2)

# Mark the background
	y0 = amedr (yvec,nlines)
	y2 = y0 + dy
	do n = 1, nsect {
	    x = bsect[n,1]
	    call gamove (gfd,x,wy1)
	    call gadraw (gfd,x,y2)
	    x = bsect[n,2]
	    call gamove (gfd,x,wy1)
	    call gadraw (gfd,x,y2)
	    }
	call gflush (gfd)

# Print spectrum range
	x = wx1
	y = wy2 - (dy * 0.4)
	call sprintf (Memc[caption],SZ_LINE,"Spectrum at %d, %d:%d [%4.2f]")
	    call pargi (imax)
	    call pargi (imax+il)
	    call pargi (imax+ir)
	    call pargr (pfrac)
	call gtext (gfd, x,y,Memc[caption],"q=h;f=r;h=l;s=0.7")
	call gflush (gfd)

# Print background sample range
	bsample[1] = EOS
	do n = 1, nsect {
	    call sprintf (temp,SZ_LINE,"%d:%d")
		call pargi (bsect[n,1])
		call pargi (bsect[n,2])
	    call strcat (temp,bsample,SZ_LINE)
	    if (n < nsect)
		call strcat (",",bsample,SZ_LINE)
	    }
	y = y - (dy * 0.4)
	call sprintf (Memc[caption],SZ_LINE,"Background from %s [%4.2f]")
	    call pargstr (bsample)
	    call pargr (bfrac)
	call gtext (gfd, x,y, Memc[caption],"q=h;f=r;h=l;s=0.7")

	call gflush (gfd)
	call sfree (sp)
end
# Oct 12 1995	New program
# Nov  1 1995	Clear screen before plotting
# Nov  1 1995	Compute ZMIN and ZMAX locally
# Nov  2 1995	Scale plot to background and peak used
# Nov  3 1995	Pass arguments through FSPEC labelled common
# Nov  6 1995	Mark background and spectrum from bottom axis of graph
# Nov  6 1995	Move spectrum and background sections to upper left corner

# Feb 21 1996	Drop unused varieables
