# File procdata/Src/fsback.x
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
# February 21, 1996

# FSBACK -- Given background sections, remove bad pieces, such as stars

define	MAXBSECT	10

procedure fsback (vector, ipix0, imax, verbose)

real	vector[ARB]		# Data vector containing object and background
int	ipix0			# Starting pixel of vector - 1
int	imax			# Object peak pixel
bool	verbose			# TRUE to print diagnostic messages

int	nsect1			# Number of sections used
int	bsect1[MAXBSECT,2]	# Background sections after sigma rejection
char	bsample[SZ_LINE]	# Pixel range for background
char	temp[SZ_LINE]
int	i, n, nbpix, igrow, iteration
bool	offscale
real	sum,sumsq,mean,sigma,value, bmin, bmax, sigma2

include	"fspec.com"

begin

	if (bnit < 1)
	    bnit = 1
	do iteration = 1, bnit {

	if (verbose) {
	    bsample[1] = EOS
	    do n = 1, nsect {
		call sprintf (temp,SZ_LINE,"%d:%d")
		    call pargi (bsect[n,1])
		    call pargi (bsect[n,2])
		call strcat (temp,bsample,SZ_LINE)
		if (n < nsect)
		    call strcat (",",bsample,SZ_LINE)
		}
	    call printf ("FSBACK: %d: initial absolute background %s\n")
		call pargi (iteration)
		call pargstr (bsample)

	    bsample[1] = EOS
	    do n = 1, nsect {
		call sprintf (temp,SZ_LINE,"%d:%d")
		    call pargi (bsect[n,1]-imax)
		    call pargi (bsect[n,2]-imax)
		call strcat (temp,bsample,SZ_LINE)
		if (n < nsect)
		    call strcat (",",bsample,SZ_LINE)
		}
	    call printf ("FSBACK: %d: initial relative background %s\n")
		call pargi (iteration)
		call pargstr (bsample)
	    call flush (STDOUT)
	    }

# Remove objects from limits
# Compute mean and sigma
	sum = 0.0
	sumsq = 0.0
	nbpix = 0
	do n = 1, nsect {
	    do i = bsect[n,1], bsect[n,2] {
		value = vector[i-ipix0]
		sum = sum + value
		sumsq = sumsq + (value * value)
		nbpix = nbpix + 1
		}
	    }
	switch (nbpix) {
	    case 0:
		mean = INDEF
		sigma = INDEF
	    case 1:
		mean = sum
		sigma = INDEF
	    default:
		mean = sum / nbpix
		sigma2 = (sumsq - mean * sum) / (nbpix - 1)
		if (sigma2 < 0)               # possible with roundoff error
		    sigma = 0.0
		else
		    sigma = sqrt (sigma2)
	    }

# Set limiting values for good background
	bmax = mean + (bsig * sigma)
	bmin = mean - (bsig * sigma)
	if (verbose) {
	    call printf ("FSBACK: mean = %.2f +- %.2f, accept %.2f - %.2f\n")
		call pargr (mean)
		call pargr (sigma)
		call pargr (bmin)
		call pargr (bmax)
	    call flush (STDOUT)
	    }

# Set new background sections, rejecting points outside of range
	do n = 1, MAXBSECT {
	    bsect1[n,1] = 0
	    bsect1[n,2] = 0
	    }
	nsect1 = 0
	do n = 1, nsect {
	    offscale = TRUE
	    igrow = 0
	    do i = bsect[n,1], bsect[n,2] {
		value = vector[i-ipix0]

	# if this point is offscale and previous point was OK, end section
		if (value < bmin || value > bmax) {
		    if (!offscale ) {
			if (bsect1[nsect1,1] < i-1-bgrow)
			    bsect1[nsect1,2] = i - 1 - bgrow
			else {
			    bsect1[nsect1,1] = 0
			    bsect1[nsect1,2] = 0
			    nsect1 = nsect1 - 1
			    }
#			if (verbose) {
#			    call printf ("FSBACK: output %d: %d-%d\n")
#				call pargi (nsect1)
#				call pargi (bsect1[nsect1,1])
#				call pargi (bsect1[nsect1,2])
#			    call flush (STDOUT)
#			    }
			}
		    offscale = TRUE
		    igrow = bgrow
		    }

	# if previous point was offscale, start a new section
		else if (offscale) {
		    if (igrow > 0)
			igrow = igrow - 1
		    else {
			offscale = FALSE
			nsect1 = nsect1 + 1
			if (nsect1 > MAXBSECT)
			    break
			bsect1[nsect1,1] = i
			bsect1[nsect1,2] = i
			}
		    }

	# if point and previous point OK, continue section
		else
		    bsect1[nsect1,2] = i
#		if (verbose) {
#		    call printf ("FSBACK: %d[%d->%d]: %.2f %b\n")
#			call pargi (i)
#			call pargi (n)
#			call pargi (nsect1)
#			call pargr (value)
#			call pargb (offscale)
#		    }
		}
	    if (nsect1 > MAXBSECT)
		break

	    }
	if (nsect1 > MAXBSECT)
	    nsect1 = MAXBSECT

# Reset section
	nsect = 0
	do n = 1, nsect1 {
	    if (bsect1[n,2] - bsect1[n,1] + 1 >= bminsect) {
		nsect = nsect + 1
		bsect[nsect,1] = bsect1[n,1]
		bsect[nsect,2] = bsect1[n,2]
		}
	    }

	}

# List input and output sections
	if (verbose) {
	    bsample[1] = EOS
	    do n = 1, nsect {
		call sprintf (temp,SZ_LINE,"%d:%d")
		    call pargi (bsect[n,1]-imax)
		    call pargi (bsect[n,2]-imax)
		call strcat (temp,bsample,SZ_LINE)
		if (n < nsect)
		    call strcat (",",bsample,SZ_LINE)
		}
	    call printf ("FSBACK: final relative background %s\n")
		call pargstr (bsample)

	    bsample[1] = EOS
	    do n = 1, nsect {
		call sprintf (temp,SZ_LINE,"%d:%d")
		    call pargi (bsect[n,1])
		    call pargi (bsect[n,2])
		call strcat (temp,bsample,SZ_LINE)
		if (n < nsect)
		    call strcat (",",bsample,SZ_LINE)
		}
	    call printf ("FSBACK: final absolute background %s\n")
		call pargstr (bsample)
	    call flush (STDOUT)
	    }
	return
end
# Nov  3 1995	Pass background arguments in FSPEC common

# Feb 21 1996	Drop unused arguments NPTS, IPIX1, and IPIX2
