# File procdata/Src/fspeak.x
# February 21, 1996
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# Copyright(c) 1994,1995 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.

define	MAXPEAKS	50
define	MAXBSECT	10


#--- Find peak and full width at given fraction of peak value of vector

procedure fspeak (npts, ipt0, vector, imax, il, ir, ilb, irb, debug)

int	npts		# Length of vector
int	ipt0		# First coordinate in vector
real	vector[ARB]	# Data vector
int	imax		# Bin containing maximum (computed if zero)
int	il		# First bin for object spectum (returned)
int	ir		# Last bin for object spectrum (returned)
int	ilb		# Last bin for left background (returned)
int	irb		# First bin for right background (returned)
bool	debug		# If TRUE, print extra diagnostics

real	height		# Actual height of peak
real	fracp, fracb
real	mean, sigma, vmax
#real	vmin
int	iwidth, i, ksig
#int	iterate
int	ipt, cpix, cpeak, cdist, maxpeak, nvec
int	pklist[MAXPEAKS], pkheight[MAXPEAKS], pkdist[MAXPEAKS], npeaks
real	background, amedr()

include	"fspec.com"

begin

# Use the input peak position if it is set
	if (imax > 0) {
	    imax = imax - ipt0
	    height = vector[imax]
	    }

# Find the peak in the vector if it's not already set.
	else {
	    height = 0.
	    npeaks = 0
	    maxpeak = 0
	    cpeak = 0
	    cpix = npts / 2
	    cpeak = 0
	    cdist = npts
	    ksig = int (bsig)
	    nvec = pmax - pmin + 1
	    call aravr (vector[pmin-ipt0],nvec,mean,sigma,ksig)
	    vmax = mean + (psig * sigma)
#	    call aavgr (vector,npts,mean,sigma)
#	    if (pnit > 1) {
#		do iterate = 2, pnit {
#		    vmin = mean - (bsig * sigma)
#		    vmax = mean + (bsig * sigma)
#		    call aavgr (vector,npts,mean,sigma,vmin,vmax)
#		    }
#		}

# Find peaks in vector, where each peak is the local maximum of 5 points
	    do ipt = 3, npts-2 {
		if (vector[ipt] > vmax &&
		    vector[ipt] >= vector[ipt-1] &&
		    vector[ipt] >= vector[ipt+1]) {
		    if (npeaks < MAXPEAKS) {
			npeaks = npeaks + 1
			pkheight[npeaks] = int (vector[ipt])
			pklist[npeaks] = ipt
			pkdist[npeaks] = abs (ipt - cpix)
			if (pkdist[npeaks] < cdist){
			    cpeak = npeaks
			    cdist = pkdist[npeaks]
			    }
			if (pkheight[npeaks] > height){
			    maxpeak = npeaks
			    height = pkheight[npeaks]
			    }
			}
		    }
		}

	    if (debug) {
		call printf ("FSPEAK: %d peaks")
		    call pargi (npeaks)
		if (npeaks > 0) {
		    call printf (":")
		    do i = 1, npeaks {
			call printf (" %d")
			    call pargi (pklist[i])
			if (i < npeaks)
			    call printf (",")
			else
			    call printf ("\n")
			}
		    }
		else
		    call printf ("\n")
		if (maxpeak > 0) {
		    call printf ("FSPEAK: max peak at %d = %d, %d from center\n")
			call pargi (pklist[maxpeak])
			call pargi (pkheight[maxpeak])
			call pargi (pkdist[maxpeak])
		    }
		else
		    call printf ("FSPEAK:  no max peak!\n")
		if (cpeak > 0) {
		    call printf ("FSPEAK: center peak at %d = %d, %d from center\n")
			call pargi (pklist[cpeak])
			call pargi (pkheight[cpeak])
			call pargi (pkdist[cpeak])
		    }
		else
		    call printf ("FSPEAK:  no center peak!\n")
		call flush (STDOUT)
		}
	    if (npeaks == 0) {
		imax = 0
		il = 0
		ir = 0
		ilb = 0
		irb = 0
		return
		}

# Set best peak
	    if (maxpeak == cpeak) {
		imax = pklist[cpeak]
		height = pkheight[cpeak]
		}
#	    else if (pkdist[maxpeak] < 2*cdist) {
#		imax = pklist[maxpeak]
#		height = pkheight[maxpeak]
#		}
	    else {
		imax = pklist[cpeak]
		height = pkheight[cpeak]
		}
	    }

# Set limits and width if number of points to fit is given
	background = amedr (vector, npts)
	if (pfrac > 1) {
	    iwidth = int (pfrac * 0.5d0)
	    il = imax - iwidth
	    ir = imax + iwidth
	    }

# Set actual level for fraction of peak
	else {
	    if (pfrac == 0)
		fracp = ((height - background) * 0.5d0) + background
	    else
		fracp = ((height - background) * pfrac) + background

	# Find left fractional power point
	    do il = imax, 1, -1 {
		if (vector[il] < fracp)
		    break
		}

	#  Find right fractional power point
	    do ir = imax, npts {
		if (vector[ir] < fracp)
		    break
		}
	    }

# Find background limits
	if (pfrac == 0)
	    fracb = ((height - background) * 0.05d0) + background
	else
	    fracb = ((height - background) * bfrac) + background

	do ilb = il, 1, -1 {
	    if (vector[ilb] < fracb)
		break
	    }
	ilb = ilb - bdiff + ipt0 - 1

	do irb = ir, npts {
	    if (vector[irb] < fracb)
		break
	    }
	irb = irb + bdiff + ipt0 - 1

# Set peak and edges to real pixels
	imax = imax + ipt0
	il = il + ipt0
	ir = ir + ipt0

	if (debug) {
	    call printf ("FSPEAK:  c= %d, h= %6.4f - %6.4f, obj %d - %d, sky < %d, > %d\n")
	    call pargi (imax)
	    call pargr (height)
	    call pargr (background)
	    call pargi (il)
	    call pargi (ir)
	    call pargi (ilb)
	    call pargi (irb)
	    }
	return
end
# Jan 21 1994	New program

# Jul 14 1995	Use do instead of while due to Solaris problems
# Sep 29 1995	Check both max peak and peak closest to the center
# Sep 29 1995	Check for maximum of 3 points and > dsig sigma
# Sep 30 1995	Add variable number of iterations for bacground sigma
# Nov  1 1995	Change name from PKWIDTH to FSPEAK
# Nov  1 1995	Use ARAVR instead of AAVGR to get background
# Nov  2 1995	Clean up indices
# Nov  3 1995	Move all CL interaction to T_FINDSPEC

# Feb 21 1996	Improve error handling
