# File procdata/Src/fsplot.x
# November 21, 1996
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

# Copyright(c) 1995 Smithsonian Astrophysical Observatory
# You may do anything you like with this file except remove this copyright.
# The Smithsonian Astrophysical Observatory makes no representations about
# the suitability of this software for any purpose.  It is provided "as is"
# without express or implied warranty.

#  FSPLOT uses IRAF plot routines to plot a spectrum on STDGRAPH.
#  Adapted from splot in the IRAF onedspec package
 
include	<gset.h>
include <mach.h>
include <imhdr.h>
include <imio.h>
include	<pkg/gtools.h>

define	MAXBSECT	10

procedure fsplot (image,xvec,svec,yvec,row1,row2,col1,col2,wcslab,format,
		  imax,il,ir,ns,ipeak,rerun)

char	image[SZ_FNAME]		# Image name
real	xvec[ARB]	# Image rows (X coordinates) for plotted vector
real	svec[ARB]	# Smoothed pixel medians (Y coordinates) to plot
real	yvec[ARB]	# Pixel medians (Y coordinates)
int	row1, row2	# Selected rows
int	col1, col2	# Selected columns
char	wcslab[ARB]	# WCS label
char	format[ARB]	# WCS format
int	imax		# Line of maximum peak
int	il,ir		# Left and right limits of peak
int	ns		# Number of times spectrum is smoothed
int	ipeak		# peak pixel, if returned
bool	rerun		# True to look for peak again (returned)
 
pointer	gfd,gt		# Pointers for graphics control structures
int	ipix1		# First pixel to plot
int	ipix2		# Last pixel to plot
int	nrow		# Number of pixels in vector
char	command[SZ_FNAME]
int	wc, key
bool	newgraph, zfirst, interact
int	i, ix, i1, i2, ipix0, itemp, ic, ip
real	wx, wy, rtemp
char    device[SZ_FNAME] # Display on which to plot data
char    plotter[SZ_FNAME] # Device on which to make hard copies
char    linebuf[SZ_LINE] # Line buffer for input
bool	autoscale	# True to scale by peak and background, else all
 
int	gt_gcur(), getline(), ctoi(), ctor()
bool	clgetb()
pointer	gopen(), gt_init()

include	"fspec.com"
 
begin
	rerun = FALSE
	ipeak = 0

# Open display and eliminate y-axis minor ticks.
	call clgstr("device", device, SZ_FNAME)
	gfd = gopen (device, NEW_FILE, STDGRAPH)
	If (gfd == ERR) {
	    call printf ("PLOTSPEC:  cannot open display %s\n")
		call pargstr (device)
	    return
	    }
	call gseti (gfd, G_YNMINOR, NO)
	call gseti (gfd, G_XNMINOR, YES)
	call gsetr (gfd, G_TICKLABELSIZE, 0.8)
 
# Initialize graph format
	gt = gt_init()
	call loghead ("FINDSPEC",linebuf)
	call gt_seti (gt, GTSYSID, NO)
	call gt_sets (gt, GTTITLE, linebuf)
	call gt_sets (gt, GTSUBTITLE, image)
	call gt_sets (gt, GTXLABEL, "Row")

# Initial plot of data array
	ipix0 = row1 - 1
	ipix1 = row1
	ipix2 = row2
	nrow = ipix2 - ipix0
	autoscale = TRUE
	call fsaplot (gfd, gt, image, xvec[ipix1-ipix0], svec[ipix1-ipix0],
		      ipix1,ipix2, col1, col2, wcslab, format, imax,il,ir,
		      ns, autoscale)
	newgraph = false
	zfirst = true
	interact = clgetb ("interactive")
	if (!interact) {
	    call gclose (gfd)
	    call gt_free (gt)
	    return
	    }
 
# Handle cursor interaction
	while (gt_gcur ("cursor", wx, wy, wc, key, command, SZ_FNAME) != EOF) {

# Find closest pixel within display limits
	    if (wx <= xvec[ipix1-ipix0])
		ix = ipix1
	    else if (wx >= xvec[ipix2-ipix0])
		ix = ipix2
	    else {
		i1 = ipix1-ipix0
		i2 = ipix2-ipix0 - 1
		do i = i1, i2 {
		    if (wx >= xvec[i] && wx < xvec[i+1]) {
			if (wx - xvec[i] > 0.5)
			    ix = i + ipix0 + 1
			else
			    ix = i + ipix0
			break
			}
		    }
		}
	    switch (key) {

#	Make a hardcopy of the current display
		case '@':
		    call gflush (gfd)
		    call gclose (gfd)
		    call printf ("Making hard copy on stdplot\n")
		    call flush (STDOUT)
		    call clgstr("plotter", plotter, SZ_FNAME)
		    gfd = ERR
		    gfd = gopen (plotter, APPEND, STDGRAPH)
		    If (gfd == ERR) {
			call printf ("PLOTSPEC:  cannot open plotter %s\n")
			    call pargstr (plotter)
			next
			}
		    call fsaplot (gfd, gt, image, xvec[ipix1-ipix0],
				  svec[ipix1-ipix0], ipix1,ipix2, col1, col2,
				  wcslab, format, imax,il,ir, ns, autoscale)
		    call gflush (gfd)
		    call gclose (gfd)
		    gfd = gopen (device, APPEND, STDGRAPH)
		    call gseti (gfd, G_YNMINOR, NO)
		    call gseti (gfd, G_XNMINOR, YES)
		    call gsetr (gfd, G_TICKLABELSIZE, 0.8)

		case 'B','b':
		    call gclose (gfd)
		    call printf("Number of sigma at which to clip background (%.1f): ")
			call pargr (bsig)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 0) {
			ip = 1
			ic = ctor (linebuf, ip, rtemp)
			if (ic > 0)
			    bsig = rtemp
			}
		    call printf("Number of sigma for peaks (%.1f): ")
			call pargr (psig)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 0) {
			ip = 1
			ic = ctor (linebuf,ip,rtemp)
			if (ic > 0)
			    psig = rtemp
			}
		    call printf("%.1f max sigma for background, %.1f min sigma for peaks\n")
			call pargr (bsig)
			call pargr (psig)
		    call gt_free (gt)
		    rerun = TRUE
		    return

		case 'C','c':
		    call gclose (gfd)
		    call printf ("x = %d(%.0f), y = %.1f\n")
			call pargi (ix)
			call pargr (xvec[ix])
			call pargr (wy)
		    call flush (STDOUT)
		    gfd = gopen (device, APPEND, STDGRAPH)

		case 'K','k':
		    call gclose (gfd)
		    if (dbap) {
			dbap = FALSE
			call printf ("DO NOT save aperture and background limits in database\n")
			}
		    call flush (STDOUT)
		    call gt_free (gt)
		    return

		case 'D','d':
		    call gclose (gfd)
		    if (imup) {
			dbap = TRUE
			call printf ("Save aperture and background limits in database\n")
			}
		    else
			call printf ("Cannot save aperture and background limits in image\n")
		    call flush (STDOUT)
		    call gt_free (gt)
		    return

		case 'F','f':
		    call gclose (gfd)
		    call printf("Minimum peak fraction for spectrum (%.3f): ")
			call pargr (pfrac)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 0) {
			ip = 1
			ic = ctor (linebuf, ip, rtemp)
			if (ic > 0)
			    pfrac = rtemp
			}
		    call printf("Maximum peak fraction for background (%.3f): ")
			call pargr (bfrac)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 0) {
			ip = 1
			ic = ctor (linebuf, ip, rtemp)
			if (ic > 0)
			    bfrac = rtemp
			}
		    call printf("spectrum > %.3f of peak, background < %.3f of peak\n")
			call pargr (pfrac)
			call pargr (bfrac)
		    call gt_free (gt)
		    rerun = TRUE
		    return

#	Set number of times to smooth spectrum
		case 'G','g':
		    call gclose (gfd)
		    call printf("Growth radius around spurious peak (%d): ")
			call pargi (bgrow)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 0) {
			ip = 1
			ic = ctoi (linebuf,ip,itemp)
			if (ic > 0)
			    bgrow = itemp
			}
		    call printf("Number of pixels between background and peak (%d): ")
			call pargi (bdiff)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 0) {
			ip = 1
			ic = ctoi (linebuf,ip,itemp)
			if (ic > 0)
			    bdiff = itemp
			}
		    call gt_free (gt)
		    rerun = TRUE
		    return

#	Print menu
		case 'H','h','?',' ':
		    call gclose (gfd)
		    call printf ("@  Make hard copy of screen\n")
		    call printf ("a  Select aperture center\n")
		    call printf ("b  Change background and peak limits\n")
		    call printf ("c  Show x and y coordinates\n")
		    call printf ("d  Save aperture and background limits in database\n")
		    call printf ("f  Change fraction of peak for spectrum and background\n")
		    call printf ("g  Change growth radius around bad peaks\n")
		    call printf ("h  This menu\n")
		    call printf ("k  Do not save aperture and background limits in database\n")
		    call printf ("m  Change number of times to smooth vector\n")
		    call printf ("n  Change number of peak and back iterations\n")
		    call printf ("p  Set aperture peak manually\n")
		    call printf ("q  Leave plot\n")
		    call printf ("r  Replot current graph\n")
		    call printf ("s  Toggle scaling region between peak and all\n")
		    call printf ("u  Unzoom\n")
		    call printf ("x  Cancel zoom after 1st z\n")
		    call printf ("z  Zoom between 1st and 2nd positions\n")
		    call printf ("?  This menu\n")
		    call flush (STDOUT)
		    gfd = gopen (device, APPEND, STDGRAPH)

		case 'M','m':
		    call gclose (gfd)
		    call printf("Number of times to smooth spectrum (%d): ")
			call pargi (ns)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 0) {
			ip = 1
			ic = ctoi (linebuf,ip,itemp)
			if (ic > 0) {
			    ns = itemp
			    call amovr (yvec, svec, nrow)
			    if (ns > 0)
				call smooth (svec,nrow,ns)
			    }
			}
		    gfd = gopen (device, APPEND, STDGRAPH)
		    call gt_free (gt)
		    rerun = TRUE
		    return

		case 'N','n':
		    call gclose (gfd)
		    call printf("Number of background pixels (%d): ")
			call pargi (bnpix)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 0) {
			ip = 1
			ic = ctoi (linebuf,ip,itemp)
			if (ic > 0)
			    bnpix = itemp
			}
		    call printf("Number of iterations for background (%d): ")
			call pargi (bnit)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 0) {
			ip = 1
			ic = ctoi (linebuf,ip,itemp)
			if (ic > 0)
			    bnit = itemp
			}
		    call printf("Number of iterations for peak background (%d): ")
			call pargi (pnit)
		    call flush (STDOUT)
		    i = getline (STDIN,linebuf)
		    if (i > 0) {
			ip = 1
			ic = ctoi (linebuf,ip,itemp)
			if (ic > 0)
			    pnit = itemp
			}
		    call gt_free (gt)
		    rerun = TRUE
		    return

#	Set estimate of spectrum center to the cursor value (+-1)
		case 'P','p':
		    call gclose (gfd)
		    ipeak = ix
		    if (svec[ix-ipix0+1] > svec[ix-ipix0])
			ipeak = ipeak + 1
		    if (svec[ix-ipix0-1] > svec[ix-ipix0])
			ipeak = ipeak - 1
		    call printf ("Set spectrum center at %d\n")
			call pargi (ipeak)
		    call flush (STDOUT)
		    call gt_free (gt)
		    rerun = TRUE
		    return

		case 'Q','q':
		    break

		case 'R','r':
		    newgraph = true

		case 'S','s':
		    autoscale = !autoscale
		    call gclose (gfd)
		    if (autoscale)
			call printf("scale using background and peak only\n")
		    else
			call printf("scale using entire vector\n")
		    call flush (STDOUT)
		    newgraph = true
		    gfd = gopen (device, APPEND, STDGRAPH)

#	Unzoom
		case 'U','u':
		    ipix1 = row1
		    ipix2 = row2
		    call gclose (gfd)
		    call printf("plotting from %d to %d, %5.0f to %5.0f\n")
			call pargi (ipix1)
			call pargi (ipix2)
			call pargr (xvec[ipix1-ipix0])
			call pargr (xvec[ipix2-ipix0])
		    call flush (STDOUT)
		    newgraph = true
		    gfd = gopen (device, APPEND, STDGRAPH)

#	Reinitialize two-keystroke commands
		case 'X','x':
		    if (!zfirst) {
			call gclose (gfd)
			call printf("zoom cancelled\n")
			call flush (STDOUT)
			gfd = gopen (device, APPEND, STDGRAPH)
			zfirst = true
			}

#	Zoom in on section of graph
		case 'Z','z':
		    if (zfirst) {
			ipix1 = ix
			zfirst = false
			call gclose (gfd)
			call printf("z again to zoom from %5.0f\n")
			    call pargr (xvec[ipix1-ipix0])
			call flush (STDOUT)
			}
		    else {
			ipix2 = ix
			zfirst = true
			if (ipix2 < ipix1) {
			    itemp = ipix1
			    ipix1 = ipix2
			    ipix2 = itemp
			    }
			call gclose (gfd)
			call printf("zooming from %5.0f to %5.0f\n")
			    call pargr (xvec[ipix1-ipix0])
			    call pargr (xvec[ipix2-ipix0])
			call flush (STDOUT)
			autoscale = FALSE
			newgraph = true
			}
		    gfd = gopen (device, APPEND, STDGRAPH)

#	Default is to print coordinates of cursor
		default:
		    call gclose (gfd)
		    call printf ("x,y: %10.3f %10.4g\n")
			call pargr (wx)
			call pargr (wy)
		    call flush (STDOUT)
		    gfd = gopen (device, APPEND, STDGRAPH)
		}
 
#	Replot graph if necessary
	    if (newgraph) {
		call fsaplot (gfd, gt, image, xvec[ipix1-ipix0],
			      svec[ipix1-ipix0], ipix1,ipix2, col1, col2,
			      wcslab, format, imax,il,ir, ns, autoscale)
		newgraph = false
		}
	    }
 
# Close graph window
 
	call gclose (gfd)
	call gt_free (gt)
	return
end

# Nov  6 1995	Add unsmoothed vector as an argument
# Nov 27 1995	Add command to toggle saving of result in database (DBAP)

# Feb 21 1996	Drop unused variables; fix PARGI of real
# Nov 21 1996	Add new command K to cancel writing to database and return
