# File redshift/Util/imgpar.x
# July 14, 1995
# Copyright(c) 1986 Association of Universities for Research in Astronomy Inc.
# except IMGDATE by Doug Mink, Center for Astrophysics, 1991

include	<syserr.h>
include	<ctype.h>
include <ctotok.h>
define	IDB_ENDVALUE	30

# IMGBPAR -- Get an image header parameter of type boolean.  False is returned
# if the parameter cannot be found or if the value is not true.

procedure imgbpar (im, key, bval)

pointer	im		# image descriptor
char	key[ARB]	# parameter to be returned
bool	bval		# value returned

pointer	rp
pointer	idb_findrecord()

begin
	if (idb_findrecord (im, key, rp) != 0)
	    bval = (Memc[rp+IDB_ENDVALUE-1] == 'T')
	return
end


# IMGCPAR -- Get an image header parameter of type char.

procedure imgcpar (im, key, cval)

pointer	im		# image descriptor
char	key[ARB]	# parameter to be returned
char	cval		# parameter value returned

pointer	idb_findrecord()
pointer	rp, sp, vp
int	ip, cctoc()
errchk	syserrs, imgstr

begin
	if (idb_findrecord (im, key, rp) != 0) {
	    call smark (sp)
	    call salloc (vp, SZ_LINE, TY_CHAR)
	    ip = 1
	    call imgstr (im, key, Memc[vp], SZ_LINE)
	    if (cctoc (Memc[vp], ip, cval) == 0)
		call syserrs (SYS_IDBTYPE, key)
	    call sfree (sp)
	    }
	return
end


# IMGIPAR -- Get an image header parameter of type integer.

procedure imgipar (im, key, ival)

pointer	im		# image descriptor
char	key[ARB]	# parameter to be returned
int	ival		# parameter value returned

pointer	idb_findrecord()
pointer	rp, sp, vp
int	ip, ctoi()
errchk	syserrs, imgstr

begin
	if (idb_findrecord (im, key, rp) != 0) {
	    call smark (sp)
	    call salloc (vp, SZ_LINE, TY_CHAR)
	    ip = 1
	    call imgstr (im, key, Memc[vp], SZ_LINE)
	    if (ctoi (Memc[vp], ip, ival) == 0)
		call syserrs (SYS_IDBTYPE, key)
	    call sfree (sp)
	    }
	return
end


# IMGLPAR -- Get an image header parameter of type long integer.

procedure imglpar (im, key)

pointer	im		# image descriptor
char	key[ARB]	# parameter to be returned
long	lval		# parameter value returned

pointer	idb_findrecord()
pointer	rp, sp, vp
int	ip, ctol()
errchk	syserrs, imgstr

begin
	if (idb_findrecord (im, key, rp) != 0) {
	    call smark (sp)
	    call salloc (vp, SZ_LINE, TY_CHAR)
	    ip = 1
	    call imgstr (im, key, Memc[vp], SZ_LINE)
	    if (ctol (Memc[vp], ip, lval) == 0)
		call syserrs (SYS_IDBTYPE, key)
	    call sfree (sp)
	    }
	return
end


# IMGSPAR -- Get an image header string parameter

procedure imgspar (im, key, sval,lsval)

pointer	im		# image descriptor
char	key[ARB]	# parameter to be returned
char	sval[ARB]	# parameter value returned
int	lsval		# Maximum length for sval

pointer	idb_findrecord()
pointer	rp
errchk	imgstr

begin
	if (idb_findrecord (im, key, rp) != 0)
	    call imgstr (im, key, sval, lsval)
	return
end


# IMGRPAR -- Get an image header parameter of type real.

procedure imgrpar (im, key, rval)

pointer	im		# image descriptor
char	key[ARB]	# parameter to be returned
real	rval		# parameter value returned

pointer	idb_findrecord()
pointer	rp, sp, vp
int	ip, ctor()
errchk	syserrs, imgstr

begin
	if (idb_findrecord (im, key, rp) != 0) {
	    call smark (sp)
	    call salloc (vp, SZ_LINE, TY_CHAR)
	    ip = 1
	    call imgstr (im, key, Memc[vp], SZ_LINE)
	    if (ctor (Memc[vp], ip, rval) == 0)
		call syserrs (SYS_IDBTYPE, key)
	    call sfree (sp)
	    }
	return
end


# IMGDPAR -- Get an image header parameter of type double floating.  If the
# named parameter is a standard parameter return the value directly,
# else scan the user area for the named parameter and decode the value.

procedure imgdpar (im, key, dval)

pointer	im		# image descriptor
char	key[ARB]	# parameter to be returned
double	dval		# parameter value returned

pointer	idb_findrecord()
pointer	rp, sp, vp
int	ip, ctod()
errchk	syserrs, imgstr

begin
	if (idb_findrecord (im, key, rp) != 0) {
	    call smark (sp)
	    call salloc (vp, SZ_LINE, TY_CHAR)
	    ip = 1
	    call imgstr (im, key, Memc[vp], SZ_LINE)
	    if (ctod (Memc[vp], ip, dval) == 0)
		call syserrs (SYS_IDBTYPE, key)
	    call sfree (sp)
	    }
	return
end


# IMGDATE -- Get a date image header string parameter as day/month/year
#	     of day-month-year

procedure imgdate (im, key, mm, dd, yyyy)

pointer	im		# image descriptor
char	key[ARB]	# parameter to be returned
int	mm, dd, yyyy

int	i, j, lstr, ix1, ix2, np
int	ctoi(), strlen(), stridx(), strldx()
pointer	idb_findrecord()
pointer	rp, vp, sp, ip, tp

begin
	call smark (sp)
	call salloc (vp, SZ_LINE, TY_CHAR)
	call salloc (tp, SZ_LINE, TY_CHAR)
	if (idb_findrecord (im, key, rp) != 0) {
	    call imgstr (im, key, Memc[vp], SZ_LINE)
	    lstr = strlen (Memc[vp])
	    if (lstr < 1) {
		return
		}

	    ix1 = stridx ("/", Memc[vp])
	    if (ix1 == 0) ix1 = stridx ("-", Memc[vp])
	    ip = vp + ix1 - 1
	    np = ix1 - 1
	    call strcpy (Memc[vp], Memc[tp], np)
	    i = 1
	    if (ctoi(Memc[tp],i,j) > 0)
		dd = j
	    else
		dd = 0

	    ix2 = strldx ("/", Memc[vp])
	    if (ix2 == 0) ix2 = strldx ("-", Memc[vp])
	    np = ix2 - ix1 - 1
	    call strcpy (Memc[ip+1], Memc[tp], np)
	    i = 1
	    if (ctoi(Memc[tp],i,j) > 0)
		mm = j
	    else
		mm = 0

	    ip = vp + ix2 - 1
	    call strcpy (Memc[ip+1],Memc[tp],4)
	    i = 1
	    if (ctoi(Memc[tp],i,j) > 0) {
		yyyy = j
		if (yyyy < 100)
		    yyyy = 1900 + yyyy
		}
	    else
		yyyy = 0
	    call sfree (sp)
	    }
end                                     


# IMGTOK -- Return the character string value for a given token

procedure imgtok (im, key0, value)

pointer	im		# image descriptor
char	key0[ARB]	# parameter name (may include ,<token number>)
char	value[ARB]	# parameter value (returned)

char	key[SZ_FNAME]	# parameter name without token number
int	itok		# token of parameter value string to be returned
			# 0=whole string

pointer	tvalue
int	i,icomma, ip, nc
int	ctoi(), stridx(), ctotok
char	comma

	call smark (sp)
	call salloc (tvalue, SZ_LINE, TY_CHAR)

#  Get token number, if part of parameter name
	comma = ','
	icomma = stridx (comma,Memc[param])
	if (icomma > 0) {
	    nc = icomma - 1
	    call strcpy (key0,key,nc)
	    ip = icomma + 1
	    nc = ctoi (key0,ip,itok)
	    }
	else {
	    call strcpy (key0,key,SZ_FNAME)
	    itok = 0
	    }

	if (idb_findrecord (im, key, rp) != 0) {
	    call imgstr (im, key, Memc[tvalue], SZ_LINE)
	    lstr = strlen (Memc[tvalue])
	    if (lstr < 1) {
		call sfree (sp)
		return
		}
	    if (itok > 0) {
		ip = 1
		do i = 1, itok {
		    if (ctotok (Memc[tvalue],ip,value,SZ_LINE) == TOK_EOS) {
			call eprintf ("IMGETP:  Token %d not found; only %d\n")
			    call pargi (itok)
			    call pargi (i-1)
			break
			}
		    }
		}
	    else
		call strcpy (Memc[tvalue],value,SZ_LINE)
	    call sfree (sp)
	    return 1
	    }
	else {
	    call sfree (sp)
	    return 0
	    }
end
# Jul 24 1991	New program

# Jun  1 1995	Add string parser to handle multiple values per parameter
