# File procdata/Src/ircorrect.x
# February 21, 1996
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics

include	<imhdr.h>
 
define	MAXCOEF	3
 
# IRCORRECT -- Corrects IR images for non-linearity, bias, and frame rate
 
procedure t_ircorrect ()
 
char	input[SZ_LINE]		# List of input images to be corrected.
char	output[SZ_LINE]		# List of output corrected images
 
int	list1, list2, verbose, nlcoef
char	image1[SZ_FNAME], image2[SZ_FNAME], image3[SZ_FNAME]
char	imbias0[SZ_FNAME], imbias[SZ_FNAME]
char	imlin[SZ_FNAME], imlin0[SZ_FNAME]
pointer	im1, im2, imb, iml

bool	clgetb()
int	imtopen(), imtgetim(), imtlen(), btoi(), strlen()
real	clgetr ()
pointer	immap()
real	expfact, exposure, framerate
int	linear0, normal0, subbias0, linimg0, linear, iexpf

common	/irlin/	linearize, normalize, subbias, linimg, ncoef, coeff
int linearize, normalize, subbias, linimg, ncoef
real coeff[MAXCOEF]
 
begin

	verbose = btoi (clgetb ("verbose"))
	linear0 = btoi (clgetb ("linearize"))
	linimg0 = btoi (clgetb ("linimg"))
	subbias0 = btoi (clgetb ("subbias"))
	normal0 = btoi (clgetb ("normalize"))
 
	# Get image template list.
	call clgstr ("input", input, SZ_LINE)
	call clgstr ("output", output, SZ_LINE)
	list1 = imtopen (input)
	list2 = imtopen (output)
	if (imtlen (list1) != imtlen (list2)) {
	    call imtclose (list1)
	    call imtclose (list2)
	    call error (1, "Input and output image lists don't match")
	    }

	if (subbias0 == YES) {
	    call clgstr ("bias", imbias0, SZ_FNAME)
	    imb = immap (imbias0, READ_WRITE, im1)
	    }

	# Get coefficient image for linearization
	if (linear0 == YES) {
	    if (linimg0 == YES) {
		call clgstr ("coeffim", imlin0, SZ_FNAME)
		iml = immap (imlin0, READ_WRITE, im1)
		}

	# Get coeficients of correction function
	    else {
		ncoef		= MAXCOEF
		coeff[1]	= clgetr ("coeff1")
		coeff[2]	= clgetr ("coeff2")
		coeff[3]	= clgetr ("coeff3")
		}
	    }
 
	# Apply correction one image at a time.
 
	while ((imtgetim (list1, image1, SZ_FNAME) != EOF) &&
	    (imtgetim (list2, image2, SZ_FNAME) != EOF)) {
 
	    call xt_mkimtemp (image1, image2, image3, SZ_FNAME)
	    im1 = immap (image1, READ_WRITE, 0)
	    im2 = immap (image2, NEW_COPY, im1)

	    # Compute exposure / frame rate normalization factor if needed
	    normalize = normal0
	    expfact = INDEFR
	    call imgrpar (im1,"NORMFACT",expfact)
	    if (normalize == YES && expfact == INDEFR) {
		exposure = -1.
		call imgrpar (im1,"EXPOSURE", exposure)
		if (exposure <= 0.)
		    call imgrpar (im1,"EXPTIME", exposure)
		call imgrpar (im1,"FRAMERAT",framerate)
		if (framerate == INDEFR)
		    expfact = 1.0
		else {
		    expfact = exposure / framerate
		    iexpf = nint (expfact)
		    expfact = real (iexpf)
		    if (verbose == YES) {
			call fprintf (STDERR,"%s factor is %d / %d = %d\n")
			call pargstr (image1)
			call pargr (exposure)
			call pargr (framerate)
			call pargr (expfact)
			}
		    }
		}
	    else
		normalize = NO

	    # Set bias file name in header if image is bias-subtracted
	    subbias = subbias0
	    imbias(1) = EOS
	    call imgspar (im1,"BIASFILE",imbias,SZ_FNAME)
	    if (subbias == YES && strlen(imbias) == 0) {
		if (verbose == YES) {
		    call fprintf (STDERR,"%s: bias image %s subtracted\n")
		    call pargstr (image1)
		    call pargstr (imbias0)
		    }
		}
	    else
		subbias = NO
 
	    # Set linearization factors in header if image is not linearized
	    linearize = linear0
	    nlcoef = INDEFI
	    call imgipar (im1,"NLINCOEF",nlcoef)
	    imlin(1) = EOS
	    call imgspar (im1,"LINFILE",imlin,SZ_FNAME)
	    if (nlcoef == INDEFI && strlen(imlin) == 0)
		linear = NO
	    else
		linear = YES
	    if (linearize == YES && linear == NO) {
		if (linimg0 == YES && strlen (imlin0) > 0) {
		    linearize = YES
		    linimg = YES
		    if (verbose == YES) {
			call fprintf (STDERR,"%s linearized by file %s\n")
			call pargstr (image1)
			call pargstr (imlin0)
			}
		    }
		if (linimg0 == NO && ncoef > 0) {
		    linearize = YES
		    linimg = NO
		    if (verbose == YES) {
			call fprintf (STDERR,"%s: linearized, order = %d\n")
			call pargstr (image1)
			call pargi (ncoef-1)
			}
		    }
		}
	    else
		linearize = NO

	    # Apply correction to each input image.
	    call irlinpolr (im1, im2, imb, iml, expfact)

	    # Save appropriate processing information in image header
	    if (normalize == YES)
		call imaddr (im2,"NORMFACT",expfact)
	    if (subbias == YES)
		call imastr (im2,"BIASFILE",imbias0)
	    if (linearize == YES) {
		if (linimg == YES) {
		    call imastr (im2,"LINFILE",imlin0)
		    ncoef = IM_LEN(iml, 3)
		    call imaddi (im2,"NLCOEFF",ncoef)
		    }
		else {
		    call imaddi (im2,"NLCOEFF",ncoef)
		    call imaddr (im2,"LCOEFF1",coeff[1])
		    call imaddr (im2,"LCOEFF2",coeff[2])
		    call imaddr (im2,"LCOEFF3",coeff[3])
		    }
		}

	    call imunmap (im1)
	    call imunmap (im2)
	    call xt_delimtemp (image2, image3)
	    if (verbose == YES) {
		call fprintf (STDERR,"%s corrected to %s \n")
		call pargstr (image1)
		call pargstr (image2)
		}
	    }
 
	if (subbias0 == YES)
	    call imunmap (imb)
	if (linear0 == YES && linimg0 == YES)
	    call imunmap (iml)
	call imtclose (list1)
	call imtclose (list2)
end
 
 
include <imhdr.h>
 
 
# IRLINPOLR -- Corrects an IR imager frame for non-linearity using a simple 
# power series polynomial correction function:
#
# ADU' = ADU * [ a + b * (ADU / MAXADU) + c * (ADU / MAXADU) **2 ]
#
 
procedure irlinpolr (im1, im2, imb, iml, expfact)
 
pointer	im1				# Input IMIO pointer
pointer	im2				# Output IMIO pointer
pointer	imb				# Bias IMIO pointer
pointer	iml				# Linearization IMIO pointer
real	expfact				# Exposure / frame rate
 
int	npix, nplin, nclin, nllin, ic
#int	first, ib
long	v1[IM_MAXDIM], v2[IM_MAXDIM], vb[IM_MAXDIM], vl[IM_MAXDIM,10]
pointer	buf1, buf2, bufb, bufl
real	rfac
 
int	imgnlr(), impnlr()

common	/irlin/	linearize, normalize, subbias, linimg, ncoef, coeff
int linearize, normalize, subbias, linimg, ncoef
real coeff[MAXCOEF]
 
begin
	npix = IM_LEN(im1, 1)
	call amovkl (long(1), v1, IM_MAXDIM)
	call amovkl (long(1), v2, IM_MAXDIM)
	call amovkl (long(1), vb, IM_MAXDIM)
#	first = YES
#	ib = 0

	if (linearize == YES && linimg == YES) {
	    nplin = IM_LEN(iml, 1) * IM_LEN(iml,2)
	    nllin = IM_LEN(iml, 1)
	    nclin = IM_LEN(iml, 3)
	    call amovkl (long(1), vl, IM_MAXDIM*10)
	    do ic = 1, 10 {
		vl(3,ic) = ic
		}
	    }
	# Loop through image one line at a time

	while ((imgnlr (im1, buf1, v1) != EOF) &&
	(impnlr (im2, buf2, v2) != EOF)) {

#	if (first == YES) {
#	    call printf ("irlinpolr: raw %g\n")
#	    call pargr (Memr[buf1+ib])
#	    }

	    # Divide by exposure / frame rate normalization factor
	    if (normalize == YES)
		    call adivkr (Memr[buf1], expfact, Memr[buf1], npix)

#	if (first == YES) {
#	    call printf ("irlinpolr: normalized %g\n")
#	    call pargr (Memr[buf1])
#	    }

	    # Subtract bias from one line of image
	    if (subbias == YES) {
		if (imgnlr (imb, bufb, vb) != EOF)
		    call asubr (Memr[buf1], Memr[bufb], Memr[buf1], npix)
		else {
		    call fprintf (STDERR,"Bias subtract failed at line %d\n")
			call pargi (vb(2))
		    return
		    }
		}

#	if (first == YES) {
#	    call printf ("irlinpolr: bias subtrcted %g\n")
#	    call pargr (Memr[buf1+ib])
#	    }

	    # Linearize one line of image
	    if (linearize == YES) {

		# Use image of coefficients
		if (linimg == YES) {
		    rfac = 10000.
		    call adivkr (Memr[buf1], rfac, Memr[buf1], npix)

#	if (first == YES) {
#	    call printf ("irlinpolr: normalized %g\n")
#	    call pargr (Memr[buf1+ib])
#	    }

		    call amovkr (0., Memr[buf2], npix)
		    do ic = nclin, 1, -1 {
			if (imgnlr (iml, bufl, vl(1,ic)) != EOF) {
			    call amulr (Memr[buf2],Memr[buf1],Memr[buf2],npix)
			    call aaddr (Memr[buf2],Memr[bufl],Memr[buf2],npix)

#	if (first == YES) {
#	    call printf ("irlinpolr: %d : %g %g \n")
#	    call pargi (ic)
#	    call pargr (Memr[bufl+ib])
#	    call pargr (Memr[buf2+ib])
#	    }

			    }
			else {
			    call fprintf (STDERR,"Linearization failed at line %d\n")
				call pargi (vb(2))
			    return
			    }
			}
		    call amulkr (Memr[buf1], rfac, Memr[buf1], npix)
		    call amulr (Memr[buf1], Memr[buf2], Memr[buf2], npix)

#	if (first == YES) {
#	    call printf ("irlinpolr: linearized %g\n")
#	    call pargr (Memr[buf2+ib])
#	    }
#	first = NO
		    }

		# Use constant coefficients
		else {
		    rfac = 32767.0
		    call adivkr (Memr[buf1], rfac, Memr[buf1], npix)
		    call amovkr (0., Memr[buf2], npix)
		    do ic = ncoef, 1, -1 {
			call amulr (Memr[buf2],Memr[buf1],Memr[buf2],npix)
			call aaddkr (Memr[buf2],coeff[ic],Memr[buf2],npix)
			}
		    call amulkr (Memr[buf1], rfac, Memr[buf1], npix)
		    call amulr (Memr[buf1], Memr[buf2], Memr[buf2], npix)
		    }
		}
	    else
		call amovr (Memr[buf1], Memr[buf2], npix)
	
	   }
end

# Mar 31 1993	Round expfact to nearest integer
# Apr 23 1993	Check for exptime as well as exposure in header
# Jun 22 1993	Match diagnostic and header processing order to reality

# Feb 21 1996	Change PARGR to PARGI for int
