# File procdata/Src/t_findspec.x
# By Doug Mink, Harvard-Smithsonian Center for Astrophysics
# November 22, 2004

# T_FINDSPEC -- Find the spectrum in a range of columns from an image.

define	MAXBSECT	10
define	VERSION		"2.3"

include	<gset.h>
include	<mach.h>
include	<imhdr.h>
include	<imio.h>
include	<mwset.h>
include	<math/curfit.h>
include "apertures.h"

define	SZ_HSTRING	68

procedure t_findspec ()

pointer	image		# Image file name
pointer	wcslab		# World Coordinate System type
pointer	im		# image header structure
pointer	mw		# World Coordinate System structure
pointer	fmt
pointer	ct, sp, xvec, yvec, svec, impath
int	col1, col2, ncols, nlines, row0, row1, row2, itemp, nap, lpath
real	xmin, xmax
int	clgeti(), strlen()
pointer	immap(), mw_openim(), mw_sctran()
real	clgetr()
bool	verbose, clgetb()
bool	dispap		# Plot aperture and background limits

int	apaxis		# Aperture axis
real	dispcenter	# Center along the dispersion axis
pointer	ap		# Aperture pointer

int	imax		# Bin containing maximum (computed if zero)
int	il		# first bin for peak fit (returned)
int	ir		# last bin for peak fit (returned)
int	ilb1,ilb2	# Left background limits
int	irb1,irb2	# Right background limits
int	nrows		# Number of rows to check and plot
int	nsmooth		# Number of times to smooth vector before search
int	dispaxis	# Dispersion axis
char	bsample[SZ_LINE] # Pixel range for background
char	temp[SZ_LINE]
char	dstr[SZ_LINE]
char	hstr[SZ_LINE]
char	hstr0[SZ_LINE]
int	n, ipk, ip
bool	rerun
real	a, b, c[1]
pointer	ic, str
int	imaccess(), strsearch()
bool	interact

define	findpeak_ 30
define	plotfind_ 50
define	error_exit_ 80
define	endfind_ 90

include	"fspec.com"

begin
	call smark (sp)
	call salloc (image, SZ_FNAME, TY_CHAR)
	call salloc (impath, SZ_FNAME, TY_CHAR)
	call salloc (wcslab, SZ_LINE, TY_CHAR)
	call salloc (fmt, SZ_LINE, TY_CHAR)
	bsample[1] = EOS

# Open image
	call clgstr ("image", Memc[image], SZ_FNAME)

# Remove trailing .imh, if present, so database file is named correctly
	ip = strsearch (Memc[image],".imh")
	if (ip > 0) {
	    Memc[image+ip-5] = EOS
	    Memc[image+ip-4] = EOS
	    Memc[image+ip-3] = EOS
	    Memc[image+ip-2] = EOS
	    }

# Remove trailing .fit, if present, so database file is named correctly
	ip = strsearch (Memc[image],".fit")
	if (ip > 0) {
	    Memc[image+ip-5] = EOS
	    Memc[image+ip-4] = EOS
	    Memc[image+ip-3] = EOS
	    Memc[image+ip-2] = EOS
	    }

# Remove trailing .fits, if present, so database file is named correctly
	ip = strsearch (Memc[image],".fits")
	if (ip > 0) {
	    Memc[image+ip-6] = EOS
	    Memc[image+ip-5] = EOS
	    Memc[image+ip-4] = EOS
	    Memc[image+ip-3] = EOS
	    Memc[image+ip-2] = EOS
	    }

	call clgstr ("imdir", Memc[impath], SZ_FNAME)
	lpath = strlen (Memc[impath])
	if (lpath > 0) {
	    if (Memc[impath+lpath-1] != '/') {
		Memc[impath+lpath] = '/'
		Memc[impath+lpath+1] = EOS
		}
	    call strcat (Memc[image], Memc[impath], SZ_FNAME)
	    }
	else
	    call strcpy (Memc[image], Memc[impath], SZ_FNAME)

	dbap = clgetb ("dbap")
	if (imaccess (Memc[impath], READ_WRITE) == NO) {
	    im = immap (Memc[impath], READ_ONLY, 0)
	    imup = FALSE
	    if (dbap) {
		call eprintf ("FINDSPEC: Cannot write to %s; not extracting spectrum\n")
		    call pargstr (Memc[impath])
		dbap = FALSE
		}
	    }
	else {
	    im = immap (Memc[impath], READ_WRITE, 0)
	    imup = TRUE
	    }
	call strcpy ("logical", Memc[wcslab], SZ_LINE)
	mw = mw_openim (im)
	call mw_seti (mw, MW_USEAXMAP, NO)
	ct = mw_sctran (mw, "logical", Memc[wcslab], 0)

# Dimensions of image
	ncols  = IM_LEN(im,1)
	nlines = IM_LEN(im,2)

# Range of columns within which to find median
	col1 = clgeti ("col1")
	col2 = clgeti ("col2")
	if (col2 < col1) {
	    itemp = col1
	    col1 = col2
	    col2 = itemp
	    }
	if (col1 == INDEFI && col2 == INDEFI) {
	    col1 = (ncols / 2) - 50
	    col2 = (ncols / 2) + 50
	    }
	else if (col1 != INDEFI && col2 == INDEFI) {
	    col2 = col1 + 100
	    }
	else if (col1 == INDEFI && col2 != INDEFI) {
	    col1 = col2 - 100
	    }
	if (min(col1,col2) < 1 || max(col1,col2) > ncols) {
	    call eprintf ("column index references outside image\n")
	    go to error_exit_
	    }

# Range of rows within which to find object spectrum and background
	row1 = clgeti ("row1")
	row2 = clgeti ("row2")
	if (row1 == INDEFI && row2 == INDEFI) {
	    row1 = 1
	    row2 = nlines
	    }
	else if (row1 != INDEFI && row2 == INDEFI) {
	    row2 = nlines
	    }
	else if (row1 == INDEFI && row2 != INDEFI) {
	    row1 = 1
	    }
	if (row2 < row1) {
	    itemp = row1
	    row1 = row2
	    row2 = itemp
	    }
	nrows = row2 - row1 + 1
	row0 = row1 - 1

# Get the median for the requested columns from the image.
	call malloc (xvec, nlines, TY_REAL)
	call malloc (yvec, nlines, TY_REAL)
	call malloc (svec, nlines, TY_REAL)
	verbose = clgetb ("verbose")
	if (verbose) {
	    call printf ("FINDSPEC: Columns %d - %d, Rows %d - %d\n")
		call pargi (col1)
		call pargi (col2)
		call pargi (row1)
		call pargi (row2)
	    }
	call fsmedian (im, mw, ct, col1,col2, row1,row2,
	    Memr[xvec], Memr[yvec], Memc[wcslab], Memc[fmt], SZ_LINE)

# Smooth data, if requested
	nsmooth = 0
	nsmooth = clgeti ("nsmooth")
	call amovr (Memr[yvec], Memr[svec], nrows)
	if (nsmooth > 0)
	    call smooth (Memr[svec],nrows,nsmooth)

# Initialize parameters for peak search
	imax = 0
	psig = clgetr ("peak_sigma")
	pfrac = clgetr ("fracpeak")
	pnit = clgeti ("peak_iterate")
	pmin = clgeti ("peak_min")
	if (pmin == INDEFI)
	    pmin = row1
	pmax = clgeti ("peak_max")
	if (pmax == INDEFI)
	    pmax = row2
	bsig = clgetr ("back_sigma")
	bfrac = clgetr ("fracback")
	bdiff = clgeti ("diffback")
	bgrow = clgeti ("back_grow")
	bnit = clgeti ("back_iterate")
	bminsect = clgeti ("back_minsect")
	bnpix = clgeti ("nbackpix")

# Find most central peak and set background limits in cross-dispersion vector
findpeak_
	call fspeak (nrows,row0,Memr[svec],imax,il,ir,ilb1,irb1,verbose)
	if (imax == 0) {
	    dispap = clgetb ("dispap")
	    interact = clgetb ("interactive")
	    if (dispap && interact)
		go to plotfind_
	    go to error_exit_
	    }

# Set spectrum limits
	il = il - imax
	ir = ir - imax
	call clputi ("center",imax)
	call clputi ("lower",il)
	call clputi ("upper",ir)
	if (verbose) {
	    call printf ("FINDSPEC: %s spectrum centered at %d, %d:%d\n")
		call pargstr (Memc[image])
		call pargi (imax)
		call pargi (imax+il)
		call pargi (imax+ir)
	    call flush (STDOUT)
	    }

# Set background limits
	ilb2 = ilb1 - bnpix
	irb2 = irb1 + bnpix
	nsect = 2
	bsect[1,1] = ilb2
	bsect[1,2] = ilb1
	bsect[2,1] = irb1
	bsect[2,2] = irb2
	do n = 1, nsect {
	    call sprintf (temp,SZ_LINE,"%d:%d")
		call pargi (bsect[n,1])
		call pargi (bsect[n,2])
	    call strcat (temp,bsample,SZ_LINE)
	    if (n < nsect)
		call strcat (",",bsample,SZ_LINE)
	    }
	if (bsect[1,1] < row1) {
	    if (bsect[1,2] > 0)
		bsect[1,1] = row1
	    else {
		nsect = 1
		bsect[1,1] = bsect[2,1]
		bsect[1,2] = bsect[2,2]
		}
	    }
	if (bsect[2,2] > row2) {
	    if (bsect[2,1] < row2)
		bsect[2,2] = row2
	    else
		nsect = nsect - 1
	    }

	if (nsect < 1) {
	    call eprintf ("FINDSPEC: cannot use background %s\n")
		call pargstr (bsample)
	    go to error_exit_
	    }

# Remove objects from background
	call fsback (Memr[svec], row0, imax, verbose)

# Save background sample range as a parameter
	bsample[1] = EOS
	do n = 1, nsect {
	    call sprintf (temp,SZ_LINE,"%d:%d")
		call pargi (bsect[n,1]-imax)
		call pargi (bsect[n,2]-imax)
	    call strcat (temp,bsample,SZ_LINE)
	    if (n < nsect)
		call strcat (",",bsample,SZ_LINE)
	    }
	call clpstr ("b_sample",bsample)

	if (verbose) {
	    call printf ("FINDSPEC: Background sampled from %s\n")
		call pargstr (bsample)
	    call flush (STDOUT)
	    }

	# Draw the vector to the screen.
plotfind_
	dispap = clgetb ("dispap")
	if (dispap) {
	    call fsplot (Memc[image], Memr[xvec], Memr[svec], Memr[yvec],
			 row1,row2, col1,col2,Memc[wcslab],Memc[fmt],imax,
			 il,ir, nsmooth, ipk, rerun)
	    call clputb ("dbap", dbap)
	    if (ipk > 0)
		imax = ipk
	    if (rerun)
		go to findpeak_
	    }

# Create a default aperture.
	if (dbap) {
#	Specify aperture ID, beam, axis, and center in both dimensions.
#	Aperture limits along the dispersion axis are set to the full size
#	of the image while along the dispersion axis they are queried.
#	The default offset curve is a constant zero curve.
	    call ap_alloc (ap)
	    AP_ID(ap) = 1
	    AP_BEAM(ap) = 1
	    dispaxis = 1
	    call imgipar (im, "DISPAXIS", dispaxis)
	    if (dispaxis == 1) {
		apaxis = 2
		dispcenter = (col1 + col2) * 0.5
		}
	    else {
		apaxis = 1
		dispcenter = (row1 + row2) * 0.5
		}

	    AP_AXIS(ap) = apaxis

# Set the limits for the aperture
	    AP_CEN(ap, apaxis) = imax
	    AP_LOW(ap, apaxis) = il
	    AP_HIGH(ap, apaxis) = ir

# Set the limits for the spectrum trace
	    AP_CEN(ap, dispaxis) = dispcenter
	    AP_LOW(ap, dispaxis) = 1 - dispcenter
	    AP_HIGH(ap, dispaxis) = IM_LEN(im, dispaxis) - dispcenter

# Set the trace curve.
# If the input template aperture is NULL then the output trace curve
# is set to a constant zero otherwise a copy from the input template
# aperture is made.

	# Determine the limits over which the curve is defined.
	    a = 1
	    b = IM_LEN(im, dispaxis)
	    if (a == b)
		b = b + 1

	# Set the curve to a legendre polynomial of order 1 and value 0.
	    c[1] = 0.
	    call cvset (AP_CV(ap), LEGENDRE, a, b, c, 1)

# Set the background fitting ICFIT structure for an aperture.
# If the input template aperture is NULL then the output background fitting
# ICFIT pointer is initialized otherwise a copy from the input template
# aperture is made.

	    call ic_open (AP_IC(ap))
	    ic = AP_IC(ap)

	    call salloc (str, SZ_LINE, TY_CHAR)
	    call clgstr ("b_function", Memc[str], SZ_LINE)
	    call ic_pstr (ic, "function", Memc[str])
	    call ic_puti (ic, "order", clgeti ("b_order"))
	    call ic_pstr (ic, "sample", bsample)
	    call ic_puti (ic, "naverage", clgeti ("b_naverage"))
	    call ic_puti (ic, "niterate", clgeti ("b_niterate"))
	    call ic_putr (ic, "low", clgetr ("b_low"))
	    call ic_putr (ic, "high", clgetr ("b_high"))
	    call ic_putr (ic, "grow", clgetr ("b_grow"))
	    if (AP_AXIS(ap) == 1)
		call ic_pstr (ic, "xlabel", "Column")
	    else
		call ic_pstr (ic, "xlabel", "Line")

	# Set background min and max based on sample regions.
	    xmin = ilb2
	    xmax = irb2
	    call ic_putr (ic, "xmin", xmin)
	    call ic_putr (ic, "xmax", xmax)

	    nap = 1
	    call ap_dbwrite (Memc[image], ap)
	    call ap_free (ap)

	# Write spectrum range to FINDOBJ header record
	    call sprintf (hstr, SZ_LINE,"%d:%d")
		call pargi (imax+il)
		call pargi (imax+ir)
	    call imastr (im,"FINDOBJ",hstr,SZ_HSTRING)

	# Write background range to FINDBACK header record
	    bsample[1] = EOS
	    do n = 1, nsect {
		call sprintf (temp,SZ_LINE,"%d:%d")
		    call pargi (bsect[n,1])
		    call pargi (bsect[n,2])
		call strcat (temp,bsample,SZ_LINE)
		if (n < nsect)
		    call strcat (",",bsample,SZ_LINE)
		}
	    call imastr (im,"FINDBACK",bsample,SZ_HSTRING)

	# Write history to FINDSPEC header record
	    call logtime (dstr, SZ_LINE)
	    call sprintf (hstr, SZ_LINE,
		"procdata.findspec %s%s obj = %d:%d")
		call pargstr (VERSION)
		call pargstr (dstr)
		call pargi (imax+il)
		call pargi (imax+ir)
	    hstr0[1] = EOS
	    call imgspar (im,"FINDSPEC",hstr0,SZ_HSTRING)
	    if (hstr0[1] != EOS)
		call imputh (im, "HISTORY", hstr0)
	    call imastr (im, "FINDSPEC", hstr,SZ_HSTRING)
	    IM_UPDATE(im) = YES
	    }
	else
	    IM_UPDATE(im) = NO
	go to endfind_

error_exit_
	call clputi ("center",0)
	call clputi ("lower",0)
	call clputi ("upper",0)
	call clpstr ("b_sample","")

	# Free resources.
endfind_
	call mfree (xvec, TY_REAL) 
	call mfree (yvec, TY_REAL) 
	call mfree (svec, TY_REAL) 
	call mw_close (mw)
	call sfree (sp) 
	call imunmap (im) 
end


# fsmedian -- Get median of specified columns from an image.  The average
# data vector is returned as y_vector; the line coordinates are returned in
# x_vector.  The data vector min and max are also returned.
 
procedure fsmedian (im, mw, ct, col1,col2, row1,row2, x_vector, y_vector,
		    wcslab, format, sz_wcslab)
 
pointer im              # Pointer to image header
pointer	mw		# MWCS  pointer
pointer	ct		# CT pointer
int     col1            # First column to extract
int     col2            # Last column to extract
int     row1            # First row to extract
int     row2            # Last row to extract
real    x_vector[ARB]   # The row ordinal values (returned)
real    y_vector[ARB]   # The column data values (returned)
char	wcslab[sz_wcslab]	# WCS label if present
char	format[sz_wcslab]	# WCS format if present
int	sz_wcslab	# String length
 
int     i, nrows, ncols, irow
pointer	sp, axvals, off, imgl2r()
real    amedr()
 
begin
	call smark (sp)
	call salloc (axvals, IM_MAXDIM, TY_REAL)

        # Fill x and y arrays.
        nrows = row2 - row1 + 1
        ncols = col2 - col1 + 1
 
	Memr[axvals] = (col1 + col2) / 2.
	call plt_wcs (im, mw, ct, 2, Memr[axvals], real(row1), real(row2),
	    x_vector, nrows, wcslab, format, sz_wcslab)

	i = 0
        do irow = row1, row2 {
            off = imgl2r (im, irow)
	    i = i + 1
	    x_vector[i] = real (irow)
            y_vector[i] = amedr (Memr[off+col1-1], ncols)
            }

	call sfree (sp)
end
# Feb  1 1994	New program

# Jul 14 1995	Do not add / if there is no directory
# Aug  2 1995	Initialize POINTMODE
# Sep 29 1995	Set background sections in separate subroutine
# Sep 30 1995	Set peak finding background iterations
# Oct  2 1995	Set minimum background section size
# Oct  5 1995	Write FINDSPEC header record recording version and results
# Oct 10 1995	Call FSPLOT to interactively plot image cross-section
# Oct 12 1995	Close MW structure; free XVEC and YVEC
# Oct 31 1995	If not writing to heder, open file read-only
# Nov  1 1995	Change subroutine names PKWIDTH->FSPEAK, BACKSECT->FSBACK
# Nov  1 1995	Compute y range in plot subroutine
# Nov  1 1995	Add peak return from FSPLOT
# Nov  1 1995	Add initial pixel in vector to FSBACK
# Nov  3 1995	Pass background parameters through FSPEC common
# Nov  6 1995	Pass smoothed vector to FSPEAK, FSBACK, and FSPLOT
# Nov 27 1995	Move DBAP to labelled common
# Nov 27 1995	Open file READ_WRITE based on access, not DBAP

# Jan 17 1996	Drop .imh at end of filename, if it is there
# Feb 21 1996	Drop unused arguments from FSBACK call
# Feb 21 1996	Drop unused variables; fix PNIT CLGET error
# Feb 21 1996	Return center =0 if object or background not found
# Apr 11 1996	Explicitly set limits along dispersion axis

# May 22 2000	Drop trailing .fit or .fits as well as .imh from filenames

# Nov 22 2004	Add string length to last call to imastr()
