include	<imhdr.h>
include	<smw.h>

define	LEN_USER_AREA	100000

# T_FIXTRACT - Replace pixels in a reduced spectrum with those from a similar
# reduced spectrum.  Both must cover exactly the same wavelength range at the
# same resolution.

procedure t_fixtract()

int	list			# list of input spectra
int	listfix			# list of input replacement spectra

char	image[SZ_FNAME]		# input image file name
char	imfix[SZ_FNAME]		# replacement image file name
char	imout[SZ_FNAME]		# output image file name
char	imname[SZ_FNAME]	# image name
char	outname[SZ_FNAME]	# Output file name
int	mspec			# Number of spectrum to read from multispec file
int	mband			# Number of band to read from multispec file
int	i
pointer	im,imf,imo		# Image header structures
pointer	sh,shf,sho		# Spectrum header structures
pointer	pix,pixf,pixo		# pointers to pixel values
pointer	wav			# pointer to pixel wavelengths
int	npts			# Number of pixels in spectrum
real	wavelength		# Wavelength at pixel in angstroms
real	value			# Image value at pixel
real	minwav			# Minimum wavelength in angstroms
real	maxwav			# Maximum wavelength in angstroms
int	minpix, maxpix
real	fpix, fmin, fmax, fnorm
int	outfd
 
bool	debug, verbose, heading
int	mode, nc, ns
double	fsum

real	clgetr()
int	clpopnu(), clgfil()
int	clgeti(), clscan()
bool	clgetb()
int	open()
int	strldx(), strncmp()
real	renorm
bool	outfile
char	cdot
char	outform[32]
char	fluxform[16]

begin
	cdot = '.'

	# Open input list of images
	list = clpopnu ("input")
	listf = clpopnu ("fix")
	mspec = clgeti ("specnum")
	mband = clgeti ("specband")

	minwav = clgetr ("lambda1")
	maxwav = clgetr ("lambda2")

	minpix = clgeti ("pix1")
	maxpix = clgeti ("pix2")

	verbose = clgetb ("verbose")

	debug = FALSE
	if (clscan ("debug") != EOF)
	    debug = clgetb ("debug")

	heading = FALSE
	if (clscan ("heading") != EOF)
	    heading = clgetb ("heading")

	renorm = 0.0
	if (clscan ("renormalize") != EOF)
	    renorm = clgetr ("renormalize")
	outfile = FALSE
	if (clscan ("outfile") != EOF)
	    outfile = clgetb ("outfile")
	call strcpy ("%g", fluxform, 16)
	if (clscan ("fluxform") != EOF)
	    call clgstr ("fluxform", fluxform, 16)
	mode = READ_ONLY

	# Loop over spectra made with clean=yes
	while (clgfil (list, image, SZ_FNAME) != EOF) {

	    # Try to open the spectrum
	    call getimage (image, mspec, mband, pix, im, sh, npts, imname,
		mode, debug)
	    if (im == ERR)
		next

	    # Get name of image run with clean=no
	    call clgfil (listf, imfix, SZ_FNAME)

	    # Create output file name
	    ns = strldx ('/', image)
	    if (strsearch (image,".ms.imh") > 0) {
		nc = strsearch (image,".ms.imh")
		call strcpy (image[ns+1], outname, nc-ns-1)
		outname[nc] = EOS
		call strcat ("f.ms.imh", outname, SZ_PATHNAME)
		}
	    else if (strsearch (image,".ms.fits") >= 0) {
		nc = strsearch (image,".ms.fits")
		call strcpy (image[ns+1], outname, nc-ns-1)
		outname[nc] = EOS
		call strcat ("f.ms.fits", outname, SZ_PATHNAME)
		}
	    else if (strsearch (image,".ms.fit") >= 0) {
		nc = strsearch (image,".ms.fit")
		call strcpy (image[ns+1], outname, nc-ns-1)
		outname[nc] = EOS
		call strcat ("f.ms.fit", outname, SZ_PATHNAME)
		}
	    else if (strsearch (image,"imh") >= 0) {
		nc = strsearch (image,".imh")
		call strcpy (image[ns+1], outname, nc-ns-1)
		outname[nc] = EOS
		call strcat ("f.imh", outname, SZ_PATHNAME)
		}
	    else if (strsearch (image,".fits") >= 0) {
		nc = strsearch (image,".fits")
		call strcpy (image[ns+1], outname, nc-ns-1)
		outname[nc] = EOS
		call strcat ("f.fits", outname, SZ_PATHNAME)
		}
	    else if (strsearch (image,".fit") >= 0) {
		nc = strsearch (image,".fit")
		call strcpy (image[ns+1], outname, nc-ns-1)
		outname[nc] = EOS
		call strcat ("f.fit", outname, SZ_PATHNAME)
		}
	    else {
		nc = strlen (image)
		call strcpy (image[ns+1], outname, nc-ns-1)
		call strcat ("f", outname, SZ_PATHNAME)
		}

	    # Open output file
	    outfd = open (outname, APPEND, TEXT_FILE)
	
	# Loop over pixels
	    if (minpix == INDEFI || minpix < NP1(sh))
		minpix = NP1(sh)
	    if (maxpix == INDEFI || maxpix > NP2(sh))
		maxpix = NP2(sh)

	# Normalize data so maximum value is renorm
	    if (renorm > 0.0) {
		do i = minpix, maxpix {
		    fpix = Memr[pix + i - 1]
		    if (i == minpix) {
			fmax = fpix
			fmin = fpix
			}
		    else if (fpix > fmax)
			fmax = fpix
		    else if (fpix < fmin)
			fmin = fpix
		    }
		fnorm = renorm / (fmax - fmin)
		if (debug) {
		    call printf ("LISTWAV: pixels %d-%d: %.8g-%.8g -> %.2f\n")
			call pargi (minpix)
			call pargi (maxpix)
			call pargr (fmin)
			call pargr (fmax)
			call pargr (fnorm)
		    }
		}

	# Normalize data so mean value is renorm
	    else if (renorm < 0.0) {
		fsum = 0.d0
		do i = minpix, maxpix {
		    fpix = Memr[pix + i - 1]
		    fsum = fsum + double (fpix)
		    }
		if (debug) {
		    call printf ("LISTWAV: sum= %.2f\n")
			call pargd (fsum)
		    }
		fsum = fsum / double (maxpix - minpix + 1)
		fnorm = -renorm / real (fsum)
		fmin = 0.0
		if (debug) {
		    call printf ("LISTWAV: pixels %d-%d: mean= %.8g -> %.2f\n")
			call pargi (minpix)
			call pargi (maxpix)
			call pargd (fsum)
			call pargr (fnorm)
		    }
		}
	    else {
		fnorm = 1.0
		fmin = 0.0
		}

	#  Set up format for output
	    if (number == 1) {
		call strcpy ("%4d %9.3f ", outform, 32)
		call strcat (fluxform, outform, 32)
		}
	    else if (number == 2) {
		call strcpy ("%9.3f %4d ", outform)
		call strcat (fluxform, outform, 32)
		}
	    else if (number == 3) {
		call strcpy ("%9.3f ", outform, 32)
		call strcat (fluxform, outform, 32)
		call strcat (" %4d", outform, 32)
		}
	    else {
		call strcpy ("%9.3f ", outform, 32)
		call strcat (fluxform, outform, 32)
		}
	    call strcat ("\n", outform, 32)

	    do i = minpix, maxpix {
		wavelength = Memr[wav+i-1]
		value = Memr[pix+i-1]
		if ((minwav == INDEFR || wavelength >= minwav) &&
		    (maxwav == INDEFR || wavelength <= maxwav)) {

		    if (renorm != 0.0)
			value = (value - fmin) * fnorm

		# Write both values to the standard output
		    if (number == 1) {
			call fprintf (outfd, outform)
			    call pargi (i)
			    call pargr (wavelength)
			    call pargr (value)
			}
		    else if (number == 2) {
			call fprintf (outfd, outform)
			    call pargr (wavelength)
			    call pargi (i)
			    call pargr (value)
			}
		    else if (number == 3) {
			call fprintf (outfd, outform)
			    call pargr (wavelength)
			    call pargr (value)
			    call pargi (i)
			}
		    else {
			call fprintf (outfd, outform)
			    call pargr (wavelength)
			    call pargr (value)
			}
		    }
		}

	# Flush standard output before opening next image
	    call flush (outfd)

	# Close image
	    call close_image (im, sh)
	    }

	# Close input list
	if (outfile)
	    call close (outfd)
	call clpcls (list)
end
# Jul  8 1996	New task

# Mar 17 1998	Add options to limit pixel numbers to read
# Mar 17 1998	Add specband to deal with multispec spectrum
# Jun 11 1998	List only within limits of dispersion solution
# Sep 23 1998	Close spectral mwcs

# Jul 22 1999	Add renormalization and automatic output file naming
