include	<imhdr.h>
include	<smw.h>

# T_LIMWAV - Compute and save the first and last pixels and corresponding wavelengths

procedure t_limwav()

int	list			# list of input images

char	image[SZ_FNAME]		# image file name
char	imname[SZ_FNAME]	# image name
int	mspec			# Number of spectrum to read from multispec file
int	mband			# Number of band to read from multispec file
pointer	im			# Image header structure
pointer	sh			# Spectrum header structure
pointer	pix			# pointer to pixel values
pointer	wav			# pointer to pixel wavelengths
int	npts			# Number of pixels in spectrum
real	minwav			# Minimum wavelength in angstroms
real	maxwav			# Maximum wavelength in angstroms
int	minpix, maxpix
 
bool	debug, verbose
int	mode

int	clpopnu(), clgfil()
int	clgeti(), clscan()
bool	clgetb()

begin

	# Open input list of images
	list = clpopnu ("input")
	mspec = clgeti ("specnum")
	mband = clgeti ("specband")

	verbose = clgetb ("verbose")

	debug = FALSE
	if (clscan ("debug") != EOF)
	    debug = clgetb ("debug")

	mode = READ_ONLY

	# Loop over images
	while (clgfil (list, image, SZ_FNAME) != EOF) {

	    # Try to open the image
	    call getimage (image, mspec, mband, pix, im, sh, npts, imname,
		mode, debug)
	    if (im == ERR)
		next

	# Find first and last pixels
	    minpix = NP1(sh)
	    maxpix = NP2(sh)

	# Write limiting pixels to parameter file
	    call clputi ("pix1", minpix)
	    call clputi ("pix2", maxpix)

        # Get image wavelengths
            wav = SX(sh)

	# Find wavelengths for first and last pixels
	    minwav = Memr[wav+minpix-1]
	    maxwav = Memr[wav+maxpix-1]

	# Write limiting wavelengths to parameter file
	    call clputr ("wav1", minwav)
	    call clputr ("wav2", maxwav)

	# Flush standard output before opening next image
	    call flush (STDOUT)

	# Print image name
	    if (verbose) {
		call printf ("Spectrum %s %d - %d  %8.3f - %8.3f\n")
		    call pargstr (imname)
		    call pargi (minpix)
		    call pargi (maxpix)
		    call pargr (minwav)
		    call pargr (maxwav)
		call flush (STDOUT)
		}

	# Close image
	    call close_image (im, sh)
	    }

	# Close input list
	call clpcls (list)
end

# Jun 19 2003	New task, based on listwav
# Jul  2 2003	Print wavelength limits as well as pixel limits if verbose
