include	<imhdr.h>
include	<smw.h>

define	LEN_USER_AREA	100000

# T_LISTDISP-- Compute and print the pixel, wavelength, and delta wavelength
# across each pixel for all specified pixels in a list of one dimensional
# wavelength calibrated images.

procedure t_listdisp()

int	list			# list of input images

char	image[SZ_FNAME]		# image file name
char	imname[SZ_FNAME]	# image name
char	outname[SZ_FNAME]	# Output file name
int	mspec			# Number of spectrum to read from multispec file
int	mband			# Number of band to read from multispec file
int	i
pointer	im			# Image header structure
pointer	sh			# Spectrum header structure
pointer	pix			# pointer to pixel values
pointer	wav			# pointer to pixel wavelengths
int	npts			# Number of pixels in spectrum
real	wavelength		# Wavelength at pixel in angstroms
double	wl			# Wavelength at pixel in angstroms
real	value			# Image value at pixel
real	minwav			# Minimum wavelength in angstroms
real	maxwav			# Maximum wavelength in angstroms
int	minpix, maxpix
int	outfd
double	dpix
double	wl1, wl2, dwlmean
 
bool	debug, verbose, heading, logwav
int	mode, nc, ns

real	clgetr()
int	clpopnu(), clgfil()
int	clgeti(), clscan()
bool	clgetb()
int	open()
int	strldx(), strncmp()
double	wcs_p2w(), wcs_p2l()
bool	outfile
char	cdot, cslash
char	outform[32]
char	waveform[16]

begin
	cdot = '.'
	cslash = '/'

	# Open input list of images
	list = clpopnu ("input")
	mspec = clgeti ("specnum")
	mband = clgeti ("specband")

	minwav = clgetr ("lambda1")
	maxwav = clgetr ("lambda2")

	minpix = clgeti ("pix1")
	maxpix = clgeti ("pix2")

	verbose = clgetb ("verbose")

	debug = FALSE
	if (clscan ("debug") != EOF)
	    debug = clgetb ("debug")

	heading = FALSE
	if (clscan ("heading") != EOF)
	    heading = clgetb ("heading")

	outfile = FALSE
	if (clscan ("outfile") != EOF)
	    outfile = clgetb ("outfile")
	logwav = FALSE
	if (clscan ("logwav") != EOF)
	    logwav = clgetb ("logwav")
	call strcpy ("%9.3f", waveform, 16)
	if (clscan ("waveform") != EOF)
	    call clgstr ("waveform", waveform, 16)
	mode = READ_ONLY

	# Loop over images
	while (clgfil (list, image, SZ_FNAME) != EOF) {

	    # Try to open the image
	    call getimage (image, mspec, mband, pix, im, sh, npts, imname,
		mode, debug)
	    if (im == ERR)
		next

	    # Open output file
	    if (outfile) {
		nc = strldx (cdot, image)
		ns = strldx (cslash, image)
		if (nc > 0) {
		    if (strncmp (image[nc+1],"imh",3) == 0) {
			call strcpy (image[ns+1], outname, nc-ns-1)
			outname[nc] = EOS
			}
		    else if (strncmp (image[nc+1],"fit",3) == 0) {
			call strcpy (image[ns+1], outname, nc-ns-1)
			outname[nc] = EOS
			}
		    else
			call strcpy (image[ns+1], outname, 32)
		    }
		call strcat (".wav", outname, SZ_PATHNAME)
		outfd = open (outname, APPEND, TEXT_FILE)
		}
	    else
		outfd = STDOUT

	    if (heading) {
		if (mspec > 0 && mband > 0) {
		    call fprintf (outfd, "# %s[%d] %s band %d: %d pixels\n")
			call pargstr (image)
			call pargi (mspec)
			call pargstr (imname)
			call pargi (mband)
			call pargi (npts)
		    }
		else if (mspec > 0) {
		    call fprintf (outfd, "# %s[%d] %s : %d pixels\n")
			call pargstr (image)
			call pargi (mspec)
			call pargstr (imname)
			call pargi (npts)
		    }
	   	else if (mband > 0) {
		    call fprintf (outfd, "# %s %s band %d: %d pixels\n")
			call pargstr (image)
			call pargi (mspec)
			call pargstr (imname)
			call pargi (npts)
		    }
		else {
		    call fprintf (outfd, "# %s %s : %d pixels\n")
			call pargstr (image)
			call pargstr (imname)
			call pargi (npts)
		    }
		}

	# Get image wavelengths to list
	    wav = SX(sh)

	# Print image name
	    if (verbose) {
		if (outfile) {
		    call printf ("Spectrum %s %d - %d -> %s\n")
			call pargstr (imname)
			call pargi (NP1(sh))
			call pargi (NP2(sh))
			call pargstr (outname)
		    }
		else {
		    call printf ("Spectrum %s %d - %d\n")
			call pargstr (imname)
			call pargi (NP1(sh))
			call pargi (NP2(sh))
		    }
		call flush (STDOUT)
		}

	# Loop over pixels
	    if (minpix == INDEFI || minpix < NP1(sh))
		minpix = NP1(sh)
	    if (maxpix == INDEFI || maxpix > NP2(sh))
		maxpix = NP2(sh)

	# Find mean wavelength per pixel variation
	    wl1 = wcs_p2w (double(minpix))
	    wl2 = wcs_p2w (double(maxpix))
	    if (wl2 > wl1)
		dwlmean = (wl2 - wl1) / double (maxpix - minpix)
	    else
		dwlmean = (wl1 - wl2) / double (maxpix - minpix)
	    if (debug) {
		call printf ("LISTDISP: pixels %d-%d: %9.4f-%9.4f -> %.4f\n")
		    call pargi (minpix)
		    call pargi (maxpix)
		    call pargd (wl1)
		    call pargd (wl2)
		    call pargr (dwlmean)
		}

	    if (debug) {
		call printf ("Output format is w=%s\n")
		    call pargstr (waveform)
		call flush (STDOUT)
		}

	#  Set up format for output
	    call strcpy ("%4d ", outform, 32)
	    call strcat (waveform, outform, 32)
	    call strcat (" ", outform, 32)
	    call strcat (waveform, outform, 32)
	    call strcat ("\n", outform, 32)
	    if (debug) {
		call printf ("Output format is %s\n")
		    call pargstr (outform)
		call flush (STDOUT)
		}

	    call wcs_set (sh)
	    do i = minpix, maxpix {
		dpix = double (i)
		wavelength = Memr[wav+i-1]
		if (logwav) {
		    wl = wcs_p2l (dpix)
		    wl1 = wcs_p2l (dpix - 0.5d0)
		    wl2 = wcs_p2l (dpix + 0.5d0)
		    }
		else {
		    wl = wcs_p2w (dpix)
		    wl1 = wcs_p2w (dpix - 0.5d0)
		    wl2 = wcs_p2w (dpix + 0.5d0)
		    }
		value = Memr[pix+i-1]
		if ((minwav == INDEFR || wavelength >= minwav) &&
		    (maxwav == INDEFR || wavelength <= maxwav)) {

		# Write pixel, wavelength, and delta wavelength
		    call fprintf (outfd, outform)
			call pargi (i)
			call pargd (wl)
			if (wl2 > wl1)
			    call pargd (wl2 - wl1)
			else
			    call pargd (wl1 - wl2)
		    }
		}

	# Flush standard output before opening next image
	    call flush (outfd)

	# Close image
	    call close_image (im, sh)
	    }

	# Close input list
	if (outfile)
	    call close (outfd)
	call clpcls (list)
end
# Jul 25 2005	New task based on procdata.listwav
