include	<imhdr.h>
include	<smw.h>

define	LEN_USER_AREA	100000

# T_LISTWAV - Compute and print the pixel value and corresponding wavelength
# for all the pixels in a list of one dimensional wavelength calibrated
# images.

procedure t_listwav()

int	list			# list of input images

int	number			# print pixel number in this column
char	image[SZ_FNAME]		# image file name
char	imname[SZ_FNAME]	# image name
char	outname[SZ_FNAME]	# Output file name
int	mspec			# Number of spectrum to read from multispec file
int	mband			# Number of band to read from multispec file
int	i
pointer	im			# Image header structure
pointer	sh			# Spectrum header structure
pointer	pix			# pointer to pixel values
pointer	wav			# pointer to pixel wavelengths
int	npts			# Number of pixels in spectrum
real	wavelength		# Wavelength at pixel in angstroms
double	wl			# Wavelength at pixel in angstroms
real	value			# Image value at pixel
real	minwav			# Minimum wavelength in angstroms
real	maxwav			# Maximum wavelength in angstroms
int	minpix, maxpix
real	fpix, fmin, fmax, fnorm
int	outfd
double	dpix, wl1, wl2
 
bool	debug, verbose, heading, logwav
int	mode, nc, ns
double	fsum

real	clgetr()
int	clpopnu(), clgfil()
int	clgeti(), clscan()
bool	clgetb()
int	open()
int	strldx(), strncmp()
double	wcs_p2w(), wcs_p2l()
real	renorm
bool	outfile
char	cdot, cslash
char	outform[32]
char	fluxform[16]
char	waveform[16]
double	c0


begin
	cdot = '.'
	cslash = '/'
	c0 = 299792.5d0

	# Open input list of images
	list = clpopnu ("input")
	mspec = clgeti ("specnum")
	mband = clgeti ("specband")

	number = clgeti ("number")

	minwav = clgetr ("lambda1")
	maxwav = clgetr ("lambda2")

	minpix = clgeti ("pix1")
	maxpix = clgeti ("pix2")

	verbose = clgetb ("verbose")

	debug = FALSE
	if (clscan ("debug") != EOF)
	    debug = clgetb ("debug")

	heading = FALSE
	if (clscan ("heading") != EOF)
	    heading = clgetb ("heading")

	renorm = 0.0
	if (clscan ("renormalize") != EOF)
	    renorm = clgetr ("renormalize")
	outfile = FALSE
	if (clscan ("outfile") != EOF)
	    outfile = clgetb ("outfile")
	call strcpy ("%g", fluxform, 16)
	if (clscan ("fluxform") != EOF)
	    call clgstr ("fluxform", fluxform, 16)
	logwav = FALSE
	if (clscan ("logwav") != EOF)
	    logwav = clgetb ("logwav")
	call strcpy ("%9.3f", waveform, 16)
	if (clscan ("waveform") != EOF)
	    call clgstr ("waveform", waveform, 16)
	mode = READ_ONLY

	# Loop over images
	while (clgfil (list, image, SZ_FNAME) != EOF) {

	    # Try to open the image
	    call getimage (image, mspec, mband, pix, im, sh, npts, imname,
		mode, debug)
	    if (im == ERR)
		next

	    # Open output file
	    if (outfile) {
		nc = strldx (cdot, image)
		ns = strldx (cslash, image)
		if (nc > 0) {
		    if (strncmp (image[nc+1],"imh",3) == 0) {
			call strcpy (image[ns+1], outname, nc-ns-1)
			outname[nc] = EOS
			}
		    else if (strncmp (image[nc+1],"fit",3) == 0) {
			call strcpy (image[ns+1], outname, nc-ns-1)
			outname[nc] = EOS
			}
		    else
			call strcpy (image[ns+1], outname, 32)
		    }
		call strcat (".wav", outname, SZ_PATHNAME)
		outfd = open (outname, APPEND, TEXT_FILE)
		}
	    else
		outfd = STDOUT

	    if (heading) {
		if (mspec > 0 && mband > 0) {
		    call fprintf (outfd, "# %s[%d] %s band %d: %d pixels\n")
			call pargstr (image)
			call pargi (mspec)
			call pargstr (imname)
			call pargi (mband)
			call pargi (npts)
		    }
		else if (mspec > 0) {
		    call fprintf (outfd, "# %s[%d] %s : %d pixels\n")
			call pargstr (image)
			call pargi (mspec)
			call pargstr (imname)
			call pargi (npts)
		    }
	   	else if (mband > 0) {
		    call fprintf (outfd, "# %s %s band %d: %d pixels\n")
			call pargstr (image)
			call pargi (mspec)
			call pargstr (imname)
			call pargi (npts)
		    }
		else {
		    call fprintf (outfd, "# %s %s : %d pixels\n")
			call pargstr (image)
			call pargstr (imname)
			call pargi (npts)
		    }
		}

	# Get image wavelengths to list
	    wav = SX(sh)

	# Print image name
	    if (verbose) {
		if (outfile) {
		    call printf ("Spectrum %s %d - %d -> %s\n")
			call pargstr (imname)
			call pargi (NP1(sh))
			call pargi (NP2(sh))
			call pargstr (outname)
		    }
		else {
		    call printf ("Spectrum %s %d - %d\n")
			call pargstr (imname)
			call pargi (NP1(sh))
			call pargi (NP2(sh))
		    }
		call flush (STDOUT)
		}

	# Loop over pixels
	    if (minpix == INDEFI || minpix < NP1(sh))
		minpix = NP1(sh)
	    if (maxpix == INDEFI || maxpix > NP2(sh))
		maxpix = NP2(sh)

	# Normalize data so maximum value is renorm
	    if (renorm > 0.0) {
		do i = minpix, maxpix {
		    fpix = Memr[pix + i - 1]
		    if (i == minpix) {
			fmax = fpix
			fmin = fpix
			}
		    else if (fpix > fmax)
			fmax = fpix
		    else if (fpix < fmin)
			fmin = fpix
		    }
		fnorm = renorm / (fmax - fmin)
		if (debug) {
		    call printf ("LISTWAV: pixels %d-%d: %.8g-%.8g -> %.2f\n")
			call pargi (minpix)
			call pargi (maxpix)
			call pargr (fmin)
			call pargr (fmax)
			call pargr (fnorm)
		    }
		}

	# Normalize data so mean value is renorm
	    else if (renorm < 0.0) {
		fsum = 0.d0
		do i = minpix, maxpix {
		    fpix = Memr[pix + i - 1]
		    fsum = fsum + double (fpix)
		    }
		if (debug) {
		    call printf ("LISTWAV: sum= %.2f\n")
			call pargd (fsum)
		    }
		fsum = fsum / double (maxpix - minpix + 1)
		fnorm = -renorm / real (fsum)
		fmin = 0.0
		if (debug) {
		    call printf ("LISTWAV: pixels %d-%d: mean= %.8g -> %.2f\n")
			call pargi (minpix)
			call pargi (maxpix)
			call pargd (fsum)
			call pargr (fnorm)
		    }
		}
	    else {
		fnorm = 1.0
		fmin = 0.0
		}

	    if (debug) {
		call printf ("Output formats are w=%s, v=%s\n")
		    call pargstr (waveform)
		    call pargstr (fluxform)
		call flush (STDOUT)
		}

	#  Set up format for output
	    if (number == 1) {
		call strcpy ("%4d ", outform, 32)
		call strcat (waveform, outform, 32)
		call strcat (" ", outform, 32)
		call strcat (fluxform, outform, 32)
		}
	    else if (number == 2) {
		call strcpy (waveform, outform, 32)
		call strcat (" %4d ", outform, 32)
		call strcat (fluxform, outform, 32)
		}
	    else if (number == 3) {
		call strcpy (waveform, outform, 32)
		call strcat (" ", outform, 32)
		call strcat (fluxform, outform, 32)
		call strcat (" %4d", outform, 32)
		}
	    else if (number == 4) {
		call strcpy (" %4d ", outform, 32)
		call strcat (waveform, outform, 32)
		call strcat (" ", outform, 32)
		call strcat (waveform, outform, 32)
		call strcat (" ", outform, 32)
		call strcat (waveform, outform, 32)
		}
	    else {
		call strcpy (waveform, outform, 32)
		call strcat (" ", outform, 32)
		call strcat (fluxform, outform, 32)
		}
	    call strcat ("\n", outform, 32)
	    if (debug) {
		call printf ("Output format is %s\n")
		    call pargstr (outform)
		call flush (STDOUT)
		}

	    call wcs_set (sh)
	    do i = minpix, maxpix {
		dpix = double (i)
		wavelength = Memr[wav+i-1]
		if (logwav)
		    wl = wcs_p2l (dpix)
		else
		    wl = wcs_p2w (dpix)
		value = Memr[pix+i-1]
		if ((minwav == INDEFR || wavelength >= minwav) &&
		    (maxwav == INDEFR || wavelength <= maxwav)) {

		    if (renorm != 0.0)
			value = (value - fmin) * fnorm

		# Write both values to the standard output
		    if (number == 1) {
			call fprintf (outfd, outform)
			    call pargi (i)
			    call pargd (wl)
			    call pargr (value)
			}
		    else if (number == 2) {
			call fprintf (outfd, outform)
			    call pargd (wl)
			    call pargi (i)
			    call pargr (value)
			}
		    else if (number == 3) {
			call fprintf (outfd, outform)
			    call pargd (wl)
			    call pargr (value)
			    call pargi (i)
			}
		    else if (number == 4) {
			wl1 = wcs_p2w (dpix - 0.5d0)
			wl2 = wcs_p2w (dpix + 0.5d0)
			call fprintf (outfd, outform)
			    call pargi (i)
			    call pargd (wl)
			if (wl1 > wl2) {
			    call pargd (wl1 - wl2)
			    call pargd (((wl1 - wl2) / wl) * c0)
			    }
			else {
			    call pargd (wl2 - wl1)
			    call pargd (((wl2 - wl1) / wl) * c0)
			    }
			}
		    else {
			call fprintf (outfd, outform)
			    call pargd (wl)
			    call pargr (value)
			}
		    }
		}

	# Flush standard output before opening next image
	    call flush (outfd)

	# Close image
	    call close_image (im, sh)
	    }

	# Close input list
	if (outfile)
	    call close (outfd)
	call clpcls (list)
end
# Jul  8 1996	New task

# Mar 17 1998	Add options to limit pixel numbers to read
# Mar 17 1998	Add specband to deal with multispec spectrum
# Jun 11 1998	List only within limits of dispersion solution
# Sep 23 1998	Close spectral mwcs

# Jul 22 1999	Add renormalization and automatic output file naming

# May 10 2004	Write double precision wavelength or log wavelength
# May 10 2004	Add wavlog for log wavelength output and waveform for format

# Nov 22 2004	Fix bugs found by g77 compiler

# Jul 25 2005	Add option 4 to print wavelength and z difference per pixel
