include	<error.h>
include	"shdr.h"
include	<imhdr.h>

define	LEN_USER_AREA	100000

# T_LISTWAV - Compute and print the pixel value and corresponding wavelength
# for all the pixels in a list of one dimensional wavelength calibrated
# images.

procedure t_listwav()

int	list			# list of input images

int	number			# print pixel number in this column
char	image[SZ_FNAME]		# image file name
char	name[SZ_FNAME]		# image name
int	mspec			# Number of spectrum to read from multispec file
int	mband			# Number of band to read from multispec file
int	i
real	w0, wpc			# wavelength parameters
pointer	pix			# pointer to pixel values
pointer	wav			# pointer to pixel wavelengths
pointer	im			# Image header structure
pointer	sh			# Spectrum header structure
int	npts			# Number of pixels in spectrum
int	nband			# Band number for 3-D images
real	wavelength		# Wavelength at pixel in angstroms
real	value			# Image value at pixel
real	minwav			# Minimum wavelength in angstroms
real	maxwav			# Maximum wavelength in angstroms
int	minpix, maxpix
 
char	temp[16]
int	ltitle
bool	debug, verbose

real	clgetr()
int	clpopnu(), clgfil()
int	clgeti()
bool	clgetb()
pointer	immap()
errchk	immap()
int	imaccf()
int	nline, ap, ndim
int	strlen(),strldxs(),ip

begin
	# Open input list of images
	list = clpopnu ("input")
	mspec = clgeti ("specnum")
	mband = clgeti ("specband")

	number = clgeti ("number")

	minwav = clgetr ("lambda1")
	maxwav = clgetr ("lambda2")

	minpix = clgeti ("pix1")
	maxpix = clgeti ("pix2")

	verbose = clgetb ("verbose")
	debug = FALSE

	# Loop over images
	while (clgfil (list, image, SZ_FNAME) != EOF) {

	    # Try to open the image
	    iferr (im = immap (image, READ_ONLY, 0)) {
		call eprintf ("LISTWAV: Cannot read image %s\n")
		    call pargstr (image)
		next
	    }

	    ndim = IM_NDIM(im)
	    if (debug) {
		call printf ("LISTWAV: %d x %d x %d %d-D image: spectrum %d %d\n")
		    call pargi (IM_LEN(im,1))
		    call pargi (IM_LEN(im,2))
		    call pargi (IM_LEN(im,3))
		    call pargi (ndim)
		    call pargi (mspec)
		    call pargi (mband)
		}

	    if (ndim > 2 && IM_LEN(im,3) > 1) {
		nband = max (1, min (IM_LEN(im,3), mband))
		nline = max (1, min (IM_LEN(im,2), mspec))
		}
	    else if (ndim > 1 && IM_LEN(im,2) > 1) {
		nline = max (1, min (IM_LEN(im,2), mspec))
		nband = 1
		}
	    else {
		nline = 1
		nband = 1
		}

	# Set up FITS World Coordinate System keywords
	# if not present in 1-D spectrum
	    if (ndim < 2) {
		if (imaccf (im, "W0") == YES && imaccf (im, "CRVAL1") == NO) {
		    call imgdpar (im, "W0",w0)
		    call imaddi (im,"CRPIX1",1)
		    call imaddd (im,"CRVAL1",w0)
		    }
		if (imaccf (im, "WPC") == YES && imaccf (im, "CDELT1") == NO) {
        	    call imgdpar (im, "WPC",wpc)
		    call imaddd (im,"CDELT1",wpc)
		    }
		}

	# Set dispersion axis for 1-D spectra
	    if (imaccf (im, "DISPAXIS") == NO)
		call imaddi (im,"DISPAXIS",1)

	# Open spectrum header
            ap = nline
	    call shdr_open (im, nline, nband, ap, SHDATA, sh)

	    npts = IM_LEN (im, 1)

	# Get image line to list
	    pix = SY(sh)
	    wav = SX(sh)

	# Title is object name (and spectrum number if multispec)
	    call strcpy (IM_TITLE(im), image, SZ_FNAME)
	    ltitle = strlen (image)
	    if (ltitle > 1) {
		ip = strldxs ("$/",image)
		if (ip > 0)
		    call strcpy (image[ip+1],name,SZ_FNAME)
		else
		    call strcpy (image, name, SZ_FNAME)
		}
	    else
		name[1] = EOS

	    if (mband > 0) {
		call sprintf (temp,16,"[%d,%d]")
		    call pargi (mspec)
		    call pargi (mband)
		call strcat (temp,name,SZ_FNAME)
		}
		
	    else if (mspec > 0) {
		call sprintf (temp,16,"[%d]")
		    call pargi (mspec)
		call strcat (temp,name,SZ_FNAME)
		}

	# Print image name
	    if (verbose) {
		call printf ("Spectrum %s\n\n")
		    call pargstr (name)
		}

	# Loop over pixels
	    if (minpix == INDEFI)
		minpix = 1
	    if (maxpix == INDEFI)
		maxpix = npts
	    do i = minpix, maxpix {
		wavelength = Memr[wav+i-1]
		value = Memr[pix+i-1]
		if ((minwav == INDEFR || wavelength >= minwav) &&
		    (maxwav == INDEFR || wavelength <= maxwav)) {

		# Write both values to the standard output
		    if (number == 1) {
			call printf ("%4d %9.3f %g\n")
			    call pargi (i)
			    call pargr (wavelength)
			    call pargr (value)
			}
		    else if (number == 2) {
			call printf ("%9.3f %4d %g\n")
			    call pargr (wavelength)
			    call pargi (i)
			    call pargr (value)
			}
		    else if (number == 3) {
			call printf ("%9.3f %g %4d\n")
			    call pargr (wavelength)
			    call pargr (value)
			    call pargi (i)
			}
		    else {
			call printf ("%9.3f %g\n")
			    call pargr (wavelength)
			    call pargr (value)
			}
		    }
		}

	# Flush standard output before opening next image
	    call flush (STDOUT)

	# Close image
	    call imunmap (im)
	    call shdr_close (sh)
	    }

	# Close input list
	call clpcls (list)
end
# Jul  8 1996	New task

# Mar 17 1998	Add options to limit pixel numbers to read
# Mar 17 1998	Add specband to deal with multispec spectrum
