include	<error.h>
include	"shdr.h"
include	<imhdr.h>

define	LEN_USER_AREA	100000
define	MAX_RANGES	50

# T_PIX2WAV - Given a pixel value, compute and print the corresponding wavelength
#             in an image

procedure t_pix2wav()

int	list			# List of input images
int	pixlist[SZ_LINE]	# List of pixels for which to get wavelengths

char	image[SZ_FNAME]		# image file name
char	name[SZ_FNAME]		# image name
int	mspec			# Number of spectrum to read from multispec file
int	mband			# Number of band to read from multispec file
int	i, ipix
int	nwpix			# Number of pixels for which to list wavelengths
real	w0, wpc			# wavelength parameters
pointer	pix			# pointer to pixel values
pointer	wav			# pointer to pixel wavelengths
pointer	im			# Image header structure
pointer	sh			# Spectrum header structure
int	npts			# Number of pixels in spectrum
int	nband			# Band number for 3-D images
real	wavelength		# Wavelength at pixel in angstroms
 
char	temp[16]
int	ltitle
bool	debug, verbose
int	pixel_range[3,MAX_RANGES]

int	clpopnu(), clgfil()
int	clgeti()
bool	clgetb()
pointer	immap()
errchk	immap()
int	imaccf()
int	nline, ap, ndim
int	strlen(),strldxs(),ip
int	decode_ranges(),get_next_number()

define	nextimage_	90

begin
	# Open input list of images
	list = clpopnu ("input")
	mspec = clgeti ("specnum")
	mband = clgeti ("specband")

	call clgstr ("pixels",pixlist,SZ_LINE)
        if (decode_ranges (pixlist, pixel_range, MAX_RANGES, nwpix) == ERR)
            call error (1, "PIX2WAV: Illegal pixel list")

	verbose = clgetb ("verbose")
	debug = FALSE

	# Loop over images
	while (clgfil (list, image, SZ_FNAME) != EOF) {

	    # Try to open the image
	    iferr (im = immap (image, READ_ONLY, 0)) {
		call eprintf ("LISTWAV: Cannot read image %s\n")
		    call pargstr (image)
		next
		}

	    ndim = IM_NDIM(im)
	    if (debug) {
		call printf ("PIX2WAV: %d x %d x %d %d-D image: spectrum %d %d\n")
		    call pargi (IM_LEN(im,1))
		    call pargi (IM_LEN(im,2))
		    call pargi (IM_LEN(im,3))
		    call pargi (ndim)
		    call pargi (mspec)
		    call pargi (mband)
		}

	    if (ndim > 2 && IM_LEN(im,3) > 1) {
		nband = max (1, min (IM_LEN(im,3), mband))
		nline = max (1, min (IM_LEN(im,2), mspec))
		}
	    else if (ndim > 1 && IM_LEN(im,2) > 1) {
		nline = max (1, min (IM_LEN(im,2), mspec))
		nband = 1
		}
	    else {
		nline = 1
		nband = 1
		}

	# Set up FITS World Coordinate System keywords
	# if not present in 1-D spectrum
	    if (ndim < 2) {
		if (imaccf (im, "W0") == YES && imaccf (im, "CRVAL1") == NO) {
		    call imgdpar (im, "W0",w0)
		    call imaddi (im,"CRPIX1",1)
		    call imaddd (im,"CRVAL1",w0)
		    }
		if (imaccf (im, "WPC") == YES && imaccf (im, "CDELT1") == NO) {
        	    call imgdpar (im, "WPC",wpc)
		    call imaddd (im,"CDELT1",wpc)
		    }
		}

	# Set dispersion axis for 1-D spectra
	    if (imaccf (im, "DISPAXIS") == NO)
		call imaddi (im,"DISPAXIS",1)

	# Open spectrum header
            ap = nline
	    call shdr_open (im, nline, nband, ap, SHDATA, sh)

	    npts = IM_LEN (im, 1)

	# Get image line to list
	    pix = SY(sh)
	    wav = SX(sh)

	# Title is object name (and spectrum number if multispec)
	    call strcpy (IM_TITLE(im), image, SZ_FNAME)
	    ltitle = strlen (image)
	    if (ltitle > 1) {
		ip = strldxs ("$/",image)
		if (ip > 0)
		    call strcpy (image[ip+1],name,SZ_FNAME)
		else
		    call strcpy (image, name, SZ_FNAME)
		}
	    else
		name[1] = EOS

	    if (mband > 0) {
		call sprintf (temp,16,"[%d,%d]")
		    call pargi (mspec)
		    call pargi (mband)
		call strcat (temp,name,SZ_FNAME)
		}
		
	    else if (mspec > 0) {
		call sprintf (temp,16,"[%d]")
		    call pargi (mspec)
		call strcat (temp,name,SZ_FNAME)
		}

	# Print image name
	    if (verbose) {
		call printf ("Spectrum %s\n\n")
		    call pargstr (name)
		}

	# Loop over pixels
	    do i = 1, nwpix {
		if (get_next_number (pixel_range, ipix) == EOF)
		    go to nextimage_
		if (ipix < 1 || ipix > npts) {
		    if (verbose) {
			call printf ("%4d: offscale\n")
			    call pargi (ipix)
			}
		    else
			call printf ("offscale\n")
		    }
		else {
		    wavelength = Memr[pix+ipix]
		    if (verbose) {
			call printf ("%4d: %9.3f\n")
			    call pargi (ipix)
			    call pargr (wavelength)
			}
		    else {
			call printf ("%9.3f\n")
			    call pargr (wavelength)
			}
		    }
		}

	# Flush standard output before opening next image
nextimage_
	    call flush (STDOUT)

	# Close image
	    call imunmap (im)
	    call shdr_close (sh)
	    }

	# Close input list
	call clpcls (list)
end
# Apr 22 1998	New task
