include	<error.h>
include	<smw.h>
include	<imhdr.h>

define	LEN_USER_AREA	100000
define	MAX_RANGES	50

# T_WAV2PIX - Given a wavelength value, compute and print the corresponding
#	      pixel in an image

procedure t_wav2pix()

int	list			# List of input images

char	image[SZ_FNAME]		# image file name
char	name[SZ_FNAME]		# image name
int	mspec			# Number of spectrum to read from multispec file
int	mband			# Number of band to read from multispec file
real	w0, wpc			# wavelength parameters
double	pix			# Pixel
double	wave			# Wavelength in Angstroms
pointer	im			# Image header structure
pointer	sh			# Spectrum header structure
pointer	wav			# Vector of wavelengths for each pixel
int	nband			# Band number for 3-D images
 
char	temp[16]
int	ltitle
bool	debug, verbose
pointer	smw, smw_openim()
double	wcs_w2p()
double	minwav, maxwav
int	minpix, maxpix

int	clpopnu(), clgfil()
bool	clgetb()
int	clgeti()
double	clgetd()
pointer	immap()
errchk	immap()
int	imaccf()
int	nline, ap, ndim
int	strlen(),strldxs(),ip

define	nextimage_	90

begin
	# Open input list of images
	list = clpopnu ("input")
	mspec = clgeti ("specnum")
	mband = clgeti ("specband")
	verbose = clgetb ("verbose")
	debug = clgetb ("debug")

	# Loop over images
	while (clgfil (list, image, SZ_FNAME) != EOF) {

	    # Try to open the image
	    iferr (im = immap (image, READ_ONLY, 0)) {
		call eprintf ("WAV2PIX: Cannot read image %s\n")
		    call pargstr (image)
		next
		}

	    ndim = IM_NDIM(im)
	    if (debug) {
		call printf ("WAV2PIX: %d x %d x %d %d-D image: spectrum %d %d\n")
		    call pargi (IM_LEN(im,1))
		    call pargi (IM_LEN(im,2))
		    call pargi (IM_LEN(im,3))
		    call pargi (ndim)
		    call pargi (mspec)
		    call pargi (mband)
		}

	    if (ndim > 2 && IM_LEN(im,3) > 1) {
		nband = max (1, min (IM_LEN(im,3), mband))
		nline = max (1, min (IM_LEN(im,2), mspec))
		}
	    else if (ndim > 1 && IM_LEN(im,2) > 1) {
		nline = max (1, min (IM_LEN(im,2), mspec))
		nband = 1
		}
	    else {
		nline = 1
		nband = 1
		}

	# Set up FITS World Coordinate System keywords
	# if not present in 1-D spectrum
	    if (ndim < 2) {
		if (imaccf (im, "W0") == YES && imaccf (im, "CRVAL1") == NO) {
		    call imgdpar (im, "W0",w0)
		    call imaddi (im,"CRPIX1",1)
		    call imaddd (im,"CRVAL1",w0)
		    }
		if (imaccf (im, "WPC") == YES && imaccf (im, "CDELT1") == NO) {
        	    call imgdpar (im, "WPC",wpc)
		    call imaddd (im,"CDELT1",wpc)
		    }
		}

	# Set dispersion axis for 1-D spectra
	    if (imaccf (im, "DISPAXIS") == NO)
		call imaddi (im,"DISPAXIS",1)

	#  Open spectrum world coordinate system
	    if (debug) {
		call printf ("GETIMAGE: Ready to open MWCS\n")
		call flush (STDOUT)
		}
	    smw = smw_openim (im)
	    call smw_daxis (smw, im, 0, INDEFI, INDEFI)
	    call smw_saxes (smw, NULL, im)

	    if (smw == ERR) {
		call eprintf ("GETIMAGE: MWCS error in image %s\n")
		call pargstr (image)
		call imunmap (im)
		im = ERR
		return
		}

	# Open spectrum header
            ap = nline
	    call shdr_open (im, smw, nline, nband, ap, SHDATA, sh)

	    call wcs_set (sh)

	# Title is object name (and spectrum number if multispec)
	    call strcpy (IM_TITLE(im), image, SZ_FNAME)
	    ltitle = strlen (image)
	    if (ltitle > 1) {
		ip = strldxs ("$/",image)
		if (ip > 0)
		    call strcpy (image[ip+1],name,SZ_FNAME)
		else
		    call strcpy (image, name, SZ_FNAME)
		}
	    else
		name[1] = EOS

	    if (mband > 0) {
		call sprintf (temp,16,"[%d,%d]")
		    call pargi (mspec)
		    call pargi (mband)
		call strcat (temp,name,SZ_FNAME)
		}
		
	    else if (mspec > 0) {
		call sprintf (temp,16,"[%d]")
		    call pargi (mspec)
		call strcat (temp,name,SZ_FNAME)
		}

        # Find first and last pixels
            minpix = NP1(sh)
            maxpix = NP2(sh)

        # Get image wavelengths
            wav = SX(sh)

        # Find wavelengths for first and last pixels
            minwav = double (Memr[wav+minpix-1])
            maxwav = double (Memr[wav+maxpix-1])

	# Compute pixel only if wavelength is within limits
	    wave = clgetd ("wavelength")
	    if (wave >= minwav && wave <= maxwav) {

	# Convert wavelength
		pix = wcs_w2p (wave)
		call clputd ("pixel", pix)

	# Print image name, wavelength, and pixel
		if (verbose) {
		    call printf ("Spectrum %s: wavelength %8.3f = pixel %8.3f\n")
			call pargstr (name)
			call pargd (wave)
			call pargd (pix)
		    }
		}
	    else if (verbose) {
		if (wav < minwav) {
		    call printf ("Spectrum %s: wavelength %8.3f < %8.3f, outside of image.\n")
			call pargstr (name)
			call pargd (wave)
			call pargd (minwav)
		    }
		if (wav > maxwav) {
		    call printf ("Spectrum %s: wavelength %8.3f > %8.3f, outside of image.\n")
			call pargstr (name)
			call pargd (wave)
			call pargd (maxwav)
		    }
		}

	# Flush standard output before opening next image
nextimage_
	    call flush (STDOUT)

	# Close image
	    call imunmap (im)
	    call shdr_close (sh)
	    }

	# Close input list
	call clpcls (list)
end

# Jul  2 2003	New task based on listwav
